/* Generated by Cython 0.29.7 */

/* BEGIN: Cython Metadata
{
    "distutils": {
        "depends": [],
        "name": "traceutils.scamper.warts",
        "sources": [
            "traceutils/scamper/warts.pyx"
        ]
    },
    "module_name": "traceutils.scamper.warts"
}
END: Cython Metadata */

#define PY_SSIZE_T_CLEAN
#include "Python.h"
#ifndef Py_PYTHON_H
    #error Python headers needed to compile C extensions, please install development version of Python.
#elif PY_VERSION_HEX < 0x02060000 || (0x03000000 <= PY_VERSION_HEX && PY_VERSION_HEX < 0x03030000)
    #error Cython requires Python 2.6+ or Python 3.3+.
#else
#define CYTHON_ABI "0_29_7"
#define CYTHON_HEX_VERSION 0x001D07F0
#define CYTHON_FUTURE_DIVISION 1
#include <stddef.h>
#ifndef offsetof
  #define offsetof(type, member) ( (size_t) & ((type*)0) -> member )
#endif
#if !defined(WIN32) && !defined(MS_WINDOWS)
  #ifndef __stdcall
    #define __stdcall
  #endif
  #ifndef __cdecl
    #define __cdecl
  #endif
  #ifndef __fastcall
    #define __fastcall
  #endif
#endif
#ifndef DL_IMPORT
  #define DL_IMPORT(t) t
#endif
#ifndef DL_EXPORT
  #define DL_EXPORT(t) t
#endif
#define __PYX_COMMA ,
#ifndef HAVE_LONG_LONG
  #if PY_VERSION_HEX >= 0x02070000
    #define HAVE_LONG_LONG
  #endif
#endif
#ifndef PY_LONG_LONG
  #define PY_LONG_LONG LONG_LONG
#endif
#ifndef Py_HUGE_VAL
  #define Py_HUGE_VAL HUGE_VAL
#endif
#ifdef PYPY_VERSION
  #define CYTHON_COMPILING_IN_PYPY 1
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #if PY_VERSION_HEX < 0x03050000
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #undef CYTHON_AVOID_BORROWED_REFS
  #define CYTHON_AVOID_BORROWED_REFS 1
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
#elif defined(PYSTON_VERSION)
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 1
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #undef CYTHON_USE_ASYNC_SLOTS
  #define CYTHON_USE_ASYNC_SLOTS 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
#else
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 1
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYTYPE_LOOKUP
    #define CYTHON_USE_PYTYPE_LOOKUP 0
  #elif !defined(CYTHON_USE_PYTYPE_LOOKUP)
    #define CYTHON_USE_PYTYPE_LOOKUP 1
  #endif
  #if PY_MAJOR_VERSION < 3
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYLONG_INTERNALS
    #define CYTHON_USE_PYLONG_INTERNALS 0
  #elif !defined(CYTHON_USE_PYLONG_INTERNALS)
    #define CYTHON_USE_PYLONG_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_PYLIST_INTERNALS
    #define CYTHON_USE_PYLIST_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #if PY_VERSION_HEX < 0x030300F0
    #undef CYTHON_USE_UNICODE_WRITER
    #define CYTHON_USE_UNICODE_WRITER 0
  #elif !defined(CYTHON_USE_UNICODE_WRITER)
    #define CYTHON_USE_UNICODE_WRITER 1
  #endif
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #ifndef CYTHON_FAST_THREAD_STATE
    #define CYTHON_FAST_THREAD_STATE 1
  #endif
  #ifndef CYTHON_FAST_PYCALL
    #define CYTHON_FAST_PYCALL 1
  #endif
  #ifndef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT (PY_VERSION_HEX >= 0x03050000)
  #endif
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE (PY_VERSION_HEX >= 0x030400a1)
  #endif
  #ifndef CYTHON_USE_DICT_VERSIONS
    #define CYTHON_USE_DICT_VERSIONS (PY_VERSION_HEX >= 0x030600B1)
  #endif
  #ifndef CYTHON_USE_EXC_INFO_STACK
    #define CYTHON_USE_EXC_INFO_STACK (PY_VERSION_HEX >= 0x030700A3)
  #endif
#endif
#if !defined(CYTHON_FAST_PYCCALL)
#define CYTHON_FAST_PYCCALL  (CYTHON_FAST_PYCALL && PY_VERSION_HEX >= 0x030600B1)
#endif
#if CYTHON_USE_PYLONG_INTERNALS
  #include "longintrepr.h"
  #undef SHIFT
  #undef BASE
  #undef MASK
  #ifdef SIZEOF_VOID_P
    enum { __pyx_check_sizeof_voidp = 1 / (int)(SIZEOF_VOID_P == sizeof(void*)) };
  #endif
#endif
#ifndef __has_attribute
  #define __has_attribute(x) 0
#endif
#ifndef __has_cpp_attribute
  #define __has_cpp_attribute(x) 0
#endif
#ifndef CYTHON_RESTRICT
  #if defined(__GNUC__)
    #define CYTHON_RESTRICT __restrict__
  #elif defined(_MSC_VER) && _MSC_VER >= 1400
    #define CYTHON_RESTRICT __restrict
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_RESTRICT restrict
  #else
    #define CYTHON_RESTRICT
  #endif
#endif
#ifndef CYTHON_UNUSED
# if defined(__GNUC__)
#   if !(defined(__cplusplus)) || (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
#     define CYTHON_UNUSED __attribute__ ((__unused__))
#   else
#     define CYTHON_UNUSED
#   endif
# elif defined(__ICC) || (defined(__INTEL_COMPILER) && !defined(_MSC_VER))
#   define CYTHON_UNUSED __attribute__ ((__unused__))
# else
#   define CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_MAYBE_UNUSED_VAR
#  if defined(__cplusplus)
     template<class T> void CYTHON_MAYBE_UNUSED_VAR( const T& ) { }
#  else
#    define CYTHON_MAYBE_UNUSED_VAR(x) (void)(x)
#  endif
#endif
#ifndef CYTHON_NCP_UNUSED
# if CYTHON_COMPILING_IN_CPYTHON
#  define CYTHON_NCP_UNUSED
# else
#  define CYTHON_NCP_UNUSED CYTHON_UNUSED
# endif
#endif
#define __Pyx_void_to_None(void_result) ((void)(void_result), Py_INCREF(Py_None), Py_None)
#ifdef _MSC_VER
    #ifndef _MSC_STDINT_H_
        #if _MSC_VER < 1300
           typedef unsigned char     uint8_t;
           typedef unsigned int      uint32_t;
        #else
           typedef unsigned __int8   uint8_t;
           typedef unsigned __int32  uint32_t;
        #endif
    #endif
#else
   #include <stdint.h>
#endif
#ifndef CYTHON_FALLTHROUGH
  #if defined(__cplusplus) && __cplusplus >= 201103L
    #if __has_cpp_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH [[fallthrough]]
    #elif __has_cpp_attribute(clang::fallthrough)
      #define CYTHON_FALLTHROUGH [[clang::fallthrough]]
    #elif __has_cpp_attribute(gnu::fallthrough)
      #define CYTHON_FALLTHROUGH [[gnu::fallthrough]]
    #endif
  #endif
  #ifndef CYTHON_FALLTHROUGH
    #if __has_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH __attribute__((fallthrough))
    #else
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
  #if defined(__clang__ ) && defined(__apple_build_version__)
    #if __apple_build_version__ < 7000000
      #undef  CYTHON_FALLTHROUGH
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
#endif

#ifndef CYTHON_INLINE
  #if defined(__clang__)
    #define CYTHON_INLINE __inline__ __attribute__ ((__unused__))
  #elif defined(__GNUC__)
    #define CYTHON_INLINE __inline__
  #elif defined(_MSC_VER)
    #define CYTHON_INLINE __inline
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_INLINE inline
  #else
    #define CYTHON_INLINE
  #endif
#endif

#if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x02070600 && !defined(Py_OptimizeFlag)
  #define Py_OptimizeFlag 0
#endif
#define __PYX_BUILD_PY_SSIZE_T "n"
#define CYTHON_FORMAT_SSIZE_T "z"
#if PY_MAJOR_VERSION < 3
  #define __Pyx_BUILTIN_MODULE_NAME "__builtin__"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a+k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyClass_Type
#else
  #define __Pyx_BUILTIN_MODULE_NAME "builtins"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyType_Type
#endif
#ifndef Py_TPFLAGS_CHECKTYPES
  #define Py_TPFLAGS_CHECKTYPES 0
#endif
#ifndef Py_TPFLAGS_HAVE_INDEX
  #define Py_TPFLAGS_HAVE_INDEX 0
#endif
#ifndef Py_TPFLAGS_HAVE_NEWBUFFER
  #define Py_TPFLAGS_HAVE_NEWBUFFER 0
#endif
#ifndef Py_TPFLAGS_HAVE_FINALIZE
  #define Py_TPFLAGS_HAVE_FINALIZE 0
#endif
#ifndef METH_STACKLESS
  #define METH_STACKLESS 0
#endif
#if PY_VERSION_HEX <= 0x030700A3 || !defined(METH_FASTCALL)
  #ifndef METH_FASTCALL
     #define METH_FASTCALL 0x80
  #endif
  typedef PyObject *(*__Pyx_PyCFunctionFast) (PyObject *self, PyObject *const *args, Py_ssize_t nargs);
  typedef PyObject *(*__Pyx_PyCFunctionFastWithKeywords) (PyObject *self, PyObject *const *args,
                                                          Py_ssize_t nargs, PyObject *kwnames);
#else
  #define __Pyx_PyCFunctionFast _PyCFunctionFast
  #define __Pyx_PyCFunctionFastWithKeywords _PyCFunctionFastWithKeywords
#endif
#if CYTHON_FAST_PYCCALL
#define __Pyx_PyFastCFunction_Check(func)\
    ((PyCFunction_Check(func) && (METH_FASTCALL == (PyCFunction_GET_FLAGS(func) & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS | METH_STACKLESS)))))
#else
#define __Pyx_PyFastCFunction_Check(func) 0
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Malloc)
  #define PyObject_Malloc(s)   PyMem_Malloc(s)
  #define PyObject_Free(p)     PyMem_Free(p)
  #define PyObject_Realloc(p)  PyMem_Realloc(p)
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030400A1
  #define PyMem_RawMalloc(n)           PyMem_Malloc(n)
  #define PyMem_RawRealloc(p, n)       PyMem_Realloc(p, n)
  #define PyMem_RawFree(p)             PyMem_Free(p)
#endif
#if CYTHON_COMPILING_IN_PYSTON
  #define __Pyx_PyCode_HasFreeVars(co)  PyCode_HasFreeVars(co)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno) PyFrame_SetLineNumber(frame, lineno)
#else
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno)  (frame)->f_lineno = (lineno)
#endif
#if !CYTHON_FAST_THREAD_STATE || PY_VERSION_HEX < 0x02070000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#elif PY_VERSION_HEX >= 0x03060000
  #define __Pyx_PyThreadState_Current _PyThreadState_UncheckedGet()
#elif PY_VERSION_HEX >= 0x03000000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#else
  #define __Pyx_PyThreadState_Current _PyThreadState_Current
#endif
#if PY_VERSION_HEX < 0x030700A2 && !defined(PyThread_tss_create) && !defined(Py_tss_NEEDS_INIT)
#include "pythread.h"
#define Py_tss_NEEDS_INIT 0
typedef int Py_tss_t;
static CYTHON_INLINE int PyThread_tss_create(Py_tss_t *key) {
  *key = PyThread_create_key();
  return 0;
}
static CYTHON_INLINE Py_tss_t * PyThread_tss_alloc(void) {
  Py_tss_t *key = (Py_tss_t *)PyObject_Malloc(sizeof(Py_tss_t));
  *key = Py_tss_NEEDS_INIT;
  return key;
}
static CYTHON_INLINE void PyThread_tss_free(Py_tss_t *key) {
  PyObject_Free(key);
}
static CYTHON_INLINE int PyThread_tss_is_created(Py_tss_t *key) {
  return *key != Py_tss_NEEDS_INIT;
}
static CYTHON_INLINE void PyThread_tss_delete(Py_tss_t *key) {
  PyThread_delete_key(*key);
  *key = Py_tss_NEEDS_INIT;
}
static CYTHON_INLINE int PyThread_tss_set(Py_tss_t *key, void *value) {
  return PyThread_set_key_value(*key, value);
}
static CYTHON_INLINE void * PyThread_tss_get(Py_tss_t *key) {
  return PyThread_get_key_value(*key);
}
#endif
#if CYTHON_COMPILING_IN_CPYTHON || defined(_PyDict_NewPresized)
#define __Pyx_PyDict_NewPresized(n)  ((n <= 8) ? PyDict_New() : _PyDict_NewPresized(n))
#else
#define __Pyx_PyDict_NewPresized(n)  PyDict_New()
#endif
#if PY_MAJOR_VERSION >= 3 || CYTHON_FUTURE_DIVISION
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_TrueDivide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceTrueDivide(x,y)
#else
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_Divide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceDivide(x,y)
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030500A1 && CYTHON_USE_UNICODE_INTERNALS
#define __Pyx_PyDict_GetItemStr(dict, name)  _PyDict_GetItem_KnownHash(dict, name, ((PyASCIIObject *) name)->hash)
#else
#define __Pyx_PyDict_GetItemStr(dict, name)  PyDict_GetItem(dict, name)
#endif
#if PY_VERSION_HEX > 0x03030000 && defined(PyUnicode_KIND)
  #define CYTHON_PEP393_ENABLED 1
  #define __Pyx_PyUnicode_READY(op)       (likely(PyUnicode_IS_READY(op)) ?\
                                              0 : _PyUnicode_Ready((PyObject *)(op)))
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_LENGTH(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_READ_CHAR(u, i)
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   PyUnicode_MAX_CHAR_VALUE(u)
  #define __Pyx_PyUnicode_KIND(u)         PyUnicode_KIND(u)
  #define __Pyx_PyUnicode_DATA(u)         PyUnicode_DATA(u)
  #define __Pyx_PyUnicode_READ(k, d, i)   PyUnicode_READ(k, d, i)
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  PyUnicode_WRITE(k, d, i, ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : PyUnicode_GET_SIZE(u)))
#else
  #define CYTHON_PEP393_ENABLED 0
  #define PyUnicode_1BYTE_KIND  1
  #define PyUnicode_2BYTE_KIND  2
  #define PyUnicode_4BYTE_KIND  4
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_SIZE(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) ((Py_UCS4)(PyUnicode_AS_UNICODE(u)[i]))
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   ((sizeof(Py_UNICODE) == 2) ? 65535 : 1114111)
  #define __Pyx_PyUnicode_KIND(u)         (sizeof(Py_UNICODE))
  #define __Pyx_PyUnicode_DATA(u)         ((void*)PyUnicode_AS_UNICODE(u))
  #define __Pyx_PyUnicode_READ(k, d, i)   ((void)(k), (Py_UCS4)(((Py_UNICODE*)d)[i]))
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  (((void)(k)), ((Py_UNICODE*)d)[i] = ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_SIZE(u))
#endif
#if CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyUnicode_Concat(a, b)      PyNumber_Add(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  PyNumber_Add(a, b)
#else
  #define __Pyx_PyUnicode_Concat(a, b)      PyUnicode_Concat(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  ((unlikely((a) == Py_None) || unlikely((b) == Py_None)) ?\
      PyNumber_Add(a, b) : __Pyx_PyUnicode_Concat(a, b))
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyUnicode_Contains)
  #define PyUnicode_Contains(u, s)  PySequence_Contains(u, s)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyByteArray_Check)
  #define PyByteArray_Check(obj)  PyObject_TypeCheck(obj, &PyByteArray_Type)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Format)
  #define PyObject_Format(obj, fmt)  PyObject_CallMethod(obj, "__format__", "O", fmt)
#endif
#define __Pyx_PyString_FormatSafe(a, b)   ((unlikely((a) == Py_None || (PyString_Check(b) && !PyString_CheckExact(b)))) ? PyNumber_Remainder(a, b) : __Pyx_PyString_Format(a, b))
#define __Pyx_PyUnicode_FormatSafe(a, b)  ((unlikely((a) == Py_None || (PyUnicode_Check(b) && !PyUnicode_CheckExact(b)))) ? PyNumber_Remainder(a, b) : PyUnicode_Format(a, b))
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyString_Format(a, b)  PyUnicode_Format(a, b)
#else
  #define __Pyx_PyString_Format(a, b)  PyString_Format(a, b)
#endif
#if PY_MAJOR_VERSION < 3 && !defined(PyObject_ASCII)
  #define PyObject_ASCII(o)            PyObject_Repr(o)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBaseString_Type            PyUnicode_Type
  #define PyStringObject               PyUnicodeObject
  #define PyString_Type                PyUnicode_Type
  #define PyString_Check               PyUnicode_Check
  #define PyString_CheckExact          PyUnicode_CheckExact
  #define PyObject_Unicode             PyObject_Str
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyBaseString_Check(obj) PyUnicode_Check(obj)
  #define __Pyx_PyBaseString_CheckExact(obj) PyUnicode_CheckExact(obj)
#else
  #define __Pyx_PyBaseString_Check(obj) (PyString_Check(obj) || PyUnicode_Check(obj))
  #define __Pyx_PyBaseString_CheckExact(obj) (PyString_CheckExact(obj) || PyUnicode_CheckExact(obj))
#endif
#ifndef PySet_CheckExact
  #define PySet_CheckExact(obj)        (Py_TYPE(obj) == &PySet_Type)
#endif
#if CYTHON_ASSUME_SAFE_MACROS
  #define __Pyx_PySequence_SIZE(seq)  Py_SIZE(seq)
#else
  #define __Pyx_PySequence_SIZE(seq)  PySequence_Size(seq)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyIntObject                  PyLongObject
  #define PyInt_Type                   PyLong_Type
  #define PyInt_Check(op)              PyLong_Check(op)
  #define PyInt_CheckExact(op)         PyLong_CheckExact(op)
  #define PyInt_FromString             PyLong_FromString
  #define PyInt_FromUnicode            PyLong_FromUnicode
  #define PyInt_FromLong               PyLong_FromLong
  #define PyInt_FromSize_t             PyLong_FromSize_t
  #define PyInt_FromSsize_t            PyLong_FromSsize_t
  #define PyInt_AsLong                 PyLong_AsLong
  #define PyInt_AS_LONG                PyLong_AS_LONG
  #define PyInt_AsSsize_t              PyLong_AsSsize_t
  #define PyInt_AsUnsignedLongMask     PyLong_AsUnsignedLongMask
  #define PyInt_AsUnsignedLongLongMask PyLong_AsUnsignedLongLongMask
  #define PyNumber_Int                 PyNumber_Long
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBoolObject                 PyLongObject
#endif
#if PY_MAJOR_VERSION >= 3 && CYTHON_COMPILING_IN_PYPY
  #ifndef PyUnicode_InternFromString
    #define PyUnicode_InternFromString(s) PyUnicode_FromString(s)
  #endif
#endif
#if PY_VERSION_HEX < 0x030200A4
  typedef long Py_hash_t;
  #define __Pyx_PyInt_FromHash_t PyInt_FromLong
  #define __Pyx_PyInt_AsHash_t   PyInt_AsLong
#else
  #define __Pyx_PyInt_FromHash_t PyInt_FromSsize_t
  #define __Pyx_PyInt_AsHash_t   PyInt_AsSsize_t
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyMethod_New(func, self, klass) ((self) ? PyMethod_New(func, self) : (Py_INCREF(func), func))
#else
  #define __Pyx_PyMethod_New(func, self, klass) PyMethod_New(func, self, klass)
#endif
#if CYTHON_USE_ASYNC_SLOTS
  #if PY_VERSION_HEX >= 0x030500B1
    #define __Pyx_PyAsyncMethodsStruct PyAsyncMethods
    #define __Pyx_PyType_AsAsync(obj) (Py_TYPE(obj)->tp_as_async)
  #else
    #define __Pyx_PyType_AsAsync(obj) ((__Pyx_PyAsyncMethodsStruct*) (Py_TYPE(obj)->tp_reserved))
  #endif
#else
  #define __Pyx_PyType_AsAsync(obj) NULL
#endif
#ifndef __Pyx_PyAsyncMethodsStruct
    typedef struct {
        unaryfunc am_await;
        unaryfunc am_aiter;
        unaryfunc am_anext;
    } __Pyx_PyAsyncMethodsStruct;
#endif

#if defined(WIN32) || defined(MS_WINDOWS)
  #define _USE_MATH_DEFINES
#endif
#include <math.h>
#ifdef NAN
#define __PYX_NAN() ((float) NAN)
#else
static CYTHON_INLINE float __PYX_NAN() {
  float value;
  memset(&value, 0xFF, sizeof(value));
  return value;
}
#endif
#if defined(__CYGWIN__) && defined(_LDBL_EQ_DBL)
#define __Pyx_truncl trunc
#else
#define __Pyx_truncl truncl
#endif


#define __PYX_ERR(f_index, lineno, Ln_error) \
{ \
  __pyx_filename = __pyx_f[f_index]; __pyx_lineno = lineno; __pyx_clineno = __LINE__; goto Ln_error; \
}

#ifndef __PYX_EXTERN_C
  #ifdef __cplusplus
    #define __PYX_EXTERN_C extern "C"
  #else
    #define __PYX_EXTERN_C extern
  #endif
#endif

#define __PYX_HAVE__traceutils__scamper__warts
#define __PYX_HAVE_API__traceutils__scamper__warts
/* Early includes */
#include <arpa/inet.h>
#ifdef _OPENMP
#include <omp.h>
#endif /* _OPENMP */

#if defined(PYREX_WITHOUT_ASSERTIONS) && !defined(CYTHON_WITHOUT_ASSERTIONS)
#define CYTHON_WITHOUT_ASSERTIONS
#endif

typedef struct {PyObject **p; const char *s; const Py_ssize_t n; const char* encoding;
                const char is_unicode; const char is_str; const char intern; } __Pyx_StringTabEntry;

#define __PYX_DEFAULT_STRING_ENCODING_IS_ASCII 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_UTF8 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT (PY_MAJOR_VERSION >= 3 && __PYX_DEFAULT_STRING_ENCODING_IS_UTF8)
#define __PYX_DEFAULT_STRING_ENCODING ""
#define __Pyx_PyObject_FromString __Pyx_PyBytes_FromString
#define __Pyx_PyObject_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#define __Pyx_uchar_cast(c) ((unsigned char)c)
#define __Pyx_long_cast(x) ((long)x)
#define __Pyx_fits_Py_ssize_t(v, type, is_signed)  (\
    (sizeof(type) < sizeof(Py_ssize_t))  ||\
    (sizeof(type) > sizeof(Py_ssize_t) &&\
          likely(v < (type)PY_SSIZE_T_MAX ||\
                 v == (type)PY_SSIZE_T_MAX)  &&\
          (!is_signed || likely(v > (type)PY_SSIZE_T_MIN ||\
                                v == (type)PY_SSIZE_T_MIN)))  ||\
    (sizeof(type) == sizeof(Py_ssize_t) &&\
          (is_signed || likely(v < (type)PY_SSIZE_T_MAX ||\
                               v == (type)PY_SSIZE_T_MAX)))  )
static CYTHON_INLINE int __Pyx_is_valid_index(Py_ssize_t i, Py_ssize_t limit) {
    return (size_t) i < (size_t) limit;
}
#if defined (__cplusplus) && __cplusplus >= 201103L
    #include <cstdlib>
    #define __Pyx_sst_abs(value) std::abs(value)
#elif SIZEOF_INT >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) abs(value)
#elif SIZEOF_LONG >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) labs(value)
#elif defined (_MSC_VER)
    #define __Pyx_sst_abs(value) ((Py_ssize_t)_abs64(value))
#elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define __Pyx_sst_abs(value) llabs(value)
#elif defined (__GNUC__)
    #define __Pyx_sst_abs(value) __builtin_llabs(value)
#else
    #define __Pyx_sst_abs(value) ((value<0) ? -value : value)
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject*);
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject*, Py_ssize_t* length);
#define __Pyx_PyByteArray_FromString(s) PyByteArray_FromStringAndSize((const char*)s, strlen((const char*)s))
#define __Pyx_PyByteArray_FromStringAndSize(s, l) PyByteArray_FromStringAndSize((const char*)s, l)
#define __Pyx_PyBytes_FromString        PyBytes_FromString
#define __Pyx_PyBytes_FromStringAndSize PyBytes_FromStringAndSize
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char*);
#if PY_MAJOR_VERSION < 3
    #define __Pyx_PyStr_FromString        __Pyx_PyBytes_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#else
    #define __Pyx_PyStr_FromString        __Pyx_PyUnicode_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyUnicode_FromStringAndSize
#endif
#define __Pyx_PyBytes_AsWritableString(s)     ((char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableSString(s)    ((signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableUString(s)    ((unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsString(s)     ((const char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsSString(s)    ((const signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsUString(s)    ((const unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyObject_AsWritableString(s)    ((char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableSString(s)    ((signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableUString(s)    ((unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsSString(s)    ((const signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsUString(s)    ((const unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_FromCString(s)  __Pyx_PyObject_FromString((const char*)s)
#define __Pyx_PyBytes_FromCString(s)   __Pyx_PyBytes_FromString((const char*)s)
#define __Pyx_PyByteArray_FromCString(s)   __Pyx_PyByteArray_FromString((const char*)s)
#define __Pyx_PyStr_FromCString(s)     __Pyx_PyStr_FromString((const char*)s)
#define __Pyx_PyUnicode_FromCString(s) __Pyx_PyUnicode_FromString((const char*)s)
static CYTHON_INLINE size_t __Pyx_Py_UNICODE_strlen(const Py_UNICODE *u) {
    const Py_UNICODE *u_end = u;
    while (*u_end++) ;
    return (size_t)(u_end - u - 1);
}
#define __Pyx_PyUnicode_FromUnicode(u)       PyUnicode_FromUnicode(u, __Pyx_Py_UNICODE_strlen(u))
#define __Pyx_PyUnicode_FromUnicodeAndLength PyUnicode_FromUnicode
#define __Pyx_PyUnicode_AsUnicode            PyUnicode_AsUnicode
#define __Pyx_NewRef(obj) (Py_INCREF(obj), obj)
#define __Pyx_Owned_Py_None(b) __Pyx_NewRef(Py_None)
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b);
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject*);
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject*);
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x);
#define __Pyx_PySequence_Tuple(obj)\
    (likely(PyTuple_CheckExact(obj)) ? __Pyx_NewRef(obj) : PySequence_Tuple(obj))
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject*);
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t);
#if CYTHON_ASSUME_SAFE_MACROS
#define __pyx_PyFloat_AsDouble(x) (PyFloat_CheckExact(x) ? PyFloat_AS_DOUBLE(x) : PyFloat_AsDouble(x))
#else
#define __pyx_PyFloat_AsDouble(x) PyFloat_AsDouble(x)
#endif
#define __pyx_PyFloat_AsFloat(x) ((float) __pyx_PyFloat_AsDouble(x))
#if PY_MAJOR_VERSION >= 3
#define __Pyx_PyNumber_Int(x) (PyLong_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Long(x))
#else
#define __Pyx_PyNumber_Int(x) (PyInt_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Int(x))
#endif
#define __Pyx_PyNumber_Float(x) (PyFloat_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Float(x))
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
static int __Pyx_sys_getdefaultencoding_not_ascii;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    PyObject* ascii_chars_u = NULL;
    PyObject* ascii_chars_b = NULL;
    const char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    if (strcmp(default_encoding_c, "ascii") == 0) {
        __Pyx_sys_getdefaultencoding_not_ascii = 0;
    } else {
        char ascii_chars[128];
        int c;
        for (c = 0; c < 128; c++) {
            ascii_chars[c] = c;
        }
        __Pyx_sys_getdefaultencoding_not_ascii = 1;
        ascii_chars_u = PyUnicode_DecodeASCII(ascii_chars, 128, NULL);
        if (!ascii_chars_u) goto bad;
        ascii_chars_b = PyUnicode_AsEncodedString(ascii_chars_u, default_encoding_c, NULL);
        if (!ascii_chars_b || !PyBytes_Check(ascii_chars_b) || memcmp(ascii_chars, PyBytes_AS_STRING(ascii_chars_b), 128) != 0) {
            PyErr_Format(
                PyExc_ValueError,
                "This module compiled with c_string_encoding=ascii, but default encoding '%.200s' is not a superset of ascii.",
                default_encoding_c);
            goto bad;
        }
        Py_DECREF(ascii_chars_u);
        Py_DECREF(ascii_chars_b);
    }
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    Py_XDECREF(ascii_chars_u);
    Py_XDECREF(ascii_chars_b);
    return -1;
}
#endif
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT && PY_MAJOR_VERSION >= 3
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_DecodeUTF8(c_str, size, NULL)
#else
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_Decode(c_str, size, __PYX_DEFAULT_STRING_ENCODING, NULL)
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
static char* __PYX_DEFAULT_STRING_ENCODING;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) (const char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    __PYX_DEFAULT_STRING_ENCODING = (char*) malloc(strlen(default_encoding_c) + 1);
    if (!__PYX_DEFAULT_STRING_ENCODING) goto bad;
    strcpy(__PYX_DEFAULT_STRING_ENCODING, default_encoding_c);
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    return -1;
}
#endif
#endif


/* Test for GCC > 2.95 */
#if defined(__GNUC__)     && (__GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)))
  #define likely(x)   __builtin_expect(!!(x), 1)
  #define unlikely(x) __builtin_expect(!!(x), 0)
#else /* !__GNUC__ or GCC < 2.95 */
  #define likely(x)   (x)
  #define unlikely(x) (x)
#endif /* __GNUC__ */
static CYTHON_INLINE void __Pyx_pretend_to_initialize(void* ptr) { (void)ptr; }

static PyObject *__pyx_m = NULL;
static PyObject *__pyx_d;
static PyObject *__pyx_b;
static PyObject *__pyx_cython_runtime = NULL;
static PyObject *__pyx_empty_tuple;
static PyObject *__pyx_empty_bytes;
static PyObject *__pyx_empty_unicode;
static int __pyx_lineno;
static int __pyx_clineno = 0;
static const char * __pyx_cfilenm= __FILE__;
static const char *__pyx_filename;


static const char *__pyx_f[] = {
  "traceutils/scamper/warts.pyx",
  "traceutils/scamper/warts.pxd",
  "stringsource",
  "traceutils/radix/radix_prefix.pxd",
  "traceutils/radix/radix_node.pxd",
  "traceutils/radix/radix_tree.pxd",
  "traceutils/radix/radix.pxd",
  "traceutils/radix/ip2as.pxd",
};

/*--- Type declarations ---*/
struct __pyx_obj_10traceutils_5radix_12radix_prefix_RadixPrefix;
struct __pyx_obj_10traceutils_5radix_10radix_node_RadixNode;
struct __pyx_obj_10traceutils_5radix_10radix_tree_RadixTree;
struct __pyx_obj_10traceutils_5radix_5radix_Radix;
struct __pyx_obj_10traceutils_5radix_5ip2as_IP2AS;
struct __pyx_obj_10traceutils_7scamper_3hop_Hop;
struct __pyx_obj_10traceutils_7scamper_3hop_Trace;
struct __pyx_obj_10traceutils_7scamper_3hop_Reader;
struct __pyx_obj_10traceutils_7scamper_5warts_WartsHop;
struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace;
struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing;
struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse;
struct __pyx_obj_10traceutils_7scamper_5warts_WartsReader;
struct __pyx_obj_10traceutils_7scamper_5warts___pyx_scope_struct____iter__;
struct __pyx_opt_args_10traceutils_5radix_12radix_prefix_from_network;
struct __pyx_opt_args_10traceutils_5radix_12radix_prefix_from_packed;

/* "traceutils/radix/radix_prefix.pxd":17
 * cdef unsigned char find_family(bytes network);
 * # cdef Network partition(bytes b);
 * cdef RadixPrefix from_network(str s, char masklen=*);             # <<<<<<<<<<<<<<
 * cdef RadixPrefix from_address(str s);
 * cdef RadixPrefix from_packed(bytes addr, char masklen=*);
 */
struct __pyx_opt_args_10traceutils_5radix_12radix_prefix_from_network {
  int __pyx_n;
  char masklen;
};

/* "traceutils/radix/radix_prefix.pxd":19
 * cdef RadixPrefix from_network(str s, char masklen=*);
 * cdef RadixPrefix from_address(str s);
 * cdef RadixPrefix from_packed(bytes addr, char masklen=*);             # <<<<<<<<<<<<<<
 * cdef (char*, char*) partition(bytes b);
 */
struct __pyx_opt_args_10traceutils_5radix_12radix_prefix_from_packed {
  int __pyx_n;
  char masklen;
};
struct __pyx_opt_args_10traceutils_5radix_5radix_5Radix__add;
struct __pyx_opt_args_10traceutils_5radix_5radix_5Radix_add;
struct __pyx_opt_args_10traceutils_5radix_5radix_5Radix_add_packed;
struct __pyx_opt_args_10traceutils_5radix_5radix_5Radix_delete;
struct __pyx_opt_args_10traceutils_5radix_5radix_5Radix_search_exact_prefix;
struct __pyx_opt_args_10traceutils_5radix_5radix_5Radix_search_best_prefix;
struct __pyx_opt_args_10traceutils_5radix_5radix_5Radix_search_worst_prefix;
struct __pyx_opt_args_10traceutils_5radix_5radix_5Radix_search_covered_prefix;
struct __pyx_opt_args_10traceutils_5radix_5radix_5Radix_search_covering_prefix;

/* "traceutils/radix/radix.pxd":12
 *     cdef long gen_id
 * 
 *     cdef RadixNode _add(self, RadixPrefix prefix, long asn=*);             # <<<<<<<<<<<<<<
 *     cdef void _delete(self, RadixNode node) except *;
 *     cdef RadixNode _search_exact(self, RadixPrefix prefix);
 */
struct __pyx_opt_args_10traceutils_5radix_5radix_5Radix__add {
  int __pyx_n;
  long asn;
};

/* "traceutils/radix/radix.pxd":19
 *     cdef list _search_covered(self, RadixPrefix prefix);
 * 
 *     cpdef RadixNode add(self, str network, short masklen=*, long asn=*);             # <<<<<<<<<<<<<<
 *     cpdef RadixNode add_packed(self, bytes packed, unsigned char masklen, long asn=*);
 *     cpdef void delete(self, str network, short masklen=*) except *;
 */
struct __pyx_opt_args_10traceutils_5radix_5radix_5Radix_add {
  int __pyx_n;
  short masklen;
  long asn;
};

/* "traceutils/radix/radix.pxd":20
 * 
 *     cpdef RadixNode add(self, str network, short masklen=*, long asn=*);
 *     cpdef RadixNode add_packed(self, bytes packed, unsigned char masklen, long asn=*);             # <<<<<<<<<<<<<<
 *     cpdef void delete(self, str network, short masklen=*) except *;
 *     cpdef void delete_packed(self, bytes packed, unsigned char masklen) except *;
 */
struct __pyx_opt_args_10traceutils_5radix_5radix_5Radix_add_packed {
  int __pyx_n;
  long asn;
};

/* "traceutils/radix/radix.pxd":21
 *     cpdef RadixNode add(self, str network, short masklen=*, long asn=*);
 *     cpdef RadixNode add_packed(self, bytes packed, unsigned char masklen, long asn=*);
 *     cpdef void delete(self, str network, short masklen=*) except *;             # <<<<<<<<<<<<<<
 *     cpdef void delete_packed(self, bytes packed, unsigned char masklen) except *;
 *     cpdef RadixNode search_exact(self, str addr);
 */
struct __pyx_opt_args_10traceutils_5radix_5radix_5Radix_delete {
  int __pyx_n;
  short masklen;
};

/* "traceutils/radix/radix.pxd":24
 *     cpdef void delete_packed(self, bytes packed, unsigned char masklen) except *;
 *     cpdef RadixNode search_exact(self, str addr);
 *     cpdef RadixNode search_exact_prefix(self, str network, short masklen=*);             # <<<<<<<<<<<<<<
 *     cpdef RadixNode search_exact_packed(self, bytes packed, unsigned char masklen);
 *     cpdef RadixNode search_best(self, str addr);
 */
struct __pyx_opt_args_10traceutils_5radix_5radix_5Radix_search_exact_prefix {
  int __pyx_n;
  short masklen;
};

/* "traceutils/radix/radix.pxd":27
 *     cpdef RadixNode search_exact_packed(self, bytes packed, unsigned char masklen);
 *     cpdef RadixNode search_best(self, str addr);
 *     cpdef RadixNode search_best_prefix(self, str network, short masklen=*);             # <<<<<<<<<<<<<<
 *     cpdef RadixNode search_best_packed(self, bytes packed, unsigned char masklen);
 *     cpdef RadixNode search_worst(self, str addr);
 */
struct __pyx_opt_args_10traceutils_5radix_5radix_5Radix_search_best_prefix {
  int __pyx_n;
  short masklen;
};

/* "traceutils/radix/radix.pxd":30
 *     cpdef RadixNode search_best_packed(self, bytes packed, unsigned char masklen);
 *     cpdef RadixNode search_worst(self, str addr);
 *     cpdef RadixNode search_worst_prefix(self, str network, short masklen=*);             # <<<<<<<<<<<<<<
 *     cpdef RadixNode search_worst_packed(self, bytes packed, unsigned char masklen);
 *     cpdef list search_covered(self, str addr);
 */
struct __pyx_opt_args_10traceutils_5radix_5radix_5Radix_search_worst_prefix {
  int __pyx_n;
  short masklen;
};

/* "traceutils/radix/radix.pxd":33
 *     cpdef RadixNode search_worst_packed(self, bytes packed, unsigned char masklen);
 *     cpdef list search_covered(self, str addr);
 *     cpdef list search_covered_prefix(self, str network, short masklen=*);             # <<<<<<<<<<<<<<
 *     cpdef list search_covered_packed(self, bytes packed, unsigned char masklen);
 *     cpdef list search_covering(self, str addr);
 */
struct __pyx_opt_args_10traceutils_5radix_5radix_5Radix_search_covered_prefix {
  int __pyx_n;
  short masklen;
};

/* "traceutils/radix/radix.pxd":36
 *     cpdef list search_covered_packed(self, bytes packed, unsigned char masklen);
 *     cpdef list search_covering(self, str addr);
 *     cpdef list search_covering_prefix(self, str network, short masklen=*);             # <<<<<<<<<<<<<<
 *     cpdef list search_covering_packed(self, bytes packed, unsigned char masklen);
 *     cpdef list nodes(self);
 */
struct __pyx_opt_args_10traceutils_5radix_5radix_5Radix_search_covering_prefix {
  int __pyx_n;
  short masklen;
};

/* "traceutils/scamper/hop.pxd":5
 *     cdef int AF_INET, AF_INET6
 * 
 * cpdef enum ICMPType:             # <<<<<<<<<<<<<<
 *     echo_reply = 1
 *     dest_unreach = 2
 */
enum __pyx_t_10traceutils_7scamper_3hop_ICMPType {
  __pyx_e_10traceutils_7scamper_3hop_echo_reply = 1,
  __pyx_e_10traceutils_7scamper_3hop_dest_unreach = 2,
  __pyx_e_10traceutils_7scamper_3hop_ptb = 3,
  __pyx_e_10traceutils_7scamper_3hop_time_exceeded = 4,
  __pyx_e_10traceutils_7scamper_3hop_echo_request = 5
};

/* "traceutils/radix/radix_prefix.pxd":5
 *     cdef int inet_pton(int, char*, void*)
 * 
 * cdef class RadixPrefix:             # <<<<<<<<<<<<<<
 *     cdef readonly bytes addr
 *     cdef readonly unsigned char bitlen
 */
struct __pyx_obj_10traceutils_5radix_12radix_prefix_RadixPrefix {
  PyObject_HEAD
  struct __pyx_vtabstruct_10traceutils_5radix_12radix_prefix_RadixPrefix *__pyx_vtab;
  PyObject *addr;
  unsigned char bitlen;
  unsigned char family;
};


/* "traceutils/radix/radix_node.pxd":3
 * from traceutils.radix.radix_prefix cimport RadixPrefix
 * 
 * cdef class RadixNode:             # <<<<<<<<<<<<<<
 *     cdef public RadixPrefix prefix
 *     cdef RadixNode parent, left, right
 */
struct __pyx_obj_10traceutils_5radix_10radix_node_RadixNode {
  PyObject_HEAD
  struct __pyx_obj_10traceutils_5radix_12radix_prefix_RadixPrefix *prefix;
  struct __pyx_obj_10traceutils_5radix_10radix_node_RadixNode *parent;
  struct __pyx_obj_10traceutils_5radix_10radix_node_RadixNode *left;
  struct __pyx_obj_10traceutils_5radix_10radix_node_RadixNode *right;
  unsigned char bitlen;
  long asn;
};


/* "traceutils/radix/radix_tree.pxd":9
 * # cdef bint prefix_match2(bytes left, bytes right, unsigned char bitlen);
 * 
 * cdef class RadixTree:             # <<<<<<<<<<<<<<
 *     # cdef unsigned char maxbits
 *     cdef RadixNode head
 */
struct __pyx_obj_10traceutils_5radix_10radix_tree_RadixTree {
  PyObject_HEAD
  struct __pyx_vtabstruct_10traceutils_5radix_10radix_tree_RadixTree *__pyx_vtab;
  struct __pyx_obj_10traceutils_5radix_10radix_node_RadixNode *head;
  long active_nodes;
};


/* "traceutils/radix/radix.pxd":7
 * cdef list _search_covering(RadixNode node);
 * 
 * cdef class Radix:             # <<<<<<<<<<<<<<
 *     cdef RadixTree _tree4
 *     cdef RadixTree _tree6
 */
struct __pyx_obj_10traceutils_5radix_5radix_Radix {
  PyObject_HEAD
  struct __pyx_vtabstruct_10traceutils_5radix_5radix_Radix *__pyx_vtab;
  struct __pyx_obj_10traceutils_5radix_10radix_tree_RadixTree *_tree4;
  struct __pyx_obj_10traceutils_5radix_10radix_tree_RadixTree *_tree6;
  long gen_id;
};


/* "traceutils/radix/ip2as.pxd":8
 * 
 * 
 * cdef class IP2AS(Radix):             # <<<<<<<<<<<<<<
 *     cpdef long asn(self, str asn);
 *     cpdef long asn_packed(self, bytes packed);
 */
struct __pyx_obj_10traceutils_5radix_5ip2as_IP2AS {
  struct __pyx_obj_10traceutils_5radix_5radix_Radix __pyx_base;
};


/* "traceutils/scamper/hop.pxd":14
 * cpdef ICMPType gettype(int family, int icmp_type) except *;
 * 
 * cdef class Hop:             # <<<<<<<<<<<<<<
 *     cdef public str addr
 *     cdef public unsigned char probe_ttl
 */
struct __pyx_obj_10traceutils_7scamper_3hop_Hop {
  PyObject_HEAD
  struct __pyx_vtabstruct_10traceutils_7scamper_3hop_Hop *__pyx_vtab;
  PyObject *addr;
  unsigned char probe_ttl;
  double rtt;
  unsigned char reply_ttl;
  int reply_tos;
  int reply_size;
  unsigned char icmp_type;
  unsigned char icmp_code;
  unsigned char icmp_q_ttl;
  int icmp_q_tos;
  PyObject *packed;
  int ismpls;
  int family;
  enum __pyx_t_10traceutils_7scamper_3hop_ICMPType type;
};


/* "traceutils/scamper/hop.pxd":33
 * 
 * 
 * cdef class Trace:             # <<<<<<<<<<<<<<
 *     cdef public str src
 *     cdef public str dst
 */
struct __pyx_obj_10traceutils_7scamper_3hop_Trace {
  PyObject_HEAD
  struct __pyx_vtabstruct_10traceutils_7scamper_3hop_Trace *__pyx_vtab;
  PyObject *src;
  PyObject *dst;
  PyObject *hops;
  PyObject *loop;
  int family;
};


/* "traceutils/scamper/hop.pxd":46
 *     cpdef void set_packed(self) except *;
 * 
 * cdef class Reader:             # <<<<<<<<<<<<<<
 *     cdef public str filename
 *     cdef public p
 */
struct __pyx_obj_10traceutils_7scamper_3hop_Reader {
  PyObject_HEAD
  struct __pyx_vtabstruct_10traceutils_7scamper_3hop_Reader *__pyx_vtab;
  PyObject *filename;
  PyObject *p;
};


/* "traceutils/scamper/warts.pxd":3
 * from traceutils.scamper.hop cimport Hop, Trace, Reader, ICMPType
 * 
 * cdef class WartsHop(Hop):             # <<<<<<<<<<<<<<
 *     cdef public list icmpext
 *     cdef public int probe_id
 */
struct __pyx_obj_10traceutils_7scamper_5warts_WartsHop {
  struct __pyx_obj_10traceutils_7scamper_3hop_Hop __pyx_base;
  PyObject *icmpext;
  int probe_id;
  unsigned short probe_size;
  int reply_ipid;
  int icmp_q_ipl;
  int icmp_nhmtu;
};


/* "traceutils/scamper/warts.pxd":12
 * 
 * 
 * cdef class WartsTrace(Trace):             # <<<<<<<<<<<<<<
 *     cdef public str type
 *     cdef public str version
 */
struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace {
  struct __pyx_obj_10traceutils_7scamper_3hop_Trace __pyx_base;
  PyObject *type;
  PyObject *version;
  int userid;
  PyObject *method;
  int icmp_sum;
  PyObject *stop_reason;
  int stop_data;
  PyObject *start;
  int hop_count;
  int attempts;
  unsigned char hoplimit;
  unsigned char firsthop;
  double wait;
  int wait_probe;
  int tos;
  unsigned char probe_size;
  unsigned char probe_count;
};


/* "traceutils/scamper/warts.pxd":31
 *     cdef public unsigned char probe_count
 * 
 * cdef class WartsPing:             # <<<<<<<<<<<<<<
 *     cdef public str type, version, method, src, dst
 *     cdef public dict start, statistics
 */
struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing {
  PyObject_HEAD
  PyObject *type;
  PyObject *version;
  PyObject *method;
  PyObject *src;
  PyObject *dst;
  PyObject *start;
  PyObject *statistics;
  int ping_sent;
  int probe_size;
  int userid;
  int ttl;
  double wait;
  double timeout;
  PyObject *responses;
  int family;
};


/* "traceutils/scamper/warts.pxd":41
 * cdef list create_responses(list responses, int family);
 * 
 * cdef class WartsPingResponse:             # <<<<<<<<<<<<<<
 *     cdef public int seq, reply_size, reply_ttl, probe_ipid, reply_ipid, icmp_type, icmp_code
 *     cdef public str reply_proto
 */
struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse {
  PyObject_HEAD
  int seq;
  int reply_size;
  int reply_ttl;
  int probe_ipid;
  int reply_ipid;
  int icmp_type;
  int icmp_code;
  PyObject *reply_proto;
  PyObject *tx;
  PyObject *rx;
  double rtt;
  int family;
  enum __pyx_t_10traceutils_7scamper_3hop_ICMPType type;
};


/* "traceutils/scamper/warts.pxd":49
 *     cdef public ICMPType type
 * 
 * cdef class WartsReader(Reader):             # <<<<<<<<<<<<<<
 *     cpdef void open(self) except *;
 *     cpdef void close(self) except *;
 */
struct __pyx_obj_10traceutils_7scamper_5warts_WartsReader {
  struct __pyx_obj_10traceutils_7scamper_3hop_Reader __pyx_base;
};


/* "traceutils/scamper/warts.pyx":141
 *         self.p = None
 * 
 *     def __iter__(self):             # <<<<<<<<<<<<<<
 *         cdef str line, rtype
 *         cdef dict j
 */
struct __pyx_obj_10traceutils_7scamper_5warts___pyx_scope_struct____iter__ {
  PyObject_HEAD
  PyObject *__pyx_v_j;
  PyObject *__pyx_v_line;
  PyObject *__pyx_v_rtype;
  struct __pyx_obj_10traceutils_7scamper_5warts_WartsReader *__pyx_v_self;
  PyObject *__pyx_t_0;
  Py_ssize_t __pyx_t_1;
  PyObject *(*__pyx_t_2)(PyObject *);
};



/* "traceutils/radix/radix_prefix.pxd":5
 *     cdef int inet_pton(int, char*, void*)
 * 
 * cdef class RadixPrefix:             # <<<<<<<<<<<<<<
 *     cdef readonly bytes addr
 *     cdef readonly unsigned char bitlen
 */

struct __pyx_vtabstruct_10traceutils_5radix_12radix_prefix_RadixPrefix {
  PyObject *(*network)(struct __pyx_obj_10traceutils_5radix_12radix_prefix_RadixPrefix *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_10traceutils_5radix_12radix_prefix_RadixPrefix *__pyx_vtabptr_10traceutils_5radix_12radix_prefix_RadixPrefix;


/* "traceutils/radix/radix_tree.pxd":9
 * # cdef bint prefix_match2(bytes left, bytes right, unsigned char bitlen);
 * 
 * cdef class RadixTree:             # <<<<<<<<<<<<<<
 *     # cdef unsigned char maxbits
 *     cdef RadixNode head
 */

struct __pyx_vtabstruct_10traceutils_5radix_10radix_tree_RadixTree {
  struct __pyx_obj_10traceutils_5radix_10radix_node_RadixNode *(*add)(struct __pyx_obj_10traceutils_5radix_10radix_tree_RadixTree *, struct __pyx_obj_10traceutils_5radix_12radix_prefix_RadixPrefix *);
  void (*remove)(struct __pyx_obj_10traceutils_5radix_10radix_tree_RadixTree *, struct __pyx_obj_10traceutils_5radix_10radix_node_RadixNode *);
  struct __pyx_obj_10traceutils_5radix_10radix_node_RadixNode *(*search_best)(struct __pyx_obj_10traceutils_5radix_10radix_tree_RadixTree *, struct __pyx_obj_10traceutils_5radix_12radix_prefix_RadixPrefix *);
  struct __pyx_obj_10traceutils_5radix_10radix_node_RadixNode *(*search_exact)(struct __pyx_obj_10traceutils_5radix_10radix_tree_RadixTree *, struct __pyx_obj_10traceutils_5radix_12radix_prefix_RadixPrefix *);
  struct __pyx_obj_10traceutils_5radix_10radix_node_RadixNode *(*search_worst)(struct __pyx_obj_10traceutils_5radix_10radix_tree_RadixTree *, struct __pyx_obj_10traceutils_5radix_12radix_prefix_RadixPrefix *);
  PyObject *(*search_covered)(struct __pyx_obj_10traceutils_5radix_10radix_tree_RadixTree *, struct __pyx_obj_10traceutils_5radix_12radix_prefix_RadixPrefix *);
};
static struct __pyx_vtabstruct_10traceutils_5radix_10radix_tree_RadixTree *__pyx_vtabptr_10traceutils_5radix_10radix_tree_RadixTree;


/* "traceutils/radix/radix.pxd":7
 * cdef list _search_covering(RadixNode node);
 * 
 * cdef class Radix:             # <<<<<<<<<<<<<<
 *     cdef RadixTree _tree4
 *     cdef RadixTree _tree6
 */

struct __pyx_vtabstruct_10traceutils_5radix_5radix_Radix {
  struct __pyx_obj_10traceutils_5radix_10radix_node_RadixNode *(*_add)(struct __pyx_obj_10traceutils_5radix_5radix_Radix *, struct __pyx_obj_10traceutils_5radix_12radix_prefix_RadixPrefix *, struct __pyx_opt_args_10traceutils_5radix_5radix_5Radix__add *__pyx_optional_args);
  void (*_delete)(struct __pyx_obj_10traceutils_5radix_5radix_Radix *, struct __pyx_obj_10traceutils_5radix_10radix_node_RadixNode *);
  struct __pyx_obj_10traceutils_5radix_10radix_node_RadixNode *(*_search_exact)(struct __pyx_obj_10traceutils_5radix_5radix_Radix *, struct __pyx_obj_10traceutils_5radix_12radix_prefix_RadixPrefix *);
  struct __pyx_obj_10traceutils_5radix_10radix_node_RadixNode *(*_search_best)(struct __pyx_obj_10traceutils_5radix_5radix_Radix *, struct __pyx_obj_10traceutils_5radix_12radix_prefix_RadixPrefix *);
  struct __pyx_obj_10traceutils_5radix_10radix_node_RadixNode *(*_search_worst)(struct __pyx_obj_10traceutils_5radix_5radix_Radix *, struct __pyx_obj_10traceutils_5radix_12radix_prefix_RadixPrefix *);
  PyObject *(*_search_covered)(struct __pyx_obj_10traceutils_5radix_5radix_Radix *, struct __pyx_obj_10traceutils_5radix_12radix_prefix_RadixPrefix *);
  struct __pyx_obj_10traceutils_5radix_10radix_node_RadixNode *(*add)(struct __pyx_obj_10traceutils_5radix_5radix_Radix *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_10traceutils_5radix_5radix_5Radix_add *__pyx_optional_args);
  struct __pyx_obj_10traceutils_5radix_10radix_node_RadixNode *(*add_packed)(struct __pyx_obj_10traceutils_5radix_5radix_Radix *, PyObject *, unsigned char, int __pyx_skip_dispatch, struct __pyx_opt_args_10traceutils_5radix_5radix_5Radix_add_packed *__pyx_optional_args);
  void (*delete)(struct __pyx_obj_10traceutils_5radix_5radix_Radix *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_10traceutils_5radix_5radix_5Radix_delete *__pyx_optional_args);
  void (*delete_packed)(struct __pyx_obj_10traceutils_5radix_5radix_Radix *, PyObject *, unsigned char, int __pyx_skip_dispatch);
  struct __pyx_obj_10traceutils_5radix_10radix_node_RadixNode *(*search_exact)(struct __pyx_obj_10traceutils_5radix_5radix_Radix *, PyObject *, int __pyx_skip_dispatch);
  struct __pyx_obj_10traceutils_5radix_10radix_node_RadixNode *(*search_exact_prefix)(struct __pyx_obj_10traceutils_5radix_5radix_Radix *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_10traceutils_5radix_5radix_5Radix_search_exact_prefix *__pyx_optional_args);
  struct __pyx_obj_10traceutils_5radix_10radix_node_RadixNode *(*search_exact_packed)(struct __pyx_obj_10traceutils_5radix_5radix_Radix *, PyObject *, unsigned char, int __pyx_skip_dispatch);
  struct __pyx_obj_10traceutils_5radix_10radix_node_RadixNode *(*search_best)(struct __pyx_obj_10traceutils_5radix_5radix_Radix *, PyObject *, int __pyx_skip_dispatch);
  struct __pyx_obj_10traceutils_5radix_10radix_node_RadixNode *(*search_best_prefix)(struct __pyx_obj_10traceutils_5radix_5radix_Radix *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_10traceutils_5radix_5radix_5Radix_search_best_prefix *__pyx_optional_args);
  struct __pyx_obj_10traceutils_5radix_10radix_node_RadixNode *(*search_best_packed)(struct __pyx_obj_10traceutils_5radix_5radix_Radix *, PyObject *, unsigned char, int __pyx_skip_dispatch);
  struct __pyx_obj_10traceutils_5radix_10radix_node_RadixNode *(*search_worst)(struct __pyx_obj_10traceutils_5radix_5radix_Radix *, PyObject *, int __pyx_skip_dispatch);
  struct __pyx_obj_10traceutils_5radix_10radix_node_RadixNode *(*search_worst_prefix)(struct __pyx_obj_10traceutils_5radix_5radix_Radix *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_10traceutils_5radix_5radix_5Radix_search_worst_prefix *__pyx_optional_args);
  struct __pyx_obj_10traceutils_5radix_10radix_node_RadixNode *(*search_worst_packed)(struct __pyx_obj_10traceutils_5radix_5radix_Radix *, PyObject *, unsigned char, int __pyx_skip_dispatch);
  PyObject *(*search_covered)(struct __pyx_obj_10traceutils_5radix_5radix_Radix *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*search_covered_prefix)(struct __pyx_obj_10traceutils_5radix_5radix_Radix *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_10traceutils_5radix_5radix_5Radix_search_covered_prefix *__pyx_optional_args);
  PyObject *(*search_covered_packed)(struct __pyx_obj_10traceutils_5radix_5radix_Radix *, PyObject *, unsigned char, int __pyx_skip_dispatch);
  PyObject *(*search_covering)(struct __pyx_obj_10traceutils_5radix_5radix_Radix *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*search_covering_prefix)(struct __pyx_obj_10traceutils_5radix_5radix_Radix *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_10traceutils_5radix_5radix_5Radix_search_covering_prefix *__pyx_optional_args);
  PyObject *(*search_covering_packed)(struct __pyx_obj_10traceutils_5radix_5radix_Radix *, PyObject *, unsigned char, int __pyx_skip_dispatch);
  PyObject *(*nodes)(struct __pyx_obj_10traceutils_5radix_5radix_Radix *, int __pyx_skip_dispatch);
  PyObject *(*prefixes)(struct __pyx_obj_10traceutils_5radix_5radix_Radix *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_10traceutils_5radix_5radix_Radix *__pyx_vtabptr_10traceutils_5radix_5radix_Radix;


/* "traceutils/radix/ip2as.pxd":8
 * 
 * 
 * cdef class IP2AS(Radix):             # <<<<<<<<<<<<<<
 *     cpdef long asn(self, str asn);
 *     cpdef long asn_packed(self, bytes packed);
 */

struct __pyx_vtabstruct_10traceutils_5radix_5ip2as_IP2AS {
  struct __pyx_vtabstruct_10traceutils_5radix_5radix_Radix __pyx_base;
  long (*asn)(struct __pyx_obj_10traceutils_5radix_5ip2as_IP2AS *, PyObject *, int __pyx_skip_dispatch);
  long (*asn_packed)(struct __pyx_obj_10traceutils_5radix_5ip2as_IP2AS *, PyObject *, int __pyx_skip_dispatch);
  void (*add_private)(struct __pyx_obj_10traceutils_5radix_5ip2as_IP2AS *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_10traceutils_5radix_5ip2as_IP2AS *__pyx_vtabptr_10traceutils_5radix_5ip2as_IP2AS;


/* "traceutils/scamper/hop.pxd":14
 * cpdef ICMPType gettype(int family, int icmp_type) except *;
 * 
 * cdef class Hop:             # <<<<<<<<<<<<<<
 *     cdef public str addr
 *     cdef public unsigned char probe_ttl
 */

struct __pyx_vtabstruct_10traceutils_7scamper_3hop_Hop {
  PyObject *(*set_packed)(struct __pyx_obj_10traceutils_7scamper_3hop_Hop *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_10traceutils_7scamper_3hop_Hop *__pyx_vtabptr_10traceutils_7scamper_3hop_Hop;


/* "traceutils/scamper/hop.pxd":33
 * 
 * 
 * cdef class Trace:             # <<<<<<<<<<<<<<
 *     cdef public str src
 *     cdef public str dst
 */

struct __pyx_vtabstruct_10traceutils_7scamper_3hop_Trace {
  PyObject *(*addrs)(struct __pyx_obj_10traceutils_7scamper_3hop_Trace *, int __pyx_skip_dispatch);
  void (*prune_dups)(struct __pyx_obj_10traceutils_7scamper_3hop_Trace *, int __pyx_skip_dispatch);
  void (*prune_loops)(struct __pyx_obj_10traceutils_7scamper_3hop_Trace *, int __pyx_skip_dispatch);
  void (*prune_private)(struct __pyx_obj_10traceutils_7scamper_3hop_Trace *, struct __pyx_obj_10traceutils_5radix_5ip2as_IP2AS *, int __pyx_skip_dispatch);
  void (*set_packed)(struct __pyx_obj_10traceutils_7scamper_3hop_Trace *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_10traceutils_7scamper_3hop_Trace *__pyx_vtabptr_10traceutils_7scamper_3hop_Trace;


/* "traceutils/scamper/hop.pxd":46
 *     cpdef void set_packed(self) except *;
 * 
 * cdef class Reader:             # <<<<<<<<<<<<<<
 *     cdef public str filename
 *     cdef public p
 */

struct __pyx_vtabstruct_10traceutils_7scamper_3hop_Reader {
  void (*open)(struct __pyx_obj_10traceutils_7scamper_3hop_Reader *, int __pyx_skip_dispatch);
  void (*close)(struct __pyx_obj_10traceutils_7scamper_3hop_Reader *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_10traceutils_7scamper_3hop_Reader *__pyx_vtabptr_10traceutils_7scamper_3hop_Reader;


/* "traceutils/scamper/warts.pyx":47
 * 
 * 
 * cdef class WartsHop(Hop):             # <<<<<<<<<<<<<<
 * 
 *     def __init__(
 */

struct __pyx_vtabstruct_10traceutils_7scamper_5warts_WartsHop {
  struct __pyx_vtabstruct_10traceutils_7scamper_3hop_Hop __pyx_base;
};
static struct __pyx_vtabstruct_10traceutils_7scamper_5warts_WartsHop *__pyx_vtabptr_10traceutils_7scamper_5warts_WartsHop;


/* "traceutils/scamper/warts.pyx":14
 * 
 * 
 * cdef class WartsTrace(Trace):             # <<<<<<<<<<<<<<
 * 
 *     def __init__(
 */

struct __pyx_vtabstruct_10traceutils_7scamper_5warts_WartsTrace {
  struct __pyx_vtabstruct_10traceutils_7scamper_3hop_Trace __pyx_base;
};
static struct __pyx_vtabstruct_10traceutils_7scamper_5warts_WartsTrace *__pyx_vtabptr_10traceutils_7scamper_5warts_WartsTrace;


/* "traceutils/scamper/warts.pyx":136
 * 
 * 
 * cdef class WartsReader(Reader):             # <<<<<<<<<<<<<<
 *     def __init__(self, str filename):
 *         self.filename = filename
 */

struct __pyx_vtabstruct_10traceutils_7scamper_5warts_WartsReader {
  struct __pyx_vtabstruct_10traceutils_7scamper_3hop_Reader __pyx_base;
};
static struct __pyx_vtabstruct_10traceutils_7scamper_5warts_WartsReader *__pyx_vtabptr_10traceutils_7scamper_5warts_WartsReader;

/* --- Runtime support code (head) --- */
/* Refnanny.proto */
#ifndef CYTHON_REFNANNY
  #define CYTHON_REFNANNY 0
#endif
#if CYTHON_REFNANNY
  typedef struct {
    void (*INCREF)(void*, PyObject*, int);
    void (*DECREF)(void*, PyObject*, int);
    void (*GOTREF)(void*, PyObject*, int);
    void (*GIVEREF)(void*, PyObject*, int);
    void* (*SetupContext)(const char*, int, const char*);
    void (*FinishContext)(void**);
  } __Pyx_RefNannyAPIStruct;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNanny = NULL;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname);
  #define __Pyx_RefNannyDeclarations void *__pyx_refnanny = NULL;
#ifdef WITH_THREAD
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          if (acquire_gil) {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
              PyGILState_Release(__pyx_gilstate_save);\
          } else {\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
          }
#else
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__)
#endif
  #define __Pyx_RefNannyFinishContext()\
          __Pyx_RefNanny->FinishContext(&__pyx_refnanny)
  #define __Pyx_INCREF(r)  __Pyx_RefNanny->INCREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_DECREF(r)  __Pyx_RefNanny->DECREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GOTREF(r)  __Pyx_RefNanny->GOTREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GIVEREF(r) __Pyx_RefNanny->GIVEREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_XINCREF(r)  do { if((r) != NULL) {__Pyx_INCREF(r); }} while(0)
  #define __Pyx_XDECREF(r)  do { if((r) != NULL) {__Pyx_DECREF(r); }} while(0)
  #define __Pyx_XGOTREF(r)  do { if((r) != NULL) {__Pyx_GOTREF(r); }} while(0)
  #define __Pyx_XGIVEREF(r) do { if((r) != NULL) {__Pyx_GIVEREF(r);}} while(0)
#else
  #define __Pyx_RefNannyDeclarations
  #define __Pyx_RefNannySetupContext(name, acquire_gil)
  #define __Pyx_RefNannyFinishContext()
  #define __Pyx_INCREF(r) Py_INCREF(r)
  #define __Pyx_DECREF(r) Py_DECREF(r)
  #define __Pyx_GOTREF(r)
  #define __Pyx_GIVEREF(r)
  #define __Pyx_XINCREF(r) Py_XINCREF(r)
  #define __Pyx_XDECREF(r) Py_XDECREF(r)
  #define __Pyx_XGOTREF(r)
  #define __Pyx_XGIVEREF(r)
#endif
#define __Pyx_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_XDECREF(tmp);\
    } while (0)
#define __Pyx_DECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_DECREF(tmp);\
    } while (0)
#define __Pyx_CLEAR(r)    do { PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);} while(0)
#define __Pyx_XCLEAR(r)   do { if((r) != NULL) {PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);}} while(0)

/* ListCompAppend.proto */
#if CYTHON_USE_PYLIST_INTERNALS && CYTHON_ASSUME_SAFE_MACROS
static CYTHON_INLINE int __Pyx_ListComp_Append(PyObject* list, PyObject* x) {
    PyListObject* L = (PyListObject*) list;
    Py_ssize_t len = Py_SIZE(list);
    if (likely(L->allocated > len)) {
        Py_INCREF(x);
        PyList_SET_ITEM(list, len, x);
        Py_SIZE(list) = len+1;
        return 0;
    }
    return PyList_Append(list, x);
}
#else
#define __Pyx_ListComp_Append(L,x) PyList_Append(L,x)
#endif

/* IterFinish.proto */
static CYTHON_INLINE int __Pyx_IterFinish(void);

/* PyFunctionFastCall.proto */
#if CYTHON_FAST_PYCALL
#define __Pyx_PyFunction_FastCall(func, args, nargs)\
    __Pyx_PyFunction_FastCallDict((func), (args), (nargs), NULL)
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, int nargs, PyObject *kwargs);
#else
#define __Pyx_PyFunction_FastCallDict(func, args, nargs, kwargs) _PyFunction_FastCallDict(func, args, nargs, kwargs)
#endif
#define __Pyx_BUILD_ASSERT_EXPR(cond)\
    (sizeof(char [1 - 2*!(cond)]) - 1)
#ifndef Py_MEMBER_SIZE
#define Py_MEMBER_SIZE(type, member) sizeof(((type *)0)->member)
#endif
  static size_t __pyx_pyframe_localsplus_offset = 0;
  #include "frameobject.h"
  #define __Pxy_PyFrame_Initialize_Offsets()\
    ((void)__Pyx_BUILD_ASSERT_EXPR(sizeof(PyFrameObject) == offsetof(PyFrameObject, f_localsplus) + Py_MEMBER_SIZE(PyFrameObject, f_localsplus)),\
     (void)(__pyx_pyframe_localsplus_offset = ((size_t)PyFrame_Type.tp_basicsize) - Py_MEMBER_SIZE(PyFrameObject, f_localsplus)))
  #define __Pyx_PyFrame_GetLocalsplus(frame)\
    (assert(__pyx_pyframe_localsplus_offset), (PyObject **)(((char *)(frame)) + __pyx_pyframe_localsplus_offset))
#endif

/* PyObjectCall.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw);
#else
#define __Pyx_PyObject_Call(func, arg, kw) PyObject_Call(func, arg, kw)
#endif

/* PyObjectCallMethO.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg);
#endif

/* PyObjectCallNoArg.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func);
#else
#define __Pyx_PyObject_CallNoArg(func) __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL)
#endif

/* PyCFunctionFastCall.proto */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject *__Pyx_PyCFunction_FastCall(PyObject *func, PyObject **args, Py_ssize_t nargs);
#else
#define __Pyx_PyCFunction_FastCall(func, args, nargs)  (assert(0), NULL)
#endif

/* PyObjectCallOneArg.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg);

/* PyObjectGetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GetAttrStr(o,n) PyObject_GetAttr(o,n)
#endif

/* PyObjectGetMethod.proto */
static int __Pyx_PyObject_GetMethod(PyObject *obj, PyObject *name, PyObject **method);

/* PyObjectCallMethod0.proto */
static PyObject* __Pyx_PyObject_CallMethod0(PyObject* obj, PyObject* method_name);

/* RaiseNeedMoreValuesToUnpack.proto */
static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index);

/* RaiseTooManyValuesToUnpack.proto */
static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected);

/* UnpackItemEndCheck.proto */
static int __Pyx_IternextUnpackEndCheck(PyObject *retval, Py_ssize_t expected);

/* RaiseNoneIterError.proto */
static CYTHON_INLINE void __Pyx_RaiseNoneNotIterableError(void);

/* UnpackTupleError.proto */
static void __Pyx_UnpackTupleError(PyObject *, Py_ssize_t index);

/* UnpackTuple2.proto */
#define __Pyx_unpack_tuple2(tuple, value1, value2, is_tuple, has_known_size, decref_tuple)\
    (likely(is_tuple || PyTuple_Check(tuple)) ?\
        (likely(has_known_size || PyTuple_GET_SIZE(tuple) == 2) ?\
            __Pyx_unpack_tuple2_exact(tuple, value1, value2, decref_tuple) :\
            (__Pyx_UnpackTupleError(tuple, 2), -1)) :\
        __Pyx_unpack_tuple2_generic(tuple, value1, value2, has_known_size, decref_tuple))
static CYTHON_INLINE int __Pyx_unpack_tuple2_exact(
    PyObject* tuple, PyObject** value1, PyObject** value2, int decref_tuple);
static int __Pyx_unpack_tuple2_generic(
    PyObject* tuple, PyObject** value1, PyObject** value2, int has_known_size, int decref_tuple);

/* dict_iter.proto */
static CYTHON_INLINE PyObject* __Pyx_dict_iterator(PyObject* dict, int is_dict, PyObject* method_name,
                                                   Py_ssize_t* p_orig_length, int* p_is_dict);
static CYTHON_INLINE int __Pyx_dict_iter_next(PyObject* dict_or_iter, Py_ssize_t orig_length, Py_ssize_t* ppos,
                                              PyObject** pkey, PyObject** pvalue, PyObject** pitem, int is_dict);

/* RaiseDoubleKeywords.proto */
static void __Pyx_RaiseDoubleKeywordsError(const char* func_name, PyObject* kw_name);

/* MergeKeywords.proto */
static int __Pyx_MergeKeywords(PyObject *kwdict, PyObject *source_mapping);

/* ParseKeywords.proto */
static int __Pyx_ParseOptionalKeywords(PyObject *kwds, PyObject **argnames[],\
    PyObject *kwds2, PyObject *values[], Py_ssize_t num_pos_args,\
    const char* function_name);

/* RaiseArgTupleInvalid.proto */
static void __Pyx_RaiseArgtupleInvalid(const char* func_name, int exact,
    Py_ssize_t num_min, Py_ssize_t num_max, Py_ssize_t num_found);

/* ArgTypeTest.proto */
#define __Pyx_ArgTypeTest(obj, type, none_allowed, name, exact)\
    ((likely((Py_TYPE(obj) == type) | (none_allowed && (obj == Py_None)))) ? 1 :\
        __Pyx__ArgTypeTest(obj, type, name, exact))
static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact);

/* PyErrExceptionMatches.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_ExceptionMatches(err) __Pyx_PyErr_ExceptionMatchesInState(__pyx_tstate, err)
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err);
#else
#define __Pyx_PyErr_ExceptionMatches(err)  PyErr_ExceptionMatches(err)
#endif

/* PyThreadStateGet.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyThreadState_declare  PyThreadState *__pyx_tstate;
#define __Pyx_PyThreadState_assign  __pyx_tstate = __Pyx_PyThreadState_Current;
#define __Pyx_PyErr_Occurred()  __pyx_tstate->curexc_type
#else
#define __Pyx_PyThreadState_declare
#define __Pyx_PyThreadState_assign
#define __Pyx_PyErr_Occurred()  PyErr_Occurred()
#endif

/* PyErrFetchRestore.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_Clear() __Pyx_ErrRestore(NULL, NULL, NULL)
#define __Pyx_ErrRestoreWithState(type, value, tb)  __Pyx_ErrRestoreInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)    __Pyx_ErrFetchInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  __Pyx_ErrRestoreInState(__pyx_tstate, type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)    __Pyx_ErrFetchInState(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_PyErr_SetNone(exc) (Py_INCREF(exc), __Pyx_ErrRestore((exc), NULL, NULL))
#else
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#endif
#else
#define __Pyx_PyErr_Clear() PyErr_Clear()
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#define __Pyx_ErrRestoreWithState(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestoreInState(tstate, type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchInState(tstate, type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)  PyErr_Fetch(type, value, tb)
#endif

/* GetAttr.proto */
static CYTHON_INLINE PyObject *__Pyx_GetAttr(PyObject *, PyObject *);

/* GetAttr3.proto */
static CYTHON_INLINE PyObject *__Pyx_GetAttr3(PyObject *, PyObject *, PyObject *);

/* GetBuiltinName.proto */
static PyObject *__Pyx_GetBuiltinName(PyObject *name);

/* PyDictVersioning.proto */
#if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
#define __PYX_DICT_VERSION_INIT  ((PY_UINT64_T) -1)
#define __PYX_GET_DICT_VERSION(dict)  (((PyDictObject*)(dict))->ma_version_tag)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)\
    (version_var) = __PYX_GET_DICT_VERSION(dict);\
    (cache_var) = (value);
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP) {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    if (likely(__PYX_GET_DICT_VERSION(DICT) == __pyx_dict_version)) {\
        (VAR) = __pyx_dict_cached_value;\
    } else {\
        (VAR) = __pyx_dict_cached_value = (LOOKUP);\
        __pyx_dict_version = __PYX_GET_DICT_VERSION(DICT);\
    }\
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj);
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj);
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version);
#else
#define __PYX_GET_DICT_VERSION(dict)  (0)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP)  (VAR) = (LOOKUP);
#endif

/* GetModuleGlobalName.proto */
#if CYTHON_USE_DICT_VERSIONS
#define __Pyx_GetModuleGlobalName(var, name)  {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    (var) = (likely(__pyx_dict_version == __PYX_GET_DICT_VERSION(__pyx_d))) ?\
        (likely(__pyx_dict_cached_value) ? __Pyx_NewRef(__pyx_dict_cached_value) : __Pyx_GetBuiltinName(name)) :\
        __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
}
#define __Pyx_GetModuleGlobalNameUncached(var, name)  {\
    PY_UINT64_T __pyx_dict_version;\
    PyObject *__pyx_dict_cached_value;\
    (var) = __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
}
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value);
#else
#define __Pyx_GetModuleGlobalName(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
#define __Pyx_GetModuleGlobalNameUncached(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name);
#endif

/* GetItemInt.proto */
#define __Pyx_GetItemInt(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Fast(o, (Py_ssize_t)i, is_list, wraparound, boundscheck) :\
    (is_list ? (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL) :\
               __Pyx_GetItemInt_Generic(o, to_py_func(i))))
#define __Pyx_GetItemInt_List(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_List_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
#define __Pyx_GetItemInt_Tuple(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Tuple_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "tuple index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j);
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i,
                                                     int is_list, int wraparound, int boundscheck);

/* PySequenceContains.proto */
static CYTHON_INLINE int __Pyx_PySequence_ContainsTF(PyObject* item, PyObject* seq, int eq) {
    int result = PySequence_Contains(seq, item);
    return unlikely(result < 0) ? result : (result == (eq == Py_EQ));
}

/* ListAppend.proto */
#if CYTHON_USE_PYLIST_INTERNALS && CYTHON_ASSUME_SAFE_MACROS
static CYTHON_INLINE int __Pyx_PyList_Append(PyObject* list, PyObject* x) {
    PyListObject* L = (PyListObject*) list;
    Py_ssize_t len = Py_SIZE(list);
    if (likely(L->allocated > len) & likely(len > (L->allocated >> 1))) {
        Py_INCREF(x);
        PyList_SET_ITEM(list, len, x);
        Py_SIZE(list) = len+1;
        return 0;
    }
    return PyList_Append(list, x);
}
#else
#define __Pyx_PyList_Append(L,x) PyList_Append(L,x)
#endif

/* PyObjectCall2Args.proto */
static CYTHON_UNUSED PyObject* __Pyx_PyObject_Call2Args(PyObject* function, PyObject* arg1, PyObject* arg2);

/* py_dict_pop.proto */
static CYTHON_INLINE PyObject *__Pyx_PyDict_Pop(PyObject *d, PyObject *key, PyObject *default_value);

/* UnpackUnboundCMethod.proto */
typedef struct {
    PyObject *type;
    PyObject **method_name;
    PyCFunction func;
    PyObject *method;
    int flag;
} __Pyx_CachedCFunction;

/* CallUnboundCMethod2.proto */
static PyObject* __Pyx__CallUnboundCMethod2(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg1, PyObject* arg2);
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030600B1
static CYTHON_INLINE PyObject *__Pyx_CallUnboundCMethod2(__Pyx_CachedCFunction *cfunc, PyObject *self, PyObject *arg1, PyObject *arg2);
#else
#define __Pyx_CallUnboundCMethod2(cfunc, self, arg1, arg2)  __Pyx__CallUnboundCMethod2(cfunc, self, arg1, arg2)
#endif

/* CallUnboundCMethod1.proto */
static PyObject* __Pyx__CallUnboundCMethod1(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg);
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_CallUnboundCMethod1(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg);
#else
#define __Pyx_CallUnboundCMethod1(cfunc, self, arg)  __Pyx__CallUnboundCMethod1(cfunc, self, arg)
#endif

/* DictGetItem.proto */
#if PY_MAJOR_VERSION >= 3 && !CYTHON_COMPILING_IN_PYPY
static PyObject *__Pyx_PyDict_GetItem(PyObject *d, PyObject* key);
#define __Pyx_PyObject_Dict_GetItem(obj, name)\
    (likely(PyDict_CheckExact(obj)) ?\
     __Pyx_PyDict_GetItem(obj, name) : PyObject_GetItem(obj, name))
#else
#define __Pyx_PyDict_GetItem(d, key) PyObject_GetItem(d, key)
#define __Pyx_PyObject_Dict_GetItem(obj, name)  PyObject_GetItem(obj, name)
#endif

/* IncludeStringH.proto */
#include <string.h>

/* BytesEquals.proto */
static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals);

/* UnicodeEquals.proto */
static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals);

/* unicode_tailmatch.proto */
static int __Pyx_PyUnicode_Tailmatch(
    PyObject* s, PyObject* substr, Py_ssize_t start, Py_ssize_t end, int direction);

/* Import.proto */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level);

/* ImportFrom.proto */
static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name);

/* RaiseException.proto */
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause);

/* HasAttr.proto */
static CYTHON_INLINE int __Pyx_HasAttr(PyObject *, PyObject *);

/* CallNextTpDealloc.proto */
static void __Pyx_call_next_tp_dealloc(PyObject* obj, destructor current_tp_dealloc);

/* CallNextTpTraverse.proto */
static int __Pyx_call_next_tp_traverse(PyObject* obj, visitproc v, void *a, traverseproc current_tp_traverse);

/* CallNextTpClear.proto */
static void __Pyx_call_next_tp_clear(PyObject* obj, inquiry current_tp_dealloc);

/* TypeImport.proto */
#ifndef __PYX_HAVE_RT_ImportType_proto
#define __PYX_HAVE_RT_ImportType_proto
enum __Pyx_ImportType_CheckSize {
   __Pyx_ImportType_CheckSize_Error = 0,
   __Pyx_ImportType_CheckSize_Warn = 1,
   __Pyx_ImportType_CheckSize_Ignore = 2
};
static PyTypeObject *__Pyx_ImportType(PyObject* module, const char *module_name, const char *class_name, size_t size, enum __Pyx_ImportType_CheckSize check_size);
#endif

/* GetVTable.proto */
static void* __Pyx_GetVtable(PyObject *dict);

/* PyObject_GenericGetAttrNoDict.proto */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static CYTHON_INLINE PyObject* __Pyx_PyObject_GenericGetAttrNoDict(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GenericGetAttrNoDict PyObject_GenericGetAttr
#endif

/* PyObject_GenericGetAttr.proto */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject* __Pyx_PyObject_GenericGetAttr(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GenericGetAttr PyObject_GenericGetAttr
#endif

/* SetVTable.proto */
static int __Pyx_SetVtable(PyObject *dict, void *vtable);

/* SetupReduce.proto */
static int __Pyx_setup_reduce(PyObject* type_obj);

/* CLineInTraceback.proto */
#ifdef CYTHON_CLINE_IN_TRACEBACK
#define __Pyx_CLineForTraceback(tstate, c_line)  (((CYTHON_CLINE_IN_TRACEBACK)) ? c_line : 0)
#else
static int __Pyx_CLineForTraceback(PyThreadState *tstate, int c_line);
#endif

/* CodeObjectCache.proto */
typedef struct {
    PyCodeObject* code_object;
    int code_line;
} __Pyx_CodeObjectCacheEntry;
struct __Pyx_CodeObjectCache {
    int count;
    int max_count;
    __Pyx_CodeObjectCacheEntry* entries;
};
static struct __Pyx_CodeObjectCache __pyx_code_cache = {0,0,NULL};
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line);
static PyCodeObject *__pyx_find_code_object(int code_line);
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object);

/* AddTraceback.proto */
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_unsigned_char(unsigned char value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_unsigned_short(unsigned short value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_enum____pyx_t_10traceutils_7scamper_3hop_ICMPType(enum __pyx_t_10traceutils_7scamper_3hop_ICMPType value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value);

/* CIntFromPy.proto */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE unsigned char __Pyx_PyInt_As_unsigned_char(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE unsigned short __Pyx_PyInt_As_unsigned_short(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE enum __pyx_t_10traceutils_7scamper_3hop_ICMPType __Pyx_PyInt_As_enum____pyx_t_10traceutils_7scamper_3hop_ICMPType(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *);

/* FastTypeChecks.proto */
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_TypeCheck(obj, type) __Pyx_IsSubtype(Py_TYPE(obj), (PyTypeObject *)type)
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject *type);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *type1, PyObject *type2);
#else
#define __Pyx_TypeCheck(obj, type) PyObject_TypeCheck(obj, (PyTypeObject *)type)
#define __Pyx_PyErr_GivenExceptionMatches(err, type) PyErr_GivenExceptionMatches(err, type)
#define __Pyx_PyErr_GivenExceptionMatches2(err, type1, type2) (PyErr_GivenExceptionMatches(err, type1) || PyErr_GivenExceptionMatches(err, type2))
#endif
#define __Pyx_PyException_Check(obj) __Pyx_TypeCheck(obj, PyExc_Exception)

/* FetchCommonType.proto */
static PyTypeObject* __Pyx_FetchCommonType(PyTypeObject* type);

/* GetTopmostException.proto */
#if CYTHON_USE_EXC_INFO_STACK
static _PyErr_StackItem * __Pyx_PyErr_GetTopmostException(PyThreadState *tstate);
#endif

/* SaveResetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_ExceptionSave(type, value, tb)  __Pyx__ExceptionSave(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#define __Pyx_ExceptionReset(type, value, tb)  __Pyx__ExceptionReset(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
#else
#define __Pyx_ExceptionSave(type, value, tb)   PyErr_GetExcInfo(type, value, tb)
#define __Pyx_ExceptionReset(type, value, tb)  PyErr_SetExcInfo(type, value, tb)
#endif

/* SwapException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_ExceptionSwap(type, value, tb)  __Pyx__ExceptionSwap(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionSwap(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#else
static CYTHON_INLINE void __Pyx_ExceptionSwap(PyObject **type, PyObject **value, PyObject **tb);
#endif

/* PyObjectCallMethod1.proto */
static PyObject* __Pyx_PyObject_CallMethod1(PyObject* obj, PyObject* method_name, PyObject* arg);

/* CoroutineBase.proto */
typedef PyObject *(*__pyx_coroutine_body_t)(PyObject *, PyThreadState *, PyObject *);
#if CYTHON_USE_EXC_INFO_STACK
#define __Pyx_ExcInfoStruct  _PyErr_StackItem
#else
typedef struct {
    PyObject *exc_type;
    PyObject *exc_value;
    PyObject *exc_traceback;
} __Pyx_ExcInfoStruct;
#endif
typedef struct {
    PyObject_HEAD
    __pyx_coroutine_body_t body;
    PyObject *closure;
    __Pyx_ExcInfoStruct gi_exc_state;
    PyObject *gi_weakreflist;
    PyObject *classobj;
    PyObject *yieldfrom;
    PyObject *gi_name;
    PyObject *gi_qualname;
    PyObject *gi_modulename;
    PyObject *gi_code;
    int resume_label;
    char is_running;
} __pyx_CoroutineObject;
static __pyx_CoroutineObject *__Pyx__Coroutine_New(
    PyTypeObject *type, __pyx_coroutine_body_t body, PyObject *code, PyObject *closure,
    PyObject *name, PyObject *qualname, PyObject *module_name);
static __pyx_CoroutineObject *__Pyx__Coroutine_NewInit(
            __pyx_CoroutineObject *gen, __pyx_coroutine_body_t body, PyObject *code, PyObject *closure,
            PyObject *name, PyObject *qualname, PyObject *module_name);
static CYTHON_INLINE void __Pyx_Coroutine_ExceptionClear(__Pyx_ExcInfoStruct *self);
static int __Pyx_Coroutine_clear(PyObject *self);
static PyObject *__Pyx_Coroutine_Send(PyObject *self, PyObject *value);
static PyObject *__Pyx_Coroutine_Close(PyObject *self);
static PyObject *__Pyx_Coroutine_Throw(PyObject *gen, PyObject *args);
#if CYTHON_USE_EXC_INFO_STACK
#define __Pyx_Coroutine_SwapException(self)
#define __Pyx_Coroutine_ResetAndClearException(self)  __Pyx_Coroutine_ExceptionClear(&(self)->gi_exc_state)
#else
#define __Pyx_Coroutine_SwapException(self) {\
    __Pyx_ExceptionSwap(&(self)->gi_exc_state.exc_type, &(self)->gi_exc_state.exc_value, &(self)->gi_exc_state.exc_traceback);\
    __Pyx_Coroutine_ResetFrameBackpointer(&(self)->gi_exc_state);\
    }
#define __Pyx_Coroutine_ResetAndClearException(self) {\
    __Pyx_ExceptionReset((self)->gi_exc_state.exc_type, (self)->gi_exc_state.exc_value, (self)->gi_exc_state.exc_traceback);\
    (self)->gi_exc_state.exc_type = (self)->gi_exc_state.exc_value = (self)->gi_exc_state.exc_traceback = NULL;\
    }
#endif
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyGen_FetchStopIterationValue(pvalue)\
    __Pyx_PyGen__FetchStopIterationValue(__pyx_tstate, pvalue)
#else
#define __Pyx_PyGen_FetchStopIterationValue(pvalue)\
    __Pyx_PyGen__FetchStopIterationValue(__Pyx_PyThreadState_Current, pvalue)
#endif
static int __Pyx_PyGen__FetchStopIterationValue(PyThreadState *tstate, PyObject **pvalue);
static CYTHON_INLINE void __Pyx_Coroutine_ResetFrameBackpointer(__Pyx_ExcInfoStruct *exc_state);

/* PatchModuleWithCoroutine.proto */
static PyObject* __Pyx_Coroutine_patch_module(PyObject* module, const char* py_code);

/* PatchGeneratorABC.proto */
static int __Pyx_patch_abc(void);

/* Generator.proto */
#define __Pyx_Generator_USED
static PyTypeObject *__pyx_GeneratorType = 0;
#define __Pyx_Generator_CheckExact(obj) (Py_TYPE(obj) == __pyx_GeneratorType)
#define __Pyx_Generator_New(body, code, closure, name, qualname, module_name)\
    __Pyx__Coroutine_New(__pyx_GeneratorType, body, code, closure, name, qualname, module_name)
static PyObject *__Pyx_Generator_Next(PyObject *self);
static int __pyx_Generator_init(void);

/* CheckBinaryVersion.proto */
static int __Pyx_check_binary_version(void);

/* FunctionExport.proto */
static int __Pyx_ExportFunction(const char *name, void (*f)(void), const char *sig);

/* VoidPtrImport.proto */
static int __Pyx_ImportVoidPtr(PyObject *module, const char *name, void **p, const char *sig);

/* FunctionImport.proto */
static int __Pyx_ImportFunction(PyObject *module, const char *funcname, void (**f)(void), const char *sig);

/* InitStrings.proto */
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t);

static void __pyx_f_10traceutils_7scamper_5warts_11WartsReader_open(struct __pyx_obj_10traceutils_7scamper_5warts_WartsReader *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static void __pyx_f_10traceutils_7scamper_5warts_11WartsReader_close(struct __pyx_obj_10traceutils_7scamper_5warts_WartsReader *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/

/* Module declarations from 'traceutils.radix.radix_prefix' */
static PyTypeObject *__pyx_ptype_10traceutils_5radix_12radix_prefix_RadixPrefix = 0;

/* Module declarations from 'traceutils.radix.radix_node' */
static PyTypeObject *__pyx_ptype_10traceutils_5radix_10radix_node_RadixNode = 0;

/* Module declarations from 'traceutils.radix.radix_tree' */
static PyTypeObject *__pyx_ptype_10traceutils_5radix_10radix_tree_RadixTree = 0;

/* Module declarations from 'traceutils.radix.radix' */
static PyTypeObject *__pyx_ptype_10traceutils_5radix_5radix_Radix = 0;

/* Module declarations from 'traceutils.radix.ip2as' */
static PyTypeObject *__pyx_ptype_10traceutils_5radix_5ip2as_IP2AS = 0;
static PyObject **__pyx_vp_10traceutils_5radix_5ip2as_PRIVATE4 = 0;
#define __pyx_v_10traceutils_5radix_5ip2as_PRIVATE4 (*__pyx_vp_10traceutils_5radix_5ip2as_PRIVATE4)
static PyObject **__pyx_vp_10traceutils_5radix_5ip2as_PRIVATE6 = 0;
#define __pyx_v_10traceutils_5radix_5ip2as_PRIVATE6 (*__pyx_vp_10traceutils_5radix_5ip2as_PRIVATE6)
static PyObject **__pyx_vp_10traceutils_5radix_5ip2as_MULTICAST4 = 0;
#define __pyx_v_10traceutils_5radix_5ip2as_MULTICAST4 (*__pyx_vp_10traceutils_5radix_5ip2as_MULTICAST4)
static PyObject **__pyx_vp_10traceutils_5radix_5ip2as_MULTICAST6 = 0;
#define __pyx_v_10traceutils_5radix_5ip2as_MULTICAST6 (*__pyx_vp_10traceutils_5radix_5ip2as_MULTICAST6)

/* Module declarations from 'traceutils.scamper.hop' */
static PyTypeObject *__pyx_ptype_10traceutils_7scamper_3hop_Hop = 0;
static PyTypeObject *__pyx_ptype_10traceutils_7scamper_3hop_Trace = 0;
static PyTypeObject *__pyx_ptype_10traceutils_7scamper_3hop_Reader = 0;
static enum __pyx_t_10traceutils_7scamper_3hop_ICMPType (*__pyx_f_10traceutils_7scamper_3hop_gettype)(int, int, int __pyx_skip_dispatch); /*proto*/

/* Module declarations from 'traceutils.utils.net' */
static unsigned char (*__pyx_f_10traceutils_5utils_3net_find_family)(PyObject *, int __pyx_skip_dispatch); /*proto*/

/* Module declarations from 'traceutils.scamper.warts' */
static PyTypeObject *__pyx_ptype_10traceutils_7scamper_5warts_WartsHop = 0;
static PyTypeObject *__pyx_ptype_10traceutils_7scamper_5warts_WartsTrace = 0;
static PyTypeObject *__pyx_ptype_10traceutils_7scamper_5warts_WartsPing = 0;
static PyTypeObject *__pyx_ptype_10traceutils_7scamper_5warts_WartsPingResponse = 0;
static PyTypeObject *__pyx_ptype_10traceutils_7scamper_5warts_WartsReader = 0;
static PyTypeObject *__pyx_ptype_10traceutils_7scamper_5warts___pyx_scope_struct____iter__ = 0;
static PyObject *__pyx_f_10traceutils_7scamper_5warts_create_responses(PyObject *, int); /*proto*/
static PyObject *__pyx_f_10traceutils_7scamper_5warts_create_hops(PyObject *, int); /*proto*/
static PyObject *__pyx_f_10traceutils_7scamper_5warts___pyx_unpickle_WartsTrace__set_state(struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *, PyObject *); /*proto*/
static PyObject *__pyx_f_10traceutils_7scamper_5warts___pyx_unpickle_WartsHop__set_state(struct __pyx_obj_10traceutils_7scamper_5warts_WartsHop *, PyObject *); /*proto*/
static PyObject *__pyx_f_10traceutils_7scamper_5warts___pyx_unpickle_WartsPing__set_state(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *, PyObject *); /*proto*/
static PyObject *__pyx_f_10traceutils_7scamper_5warts___pyx_unpickle_WartsPingResponse__set_state(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *, PyObject *); /*proto*/
static PyObject *__pyx_f_10traceutils_7scamper_5warts___pyx_unpickle_WartsReader__set_state(struct __pyx_obj_10traceutils_7scamper_5warts_WartsReader *, PyObject *); /*proto*/
#define __Pyx_MODULE_NAME "traceutils.scamper.warts"
extern int __pyx_module_is_main_traceutils__scamper__warts;
int __pyx_module_is_main_traceutils__scamper__warts = 0;

/* Implementation of 'traceutils.scamper.warts' */
static const char __pyx_k_[] = "";
static const char __pyx_k__2[] = "\t{}";
static const char __pyx_k__3[] = "\n";
static const char __pyx_k_gz[] = ".gz";
static const char __pyx_k_id[] = "id";
static const char __pyx_k_rx[] = "rx";
static const char __pyx_k_tx[] = "tx";
static const char __pyx_k_bz2[] = ".bz2";
static const char __pyx_k_dst[] = "dst";
static const char __pyx_k_new[] = "__new__";
static const char __pyx_k_pop[] = "pop";
static const char __pyx_k_rtt[] = "rtt";
static const char __pyx_k_seq[] = "seq";
static const char __pyx_k_src[] = "src";
static const char __pyx_k_tos[] = "tos";
static const char __pyx_k_ttl[] = "ttl";
static const char __pyx_k_PIPE[] = "PIPE";
static const char __pyx_k_addr[] = "addr";
static const char __pyx_k_args[] = "args";
static const char __pyx_k_dict[] = "__dict__";
static const char __pyx_k_from[] = "from";
static const char __pyx_k_hops[] = "hops";
static const char __pyx_k_iter[] = "__iter__";
static const char __pyx_k_json[] = "json";
static const char __pyx_k_main[] = "__main__";
static const char __pyx_k_name[] = "__name__";
static const char __pyx_k_open[] = "open";
static const char __pyx_k_ping[] = "ping";
static const char __pyx_k_send[] = "send";
static const char __pyx_k_test[] = "__test__";
static const char __pyx_k_type[] = "type";
static const char __pyx_k_wait[] = "wait";
static const char __pyx_k_Popen[] = "Popen";
static const char __pyx_k_bzip2[] = ".bzip2";
static const char __pyx_k_close[] = "close";
static const char __pyx_k_items[] = "items";
static const char __pyx_k_loads[] = "loads";
static const char __pyx_k_shell[] = "shell";
static const char __pyx_k_start[] = "start";
static const char __pyx_k_throw[] = "throw";
static const char __pyx_k_trace[] = "trace";
static const char __pyx_k_family[] = "family";
static const char __pyx_k_format[] = "format";
static const char __pyx_k_import[] = "__import__";
static const char __pyx_k_method[] = "method";
static const char __pyx_k_pickle[] = "pickle";
static const char __pyx_k_reduce[] = "__reduce__";
static const char __pyx_k_stdout[] = "stdout";
static const char __pyx_k_update[] = "update";
static const char __pyx_k_userid[] = "userid";
static const char __pyx_k_RTT_rtt[] = "RTT={rtt}";
static const char __pyx_k_icmpext[] = "icmpext";
static const char __pyx_k_timeout[] = "timeout";
static const char __pyx_k_version[] = "version";
static const char __pyx_k_WartsHop[] = "WartsHop";
static const char __pyx_k_attempts[] = "attempts";
static const char __pyx_k_filename[] = "filename";
static const char __pyx_k_firsthop[] = "firsthop";
static const char __pyx_k_getstate[] = "__getstate__";
static const char __pyx_k_hoplimit[] = "hoplimit";
static const char __pyx_k_hostname[] = "hostname";
static const char __pyx_k_icmp_sum[] = "icmp_sum";
static const char __pyx_k_probe_id[] = "probe_id";
static const char __pyx_k_pyx_type[] = "__pyx_type";
static const char __pyx_k_setstate[] = "__setstate__";
static const char __pyx_k_WartsPing[] = "WartsPing";
static const char __pyx_k_hop_count[] = "hop_count";
static const char __pyx_k_icmp_code[] = "icmp_code";
static const char __pyx_k_icmp_type[] = "icmp_type";
static const char __pyx_k_list_name[] = "list_name";
static const char __pyx_k_ping_sent[] = "ping_sent";
static const char __pyx_k_probe_ttl[] = "probe_ttl";
static const char __pyx_k_pyx_state[] = "__pyx_state";
static const char __pyx_k_reduce_ex[] = "__reduce_ex__";
static const char __pyx_k_reply_tos[] = "reply_tos";
static const char __pyx_k_reply_ttl[] = "reply_ttl";
static const char __pyx_k_responses[] = "responses";
static const char __pyx_k_stop_data[] = "stop_data";
static const char __pyx_k_WartsTrace[] = "WartsTrace";
static const char __pyx_k_icmp_nhmtu[] = "icmp_nhmtu";
static const char __pyx_k_icmp_q_ipl[] = "icmp_q_ipl";
static const char __pyx_k_icmp_q_tos[] = "icmp_q_tos";
static const char __pyx_k_icmp_q_ttl[] = "icmp_q_ttl";
static const char __pyx_k_probe_ipid[] = "probe_ipid";
static const char __pyx_k_probe_size[] = "probe_size";
static const char __pyx_k_pyx_result[] = "__pyx_result";
static const char __pyx_k_pyx_vtable[] = "__pyx_vtable__";
static const char __pyx_k_reply_ipid[] = "reply_ipid";
static const char __pyx_k_reply_size[] = "reply_size";
static const char __pyx_k_start_time[] = "start_time";
static const char __pyx_k_statistics[] = "statistics";
static const char __pyx_k_subprocess[] = "subprocess";
static const char __pyx_k_wait_probe[] = "wait_probe";
static const char __pyx_k_PickleError[] = "PickleError";
static const char __pyx_k_WartsReader[] = "WartsReader";
static const char __pyx_k_mpls_labels[] = "mpls_labels";
static const char __pyx_k_probe_count[] = "probe_count";
static const char __pyx_k_reply_proto[] = "reply_proto";
static const char __pyx_k_stop_reason[] = "stop_reason";
static const char __pyx_k_pyx_checksum[] = "__pyx_checksum";
static const char __pyx_k_stringsource[] = "stringsource";
static const char __pyx_k_reduce_cython[] = "__reduce_cython__";
static const char __pyx_k_sc_warts2json[] = "sc_warts2json {}";
static const char __pyx_k_Src_src_Dst_dst[] = "Src={src}, Dst={dst}:";
static const char __pyx_k_pyx_PickleError[] = "__pyx_PickleError";
static const char __pyx_k_setstate_cython[] = "__setstate_cython__";
static const char __pyx_k_WartsPingResponse[] = "WartsPingResponse";
static const char __pyx_k_WartsReader___iter[] = "WartsReader.__iter__";
static const char __pyx_k_cline_in_traceback[] = "cline_in_traceback";
static const char __pyx_k_universal_newlines[] = "universal_newlines";
static const char __pyx_k_pyx_unpickle_WartsHop[] = "__pyx_unpickle_WartsHop";
static const char __pyx_k_gzip_d_c_sc_warts2json[] = "gzip -d -c {} | sc_warts2json";
static const char __pyx_k_pyx_unpickle_WartsPing[] = "__pyx_unpickle_WartsPing";
static const char __pyx_k_bzip2_d_c_sc_warts2json[] = "bzip2 -d -c {} | sc_warts2json";
static const char __pyx_k_pyx_unpickle_WartsTrace[] = "__pyx_unpickle_WartsTrace";
static const char __pyx_k_pyx_unpickle_WartsReader[] = "__pyx_unpickle_WartsReader";
static const char __pyx_k_traceutils_scamper_warts[] = "traceutils.scamper.warts";
static const char __pyx_k_pyx_unpickle_WartsPingResponse[] = "__pyx_unpickle_WartsPingResponse";
static const char __pyx_k_Incompatible_checksums_s_vs_0x57[] = "Incompatible checksums (%s vs 0x5752e4d = (dst, family, method, ping_sent, probe_size, responses, src, start, statistics, timeout, ttl, type, userid, version, wait))";
static const char __pyx_k_Incompatible_checksums_s_vs_0x78[] = "Incompatible checksums (%s vs 0x78ac107 = (family, icmp_code, icmp_type, probe_ipid, reply_ipid, reply_proto, reply_size, reply_ttl, rtt, rx, seq, tx, type))";
static const char __pyx_k_Incompatible_checksums_s_vs_0x93[] = "Incompatible checksums (%s vs 0x93358b7 = (filename, p))";
static const char __pyx_k_Incompatible_checksums_s_vs_0xb9[] = "Incompatible checksums (%s vs 0xb91787d = (addr, family, icmp_code, icmp_nhmtu, icmp_q_ipl, icmp_q_tos, icmp_q_ttl, icmp_type, icmpext, ismpls, packed, probe_id, probe_size, probe_ttl, reply_ipid, reply_size, reply_tos, reply_ttl, rtt, type))";
static const char __pyx_k_Incompatible_checksums_s_vs_0xff[] = "Incompatible checksums (%s vs 0xffb4da5 = (attempts, dst, family, firsthop, hop_count, hoplimit, hops, icmp_sum, loop, method, probe_count, probe_size, src, start, stop_data, stop_reason, tos, type, userid, version, wait, wait_probe))";
static PyObject *__pyx_kp_u_;
static PyObject *__pyx_kp_s_Incompatible_checksums_s_vs_0x57;
static PyObject *__pyx_kp_s_Incompatible_checksums_s_vs_0x78;
static PyObject *__pyx_kp_s_Incompatible_checksums_s_vs_0x93;
static PyObject *__pyx_kp_s_Incompatible_checksums_s_vs_0xb9;
static PyObject *__pyx_kp_s_Incompatible_checksums_s_vs_0xff;
static PyObject *__pyx_n_s_PIPE;
static PyObject *__pyx_n_s_PickleError;
static PyObject *__pyx_n_s_Popen;
static PyObject *__pyx_kp_u_RTT_rtt;
static PyObject *__pyx_kp_u_Src_src_Dst_dst;
static PyObject *__pyx_n_s_WartsHop;
static PyObject *__pyx_n_s_WartsPing;
static PyObject *__pyx_n_s_WartsPingResponse;
static PyObject *__pyx_n_s_WartsReader;
static PyObject *__pyx_n_s_WartsReader___iter;
static PyObject *__pyx_n_s_WartsTrace;
static PyObject *__pyx_kp_u__2;
static PyObject *__pyx_kp_u__3;
static PyObject *__pyx_n_s_addr;
static PyObject *__pyx_n_s_args;
static PyObject *__pyx_n_s_attempts;
static PyObject *__pyx_kp_u_bz2;
static PyObject *__pyx_kp_u_bzip2;
static PyObject *__pyx_kp_u_bzip2_d_c_sc_warts2json;
static PyObject *__pyx_n_s_cline_in_traceback;
static PyObject *__pyx_n_s_close;
static PyObject *__pyx_n_s_dict;
static PyObject *__pyx_n_s_dst;
static PyObject *__pyx_n_s_family;
static PyObject *__pyx_n_s_filename;
static PyObject *__pyx_n_s_firsthop;
static PyObject *__pyx_n_s_format;
static PyObject *__pyx_n_u_from;
static PyObject *__pyx_n_s_getstate;
static PyObject *__pyx_kp_u_gz;
static PyObject *__pyx_kp_u_gzip_d_c_sc_warts2json;
static PyObject *__pyx_n_s_hop_count;
static PyObject *__pyx_n_s_hoplimit;
static PyObject *__pyx_n_s_hops;
static PyObject *__pyx_n_s_hostname;
static PyObject *__pyx_n_s_icmp_code;
static PyObject *__pyx_n_s_icmp_nhmtu;
static PyObject *__pyx_n_s_icmp_q_ipl;
static PyObject *__pyx_n_s_icmp_q_tos;
static PyObject *__pyx_n_s_icmp_q_ttl;
static PyObject *__pyx_n_s_icmp_sum;
static PyObject *__pyx_n_s_icmp_type;
static PyObject *__pyx_n_s_icmpext;
static PyObject *__pyx_n_s_id;
static PyObject *__pyx_n_s_import;
static PyObject *__pyx_n_s_items;
static PyObject *__pyx_n_s_iter;
static PyObject *__pyx_n_s_json;
static PyObject *__pyx_n_s_list_name;
static PyObject *__pyx_n_s_loads;
static PyObject *__pyx_n_s_main;
static PyObject *__pyx_n_s_method;
static PyObject *__pyx_n_u_mpls_labels;
static PyObject *__pyx_n_s_name;
static PyObject *__pyx_n_s_new;
static PyObject *__pyx_n_s_open;
static PyObject *__pyx_n_s_pickle;
static PyObject *__pyx_n_u_ping;
static PyObject *__pyx_n_s_ping_sent;
static PyObject *__pyx_n_s_pop;
static PyObject *__pyx_n_s_probe_count;
static PyObject *__pyx_n_s_probe_id;
static PyObject *__pyx_n_s_probe_ipid;
static PyObject *__pyx_n_s_probe_size;
static PyObject *__pyx_n_s_probe_ttl;
static PyObject *__pyx_n_s_pyx_PickleError;
static PyObject *__pyx_n_s_pyx_checksum;
static PyObject *__pyx_n_s_pyx_result;
static PyObject *__pyx_n_s_pyx_state;
static PyObject *__pyx_n_s_pyx_type;
static PyObject *__pyx_n_s_pyx_unpickle_WartsHop;
static PyObject *__pyx_n_s_pyx_unpickle_WartsPing;
static PyObject *__pyx_n_s_pyx_unpickle_WartsPingResponse;
static PyObject *__pyx_n_s_pyx_unpickle_WartsReader;
static PyObject *__pyx_n_s_pyx_unpickle_WartsTrace;
static PyObject *__pyx_n_s_pyx_vtable;
static PyObject *__pyx_n_s_reduce;
static PyObject *__pyx_n_s_reduce_cython;
static PyObject *__pyx_n_s_reduce_ex;
static PyObject *__pyx_n_s_reply_ipid;
static PyObject *__pyx_n_s_reply_proto;
static PyObject *__pyx_n_s_reply_size;
static PyObject *__pyx_n_s_reply_tos;
static PyObject *__pyx_n_s_reply_ttl;
static PyObject *__pyx_n_s_responses;
static PyObject *__pyx_n_s_rtt;
static PyObject *__pyx_n_s_rx;
static PyObject *__pyx_kp_u_sc_warts2json;
static PyObject *__pyx_n_s_send;
static PyObject *__pyx_n_s_seq;
static PyObject *__pyx_n_s_setstate;
static PyObject *__pyx_n_s_setstate_cython;
static PyObject *__pyx_n_s_shell;
static PyObject *__pyx_n_s_src;
static PyObject *__pyx_n_s_start;
static PyObject *__pyx_n_s_start_time;
static PyObject *__pyx_n_s_statistics;
static PyObject *__pyx_n_s_stdout;
static PyObject *__pyx_n_s_stop_data;
static PyObject *__pyx_n_s_stop_reason;
static PyObject *__pyx_kp_s_stringsource;
static PyObject *__pyx_n_s_subprocess;
static PyObject *__pyx_n_s_test;
static PyObject *__pyx_n_s_throw;
static PyObject *__pyx_n_s_timeout;
static PyObject *__pyx_n_s_tos;
static PyObject *__pyx_n_u_trace;
static PyObject *__pyx_n_s_traceutils_scamper_warts;
static PyObject *__pyx_n_s_ttl;
static PyObject *__pyx_n_s_tx;
static PyObject *__pyx_n_s_type;
static PyObject *__pyx_n_u_type;
static PyObject *__pyx_n_s_universal_newlines;
static PyObject *__pyx_n_s_update;
static PyObject *__pyx_n_s_userid;
static PyObject *__pyx_n_s_version;
static PyObject *__pyx_n_s_wait;
static PyObject *__pyx_n_s_wait_probe;
static int __pyx_pf_10traceutils_7scamper_5warts_10WartsTrace___init__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *__pyx_v_self, PyObject *__pyx_v_type, PyObject *__pyx_v_version, int __pyx_v_userid, PyObject *__pyx_v_method, PyObject *__pyx_v_src, PyObject *__pyx_v_dst, int __pyx_v_icmp_sum, PyObject *__pyx_v_stop_reason, int __pyx_v_stop_data, PyObject *__pyx_v_start, int __pyx_v_hop_count, int __pyx_v_attempts, unsigned char __pyx_v_hoplimit, unsigned char __pyx_v_firsthop, double __pyx_v_wait, int __pyx_v_wait_probe, int __pyx_v_tos, unsigned short __pyx_v_probe_size, unsigned char __pyx_v_probe_count, PyObject *__pyx_v_hops, CYTHON_UNUSED PyObject *__pyx_v_list_name, CYTHON_UNUSED int __pyx_v_id, CYTHON_UNUSED PyObject *__pyx_v_hostname, CYTHON_UNUSED long __pyx_v_start_time); /* proto */
static PyObject *__pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_4type___get__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *__pyx_v_self); /* proto */
static int __pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_4type_2__set__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_4type_4__del__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_7version___get__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *__pyx_v_self); /* proto */
static int __pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_7version_2__set__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_7version_4__del__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_6userid___get__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *__pyx_v_self); /* proto */
static int __pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_6userid_2__set__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_6method___get__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *__pyx_v_self); /* proto */
static int __pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_6method_2__set__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_6method_4__del__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_8icmp_sum___get__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *__pyx_v_self); /* proto */
static int __pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_8icmp_sum_2__set__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_11stop_reason___get__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *__pyx_v_self); /* proto */
static int __pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_11stop_reason_2__set__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_11stop_reason_4__del__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_9stop_data___get__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *__pyx_v_self); /* proto */
static int __pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_9stop_data_2__set__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_5start___get__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *__pyx_v_self); /* proto */
static int __pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_5start_2__set__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_5start_4__del__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_9hop_count___get__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *__pyx_v_self); /* proto */
static int __pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_9hop_count_2__set__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_8attempts___get__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *__pyx_v_self); /* proto */
static int __pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_8attempts_2__set__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_8hoplimit___get__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *__pyx_v_self); /* proto */
static int __pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_8hoplimit_2__set__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_8firsthop___get__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *__pyx_v_self); /* proto */
static int __pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_8firsthop_2__set__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_4wait___get__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *__pyx_v_self); /* proto */
static int __pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_4wait_2__set__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_10wait_probe___get__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *__pyx_v_self); /* proto */
static int __pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_10wait_probe_2__set__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_3tos___get__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *__pyx_v_self); /* proto */
static int __pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_3tos_2__set__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_10probe_size___get__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *__pyx_v_self); /* proto */
static int __pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_10probe_size_2__set__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_11probe_count___get__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *__pyx_v_self); /* proto */
static int __pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_11probe_count_2__set__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_2__reduce_cython__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_4__setstate_cython__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_10traceutils_7scamper_5warts_8WartsHop___init__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsHop *__pyx_v_self, PyObject *__pyx_v_addr, unsigned char __pyx_v_probe_ttl, int __pyx_v_probe_id, unsigned short __pyx_v_probe_size, double __pyx_v_rtt, unsigned char __pyx_v_reply_ttl, int __pyx_v_reply_tos, unsigned short __pyx_v_reply_size, int __pyx_v_reply_ipid, unsigned char __pyx_v_icmp_type, unsigned char __pyx_v_icmp_code, unsigned char __pyx_v_icmp_q_ttl, int __pyx_v_icmp_q_ipl, int __pyx_v_icmp_q_tos, PyObject *__pyx_v_icmpext, int __pyx_v_icmp_nhmtu, int __pyx_v_family, CYTHON_UNUSED PyObject *__pyx_v_kwargs); /* proto */
static PyObject *__pyx_pf_10traceutils_7scamper_5warts_8WartsHop_7icmpext___get__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsHop *__pyx_v_self); /* proto */
static int __pyx_pf_10traceutils_7scamper_5warts_8WartsHop_7icmpext_2__set__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsHop *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_10traceutils_7scamper_5warts_8WartsHop_7icmpext_4__del__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsHop *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_10traceutils_7scamper_5warts_8WartsHop_8probe_id___get__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsHop *__pyx_v_self); /* proto */
static int __pyx_pf_10traceutils_7scamper_5warts_8WartsHop_8probe_id_2__set__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsHop *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_10traceutils_7scamper_5warts_8WartsHop_10probe_size___get__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsHop *__pyx_v_self); /* proto */
static int __pyx_pf_10traceutils_7scamper_5warts_8WartsHop_10probe_size_2__set__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsHop *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_10traceutils_7scamper_5warts_8WartsHop_10reply_ipid___get__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsHop *__pyx_v_self); /* proto */
static int __pyx_pf_10traceutils_7scamper_5warts_8WartsHop_10reply_ipid_2__set__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsHop *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_10traceutils_7scamper_5warts_8WartsHop_10icmp_q_ipl___get__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsHop *__pyx_v_self); /* proto */
static int __pyx_pf_10traceutils_7scamper_5warts_8WartsHop_10icmp_q_ipl_2__set__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsHop *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_10traceutils_7scamper_5warts_8WartsHop_10icmp_nhmtu___get__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsHop *__pyx_v_self); /* proto */
static int __pyx_pf_10traceutils_7scamper_5warts_8WartsHop_10icmp_nhmtu_2__set__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsHop *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_10traceutils_7scamper_5warts_8WartsHop_2__reduce_cython__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsHop *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_10traceutils_7scamper_5warts_8WartsHop_4__setstate_cython__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsHop *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_10traceutils_7scamper_5warts_9WartsPing___init__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *__pyx_v_self, PyObject *__pyx_v_type, PyObject *__pyx_v_version, PyObject *__pyx_v_method, PyObject *__pyx_v_src, PyObject *__pyx_v_dst, PyObject *__pyx_v_start, int __pyx_v_ping_sent, int __pyx_v_probe_size, int __pyx_v_userid, int __pyx_v_ttl, double __pyx_v_wait, double __pyx_v_timeout, PyObject *__pyx_v_responses, PyObject *__pyx_v_statistics); /* proto */
static PyObject *__pyx_pf_10traceutils_7scamper_5warts_9WartsPing_2__repr__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_10traceutils_7scamper_5warts_9WartsPing_4type___get__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *__pyx_v_self); /* proto */
static int __pyx_pf_10traceutils_7scamper_5warts_9WartsPing_4type_2__set__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_10traceutils_7scamper_5warts_9WartsPing_4type_4__del__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_10traceutils_7scamper_5warts_9WartsPing_7version___get__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *__pyx_v_self); /* proto */
static int __pyx_pf_10traceutils_7scamper_5warts_9WartsPing_7version_2__set__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_10traceutils_7scamper_5warts_9WartsPing_7version_4__del__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_10traceutils_7scamper_5warts_9WartsPing_6method___get__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *__pyx_v_self); /* proto */
static int __pyx_pf_10traceutils_7scamper_5warts_9WartsPing_6method_2__set__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_10traceutils_7scamper_5warts_9WartsPing_6method_4__del__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_10traceutils_7scamper_5warts_9WartsPing_3src___get__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *__pyx_v_self); /* proto */
static int __pyx_pf_10traceutils_7scamper_5warts_9WartsPing_3src_2__set__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_10traceutils_7scamper_5warts_9WartsPing_3src_4__del__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_10traceutils_7scamper_5warts_9WartsPing_3dst___get__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *__pyx_v_self); /* proto */
static int __pyx_pf_10traceutils_7scamper_5warts_9WartsPing_3dst_2__set__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_10traceutils_7scamper_5warts_9WartsPing_3dst_4__del__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_10traceutils_7scamper_5warts_9WartsPing_5start___get__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *__pyx_v_self); /* proto */
static int __pyx_pf_10traceutils_7scamper_5warts_9WartsPing_5start_2__set__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_10traceutils_7scamper_5warts_9WartsPing_5start_4__del__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_10traceutils_7scamper_5warts_9WartsPing_10statistics___get__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *__pyx_v_self); /* proto */
static int __pyx_pf_10traceutils_7scamper_5warts_9WartsPing_10statistics_2__set__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_10traceutils_7scamper_5warts_9WartsPing_10statistics_4__del__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_10traceutils_7scamper_5warts_9WartsPing_9ping_sent___get__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *__pyx_v_self); /* proto */
static int __pyx_pf_10traceutils_7scamper_5warts_9WartsPing_9ping_sent_2__set__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_10traceutils_7scamper_5warts_9WartsPing_10probe_size___get__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *__pyx_v_self); /* proto */
static int __pyx_pf_10traceutils_7scamper_5warts_9WartsPing_10probe_size_2__set__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_10traceutils_7scamper_5warts_9WartsPing_6userid___get__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *__pyx_v_self); /* proto */
static int __pyx_pf_10traceutils_7scamper_5warts_9WartsPing_6userid_2__set__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_10traceutils_7scamper_5warts_9WartsPing_3ttl___get__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *__pyx_v_self); /* proto */
static int __pyx_pf_10traceutils_7scamper_5warts_9WartsPing_3ttl_2__set__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_10traceutils_7scamper_5warts_9WartsPing_4wait___get__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *__pyx_v_self); /* proto */
static int __pyx_pf_10traceutils_7scamper_5warts_9WartsPing_4wait_2__set__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_10traceutils_7scamper_5warts_9WartsPing_7timeout___get__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *__pyx_v_self); /* proto */
static int __pyx_pf_10traceutils_7scamper_5warts_9WartsPing_7timeout_2__set__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_10traceutils_7scamper_5warts_9WartsPing_9responses___get__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *__pyx_v_self); /* proto */
static int __pyx_pf_10traceutils_7scamper_5warts_9WartsPing_9responses_2__set__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_10traceutils_7scamper_5warts_9WartsPing_9responses_4__del__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_10traceutils_7scamper_5warts_9WartsPing_6family___get__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *__pyx_v_self); /* proto */
static int __pyx_pf_10traceutils_7scamper_5warts_9WartsPing_6family_2__set__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_10traceutils_7scamper_5warts_9WartsPing_4__reduce_cython__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_10traceutils_7scamper_5warts_9WartsPing_6__setstate_cython__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_10traceutils_7scamper_5warts_17WartsPingResponse___init__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *__pyx_v_self, int __pyx_v_seq, int __pyx_v_reply_size, int __pyx_v_reply_ttl, PyObject *__pyx_v_reply_proto, PyObject *__pyx_v_tx, PyObject *__pyx_v_rx, double __pyx_v_rtt, int __pyx_v_probe_ipid, int __pyx_v_reply_ipid, int __pyx_v_icmp_type, int __pyx_v_icmp_code, int __pyx_v_family); /* proto */
static PyObject *__pyx_pf_10traceutils_7scamper_5warts_17WartsPingResponse_2__repr__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_10traceutils_7scamper_5warts_17WartsPingResponse_3seq___get__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *__pyx_v_self); /* proto */
static int __pyx_pf_10traceutils_7scamper_5warts_17WartsPingResponse_3seq_2__set__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_10traceutils_7scamper_5warts_17WartsPingResponse_10reply_size___get__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *__pyx_v_self); /* proto */
static int __pyx_pf_10traceutils_7scamper_5warts_17WartsPingResponse_10reply_size_2__set__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_10traceutils_7scamper_5warts_17WartsPingResponse_9reply_ttl___get__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *__pyx_v_self); /* proto */
static int __pyx_pf_10traceutils_7scamper_5warts_17WartsPingResponse_9reply_ttl_2__set__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_10traceutils_7scamper_5warts_17WartsPingResponse_10probe_ipid___get__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *__pyx_v_self); /* proto */
static int __pyx_pf_10traceutils_7scamper_5warts_17WartsPingResponse_10probe_ipid_2__set__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_10traceutils_7scamper_5warts_17WartsPingResponse_10reply_ipid___get__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *__pyx_v_self); /* proto */
static int __pyx_pf_10traceutils_7scamper_5warts_17WartsPingResponse_10reply_ipid_2__set__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_10traceutils_7scamper_5warts_17WartsPingResponse_9icmp_type___get__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *__pyx_v_self); /* proto */
static int __pyx_pf_10traceutils_7scamper_5warts_17WartsPingResponse_9icmp_type_2__set__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_10traceutils_7scamper_5warts_17WartsPingResponse_9icmp_code___get__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *__pyx_v_self); /* proto */
static int __pyx_pf_10traceutils_7scamper_5warts_17WartsPingResponse_9icmp_code_2__set__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_10traceutils_7scamper_5warts_17WartsPingResponse_11reply_proto___get__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *__pyx_v_self); /* proto */
static int __pyx_pf_10traceutils_7scamper_5warts_17WartsPingResponse_11reply_proto_2__set__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_10traceutils_7scamper_5warts_17WartsPingResponse_11reply_proto_4__del__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_10traceutils_7scamper_5warts_17WartsPingResponse_2tx___get__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *__pyx_v_self); /* proto */
static int __pyx_pf_10traceutils_7scamper_5warts_17WartsPingResponse_2tx_2__set__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_10traceutils_7scamper_5warts_17WartsPingResponse_2tx_4__del__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_10traceutils_7scamper_5warts_17WartsPingResponse_2rx___get__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *__pyx_v_self); /* proto */
static int __pyx_pf_10traceutils_7scamper_5warts_17WartsPingResponse_2rx_2__set__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_10traceutils_7scamper_5warts_17WartsPingResponse_2rx_4__del__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_10traceutils_7scamper_5warts_17WartsPingResponse_3rtt___get__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *__pyx_v_self); /* proto */
static int __pyx_pf_10traceutils_7scamper_5warts_17WartsPingResponse_3rtt_2__set__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_10traceutils_7scamper_5warts_17WartsPingResponse_6family___get__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *__pyx_v_self); /* proto */
static int __pyx_pf_10traceutils_7scamper_5warts_17WartsPingResponse_6family_2__set__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_10traceutils_7scamper_5warts_17WartsPingResponse_4type___get__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *__pyx_v_self); /* proto */
static int __pyx_pf_10traceutils_7scamper_5warts_17WartsPingResponse_4type_2__set__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_10traceutils_7scamper_5warts_17WartsPingResponse_4__reduce_cython__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_10traceutils_7scamper_5warts_17WartsPingResponse_6__setstate_cython__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_10traceutils_7scamper_5warts_11WartsReader___init__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsReader *__pyx_v_self, PyObject *__pyx_v_filename); /* proto */
static PyObject *__pyx_pf_10traceutils_7scamper_5warts_11WartsReader_2__iter__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsReader *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_10traceutils_7scamper_5warts_11WartsReader_5open(struct __pyx_obj_10traceutils_7scamper_5warts_WartsReader *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_10traceutils_7scamper_5warts_11WartsReader_7close(struct __pyx_obj_10traceutils_7scamper_5warts_WartsReader *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_10traceutils_7scamper_5warts_11WartsReader_9__reduce_cython__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsReader *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_10traceutils_7scamper_5warts_11WartsReader_11__setstate_cython__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsReader *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_10traceutils_7scamper_5warts___pyx_unpickle_WartsTrace(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_10traceutils_7scamper_5warts_2__pyx_unpickle_WartsHop(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_10traceutils_7scamper_5warts_4__pyx_unpickle_WartsPing(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_10traceutils_7scamper_5warts_6__pyx_unpickle_WartsPingResponse(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_10traceutils_7scamper_5warts_8__pyx_unpickle_WartsReader(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_tp_new_10traceutils_7scamper_5warts_WartsHop(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_10traceutils_7scamper_5warts_WartsTrace(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_10traceutils_7scamper_5warts_WartsPing(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_10traceutils_7scamper_5warts_WartsPingResponse(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_10traceutils_7scamper_5warts_WartsReader(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_10traceutils_7scamper_5warts___pyx_scope_struct____iter__(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static __Pyx_CachedCFunction __pyx_umethod_PyDict_Type_pop = {0, &__pyx_n_s_pop, 0, 0, 0};
static __Pyx_CachedCFunction __pyx_umethod_PyUnicode_Type_format = {0, &__pyx_n_s_format, 0, 0, 0};
static PyObject *__pyx_int_91565645;
static PyObject *__pyx_int_126533895;
static PyObject *__pyx_int_154359991;
static PyObject *__pyx_int_194082941;
static PyObject *__pyx_int_268127653;
static PyObject *__pyx_tuple__4;
static PyObject *__pyx_tuple__6;
static PyObject *__pyx_tuple__8;
static PyObject *__pyx_tuple__10;
static PyObject *__pyx_tuple__12;
static PyObject *__pyx_codeobj__5;
static PyObject *__pyx_codeobj__7;
static PyObject *__pyx_codeobj__9;
static PyObject *__pyx_codeobj__11;
static PyObject *__pyx_codeobj__13;
/* Late includes */

/* "traceutils/scamper/warts.pyx":7
 * from traceutils.utils.net cimport find_family
 * 
 * cdef list create_hops(list hops, int family):             # <<<<<<<<<<<<<<
 *         cdef dict h
 *         if hops is not None:
 */

static PyObject *__pyx_f_10traceutils_7scamper_5warts_create_hops(PyObject *__pyx_v_hops, int __pyx_v_family) {
  PyObject *__pyx_7genexpr__pyx_v_h = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  Py_ssize_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  __Pyx_RefNannySetupContext("create_hops", 0);

  /* "traceutils/scamper/warts.pyx":9
 * cdef list create_hops(list hops, int family):
 *         cdef dict h
 *         if hops is not None:             # <<<<<<<<<<<<<<
 *             return [WartsHop(family=family, **h) for h in hops]
 *         return []
 */
  __pyx_t_1 = (__pyx_v_hops != ((PyObject*)Py_None));
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "traceutils/scamper/warts.pyx":10
 *         cdef dict h
 *         if hops is not None:
 *             return [WartsHop(family=family, **h) for h in hops]             # <<<<<<<<<<<<<<
 *         return []
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    { /* enter inner scope */
      __pyx_t_3 = PyList_New(0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 10, __pyx_L6_error)
      __Pyx_GOTREF(__pyx_t_3);
      if (unlikely(__pyx_v_hops == Py_None)) {
        PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
        __PYX_ERR(0, 10, __pyx_L6_error)
      }
      __pyx_t_4 = __pyx_v_hops; __Pyx_INCREF(__pyx_t_4); __pyx_t_5 = 0;
      for (;;) {
        if (__pyx_t_5 >= PyList_GET_SIZE(__pyx_t_4)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_6 = PyList_GET_ITEM(__pyx_t_4, __pyx_t_5); __Pyx_INCREF(__pyx_t_6); __pyx_t_5++; if (unlikely(0 < 0)) __PYX_ERR(0, 10, __pyx_L6_error)
        #else
        __pyx_t_6 = PySequence_ITEM(__pyx_t_4, __pyx_t_5); __pyx_t_5++; if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 10, __pyx_L6_error)
        __Pyx_GOTREF(__pyx_t_6);
        #endif
        if (!(likely(PyDict_CheckExact(__pyx_t_6))||((__pyx_t_6) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_t_6)->tp_name), 0))) __PYX_ERR(0, 10, __pyx_L6_error)
        __Pyx_XDECREF_SET(__pyx_7genexpr__pyx_v_h, ((PyObject*)__pyx_t_6));
        __pyx_t_6 = 0;
        __pyx_t_7 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 10, __pyx_L6_error)
        __Pyx_GOTREF(__pyx_t_7);
        __pyx_t_8 = __Pyx_PyInt_From_int(__pyx_v_family); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 10, __pyx_L6_error)
        __Pyx_GOTREF(__pyx_t_8);
        if (PyDict_SetItem(__pyx_t_7, __pyx_n_s_family, __pyx_t_8) < 0) __PYX_ERR(0, 10, __pyx_L6_error)
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __pyx_t_6 = __pyx_t_7;
        __pyx_t_7 = 0;
        if (unlikely(__pyx_7genexpr__pyx_v_h == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "argument after ** must be a mapping, not NoneType");
          __PYX_ERR(0, 10, __pyx_L6_error)
        }
        if (__Pyx_MergeKeywords(__pyx_t_6, __pyx_7genexpr__pyx_v_h) < 0) __PYX_ERR(0, 10, __pyx_L6_error)
        __pyx_t_7 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_10traceutils_7scamper_5warts_WartsHop), __pyx_empty_tuple, __pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 10, __pyx_L6_error)
        __Pyx_GOTREF(__pyx_t_7);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        if (unlikely(__Pyx_ListComp_Append(__pyx_t_3, (PyObject*)__pyx_t_7))) __PYX_ERR(0, 10, __pyx_L6_error)
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      }
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_XDECREF(__pyx_7genexpr__pyx_v_h); __pyx_7genexpr__pyx_v_h = 0;
      goto __pyx_L9_exit_scope;
      __pyx_L6_error:;
      __Pyx_XDECREF(__pyx_7genexpr__pyx_v_h); __pyx_7genexpr__pyx_v_h = 0;
      goto __pyx_L1_error;
      __pyx_L9_exit_scope:;
    } /* exit inner scope */
    __pyx_r = ((PyObject*)__pyx_t_3);
    __pyx_t_3 = 0;
    goto __pyx_L0;

    /* "traceutils/scamper/warts.pyx":9
 * cdef list create_hops(list hops, int family):
 *         cdef dict h
 *         if hops is not None:             # <<<<<<<<<<<<<<
 *             return [WartsHop(family=family, **h) for h in hops]
 *         return []
 */
  }

  /* "traceutils/scamper/warts.pyx":11
 *         if hops is not None:
 *             return [WartsHop(family=family, **h) for h in hops]
 *         return []             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_3 = PyList_New(0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 11, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_r = ((PyObject*)__pyx_t_3);
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "traceutils/scamper/warts.pyx":7
 * from traceutils.utils.net cimport find_family
 * 
 * cdef list create_hops(list hops, int family):             # <<<<<<<<<<<<<<
 *         cdef dict h
 *         if hops is not None:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("traceutils.scamper.warts.create_hops", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_7genexpr__pyx_v_h);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "traceutils/scamper/warts.pyx":16
 * cdef class WartsTrace(Trace):
 * 
 *     def __init__(             # <<<<<<<<<<<<<<
 *             self, str type='', str version='', int userid=-1, str method='', str src='', str dst='',
 *             int icmp_sum=-1, str stop_reason='', int stop_data=-1, dict start=None, int hop_count=-1,
 */

/* Python wrapper */
static int __pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_type = 0;
  PyObject *__pyx_v_version = 0;
  int __pyx_v_userid;
  PyObject *__pyx_v_method = 0;
  PyObject *__pyx_v_src = 0;
  PyObject *__pyx_v_dst = 0;
  int __pyx_v_icmp_sum;
  PyObject *__pyx_v_stop_reason = 0;
  int __pyx_v_stop_data;
  PyObject *__pyx_v_start = 0;
  int __pyx_v_hop_count;
  int __pyx_v_attempts;
  unsigned char __pyx_v_hoplimit;
  unsigned char __pyx_v_firsthop;
  double __pyx_v_wait;
  int __pyx_v_wait_probe;
  int __pyx_v_tos;
  unsigned short __pyx_v_probe_size;
  unsigned char __pyx_v_probe_count;
  PyObject *__pyx_v_hops = 0;
  CYTHON_UNUSED PyObject *__pyx_v_list_name = 0;
  CYTHON_UNUSED int __pyx_v_id;
  CYTHON_UNUSED PyObject *__pyx_v_hostname = 0;
  CYTHON_UNUSED long __pyx_v_start_time;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_type,&__pyx_n_s_version,&__pyx_n_s_userid,&__pyx_n_s_method,&__pyx_n_s_src,&__pyx_n_s_dst,&__pyx_n_s_icmp_sum,&__pyx_n_s_stop_reason,&__pyx_n_s_stop_data,&__pyx_n_s_start,&__pyx_n_s_hop_count,&__pyx_n_s_attempts,&__pyx_n_s_hoplimit,&__pyx_n_s_firsthop,&__pyx_n_s_wait,&__pyx_n_s_wait_probe,&__pyx_n_s_tos,&__pyx_n_s_probe_size,&__pyx_n_s_probe_count,&__pyx_n_s_hops,&__pyx_n_s_list_name,&__pyx_n_s_id,&__pyx_n_s_hostname,&__pyx_n_s_start_time,0};
    PyObject* values[24] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
    values[0] = ((PyObject*)__pyx_kp_u_);
    values[1] = ((PyObject*)__pyx_kp_u_);
    values[3] = ((PyObject*)__pyx_kp_u_);
    values[4] = ((PyObject*)__pyx_kp_u_);
    values[5] = ((PyObject*)__pyx_kp_u_);
    values[7] = ((PyObject*)__pyx_kp_u_);

    /* "traceutils/scamper/warts.pyx":18
 *     def __init__(
 *             self, str type='', str version='', int userid=-1, str method='', str src='', str dst='',
 *             int icmp_sum=-1, str stop_reason='', int stop_data=-1, dict start=None, int hop_count=-1,             # <<<<<<<<<<<<<<
 *             int attempts=-1, unsigned char hoplimit=0, unsigned char firsthop=1, double wait=-1,
 *             int wait_probe=-1, int tos=-1, unsigned short probe_size=0, unsigned char probe_count=0,
 */
    values[9] = ((PyObject*)Py_None);

    /* "traceutils/scamper/warts.pyx":21
 *             int attempts=-1, unsigned char hoplimit=0, unsigned char firsthop=1, double wait=-1,
 *             int wait_probe=-1, int tos=-1, unsigned short probe_size=0, unsigned char probe_count=0,
 *             list hops=None, str list_name='', int id=-1, str hostname='', long start_time=0             # <<<<<<<<<<<<<<
 *     ):
 *         self.src = src
 */
    values[19] = ((PyObject*)Py_None);
    values[20] = ((PyObject*)__pyx_kp_u_);
    values[22] = ((PyObject*)__pyx_kp_u_);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case 24: values[23] = PyTuple_GET_ITEM(__pyx_args, 23);
        CYTHON_FALLTHROUGH;
        case 23: values[22] = PyTuple_GET_ITEM(__pyx_args, 22);
        CYTHON_FALLTHROUGH;
        case 22: values[21] = PyTuple_GET_ITEM(__pyx_args, 21);
        CYTHON_FALLTHROUGH;
        case 21: values[20] = PyTuple_GET_ITEM(__pyx_args, 20);
        CYTHON_FALLTHROUGH;
        case 20: values[19] = PyTuple_GET_ITEM(__pyx_args, 19);
        CYTHON_FALLTHROUGH;
        case 19: values[18] = PyTuple_GET_ITEM(__pyx_args, 18);
        CYTHON_FALLTHROUGH;
        case 18: values[17] = PyTuple_GET_ITEM(__pyx_args, 17);
        CYTHON_FALLTHROUGH;
        case 17: values[16] = PyTuple_GET_ITEM(__pyx_args, 16);
        CYTHON_FALLTHROUGH;
        case 16: values[15] = PyTuple_GET_ITEM(__pyx_args, 15);
        CYTHON_FALLTHROUGH;
        case 15: values[14] = PyTuple_GET_ITEM(__pyx_args, 14);
        CYTHON_FALLTHROUGH;
        case 14: values[13] = PyTuple_GET_ITEM(__pyx_args, 13);
        CYTHON_FALLTHROUGH;
        case 13: values[12] = PyTuple_GET_ITEM(__pyx_args, 12);
        CYTHON_FALLTHROUGH;
        case 12: values[11] = PyTuple_GET_ITEM(__pyx_args, 11);
        CYTHON_FALLTHROUGH;
        case 11: values[10] = PyTuple_GET_ITEM(__pyx_args, 10);
        CYTHON_FALLTHROUGH;
        case 10: values[9] = PyTuple_GET_ITEM(__pyx_args, 9);
        CYTHON_FALLTHROUGH;
        case  9: values[8] = PyTuple_GET_ITEM(__pyx_args, 8);
        CYTHON_FALLTHROUGH;
        case  8: values[7] = PyTuple_GET_ITEM(__pyx_args, 7);
        CYTHON_FALLTHROUGH;
        case  7: values[6] = PyTuple_GET_ITEM(__pyx_args, 6);
        CYTHON_FALLTHROUGH;
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        CYTHON_FALLTHROUGH;
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_type);
          if (value) { values[0] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_version);
          if (value) { values[1] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_userid);
          if (value) { values[2] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_method);
          if (value) { values[3] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  4:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_src);
          if (value) { values[4] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  5:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_dst);
          if (value) { values[5] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  6:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_icmp_sum);
          if (value) { values[6] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  7:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_stop_reason);
          if (value) { values[7] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  8:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_stop_data);
          if (value) { values[8] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  9:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_start);
          if (value) { values[9] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case 10:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_hop_count);
          if (value) { values[10] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case 11:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_attempts);
          if (value) { values[11] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case 12:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_hoplimit);
          if (value) { values[12] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case 13:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_firsthop);
          if (value) { values[13] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case 14:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_wait);
          if (value) { values[14] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case 15:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_wait_probe);
          if (value) { values[15] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case 16:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_tos);
          if (value) { values[16] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case 17:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_probe_size);
          if (value) { values[17] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case 18:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_probe_count);
          if (value) { values[18] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case 19:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_hops);
          if (value) { values[19] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case 20:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_list_name);
          if (value) { values[20] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case 21:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_id);
          if (value) { values[21] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case 22:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_hostname);
          if (value) { values[22] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case 23:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_start_time);
          if (value) { values[23] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 16, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case 24: values[23] = PyTuple_GET_ITEM(__pyx_args, 23);
        CYTHON_FALLTHROUGH;
        case 23: values[22] = PyTuple_GET_ITEM(__pyx_args, 22);
        CYTHON_FALLTHROUGH;
        case 22: values[21] = PyTuple_GET_ITEM(__pyx_args, 21);
        CYTHON_FALLTHROUGH;
        case 21: values[20] = PyTuple_GET_ITEM(__pyx_args, 20);
        CYTHON_FALLTHROUGH;
        case 20: values[19] = PyTuple_GET_ITEM(__pyx_args, 19);
        CYTHON_FALLTHROUGH;
        case 19: values[18] = PyTuple_GET_ITEM(__pyx_args, 18);
        CYTHON_FALLTHROUGH;
        case 18: values[17] = PyTuple_GET_ITEM(__pyx_args, 17);
        CYTHON_FALLTHROUGH;
        case 17: values[16] = PyTuple_GET_ITEM(__pyx_args, 16);
        CYTHON_FALLTHROUGH;
        case 16: values[15] = PyTuple_GET_ITEM(__pyx_args, 15);
        CYTHON_FALLTHROUGH;
        case 15: values[14] = PyTuple_GET_ITEM(__pyx_args, 14);
        CYTHON_FALLTHROUGH;
        case 14: values[13] = PyTuple_GET_ITEM(__pyx_args, 13);
        CYTHON_FALLTHROUGH;
        case 13: values[12] = PyTuple_GET_ITEM(__pyx_args, 12);
        CYTHON_FALLTHROUGH;
        case 12: values[11] = PyTuple_GET_ITEM(__pyx_args, 11);
        CYTHON_FALLTHROUGH;
        case 11: values[10] = PyTuple_GET_ITEM(__pyx_args, 10);
        CYTHON_FALLTHROUGH;
        case 10: values[9] = PyTuple_GET_ITEM(__pyx_args, 9);
        CYTHON_FALLTHROUGH;
        case  9: values[8] = PyTuple_GET_ITEM(__pyx_args, 8);
        CYTHON_FALLTHROUGH;
        case  8: values[7] = PyTuple_GET_ITEM(__pyx_args, 7);
        CYTHON_FALLTHROUGH;
        case  7: values[6] = PyTuple_GET_ITEM(__pyx_args, 6);
        CYTHON_FALLTHROUGH;
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        CYTHON_FALLTHROUGH;
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_type = ((PyObject*)values[0]);
    __pyx_v_version = ((PyObject*)values[1]);
    if (values[2]) {
      __pyx_v_userid = __Pyx_PyInt_As_int(values[2]); if (unlikely((__pyx_v_userid == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 17, __pyx_L3_error)
    } else {
      __pyx_v_userid = ((int)-1);
    }
    __pyx_v_method = ((PyObject*)values[3]);
    __pyx_v_src = ((PyObject*)values[4]);
    __pyx_v_dst = ((PyObject*)values[5]);
    if (values[6]) {
      __pyx_v_icmp_sum = __Pyx_PyInt_As_int(values[6]); if (unlikely((__pyx_v_icmp_sum == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 18, __pyx_L3_error)
    } else {
      __pyx_v_icmp_sum = ((int)-1);
    }
    __pyx_v_stop_reason = ((PyObject*)values[7]);
    if (values[8]) {
      __pyx_v_stop_data = __Pyx_PyInt_As_int(values[8]); if (unlikely((__pyx_v_stop_data == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 18, __pyx_L3_error)
    } else {
      __pyx_v_stop_data = ((int)-1);
    }
    __pyx_v_start = ((PyObject*)values[9]);
    if (values[10]) {
      __pyx_v_hop_count = __Pyx_PyInt_As_int(values[10]); if (unlikely((__pyx_v_hop_count == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 18, __pyx_L3_error)
    } else {
      __pyx_v_hop_count = ((int)-1);
    }
    if (values[11]) {
      __pyx_v_attempts = __Pyx_PyInt_As_int(values[11]); if (unlikely((__pyx_v_attempts == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 19, __pyx_L3_error)
    } else {
      __pyx_v_attempts = ((int)-1);
    }
    if (values[12]) {
      __pyx_v_hoplimit = __Pyx_PyInt_As_unsigned_char(values[12]); if (unlikely((__pyx_v_hoplimit == (unsigned char)-1) && PyErr_Occurred())) __PYX_ERR(0, 19, __pyx_L3_error)
    } else {
      __pyx_v_hoplimit = ((unsigned char)0);
    }
    if (values[13]) {
      __pyx_v_firsthop = __Pyx_PyInt_As_unsigned_char(values[13]); if (unlikely((__pyx_v_firsthop == (unsigned char)-1) && PyErr_Occurred())) __PYX_ERR(0, 19, __pyx_L3_error)
    } else {
      __pyx_v_firsthop = ((unsigned char)1);
    }
    if (values[14]) {
      __pyx_v_wait = __pyx_PyFloat_AsDouble(values[14]); if (unlikely((__pyx_v_wait == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 19, __pyx_L3_error)
    } else {
      __pyx_v_wait = ((double)-1.0);
    }
    if (values[15]) {
      __pyx_v_wait_probe = __Pyx_PyInt_As_int(values[15]); if (unlikely((__pyx_v_wait_probe == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 20, __pyx_L3_error)
    } else {
      __pyx_v_wait_probe = ((int)-1);
    }
    if (values[16]) {
      __pyx_v_tos = __Pyx_PyInt_As_int(values[16]); if (unlikely((__pyx_v_tos == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 20, __pyx_L3_error)
    } else {
      __pyx_v_tos = ((int)-1);
    }
    if (values[17]) {
      __pyx_v_probe_size = __Pyx_PyInt_As_unsigned_short(values[17]); if (unlikely((__pyx_v_probe_size == (unsigned short)-1) && PyErr_Occurred())) __PYX_ERR(0, 20, __pyx_L3_error)
    } else {
      __pyx_v_probe_size = ((unsigned short)0);
    }
    if (values[18]) {
      __pyx_v_probe_count = __Pyx_PyInt_As_unsigned_char(values[18]); if (unlikely((__pyx_v_probe_count == (unsigned char)-1) && PyErr_Occurred())) __PYX_ERR(0, 20, __pyx_L3_error)
    } else {
      __pyx_v_probe_count = ((unsigned char)0);
    }
    __pyx_v_hops = ((PyObject*)values[19]);
    __pyx_v_list_name = ((PyObject*)values[20]);
    if (values[21]) {
      __pyx_v_id = __Pyx_PyInt_As_int(values[21]); if (unlikely((__pyx_v_id == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 21, __pyx_L3_error)
    } else {
      __pyx_v_id = ((int)-1);
    }
    __pyx_v_hostname = ((PyObject*)values[22]);
    if (values[23]) {
      __pyx_v_start_time = __Pyx_PyInt_As_long(values[23]); if (unlikely((__pyx_v_start_time == (long)-1) && PyErr_Occurred())) __PYX_ERR(0, 21, __pyx_L3_error)
    } else {
      __pyx_v_start_time = ((long)0);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 0, 0, 24, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 16, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsTrace.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_type), (&PyUnicode_Type), 1, "type", 1))) __PYX_ERR(0, 17, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_version), (&PyUnicode_Type), 1, "version", 1))) __PYX_ERR(0, 17, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_method), (&PyUnicode_Type), 1, "method", 1))) __PYX_ERR(0, 17, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_src), (&PyUnicode_Type), 1, "src", 1))) __PYX_ERR(0, 17, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_dst), (&PyUnicode_Type), 1, "dst", 1))) __PYX_ERR(0, 17, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_stop_reason), (&PyUnicode_Type), 1, "stop_reason", 1))) __PYX_ERR(0, 18, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_start), (&PyDict_Type), 1, "start", 1))) __PYX_ERR(0, 18, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_hops), (&PyList_Type), 1, "hops", 1))) __PYX_ERR(0, 21, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_list_name), (&PyUnicode_Type), 1, "list_name", 1))) __PYX_ERR(0, 21, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_hostname), (&PyUnicode_Type), 1, "hostname", 1))) __PYX_ERR(0, 21, __pyx_L1_error)
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_10WartsTrace___init__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *)__pyx_v_self), __pyx_v_type, __pyx_v_version, __pyx_v_userid, __pyx_v_method, __pyx_v_src, __pyx_v_dst, __pyx_v_icmp_sum, __pyx_v_stop_reason, __pyx_v_stop_data, __pyx_v_start, __pyx_v_hop_count, __pyx_v_attempts, __pyx_v_hoplimit, __pyx_v_firsthop, __pyx_v_wait, __pyx_v_wait_probe, __pyx_v_tos, __pyx_v_probe_size, __pyx_v_probe_count, __pyx_v_hops, __pyx_v_list_name, __pyx_v_id, __pyx_v_hostname, __pyx_v_start_time);

  /* "traceutils/scamper/warts.pyx":16
 * cdef class WartsTrace(Trace):
 * 
 *     def __init__(             # <<<<<<<<<<<<<<
 *             self, str type='', str version='', int userid=-1, str method='', str src='', str dst='',
 *             int icmp_sum=-1, str stop_reason='', int stop_data=-1, dict start=None, int hop_count=-1,
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_10traceutils_7scamper_5warts_10WartsTrace___init__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *__pyx_v_self, PyObject *__pyx_v_type, PyObject *__pyx_v_version, int __pyx_v_userid, PyObject *__pyx_v_method, PyObject *__pyx_v_src, PyObject *__pyx_v_dst, int __pyx_v_icmp_sum, PyObject *__pyx_v_stop_reason, int __pyx_v_stop_data, PyObject *__pyx_v_start, int __pyx_v_hop_count, int __pyx_v_attempts, unsigned char __pyx_v_hoplimit, unsigned char __pyx_v_firsthop, double __pyx_v_wait, int __pyx_v_wait_probe, int __pyx_v_tos, unsigned short __pyx_v_probe_size, unsigned char __pyx_v_probe_count, PyObject *__pyx_v_hops, CYTHON_UNUSED PyObject *__pyx_v_list_name, CYTHON_UNUSED int __pyx_v_id, CYTHON_UNUSED PyObject *__pyx_v_hostname, CYTHON_UNUSED long __pyx_v_start_time) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "traceutils/scamper/warts.pyx":23
 *             list hops=None, str list_name='', int id=-1, str hostname='', long start_time=0
 *     ):
 *         self.src = src             # <<<<<<<<<<<<<<
 *         self.dst = dst
 *         self.family = find_family(dst.encode())
 */
  __Pyx_INCREF(__pyx_v_src);
  __Pyx_GIVEREF(__pyx_v_src);
  __Pyx_GOTREF(__pyx_v_self->__pyx_base.src);
  __Pyx_DECREF(__pyx_v_self->__pyx_base.src);
  __pyx_v_self->__pyx_base.src = __pyx_v_src;

  /* "traceutils/scamper/warts.pyx":24
 *     ):
 *         self.src = src
 *         self.dst = dst             # <<<<<<<<<<<<<<
 *         self.family = find_family(dst.encode())
 *         self.hops = create_hops(hops, self.family)
 */
  __Pyx_INCREF(__pyx_v_dst);
  __Pyx_GIVEREF(__pyx_v_dst);
  __Pyx_GOTREF(__pyx_v_self->__pyx_base.dst);
  __Pyx_DECREF(__pyx_v_self->__pyx_base.dst);
  __pyx_v_self->__pyx_base.dst = __pyx_v_dst;

  /* "traceutils/scamper/warts.pyx":25
 *         self.src = src
 *         self.dst = dst
 *         self.family = find_family(dst.encode())             # <<<<<<<<<<<<<<
 *         self.hops = create_hops(hops, self.family)
 * 
 */
  if (unlikely(__pyx_v_dst == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "encode");
    __PYX_ERR(0, 25, __pyx_L1_error)
  }
  __pyx_t_1 = PyUnicode_AsEncodedString(__pyx_v_dst, NULL, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 25, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_self->__pyx_base.family = __pyx_f_10traceutils_5utils_3net_find_family(((PyObject*)__pyx_t_1), 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "traceutils/scamper/warts.pyx":26
 *         self.dst = dst
 *         self.family = find_family(dst.encode())
 *         self.hops = create_hops(hops, self.family)             # <<<<<<<<<<<<<<
 * 
 *         self.type = type
 */
  __pyx_t_1 = __pyx_f_10traceutils_7scamper_5warts_create_hops(__pyx_v_hops, __pyx_v_self->__pyx_base.family); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 26, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->__pyx_base.hops);
  __Pyx_DECREF(__pyx_v_self->__pyx_base.hops);
  __pyx_v_self->__pyx_base.hops = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "traceutils/scamper/warts.pyx":28
 *         self.hops = create_hops(hops, self.family)
 * 
 *         self.type = type             # <<<<<<<<<<<<<<
 *         self.version = version
 *         self.userid = userid
 */
  __Pyx_INCREF(__pyx_v_type);
  __Pyx_GIVEREF(__pyx_v_type);
  __Pyx_GOTREF(__pyx_v_self->type);
  __Pyx_DECREF(__pyx_v_self->type);
  __pyx_v_self->type = __pyx_v_type;

  /* "traceutils/scamper/warts.pyx":29
 * 
 *         self.type = type
 *         self.version = version             # <<<<<<<<<<<<<<
 *         self.userid = userid
 *         self.method = method
 */
  __Pyx_INCREF(__pyx_v_version);
  __Pyx_GIVEREF(__pyx_v_version);
  __Pyx_GOTREF(__pyx_v_self->version);
  __Pyx_DECREF(__pyx_v_self->version);
  __pyx_v_self->version = __pyx_v_version;

  /* "traceutils/scamper/warts.pyx":30
 *         self.type = type
 *         self.version = version
 *         self.userid = userid             # <<<<<<<<<<<<<<
 *         self.method = method
 *         self.icmp_sum = icmp_sum
 */
  __pyx_v_self->userid = __pyx_v_userid;

  /* "traceutils/scamper/warts.pyx":31
 *         self.version = version
 *         self.userid = userid
 *         self.method = method             # <<<<<<<<<<<<<<
 *         self.icmp_sum = icmp_sum
 *         self.stop_reason = stop_reason
 */
  __Pyx_INCREF(__pyx_v_method);
  __Pyx_GIVEREF(__pyx_v_method);
  __Pyx_GOTREF(__pyx_v_self->method);
  __Pyx_DECREF(__pyx_v_self->method);
  __pyx_v_self->method = __pyx_v_method;

  /* "traceutils/scamper/warts.pyx":32
 *         self.userid = userid
 *         self.method = method
 *         self.icmp_sum = icmp_sum             # <<<<<<<<<<<<<<
 *         self.stop_reason = stop_reason
 *         self.stop_data = stop_data
 */
  __pyx_v_self->icmp_sum = __pyx_v_icmp_sum;

  /* "traceutils/scamper/warts.pyx":33
 *         self.method = method
 *         self.icmp_sum = icmp_sum
 *         self.stop_reason = stop_reason             # <<<<<<<<<<<<<<
 *         self.stop_data = stop_data
 *         self.start = start
 */
  __Pyx_INCREF(__pyx_v_stop_reason);
  __Pyx_GIVEREF(__pyx_v_stop_reason);
  __Pyx_GOTREF(__pyx_v_self->stop_reason);
  __Pyx_DECREF(__pyx_v_self->stop_reason);
  __pyx_v_self->stop_reason = __pyx_v_stop_reason;

  /* "traceutils/scamper/warts.pyx":34
 *         self.icmp_sum = icmp_sum
 *         self.stop_reason = stop_reason
 *         self.stop_data = stop_data             # <<<<<<<<<<<<<<
 *         self.start = start
 *         self.hop_count = hop_count
 */
  __pyx_v_self->stop_data = __pyx_v_stop_data;

  /* "traceutils/scamper/warts.pyx":35
 *         self.stop_reason = stop_reason
 *         self.stop_data = stop_data
 *         self.start = start             # <<<<<<<<<<<<<<
 *         self.hop_count = hop_count
 *         self.attempts = attempts
 */
  __Pyx_INCREF(__pyx_v_start);
  __Pyx_GIVEREF(__pyx_v_start);
  __Pyx_GOTREF(__pyx_v_self->start);
  __Pyx_DECREF(__pyx_v_self->start);
  __pyx_v_self->start = __pyx_v_start;

  /* "traceutils/scamper/warts.pyx":36
 *         self.stop_data = stop_data
 *         self.start = start
 *         self.hop_count = hop_count             # <<<<<<<<<<<<<<
 *         self.attempts = attempts
 *         self.hoplimit = hoplimit
 */
  __pyx_v_self->hop_count = __pyx_v_hop_count;

  /* "traceutils/scamper/warts.pyx":37
 *         self.start = start
 *         self.hop_count = hop_count
 *         self.attempts = attempts             # <<<<<<<<<<<<<<
 *         self.hoplimit = hoplimit
 *         self.firsthop = firsthop
 */
  __pyx_v_self->attempts = __pyx_v_attempts;

  /* "traceutils/scamper/warts.pyx":38
 *         self.hop_count = hop_count
 *         self.attempts = attempts
 *         self.hoplimit = hoplimit             # <<<<<<<<<<<<<<
 *         self.firsthop = firsthop
 *         self.wait = wait
 */
  __pyx_v_self->hoplimit = __pyx_v_hoplimit;

  /* "traceutils/scamper/warts.pyx":39
 *         self.attempts = attempts
 *         self.hoplimit = hoplimit
 *         self.firsthop = firsthop             # <<<<<<<<<<<<<<
 *         self.wait = wait
 *         self.wait_probe = wait_probe
 */
  __pyx_v_self->firsthop = __pyx_v_firsthop;

  /* "traceutils/scamper/warts.pyx":40
 *         self.hoplimit = hoplimit
 *         self.firsthop = firsthop
 *         self.wait = wait             # <<<<<<<<<<<<<<
 *         self.wait_probe = wait_probe
 *         self.tos = tos
 */
  __pyx_v_self->wait = __pyx_v_wait;

  /* "traceutils/scamper/warts.pyx":41
 *         self.firsthop = firsthop
 *         self.wait = wait
 *         self.wait_probe = wait_probe             # <<<<<<<<<<<<<<
 *         self.tos = tos
 *         self.probe_size = probe_size
 */
  __pyx_v_self->wait_probe = __pyx_v_wait_probe;

  /* "traceutils/scamper/warts.pyx":42
 *         self.wait = wait
 *         self.wait_probe = wait_probe
 *         self.tos = tos             # <<<<<<<<<<<<<<
 *         self.probe_size = probe_size
 *         self.probe_count = probe_count
 */
  __pyx_v_self->tos = __pyx_v_tos;

  /* "traceutils/scamper/warts.pyx":43
 *         self.wait_probe = wait_probe
 *         self.tos = tos
 *         self.probe_size = probe_size             # <<<<<<<<<<<<<<
 *         self.probe_count = probe_count
 * 
 */
  __pyx_v_self->probe_size = __pyx_v_probe_size;

  /* "traceutils/scamper/warts.pyx":44
 *         self.tos = tos
 *         self.probe_size = probe_size
 *         self.probe_count = probe_count             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_v_self->probe_count = __pyx_v_probe_count;

  /* "traceutils/scamper/warts.pyx":16
 * cdef class WartsTrace(Trace):
 * 
 *     def __init__(             # <<<<<<<<<<<<<<
 *             self, str type='', str version='', int userid=-1, str method='', str src='', str dst='',
 *             int icmp_sum=-1, str stop_reason='', int stop_data=-1, dict start=None, int hop_count=-1,
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsTrace.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "traceutils/scamper/warts.pxd":13
 * 
 * cdef class WartsTrace(Trace):
 *     cdef public str type             # <<<<<<<<<<<<<<
 *     cdef public str version
 *     cdef public int userid
 */

/* Python wrapper */
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_4type_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_4type_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_4type___get__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_4type___get__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->type);
  __pyx_r = __pyx_v_self->type;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_4type_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_4type_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_4type_2__set__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_4type_2__set__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(PyUnicode_CheckExact(__pyx_v_value))||((__pyx_v_value) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "unicode", Py_TYPE(__pyx_v_value)->tp_name), 0))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->type);
  __Pyx_DECREF(__pyx_v_self->type);
  __pyx_v_self->type = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsTrace.type.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_4type_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_4type_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_4type_4__del__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_4type_4__del__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->type);
  __Pyx_DECREF(__pyx_v_self->type);
  __pyx_v_self->type = ((PyObject*)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "traceutils/scamper/warts.pxd":14
 * cdef class WartsTrace(Trace):
 *     cdef public str type
 *     cdef public str version             # <<<<<<<<<<<<<<
 *     cdef public int userid
 *     cdef public str method
 */

/* Python wrapper */
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_7version_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_7version_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_7version___get__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_7version___get__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->version);
  __pyx_r = __pyx_v_self->version;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_7version_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_7version_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_7version_2__set__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_7version_2__set__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(PyUnicode_CheckExact(__pyx_v_value))||((__pyx_v_value) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "unicode", Py_TYPE(__pyx_v_value)->tp_name), 0))) __PYX_ERR(1, 14, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->version);
  __Pyx_DECREF(__pyx_v_self->version);
  __pyx_v_self->version = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsTrace.version.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_7version_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_7version_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_7version_4__del__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_7version_4__del__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->version);
  __Pyx_DECREF(__pyx_v_self->version);
  __pyx_v_self->version = ((PyObject*)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "traceutils/scamper/warts.pxd":15
 *     cdef public str type
 *     cdef public str version
 *     cdef public int userid             # <<<<<<<<<<<<<<
 *     cdef public str method
 *     cdef public int icmp_sum
 */

/* Python wrapper */
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_6userid_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_6userid_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_6userid___get__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_6userid___get__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->userid); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 15, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsTrace.userid.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_6userid_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_6userid_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_6userid_2__set__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_6userid_2__set__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyInt_As_int(__pyx_v_value); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 15, __pyx_L1_error)
  __pyx_v_self->userid = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsTrace.userid.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "traceutils/scamper/warts.pxd":16
 *     cdef public str version
 *     cdef public int userid
 *     cdef public str method             # <<<<<<<<<<<<<<
 *     cdef public int icmp_sum
 *     cdef public str stop_reason
 */

/* Python wrapper */
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_6method_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_6method_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_6method___get__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_6method___get__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->method);
  __pyx_r = __pyx_v_self->method;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_6method_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_6method_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_6method_2__set__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_6method_2__set__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(PyUnicode_CheckExact(__pyx_v_value))||((__pyx_v_value) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "unicode", Py_TYPE(__pyx_v_value)->tp_name), 0))) __PYX_ERR(1, 16, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->method);
  __Pyx_DECREF(__pyx_v_self->method);
  __pyx_v_self->method = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsTrace.method.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_6method_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_6method_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_6method_4__del__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_6method_4__del__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->method);
  __Pyx_DECREF(__pyx_v_self->method);
  __pyx_v_self->method = ((PyObject*)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "traceutils/scamper/warts.pxd":17
 *     cdef public int userid
 *     cdef public str method
 *     cdef public int icmp_sum             # <<<<<<<<<<<<<<
 *     cdef public str stop_reason
 *     cdef public int stop_data
 */

/* Python wrapper */
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_8icmp_sum_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_8icmp_sum_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_8icmp_sum___get__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_8icmp_sum___get__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->icmp_sum); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsTrace.icmp_sum.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_8icmp_sum_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_8icmp_sum_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_8icmp_sum_2__set__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_8icmp_sum_2__set__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyInt_As_int(__pyx_v_value); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 17, __pyx_L1_error)
  __pyx_v_self->icmp_sum = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsTrace.icmp_sum.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "traceutils/scamper/warts.pxd":18
 *     cdef public str method
 *     cdef public int icmp_sum
 *     cdef public str stop_reason             # <<<<<<<<<<<<<<
 *     cdef public int stop_data
 *     cdef public dict start
 */

/* Python wrapper */
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_11stop_reason_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_11stop_reason_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_11stop_reason___get__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_11stop_reason___get__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->stop_reason);
  __pyx_r = __pyx_v_self->stop_reason;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_11stop_reason_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_11stop_reason_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_11stop_reason_2__set__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_11stop_reason_2__set__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(PyUnicode_CheckExact(__pyx_v_value))||((__pyx_v_value) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "unicode", Py_TYPE(__pyx_v_value)->tp_name), 0))) __PYX_ERR(1, 18, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->stop_reason);
  __Pyx_DECREF(__pyx_v_self->stop_reason);
  __pyx_v_self->stop_reason = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsTrace.stop_reason.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_11stop_reason_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_11stop_reason_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_11stop_reason_4__del__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_11stop_reason_4__del__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->stop_reason);
  __Pyx_DECREF(__pyx_v_self->stop_reason);
  __pyx_v_self->stop_reason = ((PyObject*)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "traceutils/scamper/warts.pxd":19
 *     cdef public int icmp_sum
 *     cdef public str stop_reason
 *     cdef public int stop_data             # <<<<<<<<<<<<<<
 *     cdef public dict start
 *     cdef public int hop_count
 */

/* Python wrapper */
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_9stop_data_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_9stop_data_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_9stop_data___get__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_9stop_data___get__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->stop_data); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 19, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsTrace.stop_data.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_9stop_data_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_9stop_data_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_9stop_data_2__set__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_9stop_data_2__set__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyInt_As_int(__pyx_v_value); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 19, __pyx_L1_error)
  __pyx_v_self->stop_data = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsTrace.stop_data.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "traceutils/scamper/warts.pxd":20
 *     cdef public str stop_reason
 *     cdef public int stop_data
 *     cdef public dict start             # <<<<<<<<<<<<<<
 *     cdef public int hop_count
 *     cdef public int attempts
 */

/* Python wrapper */
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_5start_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_5start_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_5start___get__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_5start___get__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->start);
  __pyx_r = __pyx_v_self->start;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_5start_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_5start_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_5start_2__set__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_5start_2__set__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(PyDict_CheckExact(__pyx_v_value))||((__pyx_v_value) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_v_value)->tp_name), 0))) __PYX_ERR(1, 20, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->start);
  __Pyx_DECREF(__pyx_v_self->start);
  __pyx_v_self->start = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsTrace.start.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_5start_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_5start_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_5start_4__del__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_5start_4__del__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->start);
  __Pyx_DECREF(__pyx_v_self->start);
  __pyx_v_self->start = ((PyObject*)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "traceutils/scamper/warts.pxd":21
 *     cdef public int stop_data
 *     cdef public dict start
 *     cdef public int hop_count             # <<<<<<<<<<<<<<
 *     cdef public int attempts
 *     cdef public unsigned char hoplimit
 */

/* Python wrapper */
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_9hop_count_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_9hop_count_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_9hop_count___get__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_9hop_count___get__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->hop_count); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 21, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsTrace.hop_count.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_9hop_count_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_9hop_count_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_9hop_count_2__set__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_9hop_count_2__set__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyInt_As_int(__pyx_v_value); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 21, __pyx_L1_error)
  __pyx_v_self->hop_count = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsTrace.hop_count.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "traceutils/scamper/warts.pxd":22
 *     cdef public dict start
 *     cdef public int hop_count
 *     cdef public int attempts             # <<<<<<<<<<<<<<
 *     cdef public unsigned char hoplimit
 *     cdef public unsigned char firsthop
 */

/* Python wrapper */
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_8attempts_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_8attempts_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_8attempts___get__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_8attempts___get__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->attempts); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 22, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsTrace.attempts.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_8attempts_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_8attempts_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_8attempts_2__set__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_8attempts_2__set__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyInt_As_int(__pyx_v_value); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 22, __pyx_L1_error)
  __pyx_v_self->attempts = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsTrace.attempts.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "traceutils/scamper/warts.pxd":23
 *     cdef public int hop_count
 *     cdef public int attempts
 *     cdef public unsigned char hoplimit             # <<<<<<<<<<<<<<
 *     cdef public unsigned char firsthop
 *     cdef public double wait
 */

/* Python wrapper */
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_8hoplimit_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_8hoplimit_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_8hoplimit___get__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_8hoplimit___get__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_unsigned_char(__pyx_v_self->hoplimit); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 23, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsTrace.hoplimit.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_8hoplimit_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_8hoplimit_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_8hoplimit_2__set__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_8hoplimit_2__set__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  unsigned char __pyx_t_1;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyInt_As_unsigned_char(__pyx_v_value); if (unlikely((__pyx_t_1 == (unsigned char)-1) && PyErr_Occurred())) __PYX_ERR(1, 23, __pyx_L1_error)
  __pyx_v_self->hoplimit = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsTrace.hoplimit.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "traceutils/scamper/warts.pxd":24
 *     cdef public int attempts
 *     cdef public unsigned char hoplimit
 *     cdef public unsigned char firsthop             # <<<<<<<<<<<<<<
 *     cdef public double wait
 *     cdef public int wait_probe
 */

/* Python wrapper */
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_8firsthop_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_8firsthop_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_8firsthop___get__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_8firsthop___get__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_unsigned_char(__pyx_v_self->firsthop); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 24, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsTrace.firsthop.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_8firsthop_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_8firsthop_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_8firsthop_2__set__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_8firsthop_2__set__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  unsigned char __pyx_t_1;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyInt_As_unsigned_char(__pyx_v_value); if (unlikely((__pyx_t_1 == (unsigned char)-1) && PyErr_Occurred())) __PYX_ERR(1, 24, __pyx_L1_error)
  __pyx_v_self->firsthop = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsTrace.firsthop.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "traceutils/scamper/warts.pxd":25
 *     cdef public unsigned char hoplimit
 *     cdef public unsigned char firsthop
 *     cdef public double wait             # <<<<<<<<<<<<<<
 *     cdef public int wait_probe
 *     cdef public int tos
 */

/* Python wrapper */
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_4wait_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_4wait_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_4wait___get__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_4wait___get__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->wait); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 25, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsTrace.wait.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_4wait_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_4wait_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_4wait_2__set__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_4wait_2__set__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  double __pyx_t_1;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __pyx_PyFloat_AsDouble(__pyx_v_value); if (unlikely((__pyx_t_1 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 25, __pyx_L1_error)
  __pyx_v_self->wait = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsTrace.wait.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "traceutils/scamper/warts.pxd":26
 *     cdef public unsigned char firsthop
 *     cdef public double wait
 *     cdef public int wait_probe             # <<<<<<<<<<<<<<
 *     cdef public int tos
 *     cdef public unsigned char probe_size
 */

/* Python wrapper */
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_10wait_probe_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_10wait_probe_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_10wait_probe___get__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_10wait_probe___get__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->wait_probe); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 26, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsTrace.wait_probe.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_10wait_probe_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_10wait_probe_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_10wait_probe_2__set__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_10wait_probe_2__set__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyInt_As_int(__pyx_v_value); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 26, __pyx_L1_error)
  __pyx_v_self->wait_probe = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsTrace.wait_probe.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "traceutils/scamper/warts.pxd":27
 *     cdef public double wait
 *     cdef public int wait_probe
 *     cdef public int tos             # <<<<<<<<<<<<<<
 *     cdef public unsigned char probe_size
 *     cdef public unsigned char probe_count
 */

/* Python wrapper */
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_3tos_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_3tos_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_3tos___get__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_3tos___get__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->tos); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 27, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsTrace.tos.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_3tos_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_3tos_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_3tos_2__set__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_3tos_2__set__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyInt_As_int(__pyx_v_value); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 27, __pyx_L1_error)
  __pyx_v_self->tos = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsTrace.tos.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "traceutils/scamper/warts.pxd":28
 *     cdef public int wait_probe
 *     cdef public int tos
 *     cdef public unsigned char probe_size             # <<<<<<<<<<<<<<
 *     cdef public unsigned char probe_count
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_10probe_size_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_10probe_size_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_10probe_size___get__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_10probe_size___get__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_unsigned_char(__pyx_v_self->probe_size); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 28, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsTrace.probe_size.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_10probe_size_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_10probe_size_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_10probe_size_2__set__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_10probe_size_2__set__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  unsigned char __pyx_t_1;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyInt_As_unsigned_char(__pyx_v_value); if (unlikely((__pyx_t_1 == (unsigned char)-1) && PyErr_Occurred())) __PYX_ERR(1, 28, __pyx_L1_error)
  __pyx_v_self->probe_size = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsTrace.probe_size.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "traceutils/scamper/warts.pxd":29
 *     cdef public int tos
 *     cdef public unsigned char probe_size
 *     cdef public unsigned char probe_count             # <<<<<<<<<<<<<<
 * 
 * cdef class WartsPing:
 */

/* Python wrapper */
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_11probe_count_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_11probe_count_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_11probe_count___get__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_11probe_count___get__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_unsigned_char(__pyx_v_self->probe_count); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 29, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsTrace.probe_count.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_11probe_count_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_11probe_count_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_11probe_count_2__set__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_11probe_count_2__set__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  unsigned char __pyx_t_1;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyInt_As_unsigned_char(__pyx_v_value); if (unlikely((__pyx_t_1 == (unsigned char)-1) && PyErr_Occurred())) __PYX_ERR(1, 29, __pyx_L1_error)
  __pyx_v_self->probe_count = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsTrace.probe_count.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_3__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_10traceutils_7scamper_5warts_10WartsTrace_2__reduce_cython__[] = "WartsTrace.__reduce_cython__(self)";
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_3__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_2__reduce_cython__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_2__reduce_cython__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  PyObject *__pyx_t_14 = NULL;
  int __pyx_t_15;
  int __pyx_t_16;
  int __pyx_t_17;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self.attempts, self.dst, self.family, self.firsthop, self.hop_count, self.hoplimit, self.hops, self.icmp_sum, self.loop, self.method, self.probe_count, self.probe_size, self.src, self.start, self.stop_data, self.stop_reason, self.tos, self.type, self.userid, self.version, self.wait, self.wait_probe)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->attempts); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_self->__pyx_base.family); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_From_unsigned_char(__pyx_v_self->firsthop); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyInt_From_int(__pyx_v_self->hop_count); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyInt_From_unsigned_char(__pyx_v_self->hoplimit); if (unlikely(!__pyx_t_5)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = __Pyx_PyInt_From_int(__pyx_v_self->icmp_sum); if (unlikely(!__pyx_t_6)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_7 = __Pyx_PyInt_From_unsigned_char(__pyx_v_self->probe_count); if (unlikely(!__pyx_t_7)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_8 = __Pyx_PyInt_From_unsigned_char(__pyx_v_self->probe_size); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_9 = __Pyx_PyInt_From_int(__pyx_v_self->stop_data); if (unlikely(!__pyx_t_9)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __pyx_t_10 = __Pyx_PyInt_From_int(__pyx_v_self->tos); if (unlikely(!__pyx_t_10)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_11 = __Pyx_PyInt_From_int(__pyx_v_self->userid); if (unlikely(!__pyx_t_11)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __pyx_t_12 = PyFloat_FromDouble(__pyx_v_self->wait); if (unlikely(!__pyx_t_12)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_t_13 = __Pyx_PyInt_From_int(__pyx_v_self->wait_probe); if (unlikely(!__pyx_t_13)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);
  __pyx_t_14 = PyTuple_New(22); if (unlikely(!__pyx_t_14)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_14);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_14, 0, __pyx_t_1);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.dst);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.dst);
  PyTuple_SET_ITEM(__pyx_t_14, 1, __pyx_v_self->__pyx_base.dst);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_14, 2, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_14, 3, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_14, 4, __pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_14, 5, __pyx_t_5);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.hops);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.hops);
  PyTuple_SET_ITEM(__pyx_t_14, 6, __pyx_v_self->__pyx_base.hops);
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_14, 7, __pyx_t_6);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.loop);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.loop);
  PyTuple_SET_ITEM(__pyx_t_14, 8, __pyx_v_self->__pyx_base.loop);
  __Pyx_INCREF(__pyx_v_self->method);
  __Pyx_GIVEREF(__pyx_v_self->method);
  PyTuple_SET_ITEM(__pyx_t_14, 9, __pyx_v_self->method);
  __Pyx_GIVEREF(__pyx_t_7);
  PyTuple_SET_ITEM(__pyx_t_14, 10, __pyx_t_7);
  __Pyx_GIVEREF(__pyx_t_8);
  PyTuple_SET_ITEM(__pyx_t_14, 11, __pyx_t_8);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.src);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.src);
  PyTuple_SET_ITEM(__pyx_t_14, 12, __pyx_v_self->__pyx_base.src);
  __Pyx_INCREF(__pyx_v_self->start);
  __Pyx_GIVEREF(__pyx_v_self->start);
  PyTuple_SET_ITEM(__pyx_t_14, 13, __pyx_v_self->start);
  __Pyx_GIVEREF(__pyx_t_9);
  PyTuple_SET_ITEM(__pyx_t_14, 14, __pyx_t_9);
  __Pyx_INCREF(__pyx_v_self->stop_reason);
  __Pyx_GIVEREF(__pyx_v_self->stop_reason);
  PyTuple_SET_ITEM(__pyx_t_14, 15, __pyx_v_self->stop_reason);
  __Pyx_GIVEREF(__pyx_t_10);
  PyTuple_SET_ITEM(__pyx_t_14, 16, __pyx_t_10);
  __Pyx_INCREF(__pyx_v_self->type);
  __Pyx_GIVEREF(__pyx_v_self->type);
  PyTuple_SET_ITEM(__pyx_t_14, 17, __pyx_v_self->type);
  __Pyx_GIVEREF(__pyx_t_11);
  PyTuple_SET_ITEM(__pyx_t_14, 18, __pyx_t_11);
  __Pyx_INCREF(__pyx_v_self->version);
  __Pyx_GIVEREF(__pyx_v_self->version);
  PyTuple_SET_ITEM(__pyx_t_14, 19, __pyx_v_self->version);
  __Pyx_GIVEREF(__pyx_t_12);
  PyTuple_SET_ITEM(__pyx_t_14, 20, __pyx_t_12);
  __Pyx_GIVEREF(__pyx_t_13);
  PyTuple_SET_ITEM(__pyx_t_14, 21, __pyx_t_13);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_5 = 0;
  __pyx_t_6 = 0;
  __pyx_t_7 = 0;
  __pyx_t_8 = 0;
  __pyx_t_9 = 0;
  __pyx_t_10 = 0;
  __pyx_t_11 = 0;
  __pyx_t_12 = 0;
  __pyx_t_13 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_14);
  __pyx_t_14 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self.attempts, self.dst, self.family, self.firsthop, self.hop_count, self.hoplimit, self.hops, self.icmp_sum, self.loop, self.method, self.probe_count, self.probe_size, self.src, self.start, self.stop_data, self.stop_reason, self.tos, self.type, self.userid, self.version, self.wait, self.wait_probe)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_14 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_14)) __PYX_ERR(2, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_14);
  __pyx_v__dict = __pyx_t_14;
  __pyx_t_14 = 0;

  /* "(tree fragment)":7
 *     state = (self.attempts, self.dst, self.family, self.firsthop, self.hop_count, self.hoplimit, self.hops, self.icmp_sum, self.loop, self.method, self.probe_count, self.probe_size, self.src, self.start, self.stop_data, self.stop_reason, self.tos, self.type, self.userid, self.version, self.wait, self.wait_probe)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_15 = (__pyx_v__dict != Py_None);
  __pyx_t_16 = (__pyx_t_15 != 0);
  if (__pyx_t_16) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_14 = PyTuple_New(1); if (unlikely(!__pyx_t_14)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_14);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_14, 0, __pyx_v__dict);
    __pyx_t_13 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_14); if (unlikely(!__pyx_t_13)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_13);
    __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_13));
    __pyx_t_13 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self.dst is not None or self.hops is not None or self.loop is not None or self.method is not None or self.src is not None or self.start is not None or self.stop_reason is not None or self.type is not None or self.version is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self.attempts, self.dst, self.family, self.firsthop, self.hop_count, self.hoplimit, self.hops, self.icmp_sum, self.loop, self.method, self.probe_count, self.probe_size, self.src, self.start, self.stop_data, self.stop_reason, self.tos, self.type, self.userid, self.version, self.wait, self.wait_probe)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self.dst is not None or self.hops is not None or self.loop is not None or self.method is not None or self.src is not None or self.start is not None or self.stop_reason is not None or self.type is not None or self.version is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_WartsTrace, (type(self), 0xffb4da5, None), state
 */
  /*else*/ {
    __pyx_t_15 = (__pyx_v_self->__pyx_base.dst != ((PyObject*)Py_None));
    __pyx_t_17 = (__pyx_t_15 != 0);
    if (!__pyx_t_17) {
    } else {
      __pyx_t_16 = __pyx_t_17;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_17 = (__pyx_v_self->__pyx_base.hops != ((PyObject*)Py_None));
    __pyx_t_15 = (__pyx_t_17 != 0);
    if (!__pyx_t_15) {
    } else {
      __pyx_t_16 = __pyx_t_15;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_15 = (__pyx_v_self->__pyx_base.loop != ((PyObject*)Py_None));
    __pyx_t_17 = (__pyx_t_15 != 0);
    if (!__pyx_t_17) {
    } else {
      __pyx_t_16 = __pyx_t_17;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_17 = (__pyx_v_self->method != ((PyObject*)Py_None));
    __pyx_t_15 = (__pyx_t_17 != 0);
    if (!__pyx_t_15) {
    } else {
      __pyx_t_16 = __pyx_t_15;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_15 = (__pyx_v_self->__pyx_base.src != ((PyObject*)Py_None));
    __pyx_t_17 = (__pyx_t_15 != 0);
    if (!__pyx_t_17) {
    } else {
      __pyx_t_16 = __pyx_t_17;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_17 = (__pyx_v_self->start != ((PyObject*)Py_None));
    __pyx_t_15 = (__pyx_t_17 != 0);
    if (!__pyx_t_15) {
    } else {
      __pyx_t_16 = __pyx_t_15;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_15 = (__pyx_v_self->stop_reason != ((PyObject*)Py_None));
    __pyx_t_17 = (__pyx_t_15 != 0);
    if (!__pyx_t_17) {
    } else {
      __pyx_t_16 = __pyx_t_17;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_17 = (__pyx_v_self->type != ((PyObject*)Py_None));
    __pyx_t_15 = (__pyx_t_17 != 0);
    if (!__pyx_t_15) {
    } else {
      __pyx_t_16 = __pyx_t_15;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_15 = (__pyx_v_self->version != ((PyObject*)Py_None));
    __pyx_t_17 = (__pyx_t_15 != 0);
    __pyx_t_16 = __pyx_t_17;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_16;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.dst is not None or self.hops is not None or self.loop is not None or self.method is not None or self.src is not None or self.start is not None or self.stop_reason is not None or self.type is not None or self.version is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_WartsTrace, (type(self), 0xffb4da5, None), state
 *     else:
 */
  __pyx_t_16 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_16) {

    /* "(tree fragment)":13
 *         use_setstate = self.dst is not None or self.hops is not None or self.loop is not None or self.method is not None or self.src is not None or self.start is not None or self.stop_reason is not None or self.type is not None or self.version is not None
 *     if use_setstate:
 *         return __pyx_unpickle_WartsTrace, (type(self), 0xffb4da5, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_WartsTrace, (type(self), 0xffb4da5, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_13, __pyx_n_s_pyx_unpickle_WartsTrace); if (unlikely(!__pyx_t_13)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_13);
    __pyx_t_14 = PyTuple_New(3); if (unlikely(!__pyx_t_14)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_14);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_14, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_268127653);
    __Pyx_GIVEREF(__pyx_int_268127653);
    PyTuple_SET_ITEM(__pyx_t_14, 1, __pyx_int_268127653);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_14, 2, Py_None);
    __pyx_t_12 = PyTuple_New(3); if (unlikely(!__pyx_t_12)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_12);
    __Pyx_GIVEREF(__pyx_t_13);
    PyTuple_SET_ITEM(__pyx_t_12, 0, __pyx_t_13);
    __Pyx_GIVEREF(__pyx_t_14);
    PyTuple_SET_ITEM(__pyx_t_12, 1, __pyx_t_14);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_12, 2, __pyx_v_state);
    __pyx_t_13 = 0;
    __pyx_t_14 = 0;
    __pyx_r = __pyx_t_12;
    __pyx_t_12 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.dst is not None or self.hops is not None or self.loop is not None or self.method is not None or self.src is not None or self.start is not None or self.stop_reason is not None or self.type is not None or self.version is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_WartsTrace, (type(self), 0xffb4da5, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_WartsTrace, (type(self), 0xffb4da5, None), state
 *     else:
 *         return __pyx_unpickle_WartsTrace, (type(self), 0xffb4da5, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_WartsTrace__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_12, __pyx_n_s_pyx_unpickle_WartsTrace); if (unlikely(!__pyx_t_12)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_12);
    __pyx_t_14 = PyTuple_New(3); if (unlikely(!__pyx_t_14)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_14);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_14, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_268127653);
    __Pyx_GIVEREF(__pyx_int_268127653);
    PyTuple_SET_ITEM(__pyx_t_14, 1, __pyx_int_268127653);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_14, 2, __pyx_v_state);
    __pyx_t_13 = PyTuple_New(2); if (unlikely(!__pyx_t_13)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_13);
    __Pyx_GIVEREF(__pyx_t_12);
    PyTuple_SET_ITEM(__pyx_t_13, 0, __pyx_t_12);
    __Pyx_GIVEREF(__pyx_t_14);
    PyTuple_SET_ITEM(__pyx_t_13, 1, __pyx_t_14);
    __pyx_t_12 = 0;
    __pyx_t_14 = 0;
    __pyx_r = __pyx_t_13;
    __pyx_t_13 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_XDECREF(__pyx_t_14);
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsTrace.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_WartsTrace, (type(self), 0xffb4da5, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_WartsTrace__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_5__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static char __pyx_doc_10traceutils_7scamper_5warts_10WartsTrace_4__setstate_cython__[] = "WartsTrace.__setstate_cython__(self, __pyx_state)";
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_5__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_4__setstate_cython__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10traceutils_7scamper_5warts_10WartsTrace_4__setstate_cython__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_WartsTrace, (type(self), 0xffb4da5, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_WartsTrace__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(2, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_10traceutils_7scamper_5warts___pyx_unpickle_WartsTrace__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_WartsTrace, (type(self), 0xffb4da5, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_WartsTrace__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsTrace.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "traceutils/scamper/warts.pyx":49
 * cdef class WartsHop(Hop):
 * 
 *     def __init__(             # <<<<<<<<<<<<<<
 *             self, str addr='', unsigned char probe_ttl=0, int probe_id=-1, unsigned short probe_size=0,
 *             double rtt=-1, unsigned char reply_ttl=0, int reply_tos=-1, unsigned short reply_size=0,
 */

/* Python wrapper */
static int __pyx_pw_10traceutils_7scamper_5warts_8WartsHop_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_10traceutils_7scamper_5warts_8WartsHop_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_addr = 0;
  unsigned char __pyx_v_probe_ttl;
  int __pyx_v_probe_id;
  unsigned short __pyx_v_probe_size;
  double __pyx_v_rtt;
  unsigned char __pyx_v_reply_ttl;
  int __pyx_v_reply_tos;
  unsigned short __pyx_v_reply_size;
  int __pyx_v_reply_ipid;
  unsigned char __pyx_v_icmp_type;
  unsigned char __pyx_v_icmp_code;
  unsigned char __pyx_v_icmp_q_ttl;
  int __pyx_v_icmp_q_ipl;
  int __pyx_v_icmp_q_tos;
  PyObject *__pyx_v_icmpext = 0;
  int __pyx_v_icmp_nhmtu;
  int __pyx_v_family;
  CYTHON_UNUSED PyObject *__pyx_v_kwargs = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  __pyx_v_kwargs = PyDict_New(); if (unlikely(!__pyx_v_kwargs)) return -1;
  __Pyx_GOTREF(__pyx_v_kwargs);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_addr,&__pyx_n_s_probe_ttl,&__pyx_n_s_probe_id,&__pyx_n_s_probe_size,&__pyx_n_s_rtt,&__pyx_n_s_reply_ttl,&__pyx_n_s_reply_tos,&__pyx_n_s_reply_size,&__pyx_n_s_reply_ipid,&__pyx_n_s_icmp_type,&__pyx_n_s_icmp_code,&__pyx_n_s_icmp_q_ttl,&__pyx_n_s_icmp_q_ipl,&__pyx_n_s_icmp_q_tos,&__pyx_n_s_icmpext,&__pyx_n_s_icmp_nhmtu,&__pyx_n_s_family,0};
    PyObject* values[17] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
    values[0] = ((PyObject*)__pyx_kp_u_);

    /* "traceutils/scamper/warts.pyx":53
 *             double rtt=-1, unsigned char reply_ttl=0, int reply_tos=-1, unsigned short reply_size=0,
 *             int reply_ipid=-1, unsigned char icmp_type=0, unsigned char icmp_code=0, unsigned char icmp_q_ttl=1,
 *             int icmp_q_ipl=-1, int icmp_q_tos=-1, list icmpext=None, int icmp_nhmtu=-1, int family=0, **kwargs             # <<<<<<<<<<<<<<
 *     ):
 *         self.addr = addr
 */
    values[14] = ((PyObject*)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case 17: values[16] = PyTuple_GET_ITEM(__pyx_args, 16);
        CYTHON_FALLTHROUGH;
        case 16: values[15] = PyTuple_GET_ITEM(__pyx_args, 15);
        CYTHON_FALLTHROUGH;
        case 15: values[14] = PyTuple_GET_ITEM(__pyx_args, 14);
        CYTHON_FALLTHROUGH;
        case 14: values[13] = PyTuple_GET_ITEM(__pyx_args, 13);
        CYTHON_FALLTHROUGH;
        case 13: values[12] = PyTuple_GET_ITEM(__pyx_args, 12);
        CYTHON_FALLTHROUGH;
        case 12: values[11] = PyTuple_GET_ITEM(__pyx_args, 11);
        CYTHON_FALLTHROUGH;
        case 11: values[10] = PyTuple_GET_ITEM(__pyx_args, 10);
        CYTHON_FALLTHROUGH;
        case 10: values[9] = PyTuple_GET_ITEM(__pyx_args, 9);
        CYTHON_FALLTHROUGH;
        case  9: values[8] = PyTuple_GET_ITEM(__pyx_args, 8);
        CYTHON_FALLTHROUGH;
        case  8: values[7] = PyTuple_GET_ITEM(__pyx_args, 7);
        CYTHON_FALLTHROUGH;
        case  7: values[6] = PyTuple_GET_ITEM(__pyx_args, 6);
        CYTHON_FALLTHROUGH;
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        CYTHON_FALLTHROUGH;
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_addr);
          if (value) { values[0] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_probe_ttl);
          if (value) { values[1] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_probe_id);
          if (value) { values[2] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_probe_size);
          if (value) { values[3] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  4:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_rtt);
          if (value) { values[4] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  5:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_reply_ttl);
          if (value) { values[5] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  6:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_reply_tos);
          if (value) { values[6] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  7:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_reply_size);
          if (value) { values[7] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  8:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_reply_ipid);
          if (value) { values[8] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  9:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_icmp_type);
          if (value) { values[9] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case 10:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_icmp_code);
          if (value) { values[10] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case 11:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_icmp_q_ttl);
          if (value) { values[11] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case 12:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_icmp_q_ipl);
          if (value) { values[12] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case 13:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_icmp_q_tos);
          if (value) { values[13] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case 14:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_icmpext);
          if (value) { values[14] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case 15:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_icmp_nhmtu);
          if (value) { values[15] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case 16:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_family);
          if (value) { values[16] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, __pyx_v_kwargs, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 49, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case 17: values[16] = PyTuple_GET_ITEM(__pyx_args, 16);
        CYTHON_FALLTHROUGH;
        case 16: values[15] = PyTuple_GET_ITEM(__pyx_args, 15);
        CYTHON_FALLTHROUGH;
        case 15: values[14] = PyTuple_GET_ITEM(__pyx_args, 14);
        CYTHON_FALLTHROUGH;
        case 14: values[13] = PyTuple_GET_ITEM(__pyx_args, 13);
        CYTHON_FALLTHROUGH;
        case 13: values[12] = PyTuple_GET_ITEM(__pyx_args, 12);
        CYTHON_FALLTHROUGH;
        case 12: values[11] = PyTuple_GET_ITEM(__pyx_args, 11);
        CYTHON_FALLTHROUGH;
        case 11: values[10] = PyTuple_GET_ITEM(__pyx_args, 10);
        CYTHON_FALLTHROUGH;
        case 10: values[9] = PyTuple_GET_ITEM(__pyx_args, 9);
        CYTHON_FALLTHROUGH;
        case  9: values[8] = PyTuple_GET_ITEM(__pyx_args, 8);
        CYTHON_FALLTHROUGH;
        case  8: values[7] = PyTuple_GET_ITEM(__pyx_args, 7);
        CYTHON_FALLTHROUGH;
        case  7: values[6] = PyTuple_GET_ITEM(__pyx_args, 6);
        CYTHON_FALLTHROUGH;
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        CYTHON_FALLTHROUGH;
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_addr = ((PyObject*)values[0]);
    if (values[1]) {
      __pyx_v_probe_ttl = __Pyx_PyInt_As_unsigned_char(values[1]); if (unlikely((__pyx_v_probe_ttl == (unsigned char)-1) && PyErr_Occurred())) __PYX_ERR(0, 50, __pyx_L3_error)
    } else {
      __pyx_v_probe_ttl = ((unsigned char)0);
    }
    if (values[2]) {
      __pyx_v_probe_id = __Pyx_PyInt_As_int(values[2]); if (unlikely((__pyx_v_probe_id == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 50, __pyx_L3_error)
    } else {
      __pyx_v_probe_id = ((int)-1);
    }
    if (values[3]) {
      __pyx_v_probe_size = __Pyx_PyInt_As_unsigned_short(values[3]); if (unlikely((__pyx_v_probe_size == (unsigned short)-1) && PyErr_Occurred())) __PYX_ERR(0, 50, __pyx_L3_error)
    } else {
      __pyx_v_probe_size = ((unsigned short)0);
    }
    if (values[4]) {
      __pyx_v_rtt = __pyx_PyFloat_AsDouble(values[4]); if (unlikely((__pyx_v_rtt == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 51, __pyx_L3_error)
    } else {
      __pyx_v_rtt = ((double)-1.0);
    }
    if (values[5]) {
      __pyx_v_reply_ttl = __Pyx_PyInt_As_unsigned_char(values[5]); if (unlikely((__pyx_v_reply_ttl == (unsigned char)-1) && PyErr_Occurred())) __PYX_ERR(0, 51, __pyx_L3_error)
    } else {
      __pyx_v_reply_ttl = ((unsigned char)0);
    }
    if (values[6]) {
      __pyx_v_reply_tos = __Pyx_PyInt_As_int(values[6]); if (unlikely((__pyx_v_reply_tos == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 51, __pyx_L3_error)
    } else {
      __pyx_v_reply_tos = ((int)-1);
    }
    if (values[7]) {
      __pyx_v_reply_size = __Pyx_PyInt_As_unsigned_short(values[7]); if (unlikely((__pyx_v_reply_size == (unsigned short)-1) && PyErr_Occurred())) __PYX_ERR(0, 51, __pyx_L3_error)
    } else {
      __pyx_v_reply_size = ((unsigned short)0);
    }
    if (values[8]) {
      __pyx_v_reply_ipid = __Pyx_PyInt_As_int(values[8]); if (unlikely((__pyx_v_reply_ipid == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 52, __pyx_L3_error)
    } else {
      __pyx_v_reply_ipid = ((int)-1);
    }
    if (values[9]) {
      __pyx_v_icmp_type = __Pyx_PyInt_As_unsigned_char(values[9]); if (unlikely((__pyx_v_icmp_type == (unsigned char)-1) && PyErr_Occurred())) __PYX_ERR(0, 52, __pyx_L3_error)
    } else {
      __pyx_v_icmp_type = ((unsigned char)0);
    }
    if (values[10]) {
      __pyx_v_icmp_code = __Pyx_PyInt_As_unsigned_char(values[10]); if (unlikely((__pyx_v_icmp_code == (unsigned char)-1) && PyErr_Occurred())) __PYX_ERR(0, 52, __pyx_L3_error)
    } else {
      __pyx_v_icmp_code = ((unsigned char)0);
    }
    if (values[11]) {
      __pyx_v_icmp_q_ttl = __Pyx_PyInt_As_unsigned_char(values[11]); if (unlikely((__pyx_v_icmp_q_ttl == (unsigned char)-1) && PyErr_Occurred())) __PYX_ERR(0, 52, __pyx_L3_error)
    } else {
      __pyx_v_icmp_q_ttl = ((unsigned char)1);
    }
    if (values[12]) {
      __pyx_v_icmp_q_ipl = __Pyx_PyInt_As_int(values[12]); if (unlikely((__pyx_v_icmp_q_ipl == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 53, __pyx_L3_error)
    } else {
      __pyx_v_icmp_q_ipl = ((int)-1);
    }
    if (values[13]) {
      __pyx_v_icmp_q_tos = __Pyx_PyInt_As_int(values[13]); if (unlikely((__pyx_v_icmp_q_tos == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 53, __pyx_L3_error)
    } else {
      __pyx_v_icmp_q_tos = ((int)-1);
    }
    __pyx_v_icmpext = ((PyObject*)values[14]);
    if (values[15]) {
      __pyx_v_icmp_nhmtu = __Pyx_PyInt_As_int(values[15]); if (unlikely((__pyx_v_icmp_nhmtu == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 53, __pyx_L3_error)
    } else {
      __pyx_v_icmp_nhmtu = ((int)-1);
    }
    if (values[16]) {
      __pyx_v_family = __Pyx_PyInt_As_int(values[16]); if (unlikely((__pyx_v_family == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 53, __pyx_L3_error)
    } else {
      __pyx_v_family = ((int)0);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 0, 0, 17, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 49, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_DECREF(__pyx_v_kwargs); __pyx_v_kwargs = 0;
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsHop.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_addr), (&PyUnicode_Type), 1, "addr", 1))) __PYX_ERR(0, 50, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_icmpext), (&PyList_Type), 1, "icmpext", 1))) __PYX_ERR(0, 53, __pyx_L1_error)
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_8WartsHop___init__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsHop *)__pyx_v_self), __pyx_v_addr, __pyx_v_probe_ttl, __pyx_v_probe_id, __pyx_v_probe_size, __pyx_v_rtt, __pyx_v_reply_ttl, __pyx_v_reply_tos, __pyx_v_reply_size, __pyx_v_reply_ipid, __pyx_v_icmp_type, __pyx_v_icmp_code, __pyx_v_icmp_q_ttl, __pyx_v_icmp_q_ipl, __pyx_v_icmp_q_tos, __pyx_v_icmpext, __pyx_v_icmp_nhmtu, __pyx_v_family, __pyx_v_kwargs);

  /* "traceutils/scamper/warts.pyx":49
 * cdef class WartsHop(Hop):
 * 
 *     def __init__(             # <<<<<<<<<<<<<<
 *             self, str addr='', unsigned char probe_ttl=0, int probe_id=-1, unsigned short probe_size=0,
 *             double rtt=-1, unsigned char reply_ttl=0, int reply_tos=-1, unsigned short reply_size=0,
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_kwargs);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_10traceutils_7scamper_5warts_8WartsHop___init__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsHop *__pyx_v_self, PyObject *__pyx_v_addr, unsigned char __pyx_v_probe_ttl, int __pyx_v_probe_id, unsigned short __pyx_v_probe_size, double __pyx_v_rtt, unsigned char __pyx_v_reply_ttl, int __pyx_v_reply_tos, unsigned short __pyx_v_reply_size, int __pyx_v_reply_ipid, unsigned char __pyx_v_icmp_type, unsigned char __pyx_v_icmp_code, unsigned char __pyx_v_icmp_q_ttl, int __pyx_v_icmp_q_ipl, int __pyx_v_icmp_q_tos, PyObject *__pyx_v_icmpext, int __pyx_v_icmp_nhmtu, int __pyx_v_family, CYTHON_UNUSED PyObject *__pyx_v_kwargs) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  enum __pyx_t_10traceutils_7scamper_3hop_ICMPType __pyx_t_3;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "traceutils/scamper/warts.pyx":55
 *             int icmp_q_ipl=-1, int icmp_q_tos=-1, list icmpext=None, int icmp_nhmtu=-1, int family=0, **kwargs
 *     ):
 *         self.addr = addr             # <<<<<<<<<<<<<<
 *         self.probe_ttl = probe_ttl
 *         self.rtt = rtt
 */
  __Pyx_INCREF(__pyx_v_addr);
  __Pyx_GIVEREF(__pyx_v_addr);
  __Pyx_GOTREF(__pyx_v_self->__pyx_base.addr);
  __Pyx_DECREF(__pyx_v_self->__pyx_base.addr);
  __pyx_v_self->__pyx_base.addr = __pyx_v_addr;

  /* "traceutils/scamper/warts.pyx":56
 *     ):
 *         self.addr = addr
 *         self.probe_ttl = probe_ttl             # <<<<<<<<<<<<<<
 *         self.rtt = rtt
 *         self.reply_ttl = reply_ttl
 */
  __pyx_v_self->__pyx_base.probe_ttl = __pyx_v_probe_ttl;

  /* "traceutils/scamper/warts.pyx":57
 *         self.addr = addr
 *         self.probe_ttl = probe_ttl
 *         self.rtt = rtt             # <<<<<<<<<<<<<<
 *         self.reply_ttl = reply_ttl
 *         self.reply_tos = reply_tos
 */
  __pyx_v_self->__pyx_base.rtt = __pyx_v_rtt;

  /* "traceutils/scamper/warts.pyx":58
 *         self.probe_ttl = probe_ttl
 *         self.rtt = rtt
 *         self.reply_ttl = reply_ttl             # <<<<<<<<<<<<<<
 *         self.reply_tos = reply_tos
 *         self.reply_size = reply_size
 */
  __pyx_v_self->__pyx_base.reply_ttl = __pyx_v_reply_ttl;

  /* "traceutils/scamper/warts.pyx":59
 *         self.rtt = rtt
 *         self.reply_ttl = reply_ttl
 *         self.reply_tos = reply_tos             # <<<<<<<<<<<<<<
 *         self.reply_size = reply_size
 *         self.icmp_type = icmp_type
 */
  __pyx_v_self->__pyx_base.reply_tos = __pyx_v_reply_tos;

  /* "traceutils/scamper/warts.pyx":60
 *         self.reply_ttl = reply_ttl
 *         self.reply_tos = reply_tos
 *         self.reply_size = reply_size             # <<<<<<<<<<<<<<
 *         self.icmp_type = icmp_type
 *         self.icmp_code = icmp_code
 */
  __pyx_v_self->__pyx_base.reply_size = __pyx_v_reply_size;

  /* "traceutils/scamper/warts.pyx":61
 *         self.reply_tos = reply_tos
 *         self.reply_size = reply_size
 *         self.icmp_type = icmp_type             # <<<<<<<<<<<<<<
 *         self.icmp_code = icmp_code
 *         self.icmp_q_ttl = icmp_q_ttl
 */
  __pyx_v_self->__pyx_base.icmp_type = __pyx_v_icmp_type;

  /* "traceutils/scamper/warts.pyx":62
 *         self.reply_size = reply_size
 *         self.icmp_type = icmp_type
 *         self.icmp_code = icmp_code             # <<<<<<<<<<<<<<
 *         self.icmp_q_ttl = icmp_q_ttl
 *         self.icmp_q_tos = icmp_q_tos
 */
  __pyx_v_self->__pyx_base.icmp_code = __pyx_v_icmp_code;

  /* "traceutils/scamper/warts.pyx":63
 *         self.icmp_type = icmp_type
 *         self.icmp_code = icmp_code
 *         self.icmp_q_ttl = icmp_q_ttl             # <<<<<<<<<<<<<<
 *         self.icmp_q_tos = icmp_q_tos
 *         self.icmpext = icmpext
 */
  __pyx_v_self->__pyx_base.icmp_q_ttl = __pyx_v_icmp_q_ttl;

  /* "traceutils/scamper/warts.pyx":64
 *         self.icmp_code = icmp_code
 *         self.icmp_q_ttl = icmp_q_ttl
 *         self.icmp_q_tos = icmp_q_tos             # <<<<<<<<<<<<<<
 *         self.icmpext = icmpext
 *         if icmpext:
 */
  __pyx_v_self->__pyx_base.icmp_q_tos = __pyx_v_icmp_q_tos;

  /* "traceutils/scamper/warts.pyx":65
 *         self.icmp_q_ttl = icmp_q_ttl
 *         self.icmp_q_tos = icmp_q_tos
 *         self.icmpext = icmpext             # <<<<<<<<<<<<<<
 *         if icmpext:
 *             self.ismpls = 'mpls_labels' in icmpext[0]
 */
  __Pyx_INCREF(__pyx_v_icmpext);
  __Pyx_GIVEREF(__pyx_v_icmpext);
  __Pyx_GOTREF(__pyx_v_self->icmpext);
  __Pyx_DECREF(__pyx_v_self->icmpext);
  __pyx_v_self->icmpext = __pyx_v_icmpext;

  /* "traceutils/scamper/warts.pyx":66
 *         self.icmp_q_tos = icmp_q_tos
 *         self.icmpext = icmpext
 *         if icmpext:             # <<<<<<<<<<<<<<
 *             self.ismpls = 'mpls_labels' in icmpext[0]
 *         else:
 */
  __pyx_t_1 = (__pyx_v_icmpext != Py_None)&&(PyList_GET_SIZE(__pyx_v_icmpext) != 0);
  if (__pyx_t_1) {

    /* "traceutils/scamper/warts.pyx":67
 *         self.icmpext = icmpext
 *         if icmpext:
 *             self.ismpls = 'mpls_labels' in icmpext[0]             # <<<<<<<<<<<<<<
 *         else:
 *             self.ismpls = False
 */
    if (unlikely(__pyx_v_icmpext == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(0, 67, __pyx_L1_error)
    }
    __pyx_t_2 = __Pyx_GetItemInt_List(__pyx_v_icmpext, 0, long, 1, __Pyx_PyInt_From_long, 1, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 67, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = (__Pyx_PySequence_ContainsTF(__pyx_n_u_mpls_labels, __pyx_t_2, Py_EQ)); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 67, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_v_self->__pyx_base.ismpls = __pyx_t_1;

    /* "traceutils/scamper/warts.pyx":66
 *         self.icmp_q_tos = icmp_q_tos
 *         self.icmpext = icmpext
 *         if icmpext:             # <<<<<<<<<<<<<<
 *             self.ismpls = 'mpls_labels' in icmpext[0]
 *         else:
 */
    goto __pyx_L3;
  }

  /* "traceutils/scamper/warts.pyx":69
 *             self.ismpls = 'mpls_labels' in icmpext[0]
 *         else:
 *             self.ismpls = False             # <<<<<<<<<<<<<<
 *         # self.num = iptoint(addr)
 * 
 */
  /*else*/ {
    __pyx_v_self->__pyx_base.ismpls = 0;
  }
  __pyx_L3:;

  /* "traceutils/scamper/warts.pyx":72
 *         # self.num = iptoint(addr)
 * 
 *         self.probe_id = probe_id             # <<<<<<<<<<<<<<
 *         self.probe_size = probe_size
 *         self.reply_ipid = reply_ipid
 */
  __pyx_v_self->probe_id = __pyx_v_probe_id;

  /* "traceutils/scamper/warts.pyx":73
 * 
 *         self.probe_id = probe_id
 *         self.probe_size = probe_size             # <<<<<<<<<<<<<<
 *         self.reply_ipid = reply_ipid
 *         self.icmp_q_ipl = icmp_q_ipl
 */
  __pyx_v_self->probe_size = __pyx_v_probe_size;

  /* "traceutils/scamper/warts.pyx":74
 *         self.probe_id = probe_id
 *         self.probe_size = probe_size
 *         self.reply_ipid = reply_ipid             # <<<<<<<<<<<<<<
 *         self.icmp_q_ipl = icmp_q_ipl
 *         self.icmp_nhmtu = icmp_nhmtu
 */
  __pyx_v_self->reply_ipid = __pyx_v_reply_ipid;

  /* "traceutils/scamper/warts.pyx":75
 *         self.probe_size = probe_size
 *         self.reply_ipid = reply_ipid
 *         self.icmp_q_ipl = icmp_q_ipl             # <<<<<<<<<<<<<<
 *         self.icmp_nhmtu = icmp_nhmtu
 *         self.family = family
 */
  __pyx_v_self->icmp_q_ipl = __pyx_v_icmp_q_ipl;

  /* "traceutils/scamper/warts.pyx":76
 *         self.reply_ipid = reply_ipid
 *         self.icmp_q_ipl = icmp_q_ipl
 *         self.icmp_nhmtu = icmp_nhmtu             # <<<<<<<<<<<<<<
 *         self.family = family
 *         self.type = gettype(family, icmp_type)
 */
  __pyx_v_self->icmp_nhmtu = __pyx_v_icmp_nhmtu;

  /* "traceutils/scamper/warts.pyx":77
 *         self.icmp_q_ipl = icmp_q_ipl
 *         self.icmp_nhmtu = icmp_nhmtu
 *         self.family = family             # <<<<<<<<<<<<<<
 *         self.type = gettype(family, icmp_type)
 * 
 */
  __pyx_v_self->__pyx_base.family = __pyx_v_family;

  /* "traceutils/scamper/warts.pyx":78
 *         self.icmp_nhmtu = icmp_nhmtu
 *         self.family = family
 *         self.type = gettype(family, icmp_type)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_3 = __pyx_f_10traceutils_7scamper_3hop_gettype(__pyx_v_family, __pyx_v_icmp_type, 0); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 78, __pyx_L1_error)
  __pyx_v_self->__pyx_base.type = __pyx_t_3;

  /* "traceutils/scamper/warts.pyx":49
 * cdef class WartsHop(Hop):
 * 
 *     def __init__(             # <<<<<<<<<<<<<<
 *             self, str addr='', unsigned char probe_ttl=0, int probe_id=-1, unsigned short probe_size=0,
 *             double rtt=-1, unsigned char reply_ttl=0, int reply_tos=-1, unsigned short reply_size=0,
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsHop.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "traceutils/scamper/warts.pxd":4
 * 
 * cdef class WartsHop(Hop):
 *     cdef public list icmpext             # <<<<<<<<<<<<<<
 *     cdef public int probe_id
 *     cdef public unsigned short probe_size
 */

/* Python wrapper */
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_8WartsHop_7icmpext_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_8WartsHop_7icmpext_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_8WartsHop_7icmpext___get__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsHop *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10traceutils_7scamper_5warts_8WartsHop_7icmpext___get__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsHop *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->icmpext);
  __pyx_r = __pyx_v_self->icmpext;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_10traceutils_7scamper_5warts_8WartsHop_7icmpext_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_10traceutils_7scamper_5warts_8WartsHop_7icmpext_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_8WartsHop_7icmpext_2__set__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsHop *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_10traceutils_7scamper_5warts_8WartsHop_7icmpext_2__set__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsHop *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(PyList_CheckExact(__pyx_v_value))||((__pyx_v_value) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_v_value)->tp_name), 0))) __PYX_ERR(1, 4, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->icmpext);
  __Pyx_DECREF(__pyx_v_self->icmpext);
  __pyx_v_self->icmpext = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsHop.icmpext.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_10traceutils_7scamper_5warts_8WartsHop_7icmpext_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_10traceutils_7scamper_5warts_8WartsHop_7icmpext_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_8WartsHop_7icmpext_4__del__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsHop *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_10traceutils_7scamper_5warts_8WartsHop_7icmpext_4__del__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsHop *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->icmpext);
  __Pyx_DECREF(__pyx_v_self->icmpext);
  __pyx_v_self->icmpext = ((PyObject*)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "traceutils/scamper/warts.pxd":5
 * cdef class WartsHop(Hop):
 *     cdef public list icmpext
 *     cdef public int probe_id             # <<<<<<<<<<<<<<
 *     cdef public unsigned short probe_size
 *     cdef public int reply_ipid
 */

/* Python wrapper */
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_8WartsHop_8probe_id_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_8WartsHop_8probe_id_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_8WartsHop_8probe_id___get__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsHop *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10traceutils_7scamper_5warts_8WartsHop_8probe_id___get__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsHop *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->probe_id); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsHop.probe_id.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_10traceutils_7scamper_5warts_8WartsHop_8probe_id_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_10traceutils_7scamper_5warts_8WartsHop_8probe_id_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_8WartsHop_8probe_id_2__set__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsHop *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_10traceutils_7scamper_5warts_8WartsHop_8probe_id_2__set__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsHop *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyInt_As_int(__pyx_v_value); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 5, __pyx_L1_error)
  __pyx_v_self->probe_id = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsHop.probe_id.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "traceutils/scamper/warts.pxd":6
 *     cdef public list icmpext
 *     cdef public int probe_id
 *     cdef public unsigned short probe_size             # <<<<<<<<<<<<<<
 *     cdef public int reply_ipid
 *     cdef public int icmp_q_ipl
 */

/* Python wrapper */
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_8WartsHop_10probe_size_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_8WartsHop_10probe_size_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_8WartsHop_10probe_size___get__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsHop *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10traceutils_7scamper_5warts_8WartsHop_10probe_size___get__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsHop *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_unsigned_short(__pyx_v_self->probe_size); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsHop.probe_size.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_10traceutils_7scamper_5warts_8WartsHop_10probe_size_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_10traceutils_7scamper_5warts_8WartsHop_10probe_size_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_8WartsHop_10probe_size_2__set__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsHop *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_10traceutils_7scamper_5warts_8WartsHop_10probe_size_2__set__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsHop *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  unsigned short __pyx_t_1;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyInt_As_unsigned_short(__pyx_v_value); if (unlikely((__pyx_t_1 == (unsigned short)-1) && PyErr_Occurred())) __PYX_ERR(1, 6, __pyx_L1_error)
  __pyx_v_self->probe_size = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsHop.probe_size.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "traceutils/scamper/warts.pxd":7
 *     cdef public int probe_id
 *     cdef public unsigned short probe_size
 *     cdef public int reply_ipid             # <<<<<<<<<<<<<<
 *     cdef public int icmp_q_ipl
 *     cdef public int icmp_nhmtu
 */

/* Python wrapper */
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_8WartsHop_10reply_ipid_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_8WartsHop_10reply_ipid_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_8WartsHop_10reply_ipid___get__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsHop *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10traceutils_7scamper_5warts_8WartsHop_10reply_ipid___get__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsHop *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->reply_ipid); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsHop.reply_ipid.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_10traceutils_7scamper_5warts_8WartsHop_10reply_ipid_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_10traceutils_7scamper_5warts_8WartsHop_10reply_ipid_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_8WartsHop_10reply_ipid_2__set__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsHop *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_10traceutils_7scamper_5warts_8WartsHop_10reply_ipid_2__set__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsHop *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyInt_As_int(__pyx_v_value); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 7, __pyx_L1_error)
  __pyx_v_self->reply_ipid = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsHop.reply_ipid.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "traceutils/scamper/warts.pxd":8
 *     cdef public unsigned short probe_size
 *     cdef public int reply_ipid
 *     cdef public int icmp_q_ipl             # <<<<<<<<<<<<<<
 *     cdef public int icmp_nhmtu
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_8WartsHop_10icmp_q_ipl_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_8WartsHop_10icmp_q_ipl_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_8WartsHop_10icmp_q_ipl___get__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsHop *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10traceutils_7scamper_5warts_8WartsHop_10icmp_q_ipl___get__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsHop *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->icmp_q_ipl); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 8, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsHop.icmp_q_ipl.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_10traceutils_7scamper_5warts_8WartsHop_10icmp_q_ipl_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_10traceutils_7scamper_5warts_8WartsHop_10icmp_q_ipl_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_8WartsHop_10icmp_q_ipl_2__set__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsHop *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_10traceutils_7scamper_5warts_8WartsHop_10icmp_q_ipl_2__set__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsHop *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyInt_As_int(__pyx_v_value); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 8, __pyx_L1_error)
  __pyx_v_self->icmp_q_ipl = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsHop.icmp_q_ipl.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "traceutils/scamper/warts.pxd":9
 *     cdef public int reply_ipid
 *     cdef public int icmp_q_ipl
 *     cdef public int icmp_nhmtu             # <<<<<<<<<<<<<<
 * 
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_8WartsHop_10icmp_nhmtu_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_8WartsHop_10icmp_nhmtu_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_8WartsHop_10icmp_nhmtu___get__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsHop *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10traceutils_7scamper_5warts_8WartsHop_10icmp_nhmtu___get__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsHop *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->icmp_nhmtu); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 9, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsHop.icmp_nhmtu.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_10traceutils_7scamper_5warts_8WartsHop_10icmp_nhmtu_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_10traceutils_7scamper_5warts_8WartsHop_10icmp_nhmtu_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_8WartsHop_10icmp_nhmtu_2__set__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsHop *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_10traceutils_7scamper_5warts_8WartsHop_10icmp_nhmtu_2__set__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsHop *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyInt_As_int(__pyx_v_value); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 9, __pyx_L1_error)
  __pyx_v_self->icmp_nhmtu = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsHop.icmp_nhmtu.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_8WartsHop_3__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_10traceutils_7scamper_5warts_8WartsHop_2__reduce_cython__[] = "WartsHop.__reduce_cython__(self)";
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_8WartsHop_3__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_8WartsHop_2__reduce_cython__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsHop *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10traceutils_7scamper_5warts_8WartsHop_2__reduce_cython__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsHop *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  PyObject *__pyx_t_14 = NULL;
  PyObject *__pyx_t_15 = NULL;
  PyObject *__pyx_t_16 = NULL;
  PyObject *__pyx_t_17 = NULL;
  PyObject *__pyx_t_18 = NULL;
  int __pyx_t_19;
  int __pyx_t_20;
  int __pyx_t_21;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self.addr, self.family, self.icmp_code, self.icmp_nhmtu, self.icmp_q_ipl, self.icmp_q_tos, self.icmp_q_ttl, self.icmp_type, self.icmpext, self.ismpls, self.packed, self.probe_id, self.probe_size, self.probe_ttl, self.reply_ipid, self.reply_size, self.reply_tos, self.reply_ttl, self.rtt, self.type)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->__pyx_base.family); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_From_unsigned_char(__pyx_v_self->__pyx_base.icmp_code); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_self->icmp_nhmtu); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyInt_From_int(__pyx_v_self->icmp_q_ipl); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyInt_From_int(__pyx_v_self->__pyx_base.icmp_q_tos); if (unlikely(!__pyx_t_5)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = __Pyx_PyInt_From_unsigned_char(__pyx_v_self->__pyx_base.icmp_q_ttl); if (unlikely(!__pyx_t_6)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_7 = __Pyx_PyInt_From_unsigned_char(__pyx_v_self->__pyx_base.icmp_type); if (unlikely(!__pyx_t_7)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_8 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.ismpls); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_9 = __Pyx_PyInt_From_int(__pyx_v_self->probe_id); if (unlikely(!__pyx_t_9)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __pyx_t_10 = __Pyx_PyInt_From_unsigned_short(__pyx_v_self->probe_size); if (unlikely(!__pyx_t_10)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_11 = __Pyx_PyInt_From_unsigned_char(__pyx_v_self->__pyx_base.probe_ttl); if (unlikely(!__pyx_t_11)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __pyx_t_12 = __Pyx_PyInt_From_int(__pyx_v_self->reply_ipid); if (unlikely(!__pyx_t_12)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_t_13 = __Pyx_PyInt_From_int(__pyx_v_self->__pyx_base.reply_size); if (unlikely(!__pyx_t_13)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);
  __pyx_t_14 = __Pyx_PyInt_From_int(__pyx_v_self->__pyx_base.reply_tos); if (unlikely(!__pyx_t_14)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_14);
  __pyx_t_15 = __Pyx_PyInt_From_unsigned_char(__pyx_v_self->__pyx_base.reply_ttl); if (unlikely(!__pyx_t_15)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_15);
  __pyx_t_16 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.rtt); if (unlikely(!__pyx_t_16)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_16);
  __pyx_t_17 = __Pyx_PyInt_From_enum____pyx_t_10traceutils_7scamper_3hop_ICMPType(__pyx_v_self->__pyx_base.type); if (unlikely(!__pyx_t_17)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_17);
  __pyx_t_18 = PyTuple_New(20); if (unlikely(!__pyx_t_18)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_18);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.addr);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.addr);
  PyTuple_SET_ITEM(__pyx_t_18, 0, __pyx_v_self->__pyx_base.addr);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_18, 1, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_18, 2, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_18, 3, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_18, 4, __pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_18, 5, __pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_18, 6, __pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_7);
  PyTuple_SET_ITEM(__pyx_t_18, 7, __pyx_t_7);
  __Pyx_INCREF(__pyx_v_self->icmpext);
  __Pyx_GIVEREF(__pyx_v_self->icmpext);
  PyTuple_SET_ITEM(__pyx_t_18, 8, __pyx_v_self->icmpext);
  __Pyx_GIVEREF(__pyx_t_8);
  PyTuple_SET_ITEM(__pyx_t_18, 9, __pyx_t_8);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.packed);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.packed);
  PyTuple_SET_ITEM(__pyx_t_18, 10, __pyx_v_self->__pyx_base.packed);
  __Pyx_GIVEREF(__pyx_t_9);
  PyTuple_SET_ITEM(__pyx_t_18, 11, __pyx_t_9);
  __Pyx_GIVEREF(__pyx_t_10);
  PyTuple_SET_ITEM(__pyx_t_18, 12, __pyx_t_10);
  __Pyx_GIVEREF(__pyx_t_11);
  PyTuple_SET_ITEM(__pyx_t_18, 13, __pyx_t_11);
  __Pyx_GIVEREF(__pyx_t_12);
  PyTuple_SET_ITEM(__pyx_t_18, 14, __pyx_t_12);
  __Pyx_GIVEREF(__pyx_t_13);
  PyTuple_SET_ITEM(__pyx_t_18, 15, __pyx_t_13);
  __Pyx_GIVEREF(__pyx_t_14);
  PyTuple_SET_ITEM(__pyx_t_18, 16, __pyx_t_14);
  __Pyx_GIVEREF(__pyx_t_15);
  PyTuple_SET_ITEM(__pyx_t_18, 17, __pyx_t_15);
  __Pyx_GIVEREF(__pyx_t_16);
  PyTuple_SET_ITEM(__pyx_t_18, 18, __pyx_t_16);
  __Pyx_GIVEREF(__pyx_t_17);
  PyTuple_SET_ITEM(__pyx_t_18, 19, __pyx_t_17);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_5 = 0;
  __pyx_t_6 = 0;
  __pyx_t_7 = 0;
  __pyx_t_8 = 0;
  __pyx_t_9 = 0;
  __pyx_t_10 = 0;
  __pyx_t_11 = 0;
  __pyx_t_12 = 0;
  __pyx_t_13 = 0;
  __pyx_t_14 = 0;
  __pyx_t_15 = 0;
  __pyx_t_16 = 0;
  __pyx_t_17 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_18);
  __pyx_t_18 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self.addr, self.family, self.icmp_code, self.icmp_nhmtu, self.icmp_q_ipl, self.icmp_q_tos, self.icmp_q_ttl, self.icmp_type, self.icmpext, self.ismpls, self.packed, self.probe_id, self.probe_size, self.probe_ttl, self.reply_ipid, self.reply_size, self.reply_tos, self.reply_ttl, self.rtt, self.type)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_18 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_18)) __PYX_ERR(2, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_18);
  __pyx_v__dict = __pyx_t_18;
  __pyx_t_18 = 0;

  /* "(tree fragment)":7
 *     state = (self.addr, self.family, self.icmp_code, self.icmp_nhmtu, self.icmp_q_ipl, self.icmp_q_tos, self.icmp_q_ttl, self.icmp_type, self.icmpext, self.ismpls, self.packed, self.probe_id, self.probe_size, self.probe_ttl, self.reply_ipid, self.reply_size, self.reply_tos, self.reply_ttl, self.rtt, self.type)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_19 = (__pyx_v__dict != Py_None);
  __pyx_t_20 = (__pyx_t_19 != 0);
  if (__pyx_t_20) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_18 = PyTuple_New(1); if (unlikely(!__pyx_t_18)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_18);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_18, 0, __pyx_v__dict);
    __pyx_t_17 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_18); if (unlikely(!__pyx_t_17)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_17);
    __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_17));
    __pyx_t_17 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self.addr is not None or self.icmpext is not None or self.packed is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self.addr, self.family, self.icmp_code, self.icmp_nhmtu, self.icmp_q_ipl, self.icmp_q_tos, self.icmp_q_ttl, self.icmp_type, self.icmpext, self.ismpls, self.packed, self.probe_id, self.probe_size, self.probe_ttl, self.reply_ipid, self.reply_size, self.reply_tos, self.reply_ttl, self.rtt, self.type)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self.addr is not None or self.icmpext is not None or self.packed is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_WartsHop, (type(self), 0xb91787d, None), state
 */
  /*else*/ {
    __pyx_t_19 = (__pyx_v_self->__pyx_base.addr != ((PyObject*)Py_None));
    __pyx_t_21 = (__pyx_t_19 != 0);
    if (!__pyx_t_21) {
    } else {
      __pyx_t_20 = __pyx_t_21;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_21 = (__pyx_v_self->icmpext != ((PyObject*)Py_None));
    __pyx_t_19 = (__pyx_t_21 != 0);
    if (!__pyx_t_19) {
    } else {
      __pyx_t_20 = __pyx_t_19;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_19 = (__pyx_v_self->__pyx_base.packed != ((PyObject*)Py_None));
    __pyx_t_21 = (__pyx_t_19 != 0);
    __pyx_t_20 = __pyx_t_21;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_20;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.addr is not None or self.icmpext is not None or self.packed is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_WartsHop, (type(self), 0xb91787d, None), state
 *     else:
 */
  __pyx_t_20 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_20) {

    /* "(tree fragment)":13
 *         use_setstate = self.addr is not None or self.icmpext is not None or self.packed is not None
 *     if use_setstate:
 *         return __pyx_unpickle_WartsHop, (type(self), 0xb91787d, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_WartsHop, (type(self), 0xb91787d, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_17, __pyx_n_s_pyx_unpickle_WartsHop); if (unlikely(!__pyx_t_17)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_17);
    __pyx_t_18 = PyTuple_New(3); if (unlikely(!__pyx_t_18)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_18);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_18, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_194082941);
    __Pyx_GIVEREF(__pyx_int_194082941);
    PyTuple_SET_ITEM(__pyx_t_18, 1, __pyx_int_194082941);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_18, 2, Py_None);
    __pyx_t_16 = PyTuple_New(3); if (unlikely(!__pyx_t_16)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_16);
    __Pyx_GIVEREF(__pyx_t_17);
    PyTuple_SET_ITEM(__pyx_t_16, 0, __pyx_t_17);
    __Pyx_GIVEREF(__pyx_t_18);
    PyTuple_SET_ITEM(__pyx_t_16, 1, __pyx_t_18);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_16, 2, __pyx_v_state);
    __pyx_t_17 = 0;
    __pyx_t_18 = 0;
    __pyx_r = __pyx_t_16;
    __pyx_t_16 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.addr is not None or self.icmpext is not None or self.packed is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_WartsHop, (type(self), 0xb91787d, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_WartsHop, (type(self), 0xb91787d, None), state
 *     else:
 *         return __pyx_unpickle_WartsHop, (type(self), 0xb91787d, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_WartsHop__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_16, __pyx_n_s_pyx_unpickle_WartsHop); if (unlikely(!__pyx_t_16)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_16);
    __pyx_t_18 = PyTuple_New(3); if (unlikely(!__pyx_t_18)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_18);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_18, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_194082941);
    __Pyx_GIVEREF(__pyx_int_194082941);
    PyTuple_SET_ITEM(__pyx_t_18, 1, __pyx_int_194082941);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_18, 2, __pyx_v_state);
    __pyx_t_17 = PyTuple_New(2); if (unlikely(!__pyx_t_17)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_17);
    __Pyx_GIVEREF(__pyx_t_16);
    PyTuple_SET_ITEM(__pyx_t_17, 0, __pyx_t_16);
    __Pyx_GIVEREF(__pyx_t_18);
    PyTuple_SET_ITEM(__pyx_t_17, 1, __pyx_t_18);
    __pyx_t_16 = 0;
    __pyx_t_18 = 0;
    __pyx_r = __pyx_t_17;
    __pyx_t_17 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_XDECREF(__pyx_t_14);
  __Pyx_XDECREF(__pyx_t_15);
  __Pyx_XDECREF(__pyx_t_16);
  __Pyx_XDECREF(__pyx_t_17);
  __Pyx_XDECREF(__pyx_t_18);
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsHop.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_WartsHop, (type(self), 0xb91787d, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_WartsHop__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_8WartsHop_5__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static char __pyx_doc_10traceutils_7scamper_5warts_8WartsHop_4__setstate_cython__[] = "WartsHop.__setstate_cython__(self, __pyx_state)";
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_8WartsHop_5__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_8WartsHop_4__setstate_cython__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsHop *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10traceutils_7scamper_5warts_8WartsHop_4__setstate_cython__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsHop *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_WartsHop, (type(self), 0xb91787d, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_WartsHop__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(2, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_10traceutils_7scamper_5warts___pyx_unpickle_WartsHop__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_WartsHop, (type(self), 0xb91787d, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_WartsHop__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsHop.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "traceutils/scamper/warts.pyx":82
 * 
 * cdef class WartsPing:
 *     def __init__(self, str type='ping', str version=None, str method=None, str src=None, str dst=None, dict start=None, int ping_sent=-1, int probe_size=-1, int userid=-1, int ttl=-1, double wait=-1, double timeout=-1, list responses=None, dict statistics=None):             # <<<<<<<<<<<<<<
 *         self.type = type
 *         self.version = version
 */

/* Python wrapper */
static int __pyx_pw_10traceutils_7scamper_5warts_9WartsPing_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_10traceutils_7scamper_5warts_9WartsPing_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_type = 0;
  PyObject *__pyx_v_version = 0;
  PyObject *__pyx_v_method = 0;
  PyObject *__pyx_v_src = 0;
  PyObject *__pyx_v_dst = 0;
  PyObject *__pyx_v_start = 0;
  int __pyx_v_ping_sent;
  int __pyx_v_probe_size;
  int __pyx_v_userid;
  int __pyx_v_ttl;
  double __pyx_v_wait;
  double __pyx_v_timeout;
  PyObject *__pyx_v_responses = 0;
  PyObject *__pyx_v_statistics = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_type,&__pyx_n_s_version,&__pyx_n_s_method,&__pyx_n_s_src,&__pyx_n_s_dst,&__pyx_n_s_start,&__pyx_n_s_ping_sent,&__pyx_n_s_probe_size,&__pyx_n_s_userid,&__pyx_n_s_ttl,&__pyx_n_s_wait,&__pyx_n_s_timeout,&__pyx_n_s_responses,&__pyx_n_s_statistics,0};
    PyObject* values[14] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0};
    values[0] = ((PyObject*)__pyx_n_u_ping);
    values[1] = ((PyObject*)Py_None);
    values[2] = ((PyObject*)Py_None);
    values[3] = ((PyObject*)Py_None);
    values[4] = ((PyObject*)Py_None);
    values[5] = ((PyObject*)Py_None);
    values[12] = ((PyObject*)Py_None);
    values[13] = ((PyObject*)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case 14: values[13] = PyTuple_GET_ITEM(__pyx_args, 13);
        CYTHON_FALLTHROUGH;
        case 13: values[12] = PyTuple_GET_ITEM(__pyx_args, 12);
        CYTHON_FALLTHROUGH;
        case 12: values[11] = PyTuple_GET_ITEM(__pyx_args, 11);
        CYTHON_FALLTHROUGH;
        case 11: values[10] = PyTuple_GET_ITEM(__pyx_args, 10);
        CYTHON_FALLTHROUGH;
        case 10: values[9] = PyTuple_GET_ITEM(__pyx_args, 9);
        CYTHON_FALLTHROUGH;
        case  9: values[8] = PyTuple_GET_ITEM(__pyx_args, 8);
        CYTHON_FALLTHROUGH;
        case  8: values[7] = PyTuple_GET_ITEM(__pyx_args, 7);
        CYTHON_FALLTHROUGH;
        case  7: values[6] = PyTuple_GET_ITEM(__pyx_args, 6);
        CYTHON_FALLTHROUGH;
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        CYTHON_FALLTHROUGH;
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_type);
          if (value) { values[0] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_version);
          if (value) { values[1] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_method);
          if (value) { values[2] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_src);
          if (value) { values[3] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  4:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_dst);
          if (value) { values[4] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  5:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_start);
          if (value) { values[5] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  6:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_ping_sent);
          if (value) { values[6] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  7:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_probe_size);
          if (value) { values[7] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  8:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_userid);
          if (value) { values[8] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  9:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_ttl);
          if (value) { values[9] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case 10:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_wait);
          if (value) { values[10] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case 11:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_timeout);
          if (value) { values[11] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case 12:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_responses);
          if (value) { values[12] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case 13:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_statistics);
          if (value) { values[13] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 82, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case 14: values[13] = PyTuple_GET_ITEM(__pyx_args, 13);
        CYTHON_FALLTHROUGH;
        case 13: values[12] = PyTuple_GET_ITEM(__pyx_args, 12);
        CYTHON_FALLTHROUGH;
        case 12: values[11] = PyTuple_GET_ITEM(__pyx_args, 11);
        CYTHON_FALLTHROUGH;
        case 11: values[10] = PyTuple_GET_ITEM(__pyx_args, 10);
        CYTHON_FALLTHROUGH;
        case 10: values[9] = PyTuple_GET_ITEM(__pyx_args, 9);
        CYTHON_FALLTHROUGH;
        case  9: values[8] = PyTuple_GET_ITEM(__pyx_args, 8);
        CYTHON_FALLTHROUGH;
        case  8: values[7] = PyTuple_GET_ITEM(__pyx_args, 7);
        CYTHON_FALLTHROUGH;
        case  7: values[6] = PyTuple_GET_ITEM(__pyx_args, 6);
        CYTHON_FALLTHROUGH;
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        CYTHON_FALLTHROUGH;
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_type = ((PyObject*)values[0]);
    __pyx_v_version = ((PyObject*)values[1]);
    __pyx_v_method = ((PyObject*)values[2]);
    __pyx_v_src = ((PyObject*)values[3]);
    __pyx_v_dst = ((PyObject*)values[4]);
    __pyx_v_start = ((PyObject*)values[5]);
    if (values[6]) {
      __pyx_v_ping_sent = __Pyx_PyInt_As_int(values[6]); if (unlikely((__pyx_v_ping_sent == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 82, __pyx_L3_error)
    } else {
      __pyx_v_ping_sent = ((int)-1);
    }
    if (values[7]) {
      __pyx_v_probe_size = __Pyx_PyInt_As_int(values[7]); if (unlikely((__pyx_v_probe_size == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 82, __pyx_L3_error)
    } else {
      __pyx_v_probe_size = ((int)-1);
    }
    if (values[8]) {
      __pyx_v_userid = __Pyx_PyInt_As_int(values[8]); if (unlikely((__pyx_v_userid == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 82, __pyx_L3_error)
    } else {
      __pyx_v_userid = ((int)-1);
    }
    if (values[9]) {
      __pyx_v_ttl = __Pyx_PyInt_As_int(values[9]); if (unlikely((__pyx_v_ttl == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 82, __pyx_L3_error)
    } else {
      __pyx_v_ttl = ((int)-1);
    }
    if (values[10]) {
      __pyx_v_wait = __pyx_PyFloat_AsDouble(values[10]); if (unlikely((__pyx_v_wait == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 82, __pyx_L3_error)
    } else {
      __pyx_v_wait = ((double)-1.0);
    }
    if (values[11]) {
      __pyx_v_timeout = __pyx_PyFloat_AsDouble(values[11]); if (unlikely((__pyx_v_timeout == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 82, __pyx_L3_error)
    } else {
      __pyx_v_timeout = ((double)-1.0);
    }
    __pyx_v_responses = ((PyObject*)values[12]);
    __pyx_v_statistics = ((PyObject*)values[13]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 0, 0, 14, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 82, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsPing.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_type), (&PyUnicode_Type), 1, "type", 1))) __PYX_ERR(0, 82, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_version), (&PyUnicode_Type), 1, "version", 1))) __PYX_ERR(0, 82, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_method), (&PyUnicode_Type), 1, "method", 1))) __PYX_ERR(0, 82, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_src), (&PyUnicode_Type), 1, "src", 1))) __PYX_ERR(0, 82, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_dst), (&PyUnicode_Type), 1, "dst", 1))) __PYX_ERR(0, 82, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_start), (&PyDict_Type), 1, "start", 1))) __PYX_ERR(0, 82, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_responses), (&PyList_Type), 1, "responses", 1))) __PYX_ERR(0, 82, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_statistics), (&PyDict_Type), 1, "statistics", 1))) __PYX_ERR(0, 82, __pyx_L1_error)
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_9WartsPing___init__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *)__pyx_v_self), __pyx_v_type, __pyx_v_version, __pyx_v_method, __pyx_v_src, __pyx_v_dst, __pyx_v_start, __pyx_v_ping_sent, __pyx_v_probe_size, __pyx_v_userid, __pyx_v_ttl, __pyx_v_wait, __pyx_v_timeout, __pyx_v_responses, __pyx_v_statistics);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_10traceutils_7scamper_5warts_9WartsPing___init__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *__pyx_v_self, PyObject *__pyx_v_type, PyObject *__pyx_v_version, PyObject *__pyx_v_method, PyObject *__pyx_v_src, PyObject *__pyx_v_dst, PyObject *__pyx_v_start, int __pyx_v_ping_sent, int __pyx_v_probe_size, int __pyx_v_userid, int __pyx_v_ttl, double __pyx_v_wait, double __pyx_v_timeout, PyObject *__pyx_v_responses, PyObject *__pyx_v_statistics) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "traceutils/scamper/warts.pyx":83
 * cdef class WartsPing:
 *     def __init__(self, str type='ping', str version=None, str method=None, str src=None, str dst=None, dict start=None, int ping_sent=-1, int probe_size=-1, int userid=-1, int ttl=-1, double wait=-1, double timeout=-1, list responses=None, dict statistics=None):
 *         self.type = type             # <<<<<<<<<<<<<<
 *         self.version = version
 *         self.method = method
 */
  __Pyx_INCREF(__pyx_v_type);
  __Pyx_GIVEREF(__pyx_v_type);
  __Pyx_GOTREF(__pyx_v_self->type);
  __Pyx_DECREF(__pyx_v_self->type);
  __pyx_v_self->type = __pyx_v_type;

  /* "traceutils/scamper/warts.pyx":84
 *     def __init__(self, str type='ping', str version=None, str method=None, str src=None, str dst=None, dict start=None, int ping_sent=-1, int probe_size=-1, int userid=-1, int ttl=-1, double wait=-1, double timeout=-1, list responses=None, dict statistics=None):
 *         self.type = type
 *         self.version = version             # <<<<<<<<<<<<<<
 *         self.method = method
 *         self.src = src
 */
  __Pyx_INCREF(__pyx_v_version);
  __Pyx_GIVEREF(__pyx_v_version);
  __Pyx_GOTREF(__pyx_v_self->version);
  __Pyx_DECREF(__pyx_v_self->version);
  __pyx_v_self->version = __pyx_v_version;

  /* "traceutils/scamper/warts.pyx":85
 *         self.type = type
 *         self.version = version
 *         self.method = method             # <<<<<<<<<<<<<<
 *         self.src = src
 *         self.dst = dst
 */
  __Pyx_INCREF(__pyx_v_method);
  __Pyx_GIVEREF(__pyx_v_method);
  __Pyx_GOTREF(__pyx_v_self->method);
  __Pyx_DECREF(__pyx_v_self->method);
  __pyx_v_self->method = __pyx_v_method;

  /* "traceutils/scamper/warts.pyx":86
 *         self.version = version
 *         self.method = method
 *         self.src = src             # <<<<<<<<<<<<<<
 *         self.dst = dst
 *         self.family = find_family(dst.encode())
 */
  __Pyx_INCREF(__pyx_v_src);
  __Pyx_GIVEREF(__pyx_v_src);
  __Pyx_GOTREF(__pyx_v_self->src);
  __Pyx_DECREF(__pyx_v_self->src);
  __pyx_v_self->src = __pyx_v_src;

  /* "traceutils/scamper/warts.pyx":87
 *         self.method = method
 *         self.src = src
 *         self.dst = dst             # <<<<<<<<<<<<<<
 *         self.family = find_family(dst.encode())
 *         self.start = start
 */
  __Pyx_INCREF(__pyx_v_dst);
  __Pyx_GIVEREF(__pyx_v_dst);
  __Pyx_GOTREF(__pyx_v_self->dst);
  __Pyx_DECREF(__pyx_v_self->dst);
  __pyx_v_self->dst = __pyx_v_dst;

  /* "traceutils/scamper/warts.pyx":88
 *         self.src = src
 *         self.dst = dst
 *         self.family = find_family(dst.encode())             # <<<<<<<<<<<<<<
 *         self.start = start
 *         self.ping_sent = ping_sent
 */
  if (unlikely(__pyx_v_dst == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "encode");
    __PYX_ERR(0, 88, __pyx_L1_error)
  }
  __pyx_t_1 = PyUnicode_AsEncodedString(__pyx_v_dst, NULL, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 88, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_self->family = __pyx_f_10traceutils_5utils_3net_find_family(((PyObject*)__pyx_t_1), 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "traceutils/scamper/warts.pyx":89
 *         self.dst = dst
 *         self.family = find_family(dst.encode())
 *         self.start = start             # <<<<<<<<<<<<<<
 *         self.ping_sent = ping_sent
 *         self.probe_size = probe_size
 */
  __Pyx_INCREF(__pyx_v_start);
  __Pyx_GIVEREF(__pyx_v_start);
  __Pyx_GOTREF(__pyx_v_self->start);
  __Pyx_DECREF(__pyx_v_self->start);
  __pyx_v_self->start = __pyx_v_start;

  /* "traceutils/scamper/warts.pyx":90
 *         self.family = find_family(dst.encode())
 *         self.start = start
 *         self.ping_sent = ping_sent             # <<<<<<<<<<<<<<
 *         self.probe_size = probe_size
 *         self.userid = userid
 */
  __pyx_v_self->ping_sent = __pyx_v_ping_sent;

  /* "traceutils/scamper/warts.pyx":91
 *         self.start = start
 *         self.ping_sent = ping_sent
 *         self.probe_size = probe_size             # <<<<<<<<<<<<<<
 *         self.userid = userid
 *         self.ttl = ttl
 */
  __pyx_v_self->probe_size = __pyx_v_probe_size;

  /* "traceutils/scamper/warts.pyx":92
 *         self.ping_sent = ping_sent
 *         self.probe_size = probe_size
 *         self.userid = userid             # <<<<<<<<<<<<<<
 *         self.ttl = ttl
 *         self.wait = wait
 */
  __pyx_v_self->userid = __pyx_v_userid;

  /* "traceutils/scamper/warts.pyx":93
 *         self.probe_size = probe_size
 *         self.userid = userid
 *         self.ttl = ttl             # <<<<<<<<<<<<<<
 *         self.wait = wait
 *         self.timeout = timeout
 */
  __pyx_v_self->ttl = __pyx_v_ttl;

  /* "traceutils/scamper/warts.pyx":94
 *         self.userid = userid
 *         self.ttl = ttl
 *         self.wait = wait             # <<<<<<<<<<<<<<
 *         self.timeout = timeout
 *         self.responses = create_responses(responses, self.family)
 */
  __pyx_v_self->wait = __pyx_v_wait;

  /* "traceutils/scamper/warts.pyx":95
 *         self.ttl = ttl
 *         self.wait = wait
 *         self.timeout = timeout             # <<<<<<<<<<<<<<
 *         self.responses = create_responses(responses, self.family)
 *         self.statistics = statistics
 */
  __pyx_v_self->timeout = __pyx_v_timeout;

  /* "traceutils/scamper/warts.pyx":96
 *         self.wait = wait
 *         self.timeout = timeout
 *         self.responses = create_responses(responses, self.family)             # <<<<<<<<<<<<<<
 *         self.statistics = statistics
 * 
 */
  __pyx_t_1 = __pyx_f_10traceutils_7scamper_5warts_create_responses(__pyx_v_responses, __pyx_v_self->family); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 96, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->responses);
  __Pyx_DECREF(__pyx_v_self->responses);
  __pyx_v_self->responses = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "traceutils/scamper/warts.pyx":97
 *         self.timeout = timeout
 *         self.responses = create_responses(responses, self.family)
 *         self.statistics = statistics             # <<<<<<<<<<<<<<
 * 
 *     def __repr__(self):
 */
  __Pyx_INCREF(__pyx_v_statistics);
  __Pyx_GIVEREF(__pyx_v_statistics);
  __Pyx_GOTREF(__pyx_v_self->statistics);
  __Pyx_DECREF(__pyx_v_self->statistics);
  __pyx_v_self->statistics = __pyx_v_statistics;

  /* "traceutils/scamper/warts.pyx":82
 * 
 * cdef class WartsPing:
 *     def __init__(self, str type='ping', str version=None, str method=None, str src=None, str dst=None, dict start=None, int ping_sent=-1, int probe_size=-1, int userid=-1, int ttl=-1, double wait=-1, double timeout=-1, list responses=None, dict statistics=None):             # <<<<<<<<<<<<<<
 *         self.type = type
 *         self.version = version
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsPing.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "traceutils/scamper/warts.pyx":99
 *         self.statistics = statistics
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         result = ['Src={src}, Dst={dst}:'.format(src=self.src, dst=self.dst)]
 *         for resp in self.responses:
 */

/* Python wrapper */
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_9WartsPing_3__repr__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_9WartsPing_3__repr__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__repr__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_9WartsPing_2__repr__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10traceutils_7scamper_5warts_9WartsPing_2__repr__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *__pyx_v_self) {
  PyObject *__pyx_v_result = NULL;
  PyObject *__pyx_v_resp = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  Py_ssize_t __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  __Pyx_RefNannySetupContext("__repr__", 0);

  /* "traceutils/scamper/warts.pyx":100
 * 
 *     def __repr__(self):
 *         result = ['Src={src}, Dst={dst}:'.format(src=self.src, dst=self.dst)]             # <<<<<<<<<<<<<<
 *         for resp in self.responses:
 *             result.append('\t{}'.format(repr(resp)))
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_Src_src_Dst_dst, __pyx_n_s_format); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 100, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyDict_NewPresized(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 100, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_src, __pyx_v_self->src) < 0) __PYX_ERR(0, 100, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_dst, __pyx_v_self->dst) < 0) __PYX_ERR(0, 100, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 100, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 100, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_t_3);
  __pyx_t_3 = 0;
  __pyx_v_result = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "traceutils/scamper/warts.pyx":101
 *     def __repr__(self):
 *         result = ['Src={src}, Dst={dst}:'.format(src=self.src, dst=self.dst)]
 *         for resp in self.responses:             # <<<<<<<<<<<<<<
 *             result.append('\t{}'.format(repr(resp)))
 *         result.append(repr(self.statistics))
 */
  if (unlikely(__pyx_v_self->responses == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    __PYX_ERR(0, 101, __pyx_L1_error)
  }
  __pyx_t_2 = __pyx_v_self->responses; __Pyx_INCREF(__pyx_t_2); __pyx_t_4 = 0;
  for (;;) {
    if (__pyx_t_4 >= PyList_GET_SIZE(__pyx_t_2)) break;
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    __pyx_t_3 = PyList_GET_ITEM(__pyx_t_2, __pyx_t_4); __Pyx_INCREF(__pyx_t_3); __pyx_t_4++; if (unlikely(0 < 0)) __PYX_ERR(0, 101, __pyx_L1_error)
    #else
    __pyx_t_3 = PySequence_ITEM(__pyx_t_2, __pyx_t_4); __pyx_t_4++; if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 101, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    #endif
    __Pyx_XDECREF_SET(__pyx_v_resp, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "traceutils/scamper/warts.pyx":102
 *         result = ['Src={src}, Dst={dst}:'.format(src=self.src, dst=self.dst)]
 *         for resp in self.responses:
 *             result.append('\t{}'.format(repr(resp)))             # <<<<<<<<<<<<<<
 *         result.append(repr(self.statistics))
 *         return '\n'.join(result)
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u__2, __pyx_n_s_format); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 102, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_5 = PyObject_Repr(__pyx_v_resp); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 102, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
      __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_1);
      if (likely(__pyx_t_6)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
        __Pyx_INCREF(__pyx_t_6);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_1, function);
      }
    }
    __pyx_t_3 = (__pyx_t_6) ? __Pyx_PyObject_Call2Args(__pyx_t_1, __pyx_t_6, __pyx_t_5) : __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_5);
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 102, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_7 = __Pyx_PyList_Append(__pyx_v_result, __pyx_t_3); if (unlikely(__pyx_t_7 == ((int)-1))) __PYX_ERR(0, 102, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "traceutils/scamper/warts.pyx":101
 *     def __repr__(self):
 *         result = ['Src={src}, Dst={dst}:'.format(src=self.src, dst=self.dst)]
 *         for resp in self.responses:             # <<<<<<<<<<<<<<
 *             result.append('\t{}'.format(repr(resp)))
 *         result.append(repr(self.statistics))
 */
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "traceutils/scamper/warts.pyx":103
 *         for resp in self.responses:
 *             result.append('\t{}'.format(repr(resp)))
 *         result.append(repr(self.statistics))             # <<<<<<<<<<<<<<
 *         return '\n'.join(result)
 * 
 */
  __pyx_t_2 = __pyx_v_self->statistics;
  __Pyx_INCREF(__pyx_t_2);
  __pyx_t_3 = PyObject_Repr(__pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 103, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7 = __Pyx_PyList_Append(__pyx_v_result, __pyx_t_3); if (unlikely(__pyx_t_7 == ((int)-1))) __PYX_ERR(0, 103, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "traceutils/scamper/warts.pyx":104
 *             result.append('\t{}'.format(repr(resp)))
 *         result.append(repr(self.statistics))
 *         return '\n'.join(result)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_3 = PyUnicode_Join(__pyx_kp_u__3, __pyx_v_result); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 104, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "traceutils/scamper/warts.pyx":99
 *         self.statistics = statistics
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         result = ['Src={src}, Dst={dst}:'.format(src=self.src, dst=self.dst)]
 *         for resp in self.responses:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsPing.__repr__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_result);
  __Pyx_XDECREF(__pyx_v_resp);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "traceutils/scamper/warts.pxd":32
 * 
 * cdef class WartsPing:
 *     cdef public str type, version, method, src, dst             # <<<<<<<<<<<<<<
 *     cdef public dict start, statistics
 *     cdef public int ping_sent, probe_size, userid, ttl
 */

/* Python wrapper */
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_9WartsPing_4type_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_9WartsPing_4type_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_9WartsPing_4type___get__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10traceutils_7scamper_5warts_9WartsPing_4type___get__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->type);
  __pyx_r = __pyx_v_self->type;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_10traceutils_7scamper_5warts_9WartsPing_4type_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_10traceutils_7scamper_5warts_9WartsPing_4type_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_9WartsPing_4type_2__set__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_10traceutils_7scamper_5warts_9WartsPing_4type_2__set__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(PyUnicode_CheckExact(__pyx_v_value))||((__pyx_v_value) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "unicode", Py_TYPE(__pyx_v_value)->tp_name), 0))) __PYX_ERR(1, 32, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->type);
  __Pyx_DECREF(__pyx_v_self->type);
  __pyx_v_self->type = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsPing.type.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_10traceutils_7scamper_5warts_9WartsPing_4type_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_10traceutils_7scamper_5warts_9WartsPing_4type_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_9WartsPing_4type_4__del__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_10traceutils_7scamper_5warts_9WartsPing_4type_4__del__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->type);
  __Pyx_DECREF(__pyx_v_self->type);
  __pyx_v_self->type = ((PyObject*)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_9WartsPing_7version_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_9WartsPing_7version_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_9WartsPing_7version___get__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10traceutils_7scamper_5warts_9WartsPing_7version___get__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->version);
  __pyx_r = __pyx_v_self->version;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_10traceutils_7scamper_5warts_9WartsPing_7version_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_10traceutils_7scamper_5warts_9WartsPing_7version_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_9WartsPing_7version_2__set__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_10traceutils_7scamper_5warts_9WartsPing_7version_2__set__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(PyUnicode_CheckExact(__pyx_v_value))||((__pyx_v_value) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "unicode", Py_TYPE(__pyx_v_value)->tp_name), 0))) __PYX_ERR(1, 32, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->version);
  __Pyx_DECREF(__pyx_v_self->version);
  __pyx_v_self->version = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsPing.version.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_10traceutils_7scamper_5warts_9WartsPing_7version_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_10traceutils_7scamper_5warts_9WartsPing_7version_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_9WartsPing_7version_4__del__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_10traceutils_7scamper_5warts_9WartsPing_7version_4__del__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->version);
  __Pyx_DECREF(__pyx_v_self->version);
  __pyx_v_self->version = ((PyObject*)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_9WartsPing_6method_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_9WartsPing_6method_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_9WartsPing_6method___get__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10traceutils_7scamper_5warts_9WartsPing_6method___get__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->method);
  __pyx_r = __pyx_v_self->method;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_10traceutils_7scamper_5warts_9WartsPing_6method_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_10traceutils_7scamper_5warts_9WartsPing_6method_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_9WartsPing_6method_2__set__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_10traceutils_7scamper_5warts_9WartsPing_6method_2__set__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(PyUnicode_CheckExact(__pyx_v_value))||((__pyx_v_value) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "unicode", Py_TYPE(__pyx_v_value)->tp_name), 0))) __PYX_ERR(1, 32, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->method);
  __Pyx_DECREF(__pyx_v_self->method);
  __pyx_v_self->method = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsPing.method.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_10traceutils_7scamper_5warts_9WartsPing_6method_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_10traceutils_7scamper_5warts_9WartsPing_6method_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_9WartsPing_6method_4__del__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_10traceutils_7scamper_5warts_9WartsPing_6method_4__del__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->method);
  __Pyx_DECREF(__pyx_v_self->method);
  __pyx_v_self->method = ((PyObject*)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_9WartsPing_3src_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_9WartsPing_3src_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_9WartsPing_3src___get__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10traceutils_7scamper_5warts_9WartsPing_3src___get__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->src);
  __pyx_r = __pyx_v_self->src;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_10traceutils_7scamper_5warts_9WartsPing_3src_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_10traceutils_7scamper_5warts_9WartsPing_3src_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_9WartsPing_3src_2__set__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_10traceutils_7scamper_5warts_9WartsPing_3src_2__set__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(PyUnicode_CheckExact(__pyx_v_value))||((__pyx_v_value) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "unicode", Py_TYPE(__pyx_v_value)->tp_name), 0))) __PYX_ERR(1, 32, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->src);
  __Pyx_DECREF(__pyx_v_self->src);
  __pyx_v_self->src = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsPing.src.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_10traceutils_7scamper_5warts_9WartsPing_3src_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_10traceutils_7scamper_5warts_9WartsPing_3src_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_9WartsPing_3src_4__del__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_10traceutils_7scamper_5warts_9WartsPing_3src_4__del__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->src);
  __Pyx_DECREF(__pyx_v_self->src);
  __pyx_v_self->src = ((PyObject*)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_9WartsPing_3dst_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_9WartsPing_3dst_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_9WartsPing_3dst___get__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10traceutils_7scamper_5warts_9WartsPing_3dst___get__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->dst);
  __pyx_r = __pyx_v_self->dst;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_10traceutils_7scamper_5warts_9WartsPing_3dst_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_10traceutils_7scamper_5warts_9WartsPing_3dst_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_9WartsPing_3dst_2__set__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_10traceutils_7scamper_5warts_9WartsPing_3dst_2__set__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(PyUnicode_CheckExact(__pyx_v_value))||((__pyx_v_value) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "unicode", Py_TYPE(__pyx_v_value)->tp_name), 0))) __PYX_ERR(1, 32, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->dst);
  __Pyx_DECREF(__pyx_v_self->dst);
  __pyx_v_self->dst = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsPing.dst.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_10traceutils_7scamper_5warts_9WartsPing_3dst_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_10traceutils_7scamper_5warts_9WartsPing_3dst_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_9WartsPing_3dst_4__del__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_10traceutils_7scamper_5warts_9WartsPing_3dst_4__del__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->dst);
  __Pyx_DECREF(__pyx_v_self->dst);
  __pyx_v_self->dst = ((PyObject*)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "traceutils/scamper/warts.pxd":33
 * cdef class WartsPing:
 *     cdef public str type, version, method, src, dst
 *     cdef public dict start, statistics             # <<<<<<<<<<<<<<
 *     cdef public int ping_sent, probe_size, userid, ttl
 *     cdef public double wait, timeout
 */

/* Python wrapper */
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_9WartsPing_5start_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_9WartsPing_5start_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_9WartsPing_5start___get__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10traceutils_7scamper_5warts_9WartsPing_5start___get__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->start);
  __pyx_r = __pyx_v_self->start;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_10traceutils_7scamper_5warts_9WartsPing_5start_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_10traceutils_7scamper_5warts_9WartsPing_5start_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_9WartsPing_5start_2__set__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_10traceutils_7scamper_5warts_9WartsPing_5start_2__set__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(PyDict_CheckExact(__pyx_v_value))||((__pyx_v_value) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_v_value)->tp_name), 0))) __PYX_ERR(1, 33, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->start);
  __Pyx_DECREF(__pyx_v_self->start);
  __pyx_v_self->start = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsPing.start.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_10traceutils_7scamper_5warts_9WartsPing_5start_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_10traceutils_7scamper_5warts_9WartsPing_5start_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_9WartsPing_5start_4__del__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_10traceutils_7scamper_5warts_9WartsPing_5start_4__del__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->start);
  __Pyx_DECREF(__pyx_v_self->start);
  __pyx_v_self->start = ((PyObject*)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_9WartsPing_10statistics_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_9WartsPing_10statistics_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_9WartsPing_10statistics___get__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10traceutils_7scamper_5warts_9WartsPing_10statistics___get__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->statistics);
  __pyx_r = __pyx_v_self->statistics;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_10traceutils_7scamper_5warts_9WartsPing_10statistics_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_10traceutils_7scamper_5warts_9WartsPing_10statistics_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_9WartsPing_10statistics_2__set__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_10traceutils_7scamper_5warts_9WartsPing_10statistics_2__set__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(PyDict_CheckExact(__pyx_v_value))||((__pyx_v_value) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_v_value)->tp_name), 0))) __PYX_ERR(1, 33, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->statistics);
  __Pyx_DECREF(__pyx_v_self->statistics);
  __pyx_v_self->statistics = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsPing.statistics.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_10traceutils_7scamper_5warts_9WartsPing_10statistics_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_10traceutils_7scamper_5warts_9WartsPing_10statistics_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_9WartsPing_10statistics_4__del__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_10traceutils_7scamper_5warts_9WartsPing_10statistics_4__del__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->statistics);
  __Pyx_DECREF(__pyx_v_self->statistics);
  __pyx_v_self->statistics = ((PyObject*)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "traceutils/scamper/warts.pxd":34
 *     cdef public str type, version, method, src, dst
 *     cdef public dict start, statistics
 *     cdef public int ping_sent, probe_size, userid, ttl             # <<<<<<<<<<<<<<
 *     cdef public double wait, timeout
 *     cdef public list responses
 */

/* Python wrapper */
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_9WartsPing_9ping_sent_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_9WartsPing_9ping_sent_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_9WartsPing_9ping_sent___get__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10traceutils_7scamper_5warts_9WartsPing_9ping_sent___get__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->ping_sent); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 34, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsPing.ping_sent.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_10traceutils_7scamper_5warts_9WartsPing_9ping_sent_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_10traceutils_7scamper_5warts_9WartsPing_9ping_sent_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_9WartsPing_9ping_sent_2__set__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_10traceutils_7scamper_5warts_9WartsPing_9ping_sent_2__set__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyInt_As_int(__pyx_v_value); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 34, __pyx_L1_error)
  __pyx_v_self->ping_sent = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsPing.ping_sent.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_9WartsPing_10probe_size_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_9WartsPing_10probe_size_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_9WartsPing_10probe_size___get__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10traceutils_7scamper_5warts_9WartsPing_10probe_size___get__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->probe_size); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 34, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsPing.probe_size.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_10traceutils_7scamper_5warts_9WartsPing_10probe_size_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_10traceutils_7scamper_5warts_9WartsPing_10probe_size_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_9WartsPing_10probe_size_2__set__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_10traceutils_7scamper_5warts_9WartsPing_10probe_size_2__set__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyInt_As_int(__pyx_v_value); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 34, __pyx_L1_error)
  __pyx_v_self->probe_size = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsPing.probe_size.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_9WartsPing_6userid_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_9WartsPing_6userid_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_9WartsPing_6userid___get__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10traceutils_7scamper_5warts_9WartsPing_6userid___get__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->userid); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 34, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsPing.userid.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_10traceutils_7scamper_5warts_9WartsPing_6userid_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_10traceutils_7scamper_5warts_9WartsPing_6userid_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_9WartsPing_6userid_2__set__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_10traceutils_7scamper_5warts_9WartsPing_6userid_2__set__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyInt_As_int(__pyx_v_value); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 34, __pyx_L1_error)
  __pyx_v_self->userid = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsPing.userid.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_9WartsPing_3ttl_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_9WartsPing_3ttl_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_9WartsPing_3ttl___get__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10traceutils_7scamper_5warts_9WartsPing_3ttl___get__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->ttl); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 34, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsPing.ttl.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_10traceutils_7scamper_5warts_9WartsPing_3ttl_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_10traceutils_7scamper_5warts_9WartsPing_3ttl_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_9WartsPing_3ttl_2__set__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_10traceutils_7scamper_5warts_9WartsPing_3ttl_2__set__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyInt_As_int(__pyx_v_value); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 34, __pyx_L1_error)
  __pyx_v_self->ttl = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsPing.ttl.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "traceutils/scamper/warts.pxd":35
 *     cdef public dict start, statistics
 *     cdef public int ping_sent, probe_size, userid, ttl
 *     cdef public double wait, timeout             # <<<<<<<<<<<<<<
 *     cdef public list responses
 *     cdef public int family
 */

/* Python wrapper */
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_9WartsPing_4wait_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_9WartsPing_4wait_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_9WartsPing_4wait___get__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10traceutils_7scamper_5warts_9WartsPing_4wait___get__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->wait); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 35, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsPing.wait.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_10traceutils_7scamper_5warts_9WartsPing_4wait_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_10traceutils_7scamper_5warts_9WartsPing_4wait_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_9WartsPing_4wait_2__set__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_10traceutils_7scamper_5warts_9WartsPing_4wait_2__set__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  double __pyx_t_1;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __pyx_PyFloat_AsDouble(__pyx_v_value); if (unlikely((__pyx_t_1 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 35, __pyx_L1_error)
  __pyx_v_self->wait = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsPing.wait.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_9WartsPing_7timeout_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_9WartsPing_7timeout_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_9WartsPing_7timeout___get__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10traceutils_7scamper_5warts_9WartsPing_7timeout___get__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->timeout); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 35, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsPing.timeout.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_10traceutils_7scamper_5warts_9WartsPing_7timeout_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_10traceutils_7scamper_5warts_9WartsPing_7timeout_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_9WartsPing_7timeout_2__set__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_10traceutils_7scamper_5warts_9WartsPing_7timeout_2__set__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  double __pyx_t_1;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __pyx_PyFloat_AsDouble(__pyx_v_value); if (unlikely((__pyx_t_1 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 35, __pyx_L1_error)
  __pyx_v_self->timeout = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsPing.timeout.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "traceutils/scamper/warts.pxd":36
 *     cdef public int ping_sent, probe_size, userid, ttl
 *     cdef public double wait, timeout
 *     cdef public list responses             # <<<<<<<<<<<<<<
 *     cdef public int family
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_9WartsPing_9responses_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_9WartsPing_9responses_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_9WartsPing_9responses___get__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10traceutils_7scamper_5warts_9WartsPing_9responses___get__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->responses);
  __pyx_r = __pyx_v_self->responses;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_10traceutils_7scamper_5warts_9WartsPing_9responses_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_10traceutils_7scamper_5warts_9WartsPing_9responses_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_9WartsPing_9responses_2__set__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_10traceutils_7scamper_5warts_9WartsPing_9responses_2__set__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(PyList_CheckExact(__pyx_v_value))||((__pyx_v_value) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_v_value)->tp_name), 0))) __PYX_ERR(1, 36, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->responses);
  __Pyx_DECREF(__pyx_v_self->responses);
  __pyx_v_self->responses = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsPing.responses.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_10traceutils_7scamper_5warts_9WartsPing_9responses_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_10traceutils_7scamper_5warts_9WartsPing_9responses_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_9WartsPing_9responses_4__del__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_10traceutils_7scamper_5warts_9WartsPing_9responses_4__del__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->responses);
  __Pyx_DECREF(__pyx_v_self->responses);
  __pyx_v_self->responses = ((PyObject*)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "traceutils/scamper/warts.pxd":37
 *     cdef public double wait, timeout
 *     cdef public list responses
 *     cdef public int family             # <<<<<<<<<<<<<<
 * 
 * cdef list create_responses(list responses, int family);
 */

/* Python wrapper */
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_9WartsPing_6family_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_9WartsPing_6family_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_9WartsPing_6family___get__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10traceutils_7scamper_5warts_9WartsPing_6family___get__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->family); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 37, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsPing.family.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_10traceutils_7scamper_5warts_9WartsPing_6family_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_10traceutils_7scamper_5warts_9WartsPing_6family_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_9WartsPing_6family_2__set__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_10traceutils_7scamper_5warts_9WartsPing_6family_2__set__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyInt_As_int(__pyx_v_value); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 37, __pyx_L1_error)
  __pyx_v_self->family = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsPing.family.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_9WartsPing_5__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_10traceutils_7scamper_5warts_9WartsPing_4__reduce_cython__[] = "WartsPing.__reduce_cython__(self)";
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_9WartsPing_5__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_9WartsPing_4__reduce_cython__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10traceutils_7scamper_5warts_9WartsPing_4__reduce_cython__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_t_9;
  int __pyx_t_10;
  int __pyx_t_11;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self.dst, self.family, self.method, self.ping_sent, self.probe_size, self.responses, self.src, self.start, self.statistics, self.timeout, self.ttl, self.type, self.userid, self.version, self.wait)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->family); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_self->ping_sent); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_self->probe_size); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyFloat_FromDouble(__pyx_v_self->timeout); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyInt_From_int(__pyx_v_self->ttl); if (unlikely(!__pyx_t_5)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = __Pyx_PyInt_From_int(__pyx_v_self->userid); if (unlikely(!__pyx_t_6)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_7 = PyFloat_FromDouble(__pyx_v_self->wait); if (unlikely(!__pyx_t_7)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_8 = PyTuple_New(15); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_INCREF(__pyx_v_self->dst);
  __Pyx_GIVEREF(__pyx_v_self->dst);
  PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_v_self->dst);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_8, 1, __pyx_t_1);
  __Pyx_INCREF(__pyx_v_self->method);
  __Pyx_GIVEREF(__pyx_v_self->method);
  PyTuple_SET_ITEM(__pyx_t_8, 2, __pyx_v_self->method);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_8, 3, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_8, 4, __pyx_t_3);
  __Pyx_INCREF(__pyx_v_self->responses);
  __Pyx_GIVEREF(__pyx_v_self->responses);
  PyTuple_SET_ITEM(__pyx_t_8, 5, __pyx_v_self->responses);
  __Pyx_INCREF(__pyx_v_self->src);
  __Pyx_GIVEREF(__pyx_v_self->src);
  PyTuple_SET_ITEM(__pyx_t_8, 6, __pyx_v_self->src);
  __Pyx_INCREF(__pyx_v_self->start);
  __Pyx_GIVEREF(__pyx_v_self->start);
  PyTuple_SET_ITEM(__pyx_t_8, 7, __pyx_v_self->start);
  __Pyx_INCREF(__pyx_v_self->statistics);
  __Pyx_GIVEREF(__pyx_v_self->statistics);
  PyTuple_SET_ITEM(__pyx_t_8, 8, __pyx_v_self->statistics);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_8, 9, __pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_8, 10, __pyx_t_5);
  __Pyx_INCREF(__pyx_v_self->type);
  __Pyx_GIVEREF(__pyx_v_self->type);
  PyTuple_SET_ITEM(__pyx_t_8, 11, __pyx_v_self->type);
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_8, 12, __pyx_t_6);
  __Pyx_INCREF(__pyx_v_self->version);
  __Pyx_GIVEREF(__pyx_v_self->version);
  PyTuple_SET_ITEM(__pyx_t_8, 13, __pyx_v_self->version);
  __Pyx_GIVEREF(__pyx_t_7);
  PyTuple_SET_ITEM(__pyx_t_8, 14, __pyx_t_7);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_5 = 0;
  __pyx_t_6 = 0;
  __pyx_t_7 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_8);
  __pyx_t_8 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self.dst, self.family, self.method, self.ping_sent, self.probe_size, self.responses, self.src, self.start, self.statistics, self.timeout, self.ttl, self.type, self.userid, self.version, self.wait)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_8 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_v__dict = __pyx_t_8;
  __pyx_t_8 = 0;

  /* "(tree fragment)":7
 *     state = (self.dst, self.family, self.method, self.ping_sent, self.probe_size, self.responses, self.src, self.start, self.statistics, self.timeout, self.ttl, self.type, self.userid, self.version, self.wait)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_9 = (__pyx_v__dict != Py_None);
  __pyx_t_10 = (__pyx_t_9 != 0);
  if (__pyx_t_10) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_8 = PyTuple_New(1); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_v__dict);
    __pyx_t_7 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_8); if (unlikely(!__pyx_t_7)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_7));
    __pyx_t_7 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self.dst is not None or self.method is not None or self.responses is not None or self.src is not None or self.start is not None or self.statistics is not None or self.type is not None or self.version is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self.dst, self.family, self.method, self.ping_sent, self.probe_size, self.responses, self.src, self.start, self.statistics, self.timeout, self.ttl, self.type, self.userid, self.version, self.wait)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self.dst is not None or self.method is not None or self.responses is not None or self.src is not None or self.start is not None or self.statistics is not None or self.type is not None or self.version is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_WartsPing, (type(self), 0x5752e4d, None), state
 */
  /*else*/ {
    __pyx_t_9 = (__pyx_v_self->dst != ((PyObject*)Py_None));
    __pyx_t_11 = (__pyx_t_9 != 0);
    if (!__pyx_t_11) {
    } else {
      __pyx_t_10 = __pyx_t_11;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_11 = (__pyx_v_self->method != ((PyObject*)Py_None));
    __pyx_t_9 = (__pyx_t_11 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_10 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->responses != ((PyObject*)Py_None));
    __pyx_t_11 = (__pyx_t_9 != 0);
    if (!__pyx_t_11) {
    } else {
      __pyx_t_10 = __pyx_t_11;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_11 = (__pyx_v_self->src != ((PyObject*)Py_None));
    __pyx_t_9 = (__pyx_t_11 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_10 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->start != ((PyObject*)Py_None));
    __pyx_t_11 = (__pyx_t_9 != 0);
    if (!__pyx_t_11) {
    } else {
      __pyx_t_10 = __pyx_t_11;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_11 = (__pyx_v_self->statistics != ((PyObject*)Py_None));
    __pyx_t_9 = (__pyx_t_11 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_10 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->type != ((PyObject*)Py_None));
    __pyx_t_11 = (__pyx_t_9 != 0);
    if (!__pyx_t_11) {
    } else {
      __pyx_t_10 = __pyx_t_11;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_11 = (__pyx_v_self->version != ((PyObject*)Py_None));
    __pyx_t_9 = (__pyx_t_11 != 0);
    __pyx_t_10 = __pyx_t_9;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_10;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.dst is not None or self.method is not None or self.responses is not None or self.src is not None or self.start is not None or self.statistics is not None or self.type is not None or self.version is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_WartsPing, (type(self), 0x5752e4d, None), state
 *     else:
 */
  __pyx_t_10 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_10) {

    /* "(tree fragment)":13
 *         use_setstate = self.dst is not None or self.method is not None or self.responses is not None or self.src is not None or self.start is not None or self.statistics is not None or self.type is not None or self.version is not None
 *     if use_setstate:
 *         return __pyx_unpickle_WartsPing, (type(self), 0x5752e4d, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_WartsPing, (type(self), 0x5752e4d, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_n_s_pyx_unpickle_WartsPing); if (unlikely(!__pyx_t_7)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_8 = PyTuple_New(3); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_8, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_91565645);
    __Pyx_GIVEREF(__pyx_int_91565645);
    PyTuple_SET_ITEM(__pyx_t_8, 1, __pyx_int_91565645);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_8, 2, Py_None);
    __pyx_t_6 = PyTuple_New(3); if (unlikely(!__pyx_t_6)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_GIVEREF(__pyx_t_7);
    PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_7);
    __Pyx_GIVEREF(__pyx_t_8);
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_8);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_v_state);
    __pyx_t_7 = 0;
    __pyx_t_8 = 0;
    __pyx_r = __pyx_t_6;
    __pyx_t_6 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.dst is not None or self.method is not None or self.responses is not None or self.src is not None or self.start is not None or self.statistics is not None or self.type is not None or self.version is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_WartsPing, (type(self), 0x5752e4d, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_WartsPing, (type(self), 0x5752e4d, None), state
 *     else:
 *         return __pyx_unpickle_WartsPing, (type(self), 0x5752e4d, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_WartsPing__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_6, __pyx_n_s_pyx_unpickle_WartsPing); if (unlikely(!__pyx_t_6)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_8 = PyTuple_New(3); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_8, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_91565645);
    __Pyx_GIVEREF(__pyx_int_91565645);
    PyTuple_SET_ITEM(__pyx_t_8, 1, __pyx_int_91565645);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_8, 2, __pyx_v_state);
    __pyx_t_7 = PyTuple_New(2); if (unlikely(!__pyx_t_7)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_6);
    __Pyx_GIVEREF(__pyx_t_8);
    PyTuple_SET_ITEM(__pyx_t_7, 1, __pyx_t_8);
    __pyx_t_6 = 0;
    __pyx_t_8 = 0;
    __pyx_r = __pyx_t_7;
    __pyx_t_7 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsPing.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_WartsPing, (type(self), 0x5752e4d, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_WartsPing__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_9WartsPing_7__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static char __pyx_doc_10traceutils_7scamper_5warts_9WartsPing_6__setstate_cython__[] = "WartsPing.__setstate_cython__(self, __pyx_state)";
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_9WartsPing_7__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_9WartsPing_6__setstate_cython__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10traceutils_7scamper_5warts_9WartsPing_6__setstate_cython__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_WartsPing, (type(self), 0x5752e4d, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_WartsPing__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(2, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_10traceutils_7scamper_5warts___pyx_unpickle_WartsPing__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_WartsPing, (type(self), 0x5752e4d, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_WartsPing__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsPing.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "traceutils/scamper/warts.pyx":107
 * 
 * 
 * cdef list create_responses(list responses, int family):             # <<<<<<<<<<<<<<
 *     cdef dict resp
 *     resps = []
 */

static PyObject *__pyx_f_10traceutils_7scamper_5warts_create_responses(PyObject *__pyx_v_responses, int __pyx_v_family) {
  PyObject *__pyx_v_resp = 0;
  PyObject *__pyx_v_resps = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  Py_ssize_t __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  __Pyx_RefNannySetupContext("create_responses", 0);

  /* "traceutils/scamper/warts.pyx":109
 * cdef list create_responses(list responses, int family):
 *     cdef dict resp
 *     resps = []             # <<<<<<<<<<<<<<
 *     for resp in responses:
 *         resp.pop('from', None)
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 109, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_resps = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "traceutils/scamper/warts.pyx":110
 *     cdef dict resp
 *     resps = []
 *     for resp in responses:             # <<<<<<<<<<<<<<
 *         resp.pop('from', None)
 *         resps.append(WartsPingResponse(family=family, **resp))
 */
  if (unlikely(__pyx_v_responses == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    __PYX_ERR(0, 110, __pyx_L1_error)
  }
  __pyx_t_1 = __pyx_v_responses; __Pyx_INCREF(__pyx_t_1); __pyx_t_2 = 0;
  for (;;) {
    if (__pyx_t_2 >= PyList_GET_SIZE(__pyx_t_1)) break;
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    __pyx_t_3 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_2); __Pyx_INCREF(__pyx_t_3); __pyx_t_2++; if (unlikely(0 < 0)) __PYX_ERR(0, 110, __pyx_L1_error)
    #else
    __pyx_t_3 = PySequence_ITEM(__pyx_t_1, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 110, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    #endif
    if (!(likely(PyDict_CheckExact(__pyx_t_3))||((__pyx_t_3) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_t_3)->tp_name), 0))) __PYX_ERR(0, 110, __pyx_L1_error)
    __Pyx_XDECREF_SET(__pyx_v_resp, ((PyObject*)__pyx_t_3));
    __pyx_t_3 = 0;

    /* "traceutils/scamper/warts.pyx":111
 *     resps = []
 *     for resp in responses:
 *         resp.pop('from', None)             # <<<<<<<<<<<<<<
 *         resps.append(WartsPingResponse(family=family, **resp))
 *     return resps
 */
    if (unlikely(__pyx_v_resp == Py_None)) {
      PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "pop");
      __PYX_ERR(0, 111, __pyx_L1_error)
    }
    __pyx_t_3 = __Pyx_PyDict_Pop(__pyx_v_resp, __pyx_n_u_from, Py_None); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 111, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "traceutils/scamper/warts.pyx":112
 *     for resp in responses:
 *         resp.pop('from', None)
 *         resps.append(WartsPingResponse(family=family, **resp))             # <<<<<<<<<<<<<<
 *     return resps
 * 
 */
    __pyx_t_4 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 112, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = __Pyx_PyInt_From_int(__pyx_v_family); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 112, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_family, __pyx_t_5) < 0) __PYX_ERR(0, 112, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_3 = __pyx_t_4;
    __pyx_t_4 = 0;
    if (unlikely(__pyx_v_resp == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "argument after ** must be a mapping, not NoneType");
      __PYX_ERR(0, 112, __pyx_L1_error)
    }
    if (__Pyx_MergeKeywords(__pyx_t_3, __pyx_v_resp) < 0) __PYX_ERR(0, 112, __pyx_L1_error)
    __pyx_t_4 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_10traceutils_7scamper_5warts_WartsPingResponse), __pyx_empty_tuple, __pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 112, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_6 = __Pyx_PyList_Append(__pyx_v_resps, __pyx_t_4); if (unlikely(__pyx_t_6 == ((int)-1))) __PYX_ERR(0, 112, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "traceutils/scamper/warts.pyx":110
 *     cdef dict resp
 *     resps = []
 *     for resp in responses:             # <<<<<<<<<<<<<<
 *         resp.pop('from', None)
 *         resps.append(WartsPingResponse(family=family, **resp))
 */
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "traceutils/scamper/warts.pyx":113
 *         resp.pop('from', None)
 *         resps.append(WartsPingResponse(family=family, **resp))
 *     return resps             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_resps);
  __pyx_r = __pyx_v_resps;
  goto __pyx_L0;

  /* "traceutils/scamper/warts.pyx":107
 * 
 * 
 * cdef list create_responses(list responses, int family):             # <<<<<<<<<<<<<<
 *     cdef dict resp
 *     resps = []
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("traceutils.scamper.warts.create_responses", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_resp);
  __Pyx_XDECREF(__pyx_v_resps);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "traceutils/scamper/warts.pyx":117
 * 
 * cdef class WartsPingResponse:
 *     def __init__(self, int seq=-1, int reply_size=-1, int reply_ttl=-1, str reply_proto=None, dict tx=None, dict rx=None, double rtt=-1, int probe_ipid=-1, int reply_ipid=-1, int icmp_type=-1, int icmp_code=-1, int family=0):             # <<<<<<<<<<<<<<
 *         self.seq = seq
 *         self.reply_size = reply_size
 */

/* Python wrapper */
static int __pyx_pw_10traceutils_7scamper_5warts_17WartsPingResponse_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_10traceutils_7scamper_5warts_17WartsPingResponse_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  int __pyx_v_seq;
  int __pyx_v_reply_size;
  int __pyx_v_reply_ttl;
  PyObject *__pyx_v_reply_proto = 0;
  PyObject *__pyx_v_tx = 0;
  PyObject *__pyx_v_rx = 0;
  double __pyx_v_rtt;
  int __pyx_v_probe_ipid;
  int __pyx_v_reply_ipid;
  int __pyx_v_icmp_type;
  int __pyx_v_icmp_code;
  int __pyx_v_family;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_seq,&__pyx_n_s_reply_size,&__pyx_n_s_reply_ttl,&__pyx_n_s_reply_proto,&__pyx_n_s_tx,&__pyx_n_s_rx,&__pyx_n_s_rtt,&__pyx_n_s_probe_ipid,&__pyx_n_s_reply_ipid,&__pyx_n_s_icmp_type,&__pyx_n_s_icmp_code,&__pyx_n_s_family,0};
    PyObject* values[12] = {0,0,0,0,0,0,0,0,0,0,0,0};
    values[3] = ((PyObject*)Py_None);
    values[4] = ((PyObject*)Py_None);
    values[5] = ((PyObject*)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case 12: values[11] = PyTuple_GET_ITEM(__pyx_args, 11);
        CYTHON_FALLTHROUGH;
        case 11: values[10] = PyTuple_GET_ITEM(__pyx_args, 10);
        CYTHON_FALLTHROUGH;
        case 10: values[9] = PyTuple_GET_ITEM(__pyx_args, 9);
        CYTHON_FALLTHROUGH;
        case  9: values[8] = PyTuple_GET_ITEM(__pyx_args, 8);
        CYTHON_FALLTHROUGH;
        case  8: values[7] = PyTuple_GET_ITEM(__pyx_args, 7);
        CYTHON_FALLTHROUGH;
        case  7: values[6] = PyTuple_GET_ITEM(__pyx_args, 6);
        CYTHON_FALLTHROUGH;
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        CYTHON_FALLTHROUGH;
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_seq);
          if (value) { values[0] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_reply_size);
          if (value) { values[1] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_reply_ttl);
          if (value) { values[2] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_reply_proto);
          if (value) { values[3] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  4:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_tx);
          if (value) { values[4] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  5:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_rx);
          if (value) { values[5] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  6:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_rtt);
          if (value) { values[6] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  7:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_probe_ipid);
          if (value) { values[7] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  8:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_reply_ipid);
          if (value) { values[8] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  9:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_icmp_type);
          if (value) { values[9] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case 10:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_icmp_code);
          if (value) { values[10] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case 11:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_family);
          if (value) { values[11] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 117, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case 12: values[11] = PyTuple_GET_ITEM(__pyx_args, 11);
        CYTHON_FALLTHROUGH;
        case 11: values[10] = PyTuple_GET_ITEM(__pyx_args, 10);
        CYTHON_FALLTHROUGH;
        case 10: values[9] = PyTuple_GET_ITEM(__pyx_args, 9);
        CYTHON_FALLTHROUGH;
        case  9: values[8] = PyTuple_GET_ITEM(__pyx_args, 8);
        CYTHON_FALLTHROUGH;
        case  8: values[7] = PyTuple_GET_ITEM(__pyx_args, 7);
        CYTHON_FALLTHROUGH;
        case  7: values[6] = PyTuple_GET_ITEM(__pyx_args, 6);
        CYTHON_FALLTHROUGH;
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        CYTHON_FALLTHROUGH;
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    if (values[0]) {
      __pyx_v_seq = __Pyx_PyInt_As_int(values[0]); if (unlikely((__pyx_v_seq == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 117, __pyx_L3_error)
    } else {
      __pyx_v_seq = ((int)-1);
    }
    if (values[1]) {
      __pyx_v_reply_size = __Pyx_PyInt_As_int(values[1]); if (unlikely((__pyx_v_reply_size == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 117, __pyx_L3_error)
    } else {
      __pyx_v_reply_size = ((int)-1);
    }
    if (values[2]) {
      __pyx_v_reply_ttl = __Pyx_PyInt_As_int(values[2]); if (unlikely((__pyx_v_reply_ttl == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 117, __pyx_L3_error)
    } else {
      __pyx_v_reply_ttl = ((int)-1);
    }
    __pyx_v_reply_proto = ((PyObject*)values[3]);
    __pyx_v_tx = ((PyObject*)values[4]);
    __pyx_v_rx = ((PyObject*)values[5]);
    if (values[6]) {
      __pyx_v_rtt = __pyx_PyFloat_AsDouble(values[6]); if (unlikely((__pyx_v_rtt == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 117, __pyx_L3_error)
    } else {
      __pyx_v_rtt = ((double)-1.0);
    }
    if (values[7]) {
      __pyx_v_probe_ipid = __Pyx_PyInt_As_int(values[7]); if (unlikely((__pyx_v_probe_ipid == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 117, __pyx_L3_error)
    } else {
      __pyx_v_probe_ipid = ((int)-1);
    }
    if (values[8]) {
      __pyx_v_reply_ipid = __Pyx_PyInt_As_int(values[8]); if (unlikely((__pyx_v_reply_ipid == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 117, __pyx_L3_error)
    } else {
      __pyx_v_reply_ipid = ((int)-1);
    }
    if (values[9]) {
      __pyx_v_icmp_type = __Pyx_PyInt_As_int(values[9]); if (unlikely((__pyx_v_icmp_type == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 117, __pyx_L3_error)
    } else {
      __pyx_v_icmp_type = ((int)-1);
    }
    if (values[10]) {
      __pyx_v_icmp_code = __Pyx_PyInt_As_int(values[10]); if (unlikely((__pyx_v_icmp_code == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 117, __pyx_L3_error)
    } else {
      __pyx_v_icmp_code = ((int)-1);
    }
    if (values[11]) {
      __pyx_v_family = __Pyx_PyInt_As_int(values[11]); if (unlikely((__pyx_v_family == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 117, __pyx_L3_error)
    } else {
      __pyx_v_family = ((int)0);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 0, 0, 12, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 117, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsPingResponse.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_reply_proto), (&PyUnicode_Type), 1, "reply_proto", 1))) __PYX_ERR(0, 117, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_tx), (&PyDict_Type), 1, "tx", 1))) __PYX_ERR(0, 117, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_rx), (&PyDict_Type), 1, "rx", 1))) __PYX_ERR(0, 117, __pyx_L1_error)
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_17WartsPingResponse___init__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *)__pyx_v_self), __pyx_v_seq, __pyx_v_reply_size, __pyx_v_reply_ttl, __pyx_v_reply_proto, __pyx_v_tx, __pyx_v_rx, __pyx_v_rtt, __pyx_v_probe_ipid, __pyx_v_reply_ipid, __pyx_v_icmp_type, __pyx_v_icmp_code, __pyx_v_family);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_10traceutils_7scamper_5warts_17WartsPingResponse___init__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *__pyx_v_self, int __pyx_v_seq, int __pyx_v_reply_size, int __pyx_v_reply_ttl, PyObject *__pyx_v_reply_proto, PyObject *__pyx_v_tx, PyObject *__pyx_v_rx, double __pyx_v_rtt, int __pyx_v_probe_ipid, int __pyx_v_reply_ipid, int __pyx_v_icmp_type, int __pyx_v_icmp_code, int __pyx_v_family) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  enum __pyx_t_10traceutils_7scamper_3hop_ICMPType __pyx_t_1;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "traceutils/scamper/warts.pyx":118
 * cdef class WartsPingResponse:
 *     def __init__(self, int seq=-1, int reply_size=-1, int reply_ttl=-1, str reply_proto=None, dict tx=None, dict rx=None, double rtt=-1, int probe_ipid=-1, int reply_ipid=-1, int icmp_type=-1, int icmp_code=-1, int family=0):
 *         self.seq = seq             # <<<<<<<<<<<<<<
 *         self.reply_size = reply_size
 *         self.reply_ttl = reply_ttl
 */
  __pyx_v_self->seq = __pyx_v_seq;

  /* "traceutils/scamper/warts.pyx":119
 *     def __init__(self, int seq=-1, int reply_size=-1, int reply_ttl=-1, str reply_proto=None, dict tx=None, dict rx=None, double rtt=-1, int probe_ipid=-1, int reply_ipid=-1, int icmp_type=-1, int icmp_code=-1, int family=0):
 *         self.seq = seq
 *         self.reply_size = reply_size             # <<<<<<<<<<<<<<
 *         self.reply_ttl = reply_ttl
 *         self.reply_proto = reply_proto
 */
  __pyx_v_self->reply_size = __pyx_v_reply_size;

  /* "traceutils/scamper/warts.pyx":120
 *         self.seq = seq
 *         self.reply_size = reply_size
 *         self.reply_ttl = reply_ttl             # <<<<<<<<<<<<<<
 *         self.reply_proto = reply_proto
 *         self.tx = tx
 */
  __pyx_v_self->reply_ttl = __pyx_v_reply_ttl;

  /* "traceutils/scamper/warts.pyx":121
 *         self.reply_size = reply_size
 *         self.reply_ttl = reply_ttl
 *         self.reply_proto = reply_proto             # <<<<<<<<<<<<<<
 *         self.tx = tx
 *         self.rx = rx
 */
  __Pyx_INCREF(__pyx_v_reply_proto);
  __Pyx_GIVEREF(__pyx_v_reply_proto);
  __Pyx_GOTREF(__pyx_v_self->reply_proto);
  __Pyx_DECREF(__pyx_v_self->reply_proto);
  __pyx_v_self->reply_proto = __pyx_v_reply_proto;

  /* "traceutils/scamper/warts.pyx":122
 *         self.reply_ttl = reply_ttl
 *         self.reply_proto = reply_proto
 *         self.tx = tx             # <<<<<<<<<<<<<<
 *         self.rx = rx
 *         self.rtt = rtt
 */
  __Pyx_INCREF(__pyx_v_tx);
  __Pyx_GIVEREF(__pyx_v_tx);
  __Pyx_GOTREF(__pyx_v_self->tx);
  __Pyx_DECREF(__pyx_v_self->tx);
  __pyx_v_self->tx = __pyx_v_tx;

  /* "traceutils/scamper/warts.pyx":123
 *         self.reply_proto = reply_proto
 *         self.tx = tx
 *         self.rx = rx             # <<<<<<<<<<<<<<
 *         self.rtt = rtt
 *         self.probe_ipid = probe_ipid
 */
  __Pyx_INCREF(__pyx_v_rx);
  __Pyx_GIVEREF(__pyx_v_rx);
  __Pyx_GOTREF(__pyx_v_self->rx);
  __Pyx_DECREF(__pyx_v_self->rx);
  __pyx_v_self->rx = __pyx_v_rx;

  /* "traceutils/scamper/warts.pyx":124
 *         self.tx = tx
 *         self.rx = rx
 *         self.rtt = rtt             # <<<<<<<<<<<<<<
 *         self.probe_ipid = probe_ipid
 *         self.reply_ipid = reply_ipid
 */
  __pyx_v_self->rtt = __pyx_v_rtt;

  /* "traceutils/scamper/warts.pyx":125
 *         self.rx = rx
 *         self.rtt = rtt
 *         self.probe_ipid = probe_ipid             # <<<<<<<<<<<<<<
 *         self.reply_ipid = reply_ipid
 *         self.icmp_type = icmp_type
 */
  __pyx_v_self->probe_ipid = __pyx_v_probe_ipid;

  /* "traceutils/scamper/warts.pyx":126
 *         self.rtt = rtt
 *         self.probe_ipid = probe_ipid
 *         self.reply_ipid = reply_ipid             # <<<<<<<<<<<<<<
 *         self.icmp_type = icmp_type
 *         self.icmp_code = icmp_code
 */
  __pyx_v_self->reply_ipid = __pyx_v_reply_ipid;

  /* "traceutils/scamper/warts.pyx":127
 *         self.probe_ipid = probe_ipid
 *         self.reply_ipid = reply_ipid
 *         self.icmp_type = icmp_type             # <<<<<<<<<<<<<<
 *         self.icmp_code = icmp_code
 *         self.family = family
 */
  __pyx_v_self->icmp_type = __pyx_v_icmp_type;

  /* "traceutils/scamper/warts.pyx":128
 *         self.reply_ipid = reply_ipid
 *         self.icmp_type = icmp_type
 *         self.icmp_code = icmp_code             # <<<<<<<<<<<<<<
 *         self.family = family
 *         self.type = gettype(family, icmp_type)
 */
  __pyx_v_self->icmp_code = __pyx_v_icmp_code;

  /* "traceutils/scamper/warts.pyx":129
 *         self.icmp_type = icmp_type
 *         self.icmp_code = icmp_code
 *         self.family = family             # <<<<<<<<<<<<<<
 *         self.type = gettype(family, icmp_type)
 * 
 */
  __pyx_v_self->family = __pyx_v_family;

  /* "traceutils/scamper/warts.pyx":130
 *         self.icmp_code = icmp_code
 *         self.family = family
 *         self.type = gettype(family, icmp_type)             # <<<<<<<<<<<<<<
 * 
 *     def __repr__(self):
 */
  __pyx_t_1 = __pyx_f_10traceutils_7scamper_3hop_gettype(__pyx_v_family, __pyx_v_icmp_type, 0); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 130, __pyx_L1_error)
  __pyx_v_self->type = __pyx_t_1;

  /* "traceutils/scamper/warts.pyx":117
 * 
 * cdef class WartsPingResponse:
 *     def __init__(self, int seq=-1, int reply_size=-1, int reply_ttl=-1, str reply_proto=None, dict tx=None, dict rx=None, double rtt=-1, int probe_ipid=-1, int reply_ipid=-1, int icmp_type=-1, int icmp_code=-1, int family=0):             # <<<<<<<<<<<<<<
 *         self.seq = seq
 *         self.reply_size = reply_size
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsPingResponse.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "traceutils/scamper/warts.pyx":132
 *         self.type = gettype(family, icmp_type)
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return 'RTT={rtt}'.format(rtt=self.rtt)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_17WartsPingResponse_3__repr__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_17WartsPingResponse_3__repr__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__repr__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_17WartsPingResponse_2__repr__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10traceutils_7scamper_5warts_17WartsPingResponse_2__repr__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_RefNannySetupContext("__repr__", 0);

  /* "traceutils/scamper/warts.pyx":133
 * 
 *     def __repr__(self):
 *         return 'RTT={rtt}'.format(rtt=self.rtt)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_RTT_rtt, __pyx_n_s_format); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 133, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 133, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyFloat_FromDouble(__pyx_v_self->rtt); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 133, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_rtt, __pyx_t_3) < 0) __PYX_ERR(0, 133, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 133, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "traceutils/scamper/warts.pyx":132
 *         self.type = gettype(family, icmp_type)
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return 'RTT={rtt}'.format(rtt=self.rtt)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsPingResponse.__repr__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "traceutils/scamper/warts.pxd":42
 * 
 * cdef class WartsPingResponse:
 *     cdef public int seq, reply_size, reply_ttl, probe_ipid, reply_ipid, icmp_type, icmp_code             # <<<<<<<<<<<<<<
 *     cdef public str reply_proto
 *     cdef public dict tx, rx
 */

/* Python wrapper */
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_17WartsPingResponse_3seq_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_17WartsPingResponse_3seq_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_17WartsPingResponse_3seq___get__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10traceutils_7scamper_5warts_17WartsPingResponse_3seq___get__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->seq); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 42, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsPingResponse.seq.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_10traceutils_7scamper_5warts_17WartsPingResponse_3seq_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_10traceutils_7scamper_5warts_17WartsPingResponse_3seq_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_17WartsPingResponse_3seq_2__set__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_10traceutils_7scamper_5warts_17WartsPingResponse_3seq_2__set__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyInt_As_int(__pyx_v_value); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 42, __pyx_L1_error)
  __pyx_v_self->seq = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsPingResponse.seq.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_17WartsPingResponse_10reply_size_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_17WartsPingResponse_10reply_size_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_17WartsPingResponse_10reply_size___get__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10traceutils_7scamper_5warts_17WartsPingResponse_10reply_size___get__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->reply_size); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 42, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsPingResponse.reply_size.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_10traceutils_7scamper_5warts_17WartsPingResponse_10reply_size_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_10traceutils_7scamper_5warts_17WartsPingResponse_10reply_size_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_17WartsPingResponse_10reply_size_2__set__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_10traceutils_7scamper_5warts_17WartsPingResponse_10reply_size_2__set__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyInt_As_int(__pyx_v_value); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 42, __pyx_L1_error)
  __pyx_v_self->reply_size = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsPingResponse.reply_size.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_17WartsPingResponse_9reply_ttl_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_17WartsPingResponse_9reply_ttl_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_17WartsPingResponse_9reply_ttl___get__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10traceutils_7scamper_5warts_17WartsPingResponse_9reply_ttl___get__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->reply_ttl); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 42, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsPingResponse.reply_ttl.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_10traceutils_7scamper_5warts_17WartsPingResponse_9reply_ttl_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_10traceutils_7scamper_5warts_17WartsPingResponse_9reply_ttl_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_17WartsPingResponse_9reply_ttl_2__set__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_10traceutils_7scamper_5warts_17WartsPingResponse_9reply_ttl_2__set__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyInt_As_int(__pyx_v_value); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 42, __pyx_L1_error)
  __pyx_v_self->reply_ttl = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsPingResponse.reply_ttl.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_17WartsPingResponse_10probe_ipid_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_17WartsPingResponse_10probe_ipid_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_17WartsPingResponse_10probe_ipid___get__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10traceutils_7scamper_5warts_17WartsPingResponse_10probe_ipid___get__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->probe_ipid); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 42, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsPingResponse.probe_ipid.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_10traceutils_7scamper_5warts_17WartsPingResponse_10probe_ipid_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_10traceutils_7scamper_5warts_17WartsPingResponse_10probe_ipid_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_17WartsPingResponse_10probe_ipid_2__set__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_10traceutils_7scamper_5warts_17WartsPingResponse_10probe_ipid_2__set__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyInt_As_int(__pyx_v_value); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 42, __pyx_L1_error)
  __pyx_v_self->probe_ipid = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsPingResponse.probe_ipid.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_17WartsPingResponse_10reply_ipid_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_17WartsPingResponse_10reply_ipid_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_17WartsPingResponse_10reply_ipid___get__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10traceutils_7scamper_5warts_17WartsPingResponse_10reply_ipid___get__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->reply_ipid); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 42, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsPingResponse.reply_ipid.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_10traceutils_7scamper_5warts_17WartsPingResponse_10reply_ipid_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_10traceutils_7scamper_5warts_17WartsPingResponse_10reply_ipid_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_17WartsPingResponse_10reply_ipid_2__set__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_10traceutils_7scamper_5warts_17WartsPingResponse_10reply_ipid_2__set__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyInt_As_int(__pyx_v_value); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 42, __pyx_L1_error)
  __pyx_v_self->reply_ipid = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsPingResponse.reply_ipid.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_17WartsPingResponse_9icmp_type_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_17WartsPingResponse_9icmp_type_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_17WartsPingResponse_9icmp_type___get__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10traceutils_7scamper_5warts_17WartsPingResponse_9icmp_type___get__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->icmp_type); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 42, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsPingResponse.icmp_type.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_10traceutils_7scamper_5warts_17WartsPingResponse_9icmp_type_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_10traceutils_7scamper_5warts_17WartsPingResponse_9icmp_type_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_17WartsPingResponse_9icmp_type_2__set__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_10traceutils_7scamper_5warts_17WartsPingResponse_9icmp_type_2__set__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyInt_As_int(__pyx_v_value); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 42, __pyx_L1_error)
  __pyx_v_self->icmp_type = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsPingResponse.icmp_type.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_17WartsPingResponse_9icmp_code_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_17WartsPingResponse_9icmp_code_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_17WartsPingResponse_9icmp_code___get__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10traceutils_7scamper_5warts_17WartsPingResponse_9icmp_code___get__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->icmp_code); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 42, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsPingResponse.icmp_code.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_10traceutils_7scamper_5warts_17WartsPingResponse_9icmp_code_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_10traceutils_7scamper_5warts_17WartsPingResponse_9icmp_code_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_17WartsPingResponse_9icmp_code_2__set__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_10traceutils_7scamper_5warts_17WartsPingResponse_9icmp_code_2__set__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyInt_As_int(__pyx_v_value); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 42, __pyx_L1_error)
  __pyx_v_self->icmp_code = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsPingResponse.icmp_code.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "traceutils/scamper/warts.pxd":43
 * cdef class WartsPingResponse:
 *     cdef public int seq, reply_size, reply_ttl, probe_ipid, reply_ipid, icmp_type, icmp_code
 *     cdef public str reply_proto             # <<<<<<<<<<<<<<
 *     cdef public dict tx, rx
 *     cdef public double rtt
 */

/* Python wrapper */
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_17WartsPingResponse_11reply_proto_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_17WartsPingResponse_11reply_proto_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_17WartsPingResponse_11reply_proto___get__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10traceutils_7scamper_5warts_17WartsPingResponse_11reply_proto___get__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->reply_proto);
  __pyx_r = __pyx_v_self->reply_proto;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_10traceutils_7scamper_5warts_17WartsPingResponse_11reply_proto_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_10traceutils_7scamper_5warts_17WartsPingResponse_11reply_proto_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_17WartsPingResponse_11reply_proto_2__set__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_10traceutils_7scamper_5warts_17WartsPingResponse_11reply_proto_2__set__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(PyUnicode_CheckExact(__pyx_v_value))||((__pyx_v_value) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "unicode", Py_TYPE(__pyx_v_value)->tp_name), 0))) __PYX_ERR(1, 43, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->reply_proto);
  __Pyx_DECREF(__pyx_v_self->reply_proto);
  __pyx_v_self->reply_proto = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsPingResponse.reply_proto.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_10traceutils_7scamper_5warts_17WartsPingResponse_11reply_proto_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_10traceutils_7scamper_5warts_17WartsPingResponse_11reply_proto_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_17WartsPingResponse_11reply_proto_4__del__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_10traceutils_7scamper_5warts_17WartsPingResponse_11reply_proto_4__del__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->reply_proto);
  __Pyx_DECREF(__pyx_v_self->reply_proto);
  __pyx_v_self->reply_proto = ((PyObject*)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "traceutils/scamper/warts.pxd":44
 *     cdef public int seq, reply_size, reply_ttl, probe_ipid, reply_ipid, icmp_type, icmp_code
 *     cdef public str reply_proto
 *     cdef public dict tx, rx             # <<<<<<<<<<<<<<
 *     cdef public double rtt
 *     cdef public int family
 */

/* Python wrapper */
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_17WartsPingResponse_2tx_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_17WartsPingResponse_2tx_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_17WartsPingResponse_2tx___get__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10traceutils_7scamper_5warts_17WartsPingResponse_2tx___get__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->tx);
  __pyx_r = __pyx_v_self->tx;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_10traceutils_7scamper_5warts_17WartsPingResponse_2tx_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_10traceutils_7scamper_5warts_17WartsPingResponse_2tx_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_17WartsPingResponse_2tx_2__set__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_10traceutils_7scamper_5warts_17WartsPingResponse_2tx_2__set__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(PyDict_CheckExact(__pyx_v_value))||((__pyx_v_value) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_v_value)->tp_name), 0))) __PYX_ERR(1, 44, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->tx);
  __Pyx_DECREF(__pyx_v_self->tx);
  __pyx_v_self->tx = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsPingResponse.tx.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_10traceutils_7scamper_5warts_17WartsPingResponse_2tx_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_10traceutils_7scamper_5warts_17WartsPingResponse_2tx_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_17WartsPingResponse_2tx_4__del__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_10traceutils_7scamper_5warts_17WartsPingResponse_2tx_4__del__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->tx);
  __Pyx_DECREF(__pyx_v_self->tx);
  __pyx_v_self->tx = ((PyObject*)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_17WartsPingResponse_2rx_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_17WartsPingResponse_2rx_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_17WartsPingResponse_2rx___get__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10traceutils_7scamper_5warts_17WartsPingResponse_2rx___get__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->rx);
  __pyx_r = __pyx_v_self->rx;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_10traceutils_7scamper_5warts_17WartsPingResponse_2rx_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_10traceutils_7scamper_5warts_17WartsPingResponse_2rx_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_17WartsPingResponse_2rx_2__set__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_10traceutils_7scamper_5warts_17WartsPingResponse_2rx_2__set__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(PyDict_CheckExact(__pyx_v_value))||((__pyx_v_value) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_v_value)->tp_name), 0))) __PYX_ERR(1, 44, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->rx);
  __Pyx_DECREF(__pyx_v_self->rx);
  __pyx_v_self->rx = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsPingResponse.rx.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_10traceutils_7scamper_5warts_17WartsPingResponse_2rx_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_10traceutils_7scamper_5warts_17WartsPingResponse_2rx_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_17WartsPingResponse_2rx_4__del__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_10traceutils_7scamper_5warts_17WartsPingResponse_2rx_4__del__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->rx);
  __Pyx_DECREF(__pyx_v_self->rx);
  __pyx_v_self->rx = ((PyObject*)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "traceutils/scamper/warts.pxd":45
 *     cdef public str reply_proto
 *     cdef public dict tx, rx
 *     cdef public double rtt             # <<<<<<<<<<<<<<
 *     cdef public int family
 *     cdef public ICMPType type
 */

/* Python wrapper */
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_17WartsPingResponse_3rtt_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_17WartsPingResponse_3rtt_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_17WartsPingResponse_3rtt___get__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10traceutils_7scamper_5warts_17WartsPingResponse_3rtt___get__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->rtt); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 45, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsPingResponse.rtt.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_10traceutils_7scamper_5warts_17WartsPingResponse_3rtt_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_10traceutils_7scamper_5warts_17WartsPingResponse_3rtt_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_17WartsPingResponse_3rtt_2__set__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_10traceutils_7scamper_5warts_17WartsPingResponse_3rtt_2__set__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  double __pyx_t_1;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __pyx_PyFloat_AsDouble(__pyx_v_value); if (unlikely((__pyx_t_1 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 45, __pyx_L1_error)
  __pyx_v_self->rtt = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsPingResponse.rtt.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "traceutils/scamper/warts.pxd":46
 *     cdef public dict tx, rx
 *     cdef public double rtt
 *     cdef public int family             # <<<<<<<<<<<<<<
 *     cdef public ICMPType type
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_17WartsPingResponse_6family_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_17WartsPingResponse_6family_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_17WartsPingResponse_6family___get__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10traceutils_7scamper_5warts_17WartsPingResponse_6family___get__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->family); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 46, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsPingResponse.family.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_10traceutils_7scamper_5warts_17WartsPingResponse_6family_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_10traceutils_7scamper_5warts_17WartsPingResponse_6family_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_17WartsPingResponse_6family_2__set__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_10traceutils_7scamper_5warts_17WartsPingResponse_6family_2__set__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyInt_As_int(__pyx_v_value); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 46, __pyx_L1_error)
  __pyx_v_self->family = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsPingResponse.family.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "traceutils/scamper/warts.pxd":47
 *     cdef public double rtt
 *     cdef public int family
 *     cdef public ICMPType type             # <<<<<<<<<<<<<<
 * 
 * cdef class WartsReader(Reader):
 */

/* Python wrapper */
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_17WartsPingResponse_4type_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_17WartsPingResponse_4type_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_17WartsPingResponse_4type___get__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10traceutils_7scamper_5warts_17WartsPingResponse_4type___get__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_enum____pyx_t_10traceutils_7scamper_3hop_ICMPType(__pyx_v_self->type); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 47, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsPingResponse.type.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_10traceutils_7scamper_5warts_17WartsPingResponse_4type_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_10traceutils_7scamper_5warts_17WartsPingResponse_4type_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_17WartsPingResponse_4type_2__set__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_10traceutils_7scamper_5warts_17WartsPingResponse_4type_2__set__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  enum __pyx_t_10traceutils_7scamper_3hop_ICMPType __pyx_t_1;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = ((enum __pyx_t_10traceutils_7scamper_3hop_ICMPType)__Pyx_PyInt_As_enum____pyx_t_10traceutils_7scamper_3hop_ICMPType(__pyx_v_value)); if (unlikely(PyErr_Occurred())) __PYX_ERR(1, 47, __pyx_L1_error)
  __pyx_v_self->type = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsPingResponse.type.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_17WartsPingResponse_5__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_10traceutils_7scamper_5warts_17WartsPingResponse_4__reduce_cython__[] = "WartsPingResponse.__reduce_cython__(self)";
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_17WartsPingResponse_5__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_17WartsPingResponse_4__reduce_cython__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10traceutils_7scamper_5warts_17WartsPingResponse_4__reduce_cython__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  int __pyx_t_12;
  int __pyx_t_13;
  int __pyx_t_14;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self.family, self.icmp_code, self.icmp_type, self.probe_ipid, self.reply_ipid, self.reply_proto, self.reply_size, self.reply_ttl, self.rtt, self.rx, self.seq, self.tx, self.type)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->family); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_self->icmp_code); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_self->icmp_type); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyInt_From_int(__pyx_v_self->probe_ipid); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyInt_From_int(__pyx_v_self->reply_ipid); if (unlikely(!__pyx_t_5)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = __Pyx_PyInt_From_int(__pyx_v_self->reply_size); if (unlikely(!__pyx_t_6)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_7 = __Pyx_PyInt_From_int(__pyx_v_self->reply_ttl); if (unlikely(!__pyx_t_7)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_8 = PyFloat_FromDouble(__pyx_v_self->rtt); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_9 = __Pyx_PyInt_From_int(__pyx_v_self->seq); if (unlikely(!__pyx_t_9)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __pyx_t_10 = __Pyx_PyInt_From_enum____pyx_t_10traceutils_7scamper_3hop_ICMPType(__pyx_v_self->type); if (unlikely(!__pyx_t_10)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_11 = PyTuple_New(13); if (unlikely(!__pyx_t_11)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_11, 1, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_11, 2, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_11, 3, __pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_11, 4, __pyx_t_5);
  __Pyx_INCREF(__pyx_v_self->reply_proto);
  __Pyx_GIVEREF(__pyx_v_self->reply_proto);
  PyTuple_SET_ITEM(__pyx_t_11, 5, __pyx_v_self->reply_proto);
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_11, 6, __pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_7);
  PyTuple_SET_ITEM(__pyx_t_11, 7, __pyx_t_7);
  __Pyx_GIVEREF(__pyx_t_8);
  PyTuple_SET_ITEM(__pyx_t_11, 8, __pyx_t_8);
  __Pyx_INCREF(__pyx_v_self->rx);
  __Pyx_GIVEREF(__pyx_v_self->rx);
  PyTuple_SET_ITEM(__pyx_t_11, 9, __pyx_v_self->rx);
  __Pyx_GIVEREF(__pyx_t_9);
  PyTuple_SET_ITEM(__pyx_t_11, 10, __pyx_t_9);
  __Pyx_INCREF(__pyx_v_self->tx);
  __Pyx_GIVEREF(__pyx_v_self->tx);
  PyTuple_SET_ITEM(__pyx_t_11, 11, __pyx_v_self->tx);
  __Pyx_GIVEREF(__pyx_t_10);
  PyTuple_SET_ITEM(__pyx_t_11, 12, __pyx_t_10);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_5 = 0;
  __pyx_t_6 = 0;
  __pyx_t_7 = 0;
  __pyx_t_8 = 0;
  __pyx_t_9 = 0;
  __pyx_t_10 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_11);
  __pyx_t_11 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self.family, self.icmp_code, self.icmp_type, self.probe_ipid, self.reply_ipid, self.reply_proto, self.reply_size, self.reply_ttl, self.rtt, self.rx, self.seq, self.tx, self.type)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_11 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_11)) __PYX_ERR(2, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __pyx_v__dict = __pyx_t_11;
  __pyx_t_11 = 0;

  /* "(tree fragment)":7
 *     state = (self.family, self.icmp_code, self.icmp_type, self.probe_ipid, self.reply_ipid, self.reply_proto, self.reply_size, self.reply_ttl, self.rtt, self.rx, self.seq, self.tx, self.type)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_12 = (__pyx_v__dict != Py_None);
  __pyx_t_13 = (__pyx_t_12 != 0);
  if (__pyx_t_13) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_11 = PyTuple_New(1); if (unlikely(!__pyx_t_11)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_v__dict);
    __pyx_t_10 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_11); if (unlikely(!__pyx_t_10)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_10));
    __pyx_t_10 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self.reply_proto is not None or self.rx is not None or self.tx is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self.family, self.icmp_code, self.icmp_type, self.probe_ipid, self.reply_ipid, self.reply_proto, self.reply_size, self.reply_ttl, self.rtt, self.rx, self.seq, self.tx, self.type)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self.reply_proto is not None or self.rx is not None or self.tx is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_WartsPingResponse, (type(self), 0x78ac107, None), state
 */
  /*else*/ {
    __pyx_t_12 = (__pyx_v_self->reply_proto != ((PyObject*)Py_None));
    __pyx_t_14 = (__pyx_t_12 != 0);
    if (!__pyx_t_14) {
    } else {
      __pyx_t_13 = __pyx_t_14;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_14 = (__pyx_v_self->rx != ((PyObject*)Py_None));
    __pyx_t_12 = (__pyx_t_14 != 0);
    if (!__pyx_t_12) {
    } else {
      __pyx_t_13 = __pyx_t_12;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_12 = (__pyx_v_self->tx != ((PyObject*)Py_None));
    __pyx_t_14 = (__pyx_t_12 != 0);
    __pyx_t_13 = __pyx_t_14;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_13;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.reply_proto is not None or self.rx is not None or self.tx is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_WartsPingResponse, (type(self), 0x78ac107, None), state
 *     else:
 */
  __pyx_t_13 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_13) {

    /* "(tree fragment)":13
 *         use_setstate = self.reply_proto is not None or self.rx is not None or self.tx is not None
 *     if use_setstate:
 *         return __pyx_unpickle_WartsPingResponse, (type(self), 0x78ac107, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_WartsPingResponse, (type(self), 0x78ac107, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_10, __pyx_n_s_pyx_unpickle_WartsPingResponse); if (unlikely(!__pyx_t_10)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __pyx_t_11 = PyTuple_New(3); if (unlikely(!__pyx_t_11)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_11, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_126533895);
    __Pyx_GIVEREF(__pyx_int_126533895);
    PyTuple_SET_ITEM(__pyx_t_11, 1, __pyx_int_126533895);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_11, 2, Py_None);
    __pyx_t_9 = PyTuple_New(3); if (unlikely(!__pyx_t_9)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_GIVEREF(__pyx_t_10);
    PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_10);
    __Pyx_GIVEREF(__pyx_t_11);
    PyTuple_SET_ITEM(__pyx_t_9, 1, __pyx_t_11);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_9, 2, __pyx_v_state);
    __pyx_t_10 = 0;
    __pyx_t_11 = 0;
    __pyx_r = __pyx_t_9;
    __pyx_t_9 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.reply_proto is not None or self.rx is not None or self.tx is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_WartsPingResponse, (type(self), 0x78ac107, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_WartsPingResponse, (type(self), 0x78ac107, None), state
 *     else:
 *         return __pyx_unpickle_WartsPingResponse, (type(self), 0x78ac107, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_WartsPingResponse__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_9, __pyx_n_s_pyx_unpickle_WartsPingResponse); if (unlikely(!__pyx_t_9)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_11 = PyTuple_New(3); if (unlikely(!__pyx_t_11)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_11, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_126533895);
    __Pyx_GIVEREF(__pyx_int_126533895);
    PyTuple_SET_ITEM(__pyx_t_11, 1, __pyx_int_126533895);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_11, 2, __pyx_v_state);
    __pyx_t_10 = PyTuple_New(2); if (unlikely(!__pyx_t_10)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_GIVEREF(__pyx_t_9);
    PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_t_9);
    __Pyx_GIVEREF(__pyx_t_11);
    PyTuple_SET_ITEM(__pyx_t_10, 1, __pyx_t_11);
    __pyx_t_9 = 0;
    __pyx_t_11 = 0;
    __pyx_r = __pyx_t_10;
    __pyx_t_10 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsPingResponse.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_WartsPingResponse, (type(self), 0x78ac107, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_WartsPingResponse__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_17WartsPingResponse_7__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static char __pyx_doc_10traceutils_7scamper_5warts_17WartsPingResponse_6__setstate_cython__[] = "WartsPingResponse.__setstate_cython__(self, __pyx_state)";
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_17WartsPingResponse_7__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_17WartsPingResponse_6__setstate_cython__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10traceutils_7scamper_5warts_17WartsPingResponse_6__setstate_cython__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_WartsPingResponse, (type(self), 0x78ac107, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_WartsPingResponse__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(2, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_10traceutils_7scamper_5warts___pyx_unpickle_WartsPingResponse__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_WartsPingResponse, (type(self), 0x78ac107, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_WartsPingResponse__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsPingResponse.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "traceutils/scamper/warts.pyx":137
 * 
 * cdef class WartsReader(Reader):
 *     def __init__(self, str filename):             # <<<<<<<<<<<<<<
 *         self.filename = filename
 *         self.p = None
 */

/* Python wrapper */
static int __pyx_pw_10traceutils_7scamper_5warts_11WartsReader_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_10traceutils_7scamper_5warts_11WartsReader_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_filename = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_filename,0};
    PyObject* values[1] = {0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_filename)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 137, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 1) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
    }
    __pyx_v_filename = ((PyObject*)values[0]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 1, 1, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 137, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsReader.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_filename), (&PyUnicode_Type), 1, "filename", 1))) __PYX_ERR(0, 137, __pyx_L1_error)
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_11WartsReader___init__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsReader *)__pyx_v_self), __pyx_v_filename);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_10traceutils_7scamper_5warts_11WartsReader___init__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsReader *__pyx_v_self, PyObject *__pyx_v_filename) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "traceutils/scamper/warts.pyx":138
 * cdef class WartsReader(Reader):
 *     def __init__(self, str filename):
 *         self.filename = filename             # <<<<<<<<<<<<<<
 *         self.p = None
 * 
 */
  __Pyx_INCREF(__pyx_v_filename);
  __Pyx_GIVEREF(__pyx_v_filename);
  __Pyx_GOTREF(__pyx_v_self->__pyx_base.filename);
  __Pyx_DECREF(__pyx_v_self->__pyx_base.filename);
  __pyx_v_self->__pyx_base.filename = __pyx_v_filename;

  /* "traceutils/scamper/warts.pyx":139
 *     def __init__(self, str filename):
 *         self.filename = filename
 *         self.p = None             # <<<<<<<<<<<<<<
 * 
 *     def __iter__(self):
 */
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->__pyx_base.p);
  __Pyx_DECREF(__pyx_v_self->__pyx_base.p);
  __pyx_v_self->__pyx_base.p = Py_None;

  /* "traceutils/scamper/warts.pyx":137
 * 
 * cdef class WartsReader(Reader):
 *     def __init__(self, str filename):             # <<<<<<<<<<<<<<
 *         self.filename = filename
 *         self.p = None
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
static PyObject *__pyx_gb_10traceutils_7scamper_5warts_11WartsReader_4generator(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value); /* proto */

/* "traceutils/scamper/warts.pyx":141
 *         self.p = None
 * 
 *     def __iter__(self):             # <<<<<<<<<<<<<<
 *         cdef str line, rtype
 *         cdef dict j
 */

/* Python wrapper */
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_11WartsReader_3__iter__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_11WartsReader_3__iter__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__iter__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_11WartsReader_2__iter__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsReader *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10traceutils_7scamper_5warts_11WartsReader_2__iter__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsReader *__pyx_v_self) {
  struct __pyx_obj_10traceutils_7scamper_5warts___pyx_scope_struct____iter__ *__pyx_cur_scope;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__iter__", 0);
  __pyx_cur_scope = (struct __pyx_obj_10traceutils_7scamper_5warts___pyx_scope_struct____iter__ *)__pyx_tp_new_10traceutils_7scamper_5warts___pyx_scope_struct____iter__(__pyx_ptype_10traceutils_7scamper_5warts___pyx_scope_struct____iter__, __pyx_empty_tuple, NULL);
  if (unlikely(!__pyx_cur_scope)) {
    __pyx_cur_scope = ((struct __pyx_obj_10traceutils_7scamper_5warts___pyx_scope_struct____iter__ *)Py_None);
    __Pyx_INCREF(Py_None);
    __PYX_ERR(0, 141, __pyx_L1_error)
  } else {
    __Pyx_GOTREF(__pyx_cur_scope);
  }
  __pyx_cur_scope->__pyx_v_self = __pyx_v_self;
  __Pyx_INCREF((PyObject *)__pyx_cur_scope->__pyx_v_self);
  __Pyx_GIVEREF((PyObject *)__pyx_cur_scope->__pyx_v_self);
  {
    __pyx_CoroutineObject *gen = __Pyx_Generator_New((__pyx_coroutine_body_t) __pyx_gb_10traceutils_7scamper_5warts_11WartsReader_4generator, NULL, (PyObject *) __pyx_cur_scope, __pyx_n_s_iter, __pyx_n_s_WartsReader___iter, __pyx_n_s_traceutils_scamper_warts); if (unlikely(!gen)) __PYX_ERR(0, 141, __pyx_L1_error)
    __Pyx_DECREF(__pyx_cur_scope);
    __Pyx_RefNannyFinishContext();
    return (PyObject *) gen;
  }

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsReader.__iter__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_DECREF(((PyObject *)__pyx_cur_scope));
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_gb_10traceutils_7scamper_5warts_11WartsReader_4generator(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value) /* generator body */
{
  struct __pyx_obj_10traceutils_7scamper_5warts___pyx_scope_struct____iter__ *__pyx_cur_scope = ((struct __pyx_obj_10traceutils_7scamper_5warts___pyx_scope_struct____iter__ *)__pyx_generator->closure);
  PyObject *__pyx_r = NULL;
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  Py_ssize_t __pyx_t_3;
  PyObject *(*__pyx_t_4)(PyObject *);
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_t_8;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__iter__", 0);
  switch (__pyx_generator->resume_label) {
    case 0: goto __pyx_L3_first_run;
    case 1: goto __pyx_L7_resume_from_yield;
    case 2: goto __pyx_L8_resume_from_yield;
    default: /* CPython raises the right error here */
    __Pyx_RefNannyFinishContext();
    return NULL;
  }
  __pyx_L3_first_run:;
  if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 141, __pyx_L1_error)

  /* "traceutils/scamper/warts.pyx":144
 *         cdef str line, rtype
 *         cdef dict j
 *         for line in self.p.stdout:             # <<<<<<<<<<<<<<
 *             j = json.loads(line)
 *             rtype = j['type']
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_self->__pyx_base.p, __pyx_n_s_stdout); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 144, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (likely(PyList_CheckExact(__pyx_t_1)) || PyTuple_CheckExact(__pyx_t_1)) {
    __pyx_t_2 = __pyx_t_1; __Pyx_INCREF(__pyx_t_2); __pyx_t_3 = 0;
    __pyx_t_4 = NULL;
  } else {
    __pyx_t_3 = -1; __pyx_t_2 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 144, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = Py_TYPE(__pyx_t_2)->tp_iternext; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 144, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  for (;;) {
    if (likely(!__pyx_t_4)) {
      if (likely(PyList_CheckExact(__pyx_t_2))) {
        if (__pyx_t_3 >= PyList_GET_SIZE(__pyx_t_2)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_1 = PyList_GET_ITEM(__pyx_t_2, __pyx_t_3); __Pyx_INCREF(__pyx_t_1); __pyx_t_3++; if (unlikely(0 < 0)) __PYX_ERR(0, 144, __pyx_L1_error)
        #else
        __pyx_t_1 = PySequence_ITEM(__pyx_t_2, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 144, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        #endif
      } else {
        if (__pyx_t_3 >= PyTuple_GET_SIZE(__pyx_t_2)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_1 = PyTuple_GET_ITEM(__pyx_t_2, __pyx_t_3); __Pyx_INCREF(__pyx_t_1); __pyx_t_3++; if (unlikely(0 < 0)) __PYX_ERR(0, 144, __pyx_L1_error)
        #else
        __pyx_t_1 = PySequence_ITEM(__pyx_t_2, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 144, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        #endif
      }
    } else {
      __pyx_t_1 = __pyx_t_4(__pyx_t_2);
      if (unlikely(!__pyx_t_1)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 144, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_1);
    }
    if (!(likely(PyUnicode_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "unicode", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(0, 144, __pyx_L1_error)
    __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_line);
    __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_line, ((PyObject*)__pyx_t_1));
    __Pyx_GIVEREF(__pyx_t_1);
    __pyx_t_1 = 0;

    /* "traceutils/scamper/warts.pyx":145
 *         cdef dict j
 *         for line in self.p.stdout:
 *             j = json.loads(line)             # <<<<<<<<<<<<<<
 *             rtype = j['type']
 *             if rtype == 'trace':
 */
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_json); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 145, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_loads); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 145, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_6))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_6);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_6, function);
      }
    }
    __pyx_t_1 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_6, __pyx_t_5, __pyx_cur_scope->__pyx_v_line) : __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_cur_scope->__pyx_v_line);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 145, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (!(likely(PyDict_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(0, 145, __pyx_L1_error)
    __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_j);
    __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_j, ((PyObject*)__pyx_t_1));
    __Pyx_GIVEREF(__pyx_t_1);
    __pyx_t_1 = 0;

    /* "traceutils/scamper/warts.pyx":146
 *         for line in self.p.stdout:
 *             j = json.loads(line)
 *             rtype = j['type']             # <<<<<<<<<<<<<<
 *             if rtype == 'trace':
 *                 yield WartsTrace(**j)
 */
    if (unlikely(__pyx_cur_scope->__pyx_v_j == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(0, 146, __pyx_L1_error)
    }
    __pyx_t_1 = __Pyx_PyDict_GetItem(__pyx_cur_scope->__pyx_v_j, __pyx_n_u_type); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 146, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!(likely(PyUnicode_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "unicode", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(0, 146, __pyx_L1_error)
    __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_rtype);
    __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_rtype, ((PyObject*)__pyx_t_1));
    __Pyx_GIVEREF(__pyx_t_1);
    __pyx_t_1 = 0;

    /* "traceutils/scamper/warts.pyx":147
 *             j = json.loads(line)
 *             rtype = j['type']
 *             if rtype == 'trace':             # <<<<<<<<<<<<<<
 *                 yield WartsTrace(**j)
 *             elif rtype == 'ping':
 */
    __pyx_t_7 = (__Pyx_PyUnicode_Equals(__pyx_cur_scope->__pyx_v_rtype, __pyx_n_u_trace, Py_EQ)); if (unlikely(__pyx_t_7 < 0)) __PYX_ERR(0, 147, __pyx_L1_error)
    __pyx_t_8 = (__pyx_t_7 != 0);
    if (__pyx_t_8) {

      /* "traceutils/scamper/warts.pyx":148
 *             rtype = j['type']
 *             if rtype == 'trace':
 *                 yield WartsTrace(**j)             # <<<<<<<<<<<<<<
 *             elif rtype == 'ping':
 *                 yield WartsPing(**j)
 */
      if (unlikely(__pyx_cur_scope->__pyx_v_j == Py_None)) {
        PyErr_SetString(PyExc_TypeError, "argument after ** must be a mapping, not NoneType");
        __PYX_ERR(0, 148, __pyx_L1_error)
      }
      __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_10traceutils_7scamper_5warts_WartsTrace), __pyx_empty_tuple, __pyx_cur_scope->__pyx_v_j); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 148, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_r = __pyx_t_1;
      __pyx_t_1 = 0;
      __Pyx_XGIVEREF(__pyx_t_2);
      __pyx_cur_scope->__pyx_t_0 = __pyx_t_2;
      __pyx_cur_scope->__pyx_t_1 = __pyx_t_3;
      __pyx_cur_scope->__pyx_t_2 = __pyx_t_4;
      __Pyx_XGIVEREF(__pyx_r);
      __Pyx_RefNannyFinishContext();
      __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
      /* return from generator, yielding value */
      __pyx_generator->resume_label = 1;
      return __pyx_r;
      __pyx_L7_resume_from_yield:;
      __pyx_t_2 = __pyx_cur_scope->__pyx_t_0;
      __pyx_cur_scope->__pyx_t_0 = 0;
      __Pyx_XGOTREF(__pyx_t_2);
      __pyx_t_3 = __pyx_cur_scope->__pyx_t_1;
      __pyx_t_4 = __pyx_cur_scope->__pyx_t_2;
      if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 148, __pyx_L1_error)

      /* "traceutils/scamper/warts.pyx":147
 *             j = json.loads(line)
 *             rtype = j['type']
 *             if rtype == 'trace':             # <<<<<<<<<<<<<<
 *                 yield WartsTrace(**j)
 *             elif rtype == 'ping':
 */
      goto __pyx_L6;
    }

    /* "traceutils/scamper/warts.pyx":149
 *             if rtype == 'trace':
 *                 yield WartsTrace(**j)
 *             elif rtype == 'ping':             # <<<<<<<<<<<<<<
 *                 yield WartsPing(**j)
 * 
 */
    __pyx_t_8 = (__Pyx_PyUnicode_Equals(__pyx_cur_scope->__pyx_v_rtype, __pyx_n_u_ping, Py_EQ)); if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(0, 149, __pyx_L1_error)
    __pyx_t_7 = (__pyx_t_8 != 0);
    if (__pyx_t_7) {

      /* "traceutils/scamper/warts.pyx":150
 *                 yield WartsTrace(**j)
 *             elif rtype == 'ping':
 *                 yield WartsPing(**j)             # <<<<<<<<<<<<<<
 * 
 *     cpdef void open(self) except *:
 */
      if (unlikely(__pyx_cur_scope->__pyx_v_j == Py_None)) {
        PyErr_SetString(PyExc_TypeError, "argument after ** must be a mapping, not NoneType");
        __PYX_ERR(0, 150, __pyx_L1_error)
      }
      __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_10traceutils_7scamper_5warts_WartsPing), __pyx_empty_tuple, __pyx_cur_scope->__pyx_v_j); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 150, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_r = __pyx_t_1;
      __pyx_t_1 = 0;
      __Pyx_XGIVEREF(__pyx_t_2);
      __pyx_cur_scope->__pyx_t_0 = __pyx_t_2;
      __pyx_cur_scope->__pyx_t_1 = __pyx_t_3;
      __pyx_cur_scope->__pyx_t_2 = __pyx_t_4;
      __Pyx_XGIVEREF(__pyx_r);
      __Pyx_RefNannyFinishContext();
      __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
      /* return from generator, yielding value */
      __pyx_generator->resume_label = 2;
      return __pyx_r;
      __pyx_L8_resume_from_yield:;
      __pyx_t_2 = __pyx_cur_scope->__pyx_t_0;
      __pyx_cur_scope->__pyx_t_0 = 0;
      __Pyx_XGOTREF(__pyx_t_2);
      __pyx_t_3 = __pyx_cur_scope->__pyx_t_1;
      __pyx_t_4 = __pyx_cur_scope->__pyx_t_2;
      if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 150, __pyx_L1_error)

      /* "traceutils/scamper/warts.pyx":149
 *             if rtype == 'trace':
 *                 yield WartsTrace(**j)
 *             elif rtype == 'ping':             # <<<<<<<<<<<<<<
 *                 yield WartsPing(**j)
 * 
 */
    }
    __pyx_L6:;

    /* "traceutils/scamper/warts.pyx":144
 *         cdef str line, rtype
 *         cdef dict j
 *         for line in self.p.stdout:             # <<<<<<<<<<<<<<
 *             j = json.loads(line)
 *             rtype = j['type']
 */
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  CYTHON_MAYBE_UNUSED_VAR(__pyx_cur_scope);

  /* "traceutils/scamper/warts.pyx":141
 *         self.p = None
 * 
 *     def __iter__(self):             # <<<<<<<<<<<<<<
 *         cdef str line, rtype
 *         cdef dict j
 */

  /* function exit code */
  PyErr_SetNone(PyExc_StopIteration);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("__iter__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_r); __pyx_r = 0;
  #if !CYTHON_USE_EXC_INFO_STACK
  __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
  #endif
  __pyx_generator->resume_label = -1;
  __Pyx_Coroutine_clear((PyObject*)__pyx_generator);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "traceutils/scamper/warts.pyx":152
 *                 yield WartsPing(**j)
 * 
 *     cpdef void open(self) except *:             # <<<<<<<<<<<<<<
 *         cdef str cmd
 *         if self.filename.endswith('.bz2') or self.filename.endswith('.bzip2'):
 */

static PyObject *__pyx_pw_10traceutils_7scamper_5warts_11WartsReader_6open(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static void __pyx_f_10traceutils_7scamper_5warts_11WartsReader_open(struct __pyx_obj_10traceutils_7scamper_5warts_WartsReader *__pyx_v_self, int __pyx_skip_dispatch) {
  PyObject *__pyx_v_cmd = 0;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  __Pyx_RefNannySetupContext("open", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_open); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 152, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_10traceutils_7scamper_5warts_11WartsReader_6open)) {
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 152, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "traceutils/scamper/warts.pyx":154
 *     cpdef void open(self) except *:
 *         cdef str cmd
 *         if self.filename.endswith('.bz2') or self.filename.endswith('.bzip2'):             # <<<<<<<<<<<<<<
 *             cmd = 'bzip2 -d -c {} | sc_warts2json'
 *         elif self.filename.endswith('.gz'):
 */
  if (unlikely(__pyx_v_self->__pyx_base.filename == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "endswith");
    __PYX_ERR(0, 154, __pyx_L1_error)
  }
  __pyx_t_6 = __Pyx_PyUnicode_Tailmatch(__pyx_v_self->__pyx_base.filename, __pyx_kp_u_bz2, 0, PY_SSIZE_T_MAX, 1); if (unlikely(__pyx_t_6 == ((int)-1))) __PYX_ERR(0, 154, __pyx_L1_error)
  if (!(__pyx_t_6 != 0)) {
  } else {
    __pyx_t_5 = (__pyx_t_6 != 0);
    goto __pyx_L4_bool_binop_done;
  }
  if (unlikely(__pyx_v_self->__pyx_base.filename == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "endswith");
    __PYX_ERR(0, 154, __pyx_L1_error)
  }
  __pyx_t_6 = __Pyx_PyUnicode_Tailmatch(__pyx_v_self->__pyx_base.filename, __pyx_kp_u_bzip2, 0, PY_SSIZE_T_MAX, 1); if (unlikely(__pyx_t_6 == ((int)-1))) __PYX_ERR(0, 154, __pyx_L1_error)
  __pyx_t_5 = (__pyx_t_6 != 0);
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_5) {

    /* "traceutils/scamper/warts.pyx":155
 *         cdef str cmd
 *         if self.filename.endswith('.bz2') or self.filename.endswith('.bzip2'):
 *             cmd = 'bzip2 -d -c {} | sc_warts2json'             # <<<<<<<<<<<<<<
 *         elif self.filename.endswith('.gz'):
 *             cmd = 'gzip -d -c {} | sc_warts2json'
 */
    __Pyx_INCREF(__pyx_kp_u_bzip2_d_c_sc_warts2json);
    __pyx_v_cmd = __pyx_kp_u_bzip2_d_c_sc_warts2json;

    /* "traceutils/scamper/warts.pyx":154
 *     cpdef void open(self) except *:
 *         cdef str cmd
 *         if self.filename.endswith('.bz2') or self.filename.endswith('.bzip2'):             # <<<<<<<<<<<<<<
 *             cmd = 'bzip2 -d -c {} | sc_warts2json'
 *         elif self.filename.endswith('.gz'):
 */
    goto __pyx_L3;
  }

  /* "traceutils/scamper/warts.pyx":156
 *         if self.filename.endswith('.bz2') or self.filename.endswith('.bzip2'):
 *             cmd = 'bzip2 -d -c {} | sc_warts2json'
 *         elif self.filename.endswith('.gz'):             # <<<<<<<<<<<<<<
 *             cmd = 'gzip -d -c {} | sc_warts2json'
 *         else:
 */
  if (unlikely(__pyx_v_self->__pyx_base.filename == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "endswith");
    __PYX_ERR(0, 156, __pyx_L1_error)
  }
  __pyx_t_5 = __Pyx_PyUnicode_Tailmatch(__pyx_v_self->__pyx_base.filename, __pyx_kp_u_gz, 0, PY_SSIZE_T_MAX, 1); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(0, 156, __pyx_L1_error)
  if ((__pyx_t_5 != 0)) {

    /* "traceutils/scamper/warts.pyx":157
 *             cmd = 'bzip2 -d -c {} | sc_warts2json'
 *         elif self.filename.endswith('.gz'):
 *             cmd = 'gzip -d -c {} | sc_warts2json'             # <<<<<<<<<<<<<<
 *         else:
 *             cmd = 'sc_warts2json {}'
 */
    __Pyx_INCREF(__pyx_kp_u_gzip_d_c_sc_warts2json);
    __pyx_v_cmd = __pyx_kp_u_gzip_d_c_sc_warts2json;

    /* "traceutils/scamper/warts.pyx":156
 *         if self.filename.endswith('.bz2') or self.filename.endswith('.bzip2'):
 *             cmd = 'bzip2 -d -c {} | sc_warts2json'
 *         elif self.filename.endswith('.gz'):             # <<<<<<<<<<<<<<
 *             cmd = 'gzip -d -c {} | sc_warts2json'
 *         else:
 */
    goto __pyx_L3;
  }

  /* "traceutils/scamper/warts.pyx":159
 *             cmd = 'gzip -d -c {} | sc_warts2json'
 *         else:
 *             cmd = 'sc_warts2json {}'             # <<<<<<<<<<<<<<
 *         self.p = Popen(cmd.format(self.filename), stdout=PIPE, shell=True, universal_newlines=True)
 * 
 */
  /*else*/ {
    __Pyx_INCREF(__pyx_kp_u_sc_warts2json);
    __pyx_v_cmd = __pyx_kp_u_sc_warts2json;
  }
  __pyx_L3:;

  /* "traceutils/scamper/warts.pyx":160
 *         else:
 *             cmd = 'sc_warts2json {}'
 *         self.p = Popen(cmd.format(self.filename), stdout=PIPE, shell=True, universal_newlines=True)             # <<<<<<<<<<<<<<
 * 
 *     cpdef void close(self) except *:
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_Popen); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 160, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_CallUnboundCMethod1(&__pyx_umethod_PyUnicode_Type_format, __pyx_v_cmd, __pyx_v_self->__pyx_base.filename); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 160, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 160, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyDict_NewPresized(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 160, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_PIPE); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 160, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_stdout, __pyx_t_4) < 0) __PYX_ERR(0, 160, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_shell, Py_True) < 0) __PYX_ERR(0, 160, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_universal_newlines, Py_True) < 0) __PYX_ERR(0, 160, __pyx_L1_error)
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 160, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_GIVEREF(__pyx_t_4);
  __Pyx_GOTREF(__pyx_v_self->__pyx_base.p);
  __Pyx_DECREF(__pyx_v_self->__pyx_base.p);
  __pyx_v_self->__pyx_base.p = __pyx_t_4;
  __pyx_t_4 = 0;

  /* "traceutils/scamper/warts.pyx":152
 *                 yield WartsPing(**j)
 * 
 *     cpdef void open(self) except *:             # <<<<<<<<<<<<<<
 *         cdef str cmd
 *         if self.filename.endswith('.bz2') or self.filename.endswith('.bzip2'):
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsReader.open", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_cmd);
  __Pyx_RefNannyFinishContext();
}

/* Python wrapper */
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_11WartsReader_6open(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_10traceutils_7scamper_5warts_11WartsReader_5open[] = "WartsReader.open(self) -> void";
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_11WartsReader_6open(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("open (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_11WartsReader_5open(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsReader *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10traceutils_7scamper_5warts_11WartsReader_5open(struct __pyx_obj_10traceutils_7scamper_5warts_WartsReader *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("open", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_f_10traceutils_7scamper_5warts_11WartsReader_open(__pyx_v_self, 1); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 152, __pyx_L1_error)
  __pyx_t_1 = __Pyx_void_to_None(NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 152, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsReader.open", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "traceutils/scamper/warts.pyx":162
 *         self.p = Popen(cmd.format(self.filename), stdout=PIPE, shell=True, universal_newlines=True)
 * 
 *     cpdef void close(self) except *:             # <<<<<<<<<<<<<<
 *         self.p.stdout.close()
 *         self.p.wait()
 */

static PyObject *__pyx_pw_10traceutils_7scamper_5warts_11WartsReader_8close(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static void __pyx_f_10traceutils_7scamper_5warts_11WartsReader_close(struct __pyx_obj_10traceutils_7scamper_5warts_WartsReader *__pyx_v_self, int __pyx_skip_dispatch) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("close", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_close); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 162, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_10traceutils_7scamper_5warts_11WartsReader_8close)) {
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 162, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "traceutils/scamper/warts.pyx":163
 * 
 *     cpdef void close(self) except *:
 *         self.p.stdout.close()             # <<<<<<<<<<<<<<
 *         self.p.wait()
 *         self.p = None
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->__pyx_base.p, __pyx_n_s_stdout); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 163, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_close); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 163, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  __pyx_t_1 = (__pyx_t_2) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_2) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 163, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "traceutils/scamper/warts.pyx":164
 *     cpdef void close(self) except *:
 *         self.p.stdout.close()
 *         self.p.wait()             # <<<<<<<<<<<<<<
 *         self.p = None
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->__pyx_base.p, __pyx_n_s_wait); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 164, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  __pyx_t_1 = (__pyx_t_2) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_2) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 164, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "traceutils/scamper/warts.pyx":165
 *         self.p.stdout.close()
 *         self.p.wait()
 *         self.p = None             # <<<<<<<<<<<<<<
 */
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->__pyx_base.p);
  __Pyx_DECREF(__pyx_v_self->__pyx_base.p);
  __pyx_v_self->__pyx_base.p = Py_None;

  /* "traceutils/scamper/warts.pyx":162
 *         self.p = Popen(cmd.format(self.filename), stdout=PIPE, shell=True, universal_newlines=True)
 * 
 *     cpdef void close(self) except *:             # <<<<<<<<<<<<<<
 *         self.p.stdout.close()
 *         self.p.wait()
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsReader.close", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* Python wrapper */
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_11WartsReader_8close(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_10traceutils_7scamper_5warts_11WartsReader_7close[] = "WartsReader.close(self) -> void";
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_11WartsReader_8close(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("close (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_11WartsReader_7close(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsReader *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10traceutils_7scamper_5warts_11WartsReader_7close(struct __pyx_obj_10traceutils_7scamper_5warts_WartsReader *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("close", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_f_10traceutils_7scamper_5warts_11WartsReader_close(__pyx_v_self, 1); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 162, __pyx_L1_error)
  __pyx_t_1 = __Pyx_void_to_None(NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 162, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsReader.close", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_11WartsReader_10__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_10traceutils_7scamper_5warts_11WartsReader_9__reduce_cython__[] = "WartsReader.__reduce_cython__(self)";
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_11WartsReader_10__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_11WartsReader_9__reduce_cython__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsReader *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10traceutils_7scamper_5warts_11WartsReader_9__reduce_cython__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsReader *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self.filename, self.p)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.filename);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.filename);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_self->__pyx_base.filename);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.p);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.p);
  PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_v_self->__pyx_base.p);
  __pyx_v_state = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self.filename, self.p)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_1 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v__dict = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "(tree fragment)":7
 *     state = (self.filename, self.p)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_2 = (__pyx_v__dict != Py_None);
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v__dict);
    __pyx_t_4 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_1); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_4));
    __pyx_t_4 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self.filename is not None or self.p is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self.filename, self.p)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self.filename is not None or self.p is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_WartsReader, (type(self), 0x93358b7, None), state
 */
  /*else*/ {
    __pyx_t_2 = (__pyx_v_self->__pyx_base.filename != ((PyObject*)Py_None));
    __pyx_t_5 = (__pyx_t_2 != 0);
    if (!__pyx_t_5) {
    } else {
      __pyx_t_3 = __pyx_t_5;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_5 = (__pyx_v_self->__pyx_base.p != Py_None);
    __pyx_t_2 = (__pyx_t_5 != 0);
    __pyx_t_3 = __pyx_t_2;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_3;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.filename is not None or self.p is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_WartsReader, (type(self), 0x93358b7, None), state
 *     else:
 */
  __pyx_t_3 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_3) {

    /* "(tree fragment)":13
 *         use_setstate = self.filename is not None or self.p is not None
 *     if use_setstate:
 *         return __pyx_unpickle_WartsReader, (type(self), 0x93358b7, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_WartsReader, (type(self), 0x93358b7, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_pyx_unpickle_WartsReader); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_154359991);
    __Pyx_GIVEREF(__pyx_int_154359991);
    PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_int_154359991);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_1, 2, Py_None);
    __pyx_t_6 = PyTuple_New(3); if (unlikely(!__pyx_t_6)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_1);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_v_state);
    __pyx_t_4 = 0;
    __pyx_t_1 = 0;
    __pyx_r = __pyx_t_6;
    __pyx_t_6 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.filename is not None or self.p is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_WartsReader, (type(self), 0x93358b7, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_WartsReader, (type(self), 0x93358b7, None), state
 *     else:
 *         return __pyx_unpickle_WartsReader, (type(self), 0x93358b7, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_WartsReader__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_6, __pyx_n_s_pyx_unpickle_WartsReader); if (unlikely(!__pyx_t_6)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_154359991);
    __Pyx_GIVEREF(__pyx_int_154359991);
    PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_int_154359991);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_v_state);
    __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_6);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_1);
    __pyx_t_6 = 0;
    __pyx_t_1 = 0;
    __pyx_r = __pyx_t_4;
    __pyx_t_4 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsReader.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_WartsReader, (type(self), 0x93358b7, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_WartsReader__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_11WartsReader_12__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static char __pyx_doc_10traceutils_7scamper_5warts_11WartsReader_11__setstate_cython__[] = "WartsReader.__setstate_cython__(self, __pyx_state)";
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_11WartsReader_12__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_11WartsReader_11__setstate_cython__(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsReader *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10traceutils_7scamper_5warts_11WartsReader_11__setstate_cython__(struct __pyx_obj_10traceutils_7scamper_5warts_WartsReader *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_WartsReader, (type(self), 0x93358b7, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_WartsReader__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(2, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_10traceutils_7scamper_5warts___pyx_unpickle_WartsReader__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_WartsReader, (type(self), 0x93358b7, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_WartsReader__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("traceutils.scamper.warts.WartsReader.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_WartsTrace(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_1__pyx_unpickle_WartsTrace(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_10traceutils_7scamper_5warts___pyx_unpickle_WartsTrace[] = "__pyx_unpickle_WartsTrace(__pyx_type, long __pyx_checksum, __pyx_state)";
static PyMethodDef __pyx_mdef_10traceutils_7scamper_5warts_1__pyx_unpickle_WartsTrace = {"__pyx_unpickle_WartsTrace", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_10traceutils_7scamper_5warts_1__pyx_unpickle_WartsTrace, METH_VARARGS|METH_KEYWORDS, __pyx_doc_10traceutils_7scamper_5warts___pyx_unpickle_WartsTrace};
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_1__pyx_unpickle_WartsTrace(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_WartsTrace (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_WartsTrace", 1, 3, 3, 1); __PYX_ERR(2, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_WartsTrace", 1, 3, 3, 2); __PYX_ERR(2, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_WartsTrace") < 0)) __PYX_ERR(2, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(2, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_WartsTrace", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(2, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("traceutils.scamper.warts.__pyx_unpickle_WartsTrace", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts___pyx_unpickle_WartsTrace(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10traceutils_7scamper_5warts___pyx_unpickle_WartsTrace(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  __Pyx_RefNannySetupContext("__pyx_unpickle_WartsTrace", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0xffb4da5:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xffb4da5 = (attempts, dst, family, firsthop, hop_count, hoplimit, hops, icmp_sum, loop, method, probe_count, probe_size, src, start, stop_data, stop_reason, tos, type, userid, version, wait, wait_probe))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0xffb4da5) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0xffb4da5:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xffb4da5 = (attempts, dst, family, firsthop, hop_count, hoplimit, hops, icmp_sum, loop, method, probe_count, probe_size, src, start, stop_data, stop_reason, tos, type, userid, version, wait, wait_probe))" % __pyx_checksum)
 *     __pyx_result = WartsTrace.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum != 0xffb4da5:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xffb4da5 = (attempts, dst, family, firsthop, hop_count, hoplimit, hops, icmp_sum, loop, method, probe_count, probe_size, src, start, stop_data, stop_reason, tos, type, userid, version, wait, wait_probe))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = WartsTrace.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0xff, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(2, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0xffb4da5:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xffb4da5 = (attempts, dst, family, firsthop, hop_count, hoplimit, hops, icmp_sum, loop, method, probe_count, probe_size, src, start, stop_data, stop_reason, tos, type, userid, version, wait, wait_probe))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xffb4da5 = (attempts, dst, family, firsthop, hop_count, hoplimit, hops, icmp_sum, loop, method, probe_count, probe_size, src, start, stop_data, stop_reason, tos, type, userid, version, wait, wait_probe))" % __pyx_checksum)
 *     __pyx_result = WartsTrace.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_WartsTrace__set_state(<WartsTrace> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_10traceutils_7scamper_5warts_WartsTrace), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xffb4da5 = (attempts, dst, family, firsthop, hop_count, hoplimit, hops, icmp_sum, loop, method, probe_count, probe_size, src, start, stop_data, stop_reason, tos, type, userid, version, wait, wait_probe))" % __pyx_checksum)
 *     __pyx_result = WartsTrace.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_WartsTrace__set_state(<WartsTrace> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":9
 *     __pyx_result = WartsTrace.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_WartsTrace__set_state(<WartsTrace> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_WartsTrace__set_state(WartsTrace __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(2, 9, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_10traceutils_7scamper_5warts___pyx_unpickle_WartsTrace__set_state(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xffb4da5 = (attempts, dst, family, firsthop, hop_count, hoplimit, hops, icmp_sum, loop, method, probe_count, probe_size, src, start, stop_data, stop_reason, tos, type, userid, version, wait, wait_probe))" % __pyx_checksum)
 *     __pyx_result = WartsTrace.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_WartsTrace__set_state(<WartsTrace> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_WartsTrace__set_state(<WartsTrace> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_WartsTrace__set_state(WartsTrace __pyx_result, tuple __pyx_state):
 *     __pyx_result.attempts = __pyx_state[0]; __pyx_result.dst = __pyx_state[1]; __pyx_result.family = __pyx_state[2]; __pyx_result.firsthop = __pyx_state[3]; __pyx_result.hop_count = __pyx_state[4]; __pyx_result.hoplimit = __pyx_state[5]; __pyx_result.hops = __pyx_state[6]; __pyx_result.icmp_sum = __pyx_state[7]; __pyx_result.loop = __pyx_state[8]; __pyx_result.method = __pyx_state[9]; __pyx_result.probe_count = __pyx_state[10]; __pyx_result.probe_size = __pyx_state[11]; __pyx_result.src = __pyx_state[12]; __pyx_result.start = __pyx_state[13]; __pyx_result.stop_data = __pyx_state[14]; __pyx_result.stop_reason = __pyx_state[15]; __pyx_result.tos = __pyx_state[16]; __pyx_result.type = __pyx_state[17]; __pyx_result.userid = __pyx_state[18]; __pyx_result.version = __pyx_state[19]; __pyx_result.wait = __pyx_state[20]; __pyx_result.wait_probe = __pyx_state[21]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_WartsTrace(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("traceutils.scamper.warts.__pyx_unpickle_WartsTrace", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_WartsTrace__set_state(<WartsTrace> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_WartsTrace__set_state(WartsTrace __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.attempts = __pyx_state[0]; __pyx_result.dst = __pyx_state[1]; __pyx_result.family = __pyx_state[2]; __pyx_result.firsthop = __pyx_state[3]; __pyx_result.hop_count = __pyx_state[4]; __pyx_result.hoplimit = __pyx_state[5]; __pyx_result.hops = __pyx_state[6]; __pyx_result.icmp_sum = __pyx_state[7]; __pyx_result.loop = __pyx_state[8]; __pyx_result.method = __pyx_state[9]; __pyx_result.probe_count = __pyx_state[10]; __pyx_result.probe_size = __pyx_state[11]; __pyx_result.src = __pyx_state[12]; __pyx_result.start = __pyx_state[13]; __pyx_result.stop_data = __pyx_state[14]; __pyx_result.stop_reason = __pyx_state[15]; __pyx_result.tos = __pyx_state[16]; __pyx_result.type = __pyx_state[17]; __pyx_result.userid = __pyx_state[18]; __pyx_result.version = __pyx_state[19]; __pyx_result.wait = __pyx_state[20]; __pyx_result.wait_probe = __pyx_state[21]
 *     if len(__pyx_state) > 22 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_10traceutils_7scamper_5warts___pyx_unpickle_WartsTrace__set_state(struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  unsigned char __pyx_t_3;
  double __pyx_t_4;
  int __pyx_t_5;
  Py_ssize_t __pyx_t_6;
  int __pyx_t_7;
  int __pyx_t_8;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  __Pyx_RefNannySetupContext("__pyx_unpickle_WartsTrace__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_WartsTrace__set_state(WartsTrace __pyx_result, tuple __pyx_state):
 *     __pyx_result.attempts = __pyx_state[0]; __pyx_result.dst = __pyx_state[1]; __pyx_result.family = __pyx_state[2]; __pyx_result.firsthop = __pyx_state[3]; __pyx_result.hop_count = __pyx_state[4]; __pyx_result.hoplimit = __pyx_state[5]; __pyx_result.hops = __pyx_state[6]; __pyx_result.icmp_sum = __pyx_state[7]; __pyx_result.loop = __pyx_state[8]; __pyx_result.method = __pyx_state[9]; __pyx_result.probe_count = __pyx_state[10]; __pyx_result.probe_size = __pyx_state[11]; __pyx_result.src = __pyx_state[12]; __pyx_result.start = __pyx_state[13]; __pyx_result.stop_data = __pyx_state[14]; __pyx_result.stop_reason = __pyx_state[15]; __pyx_result.tos = __pyx_state[16]; __pyx_result.type = __pyx_state[17]; __pyx_result.userid = __pyx_state[18]; __pyx_result.version = __pyx_state[19]; __pyx_result.wait = __pyx_state[20]; __pyx_result.wait_probe = __pyx_state[21]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 22 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[22])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->attempts = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyUnicode_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "unicode", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.dst);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.dst);
  __pyx_v___pyx_result->__pyx_base.dst = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.family = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_As_unsigned_char(__pyx_t_1); if (unlikely((__pyx_t_3 == (unsigned char)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->firsthop = __pyx_t_3;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->hop_count = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 5, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_As_unsigned_char(__pyx_t_1); if (unlikely((__pyx_t_3 == (unsigned char)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->hoplimit = __pyx_t_3;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 6, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.hops);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.hops);
  __pyx_v___pyx_result->__pyx_base.hops = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 7, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->icmp_sum = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 8, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.loop);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.loop);
  __pyx_v___pyx_result->__pyx_base.loop = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 9, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyUnicode_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "unicode", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->method);
  __Pyx_DECREF(__pyx_v___pyx_result->method);
  __pyx_v___pyx_result->method = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 10, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_As_unsigned_char(__pyx_t_1); if (unlikely((__pyx_t_3 == (unsigned char)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->probe_count = __pyx_t_3;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 11, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_As_unsigned_char(__pyx_t_1); if (unlikely((__pyx_t_3 == (unsigned char)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->probe_size = __pyx_t_3;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 12, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyUnicode_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "unicode", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.src);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.src);
  __pyx_v___pyx_result->__pyx_base.src = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 13, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyDict_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->start);
  __Pyx_DECREF(__pyx_v___pyx_result->start);
  __pyx_v___pyx_result->start = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 14, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->stop_data = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 15, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyUnicode_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "unicode", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->stop_reason);
  __Pyx_DECREF(__pyx_v___pyx_result->stop_reason);
  __pyx_v___pyx_result->stop_reason = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 16, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->tos = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 17, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyUnicode_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "unicode", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->type);
  __Pyx_DECREF(__pyx_v___pyx_result->type);
  __pyx_v___pyx_result->type = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 18, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->userid = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 19, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyUnicode_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "unicode", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->version);
  __Pyx_DECREF(__pyx_v___pyx_result->version);
  __pyx_v___pyx_result->version = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 20, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_4 == (double)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->wait = __pyx_t_4;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 21, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->wait_probe = __pyx_t_2;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_WartsTrace__set_state(WartsTrace __pyx_result, tuple __pyx_state):
 *     __pyx_result.attempts = __pyx_state[0]; __pyx_result.dst = __pyx_state[1]; __pyx_result.family = __pyx_state[2]; __pyx_result.firsthop = __pyx_state[3]; __pyx_result.hop_count = __pyx_state[4]; __pyx_result.hoplimit = __pyx_state[5]; __pyx_result.hops = __pyx_state[6]; __pyx_result.icmp_sum = __pyx_state[7]; __pyx_result.loop = __pyx_state[8]; __pyx_result.method = __pyx_state[9]; __pyx_result.probe_count = __pyx_state[10]; __pyx_result.probe_size = __pyx_state[11]; __pyx_result.src = __pyx_state[12]; __pyx_result.start = __pyx_state[13]; __pyx_result.stop_data = __pyx_state[14]; __pyx_result.stop_reason = __pyx_state[15]; __pyx_result.tos = __pyx_state[16]; __pyx_result.type = __pyx_state[17]; __pyx_result.userid = __pyx_state[18]; __pyx_result.version = __pyx_state[19]; __pyx_result.wait = __pyx_state[20]; __pyx_result.wait_probe = __pyx_state[21]
 *     if len(__pyx_state) > 22 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[22])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(2, 13, __pyx_L1_error)
  }
  __pyx_t_6 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_6 == ((Py_ssize_t)-1))) __PYX_ERR(2, 13, __pyx_L1_error)
  __pyx_t_7 = ((__pyx_t_6 > 22) != 0);
  if (__pyx_t_7) {
  } else {
    __pyx_t_5 = __pyx_t_7;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_7 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_7 == ((int)-1))) __PYX_ERR(2, 13, __pyx_L1_error)
  __pyx_t_8 = (__pyx_t_7 != 0);
  __pyx_t_5 = __pyx_t_8;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_5) {

    /* "(tree fragment)":14
 *     __pyx_result.attempts = __pyx_state[0]; __pyx_result.dst = __pyx_state[1]; __pyx_result.family = __pyx_state[2]; __pyx_result.firsthop = __pyx_state[3]; __pyx_result.hop_count = __pyx_state[4]; __pyx_result.hoplimit = __pyx_state[5]; __pyx_result.hops = __pyx_state[6]; __pyx_result.icmp_sum = __pyx_state[7]; __pyx_result.loop = __pyx_state[8]; __pyx_result.method = __pyx_state[9]; __pyx_result.probe_count = __pyx_state[10]; __pyx_result.probe_size = __pyx_state[11]; __pyx_result.src = __pyx_state[12]; __pyx_result.start = __pyx_state[13]; __pyx_result.stop_data = __pyx_state[14]; __pyx_result.stop_reason = __pyx_state[15]; __pyx_result.tos = __pyx_state[16]; __pyx_result.type = __pyx_state[17]; __pyx_result.userid = __pyx_state[18]; __pyx_result.version = __pyx_state[19]; __pyx_result.wait = __pyx_state[20]; __pyx_result.wait_probe = __pyx_state[21]
 *     if len(__pyx_state) > 22 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[22])             # <<<<<<<<<<<<<<
 */
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_9)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_t_9, __pyx_n_s_update); if (unlikely(!__pyx_t_10)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(2, 14, __pyx_L1_error)
    }
    __pyx_t_9 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 22, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_9)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_11 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_10))) {
      __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_10);
      if (likely(__pyx_t_11)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_10);
        __Pyx_INCREF(__pyx_t_11);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_10, function);
      }
    }
    __pyx_t_1 = (__pyx_t_11) ? __Pyx_PyObject_Call2Args(__pyx_t_10, __pyx_t_11, __pyx_t_9) : __Pyx_PyObject_CallOneArg(__pyx_t_10, __pyx_t_9);
    __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_WartsTrace__set_state(WartsTrace __pyx_result, tuple __pyx_state):
 *     __pyx_result.attempts = __pyx_state[0]; __pyx_result.dst = __pyx_state[1]; __pyx_result.family = __pyx_state[2]; __pyx_result.firsthop = __pyx_state[3]; __pyx_result.hop_count = __pyx_state[4]; __pyx_result.hoplimit = __pyx_state[5]; __pyx_result.hops = __pyx_state[6]; __pyx_result.icmp_sum = __pyx_state[7]; __pyx_result.loop = __pyx_state[8]; __pyx_result.method = __pyx_state[9]; __pyx_result.probe_count = __pyx_state[10]; __pyx_result.probe_size = __pyx_state[11]; __pyx_result.src = __pyx_state[12]; __pyx_result.start = __pyx_state[13]; __pyx_result.stop_data = __pyx_state[14]; __pyx_result.stop_reason = __pyx_state[15]; __pyx_result.tos = __pyx_state[16]; __pyx_result.type = __pyx_state[17]; __pyx_result.userid = __pyx_state[18]; __pyx_result.version = __pyx_state[19]; __pyx_result.wait = __pyx_state[20]; __pyx_result.wait_probe = __pyx_state[21]
 *     if len(__pyx_state) > 22 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[22])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_WartsTrace__set_state(<WartsTrace> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_WartsTrace__set_state(WartsTrace __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.attempts = __pyx_state[0]; __pyx_result.dst = __pyx_state[1]; __pyx_result.family = __pyx_state[2]; __pyx_result.firsthop = __pyx_state[3]; __pyx_result.hop_count = __pyx_state[4]; __pyx_result.hoplimit = __pyx_state[5]; __pyx_result.hops = __pyx_state[6]; __pyx_result.icmp_sum = __pyx_state[7]; __pyx_result.loop = __pyx_state[8]; __pyx_result.method = __pyx_state[9]; __pyx_result.probe_count = __pyx_state[10]; __pyx_result.probe_size = __pyx_state[11]; __pyx_result.src = __pyx_state[12]; __pyx_result.start = __pyx_state[13]; __pyx_result.stop_data = __pyx_state[14]; __pyx_result.stop_reason = __pyx_state[15]; __pyx_result.tos = __pyx_state[16]; __pyx_result.type = __pyx_state[17]; __pyx_result.userid = __pyx_state[18]; __pyx_result.version = __pyx_state[19]; __pyx_result.wait = __pyx_state[20]; __pyx_result.wait_probe = __pyx_state[21]
 *     if len(__pyx_state) > 22 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_AddTraceback("traceutils.scamper.warts.__pyx_unpickle_WartsTrace__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_WartsHop(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_3__pyx_unpickle_WartsHop(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_10traceutils_7scamper_5warts_2__pyx_unpickle_WartsHop[] = "__pyx_unpickle_WartsHop(__pyx_type, long __pyx_checksum, __pyx_state)";
static PyMethodDef __pyx_mdef_10traceutils_7scamper_5warts_3__pyx_unpickle_WartsHop = {"__pyx_unpickle_WartsHop", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_10traceutils_7scamper_5warts_3__pyx_unpickle_WartsHop, METH_VARARGS|METH_KEYWORDS, __pyx_doc_10traceutils_7scamper_5warts_2__pyx_unpickle_WartsHop};
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_3__pyx_unpickle_WartsHop(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_WartsHop (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_WartsHop", 1, 3, 3, 1); __PYX_ERR(2, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_WartsHop", 1, 3, 3, 2); __PYX_ERR(2, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_WartsHop") < 0)) __PYX_ERR(2, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(2, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_WartsHop", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(2, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("traceutils.scamper.warts.__pyx_unpickle_WartsHop", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_2__pyx_unpickle_WartsHop(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10traceutils_7scamper_5warts_2__pyx_unpickle_WartsHop(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  __Pyx_RefNannySetupContext("__pyx_unpickle_WartsHop", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0xb91787d:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xb91787d = (addr, family, icmp_code, icmp_nhmtu, icmp_q_ipl, icmp_q_tos, icmp_q_ttl, icmp_type, icmpext, ismpls, packed, probe_id, probe_size, probe_ttl, reply_ipid, reply_size, reply_tos, reply_ttl, rtt, type))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0xb91787d) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0xb91787d:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xb91787d = (addr, family, icmp_code, icmp_nhmtu, icmp_q_ipl, icmp_q_tos, icmp_q_ttl, icmp_type, icmpext, ismpls, packed, probe_id, probe_size, probe_ttl, reply_ipid, reply_size, reply_tos, reply_ttl, rtt, type))" % __pyx_checksum)
 *     __pyx_result = WartsHop.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum != 0xb91787d:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xb91787d = (addr, family, icmp_code, icmp_nhmtu, icmp_q_ipl, icmp_q_tos, icmp_q_ttl, icmp_type, icmpext, ismpls, packed, probe_id, probe_size, probe_ttl, reply_ipid, reply_size, reply_tos, reply_ttl, rtt, type))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = WartsHop.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0xb9, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(2, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0xb91787d:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xb91787d = (addr, family, icmp_code, icmp_nhmtu, icmp_q_ipl, icmp_q_tos, icmp_q_ttl, icmp_type, icmpext, ismpls, packed, probe_id, probe_size, probe_ttl, reply_ipid, reply_size, reply_tos, reply_ttl, rtt, type))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xb91787d = (addr, family, icmp_code, icmp_nhmtu, icmp_q_ipl, icmp_q_tos, icmp_q_ttl, icmp_type, icmpext, ismpls, packed, probe_id, probe_size, probe_ttl, reply_ipid, reply_size, reply_tos, reply_ttl, rtt, type))" % __pyx_checksum)
 *     __pyx_result = WartsHop.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_WartsHop__set_state(<WartsHop> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_10traceutils_7scamper_5warts_WartsHop), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xb91787d = (addr, family, icmp_code, icmp_nhmtu, icmp_q_ipl, icmp_q_tos, icmp_q_ttl, icmp_type, icmpext, ismpls, packed, probe_id, probe_size, probe_ttl, reply_ipid, reply_size, reply_tos, reply_ttl, rtt, type))" % __pyx_checksum)
 *     __pyx_result = WartsHop.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_WartsHop__set_state(<WartsHop> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":9
 *     __pyx_result = WartsHop.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_WartsHop__set_state(<WartsHop> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_WartsHop__set_state(WartsHop __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(2, 9, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_10traceutils_7scamper_5warts___pyx_unpickle_WartsHop__set_state(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsHop *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xb91787d = (addr, family, icmp_code, icmp_nhmtu, icmp_q_ipl, icmp_q_tos, icmp_q_ttl, icmp_type, icmpext, ismpls, packed, probe_id, probe_size, probe_ttl, reply_ipid, reply_size, reply_tos, reply_ttl, rtt, type))" % __pyx_checksum)
 *     __pyx_result = WartsHop.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_WartsHop__set_state(<WartsHop> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_WartsHop__set_state(<WartsHop> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_WartsHop__set_state(WartsHop __pyx_result, tuple __pyx_state):
 *     __pyx_result.addr = __pyx_state[0]; __pyx_result.family = __pyx_state[1]; __pyx_result.icmp_code = __pyx_state[2]; __pyx_result.icmp_nhmtu = __pyx_state[3]; __pyx_result.icmp_q_ipl = __pyx_state[4]; __pyx_result.icmp_q_tos = __pyx_state[5]; __pyx_result.icmp_q_ttl = __pyx_state[6]; __pyx_result.icmp_type = __pyx_state[7]; __pyx_result.icmpext = __pyx_state[8]; __pyx_result.ismpls = __pyx_state[9]; __pyx_result.packed = __pyx_state[10]; __pyx_result.probe_id = __pyx_state[11]; __pyx_result.probe_size = __pyx_state[12]; __pyx_result.probe_ttl = __pyx_state[13]; __pyx_result.reply_ipid = __pyx_state[14]; __pyx_result.reply_size = __pyx_state[15]; __pyx_result.reply_tos = __pyx_state[16]; __pyx_result.reply_ttl = __pyx_state[17]; __pyx_result.rtt = __pyx_state[18]; __pyx_result.type = __pyx_state[19]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_WartsHop(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("traceutils.scamper.warts.__pyx_unpickle_WartsHop", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_WartsHop__set_state(<WartsHop> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_WartsHop__set_state(WartsHop __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.addr = __pyx_state[0]; __pyx_result.family = __pyx_state[1]; __pyx_result.icmp_code = __pyx_state[2]; __pyx_result.icmp_nhmtu = __pyx_state[3]; __pyx_result.icmp_q_ipl = __pyx_state[4]; __pyx_result.icmp_q_tos = __pyx_state[5]; __pyx_result.icmp_q_ttl = __pyx_state[6]; __pyx_result.icmp_type = __pyx_state[7]; __pyx_result.icmpext = __pyx_state[8]; __pyx_result.ismpls = __pyx_state[9]; __pyx_result.packed = __pyx_state[10]; __pyx_result.probe_id = __pyx_state[11]; __pyx_result.probe_size = __pyx_state[12]; __pyx_result.probe_ttl = __pyx_state[13]; __pyx_result.reply_ipid = __pyx_state[14]; __pyx_result.reply_size = __pyx_state[15]; __pyx_result.reply_tos = __pyx_state[16]; __pyx_result.reply_ttl = __pyx_state[17]; __pyx_result.rtt = __pyx_state[18]; __pyx_result.type = __pyx_state[19]
 *     if len(__pyx_state) > 20 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_10traceutils_7scamper_5warts___pyx_unpickle_WartsHop__set_state(struct __pyx_obj_10traceutils_7scamper_5warts_WartsHop *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  unsigned char __pyx_t_3;
  int __pyx_t_4;
  unsigned short __pyx_t_5;
  double __pyx_t_6;
  enum __pyx_t_10traceutils_7scamper_3hop_ICMPType __pyx_t_7;
  Py_ssize_t __pyx_t_8;
  int __pyx_t_9;
  int __pyx_t_10;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  __Pyx_RefNannySetupContext("__pyx_unpickle_WartsHop__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_WartsHop__set_state(WartsHop __pyx_result, tuple __pyx_state):
 *     __pyx_result.addr = __pyx_state[0]; __pyx_result.family = __pyx_state[1]; __pyx_result.icmp_code = __pyx_state[2]; __pyx_result.icmp_nhmtu = __pyx_state[3]; __pyx_result.icmp_q_ipl = __pyx_state[4]; __pyx_result.icmp_q_tos = __pyx_state[5]; __pyx_result.icmp_q_ttl = __pyx_state[6]; __pyx_result.icmp_type = __pyx_state[7]; __pyx_result.icmpext = __pyx_state[8]; __pyx_result.ismpls = __pyx_state[9]; __pyx_result.packed = __pyx_state[10]; __pyx_result.probe_id = __pyx_state[11]; __pyx_result.probe_size = __pyx_state[12]; __pyx_result.probe_ttl = __pyx_state[13]; __pyx_result.reply_ipid = __pyx_state[14]; __pyx_result.reply_size = __pyx_state[15]; __pyx_result.reply_tos = __pyx_state[16]; __pyx_result.reply_ttl = __pyx_state[17]; __pyx_result.rtt = __pyx_state[18]; __pyx_result.type = __pyx_state[19]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 20 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[20])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyUnicode_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "unicode", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.addr);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.addr);
  __pyx_v___pyx_result->__pyx_base.addr = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.family = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_As_unsigned_char(__pyx_t_1); if (unlikely((__pyx_t_3 == (unsigned char)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.icmp_code = __pyx_t_3;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->icmp_nhmtu = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->icmp_q_ipl = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 5, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.icmp_q_tos = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 6, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_As_unsigned_char(__pyx_t_1); if (unlikely((__pyx_t_3 == (unsigned char)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.icmp_q_ttl = __pyx_t_3;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 7, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_As_unsigned_char(__pyx_t_1); if (unlikely((__pyx_t_3 == (unsigned char)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.icmp_type = __pyx_t_3;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 8, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->icmpext);
  __Pyx_DECREF(__pyx_v___pyx_result->icmpext);
  __pyx_v___pyx_result->icmpext = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 9, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_4 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.ismpls = __pyx_t_4;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 10, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyBytes_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "bytes", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.packed);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.packed);
  __pyx_v___pyx_result->__pyx_base.packed = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 11, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->probe_id = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 12, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_5 = __Pyx_PyInt_As_unsigned_short(__pyx_t_1); if (unlikely((__pyx_t_5 == (unsigned short)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->probe_size = __pyx_t_5;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 13, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_As_unsigned_char(__pyx_t_1); if (unlikely((__pyx_t_3 == (unsigned char)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.probe_ttl = __pyx_t_3;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 14, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->reply_ipid = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 15, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.reply_size = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 16, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.reply_tos = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 17, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_As_unsigned_char(__pyx_t_1); if (unlikely((__pyx_t_3 == (unsigned char)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.reply_ttl = __pyx_t_3;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 18, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_6 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_6 == (double)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.rtt = __pyx_t_6;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 19, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_7 = ((enum __pyx_t_10traceutils_7scamper_3hop_ICMPType)__Pyx_PyInt_As_enum____pyx_t_10traceutils_7scamper_3hop_ICMPType(__pyx_t_1)); if (unlikely(PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.type = __pyx_t_7;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_WartsHop__set_state(WartsHop __pyx_result, tuple __pyx_state):
 *     __pyx_result.addr = __pyx_state[0]; __pyx_result.family = __pyx_state[1]; __pyx_result.icmp_code = __pyx_state[2]; __pyx_result.icmp_nhmtu = __pyx_state[3]; __pyx_result.icmp_q_ipl = __pyx_state[4]; __pyx_result.icmp_q_tos = __pyx_state[5]; __pyx_result.icmp_q_ttl = __pyx_state[6]; __pyx_result.icmp_type = __pyx_state[7]; __pyx_result.icmpext = __pyx_state[8]; __pyx_result.ismpls = __pyx_state[9]; __pyx_result.packed = __pyx_state[10]; __pyx_result.probe_id = __pyx_state[11]; __pyx_result.probe_size = __pyx_state[12]; __pyx_result.probe_ttl = __pyx_state[13]; __pyx_result.reply_ipid = __pyx_state[14]; __pyx_result.reply_size = __pyx_state[15]; __pyx_result.reply_tos = __pyx_state[16]; __pyx_result.reply_ttl = __pyx_state[17]; __pyx_result.rtt = __pyx_state[18]; __pyx_result.type = __pyx_state[19]
 *     if len(__pyx_state) > 20 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[20])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(2, 13, __pyx_L1_error)
  }
  __pyx_t_8 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_8 == ((Py_ssize_t)-1))) __PYX_ERR(2, 13, __pyx_L1_error)
  __pyx_t_9 = ((__pyx_t_8 > 20) != 0);
  if (__pyx_t_9) {
  } else {
    __pyx_t_4 = __pyx_t_9;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_9 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_9 == ((int)-1))) __PYX_ERR(2, 13, __pyx_L1_error)
  __pyx_t_10 = (__pyx_t_9 != 0);
  __pyx_t_4 = __pyx_t_10;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_4) {

    /* "(tree fragment)":14
 *     __pyx_result.addr = __pyx_state[0]; __pyx_result.family = __pyx_state[1]; __pyx_result.icmp_code = __pyx_state[2]; __pyx_result.icmp_nhmtu = __pyx_state[3]; __pyx_result.icmp_q_ipl = __pyx_state[4]; __pyx_result.icmp_q_tos = __pyx_state[5]; __pyx_result.icmp_q_ttl = __pyx_state[6]; __pyx_result.icmp_type = __pyx_state[7]; __pyx_result.icmpext = __pyx_state[8]; __pyx_result.ismpls = __pyx_state[9]; __pyx_result.packed = __pyx_state[10]; __pyx_result.probe_id = __pyx_state[11]; __pyx_result.probe_size = __pyx_state[12]; __pyx_result.probe_ttl = __pyx_state[13]; __pyx_result.reply_ipid = __pyx_state[14]; __pyx_result.reply_size = __pyx_state[15]; __pyx_result.reply_tos = __pyx_state[16]; __pyx_result.reply_ttl = __pyx_state[17]; __pyx_result.rtt = __pyx_state[18]; __pyx_result.type = __pyx_state[19]
 *     if len(__pyx_state) > 20 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[20])             # <<<<<<<<<<<<<<
 */
    __pyx_t_11 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_11)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_t_11, __pyx_n_s_update); if (unlikely(!__pyx_t_12)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_12);
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(2, 14, __pyx_L1_error)
    }
    __pyx_t_11 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 20, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_11)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    __pyx_t_13 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_12))) {
      __pyx_t_13 = PyMethod_GET_SELF(__pyx_t_12);
      if (likely(__pyx_t_13)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_12);
        __Pyx_INCREF(__pyx_t_13);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_12, function);
      }
    }
    __pyx_t_1 = (__pyx_t_13) ? __Pyx_PyObject_Call2Args(__pyx_t_12, __pyx_t_13, __pyx_t_11) : __Pyx_PyObject_CallOneArg(__pyx_t_12, __pyx_t_11);
    __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_WartsHop__set_state(WartsHop __pyx_result, tuple __pyx_state):
 *     __pyx_result.addr = __pyx_state[0]; __pyx_result.family = __pyx_state[1]; __pyx_result.icmp_code = __pyx_state[2]; __pyx_result.icmp_nhmtu = __pyx_state[3]; __pyx_result.icmp_q_ipl = __pyx_state[4]; __pyx_result.icmp_q_tos = __pyx_state[5]; __pyx_result.icmp_q_ttl = __pyx_state[6]; __pyx_result.icmp_type = __pyx_state[7]; __pyx_result.icmpext = __pyx_state[8]; __pyx_result.ismpls = __pyx_state[9]; __pyx_result.packed = __pyx_state[10]; __pyx_result.probe_id = __pyx_state[11]; __pyx_result.probe_size = __pyx_state[12]; __pyx_result.probe_ttl = __pyx_state[13]; __pyx_result.reply_ipid = __pyx_state[14]; __pyx_result.reply_size = __pyx_state[15]; __pyx_result.reply_tos = __pyx_state[16]; __pyx_result.reply_ttl = __pyx_state[17]; __pyx_result.rtt = __pyx_state[18]; __pyx_result.type = __pyx_state[19]
 *     if len(__pyx_state) > 20 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[20])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_WartsHop__set_state(<WartsHop> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_WartsHop__set_state(WartsHop __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.addr = __pyx_state[0]; __pyx_result.family = __pyx_state[1]; __pyx_result.icmp_code = __pyx_state[2]; __pyx_result.icmp_nhmtu = __pyx_state[3]; __pyx_result.icmp_q_ipl = __pyx_state[4]; __pyx_result.icmp_q_tos = __pyx_state[5]; __pyx_result.icmp_q_ttl = __pyx_state[6]; __pyx_result.icmp_type = __pyx_state[7]; __pyx_result.icmpext = __pyx_state[8]; __pyx_result.ismpls = __pyx_state[9]; __pyx_result.packed = __pyx_state[10]; __pyx_result.probe_id = __pyx_state[11]; __pyx_result.probe_size = __pyx_state[12]; __pyx_result.probe_ttl = __pyx_state[13]; __pyx_result.reply_ipid = __pyx_state[14]; __pyx_result.reply_size = __pyx_state[15]; __pyx_result.reply_tos = __pyx_state[16]; __pyx_result.reply_ttl = __pyx_state[17]; __pyx_result.rtt = __pyx_state[18]; __pyx_result.type = __pyx_state[19]
 *     if len(__pyx_state) > 20 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_AddTraceback("traceutils.scamper.warts.__pyx_unpickle_WartsHop__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_WartsPing(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_5__pyx_unpickle_WartsPing(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_10traceutils_7scamper_5warts_4__pyx_unpickle_WartsPing[] = "__pyx_unpickle_WartsPing(__pyx_type, long __pyx_checksum, __pyx_state)";
static PyMethodDef __pyx_mdef_10traceutils_7scamper_5warts_5__pyx_unpickle_WartsPing = {"__pyx_unpickle_WartsPing", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_10traceutils_7scamper_5warts_5__pyx_unpickle_WartsPing, METH_VARARGS|METH_KEYWORDS, __pyx_doc_10traceutils_7scamper_5warts_4__pyx_unpickle_WartsPing};
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_5__pyx_unpickle_WartsPing(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_WartsPing (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_WartsPing", 1, 3, 3, 1); __PYX_ERR(2, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_WartsPing", 1, 3, 3, 2); __PYX_ERR(2, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_WartsPing") < 0)) __PYX_ERR(2, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(2, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_WartsPing", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(2, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("traceutils.scamper.warts.__pyx_unpickle_WartsPing", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_4__pyx_unpickle_WartsPing(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10traceutils_7scamper_5warts_4__pyx_unpickle_WartsPing(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  __Pyx_RefNannySetupContext("__pyx_unpickle_WartsPing", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x5752e4d:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x5752e4d = (dst, family, method, ping_sent, probe_size, responses, src, start, statistics, timeout, ttl, type, userid, version, wait))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0x5752e4d) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x5752e4d:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x5752e4d = (dst, family, method, ping_sent, probe_size, responses, src, start, statistics, timeout, ttl, type, userid, version, wait))" % __pyx_checksum)
 *     __pyx_result = WartsPing.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum != 0x5752e4d:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x5752e4d = (dst, family, method, ping_sent, probe_size, responses, src, start, statistics, timeout, ttl, type, userid, version, wait))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = WartsPing.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0x57, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(2, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x5752e4d:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x5752e4d = (dst, family, method, ping_sent, probe_size, responses, src, start, statistics, timeout, ttl, type, userid, version, wait))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x5752e4d = (dst, family, method, ping_sent, probe_size, responses, src, start, statistics, timeout, ttl, type, userid, version, wait))" % __pyx_checksum)
 *     __pyx_result = WartsPing.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_WartsPing__set_state(<WartsPing> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_10traceutils_7scamper_5warts_WartsPing), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x5752e4d = (dst, family, method, ping_sent, probe_size, responses, src, start, statistics, timeout, ttl, type, userid, version, wait))" % __pyx_checksum)
 *     __pyx_result = WartsPing.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_WartsPing__set_state(<WartsPing> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":9
 *     __pyx_result = WartsPing.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_WartsPing__set_state(<WartsPing> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_WartsPing__set_state(WartsPing __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(2, 9, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_10traceutils_7scamper_5warts___pyx_unpickle_WartsPing__set_state(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x5752e4d = (dst, family, method, ping_sent, probe_size, responses, src, start, statistics, timeout, ttl, type, userid, version, wait))" % __pyx_checksum)
 *     __pyx_result = WartsPing.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_WartsPing__set_state(<WartsPing> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_WartsPing__set_state(<WartsPing> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_WartsPing__set_state(WartsPing __pyx_result, tuple __pyx_state):
 *     __pyx_result.dst = __pyx_state[0]; __pyx_result.family = __pyx_state[1]; __pyx_result.method = __pyx_state[2]; __pyx_result.ping_sent = __pyx_state[3]; __pyx_result.probe_size = __pyx_state[4]; __pyx_result.responses = __pyx_state[5]; __pyx_result.src = __pyx_state[6]; __pyx_result.start = __pyx_state[7]; __pyx_result.statistics = __pyx_state[8]; __pyx_result.timeout = __pyx_state[9]; __pyx_result.ttl = __pyx_state[10]; __pyx_result.type = __pyx_state[11]; __pyx_result.userid = __pyx_state[12]; __pyx_result.version = __pyx_state[13]; __pyx_result.wait = __pyx_state[14]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_WartsPing(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("traceutils.scamper.warts.__pyx_unpickle_WartsPing", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_WartsPing__set_state(<WartsPing> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_WartsPing__set_state(WartsPing __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.dst = __pyx_state[0]; __pyx_result.family = __pyx_state[1]; __pyx_result.method = __pyx_state[2]; __pyx_result.ping_sent = __pyx_state[3]; __pyx_result.probe_size = __pyx_state[4]; __pyx_result.responses = __pyx_state[5]; __pyx_result.src = __pyx_state[6]; __pyx_result.start = __pyx_state[7]; __pyx_result.statistics = __pyx_state[8]; __pyx_result.timeout = __pyx_state[9]; __pyx_result.ttl = __pyx_state[10]; __pyx_result.type = __pyx_state[11]; __pyx_result.userid = __pyx_state[12]; __pyx_result.version = __pyx_state[13]; __pyx_result.wait = __pyx_state[14]
 *     if len(__pyx_state) > 15 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_10traceutils_7scamper_5warts___pyx_unpickle_WartsPing__set_state(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  double __pyx_t_3;
  int __pyx_t_4;
  Py_ssize_t __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  __Pyx_RefNannySetupContext("__pyx_unpickle_WartsPing__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_WartsPing__set_state(WartsPing __pyx_result, tuple __pyx_state):
 *     __pyx_result.dst = __pyx_state[0]; __pyx_result.family = __pyx_state[1]; __pyx_result.method = __pyx_state[2]; __pyx_result.ping_sent = __pyx_state[3]; __pyx_result.probe_size = __pyx_state[4]; __pyx_result.responses = __pyx_state[5]; __pyx_result.src = __pyx_state[6]; __pyx_result.start = __pyx_state[7]; __pyx_result.statistics = __pyx_state[8]; __pyx_result.timeout = __pyx_state[9]; __pyx_result.ttl = __pyx_state[10]; __pyx_result.type = __pyx_state[11]; __pyx_result.userid = __pyx_state[12]; __pyx_result.version = __pyx_state[13]; __pyx_result.wait = __pyx_state[14]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 15 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[15])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyUnicode_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "unicode", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->dst);
  __Pyx_DECREF(__pyx_v___pyx_result->dst);
  __pyx_v___pyx_result->dst = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->family = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyUnicode_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "unicode", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->method);
  __Pyx_DECREF(__pyx_v___pyx_result->method);
  __pyx_v___pyx_result->method = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->ping_sent = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->probe_size = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 5, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->responses);
  __Pyx_DECREF(__pyx_v___pyx_result->responses);
  __pyx_v___pyx_result->responses = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 6, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyUnicode_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "unicode", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->src);
  __Pyx_DECREF(__pyx_v___pyx_result->src);
  __pyx_v___pyx_result->src = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 7, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyDict_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->start);
  __Pyx_DECREF(__pyx_v___pyx_result->start);
  __pyx_v___pyx_result->start = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 8, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyDict_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->statistics);
  __Pyx_DECREF(__pyx_v___pyx_result->statistics);
  __pyx_v___pyx_result->statistics = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 9, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_3 == (double)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->timeout = __pyx_t_3;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 10, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->ttl = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 11, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyUnicode_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "unicode", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->type);
  __Pyx_DECREF(__pyx_v___pyx_result->type);
  __pyx_v___pyx_result->type = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 12, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->userid = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 13, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyUnicode_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "unicode", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->version);
  __Pyx_DECREF(__pyx_v___pyx_result->version);
  __pyx_v___pyx_result->version = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 14, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_3 == (double)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->wait = __pyx_t_3;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_WartsPing__set_state(WartsPing __pyx_result, tuple __pyx_state):
 *     __pyx_result.dst = __pyx_state[0]; __pyx_result.family = __pyx_state[1]; __pyx_result.method = __pyx_state[2]; __pyx_result.ping_sent = __pyx_state[3]; __pyx_result.probe_size = __pyx_state[4]; __pyx_result.responses = __pyx_state[5]; __pyx_result.src = __pyx_state[6]; __pyx_result.start = __pyx_state[7]; __pyx_result.statistics = __pyx_state[8]; __pyx_result.timeout = __pyx_state[9]; __pyx_result.ttl = __pyx_state[10]; __pyx_result.type = __pyx_state[11]; __pyx_result.userid = __pyx_state[12]; __pyx_result.version = __pyx_state[13]; __pyx_result.wait = __pyx_state[14]
 *     if len(__pyx_state) > 15 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[15])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(2, 13, __pyx_L1_error)
  }
  __pyx_t_5 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(2, 13, __pyx_L1_error)
  __pyx_t_6 = ((__pyx_t_5 > 15) != 0);
  if (__pyx_t_6) {
  } else {
    __pyx_t_4 = __pyx_t_6;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_6 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_6 == ((int)-1))) __PYX_ERR(2, 13, __pyx_L1_error)
  __pyx_t_7 = (__pyx_t_6 != 0);
  __pyx_t_4 = __pyx_t_7;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_4) {

    /* "(tree fragment)":14
 *     __pyx_result.dst = __pyx_state[0]; __pyx_result.family = __pyx_state[1]; __pyx_result.method = __pyx_state[2]; __pyx_result.ping_sent = __pyx_state[3]; __pyx_result.probe_size = __pyx_state[4]; __pyx_result.responses = __pyx_state[5]; __pyx_result.src = __pyx_state[6]; __pyx_result.start = __pyx_state[7]; __pyx_result.statistics = __pyx_state[8]; __pyx_result.timeout = __pyx_state[9]; __pyx_result.ttl = __pyx_state[10]; __pyx_result.type = __pyx_state[11]; __pyx_result.userid = __pyx_state[12]; __pyx_result.version = __pyx_state[13]; __pyx_result.wait = __pyx_state[14]
 *     if len(__pyx_state) > 15 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[15])             # <<<<<<<<<<<<<<
 */
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_n_s_update); if (unlikely(!__pyx_t_9)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(2, 14, __pyx_L1_error)
    }
    __pyx_t_8 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 15, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_10 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_9))) {
      __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_9);
      if (likely(__pyx_t_10)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_9);
        __Pyx_INCREF(__pyx_t_10);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_9, function);
      }
    }
    __pyx_t_1 = (__pyx_t_10) ? __Pyx_PyObject_Call2Args(__pyx_t_9, __pyx_t_10, __pyx_t_8) : __Pyx_PyObject_CallOneArg(__pyx_t_9, __pyx_t_8);
    __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_WartsPing__set_state(WartsPing __pyx_result, tuple __pyx_state):
 *     __pyx_result.dst = __pyx_state[0]; __pyx_result.family = __pyx_state[1]; __pyx_result.method = __pyx_state[2]; __pyx_result.ping_sent = __pyx_state[3]; __pyx_result.probe_size = __pyx_state[4]; __pyx_result.responses = __pyx_state[5]; __pyx_result.src = __pyx_state[6]; __pyx_result.start = __pyx_state[7]; __pyx_result.statistics = __pyx_state[8]; __pyx_result.timeout = __pyx_state[9]; __pyx_result.ttl = __pyx_state[10]; __pyx_result.type = __pyx_state[11]; __pyx_result.userid = __pyx_state[12]; __pyx_result.version = __pyx_state[13]; __pyx_result.wait = __pyx_state[14]
 *     if len(__pyx_state) > 15 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[15])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_WartsPing__set_state(<WartsPing> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_WartsPing__set_state(WartsPing __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.dst = __pyx_state[0]; __pyx_result.family = __pyx_state[1]; __pyx_result.method = __pyx_state[2]; __pyx_result.ping_sent = __pyx_state[3]; __pyx_result.probe_size = __pyx_state[4]; __pyx_result.responses = __pyx_state[5]; __pyx_result.src = __pyx_state[6]; __pyx_result.start = __pyx_state[7]; __pyx_result.statistics = __pyx_state[8]; __pyx_result.timeout = __pyx_state[9]; __pyx_result.ttl = __pyx_state[10]; __pyx_result.type = __pyx_state[11]; __pyx_result.userid = __pyx_state[12]; __pyx_result.version = __pyx_state[13]; __pyx_result.wait = __pyx_state[14]
 *     if len(__pyx_state) > 15 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_AddTraceback("traceutils.scamper.warts.__pyx_unpickle_WartsPing__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_WartsPingResponse(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_7__pyx_unpickle_WartsPingResponse(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_10traceutils_7scamper_5warts_6__pyx_unpickle_WartsPingResponse[] = "__pyx_unpickle_WartsPingResponse(__pyx_type, long __pyx_checksum, __pyx_state)";
static PyMethodDef __pyx_mdef_10traceutils_7scamper_5warts_7__pyx_unpickle_WartsPingResponse = {"__pyx_unpickle_WartsPingResponse", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_10traceutils_7scamper_5warts_7__pyx_unpickle_WartsPingResponse, METH_VARARGS|METH_KEYWORDS, __pyx_doc_10traceutils_7scamper_5warts_6__pyx_unpickle_WartsPingResponse};
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_7__pyx_unpickle_WartsPingResponse(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_WartsPingResponse (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_WartsPingResponse", 1, 3, 3, 1); __PYX_ERR(2, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_WartsPingResponse", 1, 3, 3, 2); __PYX_ERR(2, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_WartsPingResponse") < 0)) __PYX_ERR(2, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(2, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_WartsPingResponse", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(2, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("traceutils.scamper.warts.__pyx_unpickle_WartsPingResponse", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_6__pyx_unpickle_WartsPingResponse(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10traceutils_7scamper_5warts_6__pyx_unpickle_WartsPingResponse(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  __Pyx_RefNannySetupContext("__pyx_unpickle_WartsPingResponse", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x78ac107:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x78ac107 = (family, icmp_code, icmp_type, probe_ipid, reply_ipid, reply_proto, reply_size, reply_ttl, rtt, rx, seq, tx, type))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0x78ac107) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x78ac107:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x78ac107 = (family, icmp_code, icmp_type, probe_ipid, reply_ipid, reply_proto, reply_size, reply_ttl, rtt, rx, seq, tx, type))" % __pyx_checksum)
 *     __pyx_result = WartsPingResponse.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum != 0x78ac107:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x78ac107 = (family, icmp_code, icmp_type, probe_ipid, reply_ipid, reply_proto, reply_size, reply_ttl, rtt, rx, seq, tx, type))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = WartsPingResponse.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0x78, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(2, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x78ac107:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x78ac107 = (family, icmp_code, icmp_type, probe_ipid, reply_ipid, reply_proto, reply_size, reply_ttl, rtt, rx, seq, tx, type))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x78ac107 = (family, icmp_code, icmp_type, probe_ipid, reply_ipid, reply_proto, reply_size, reply_ttl, rtt, rx, seq, tx, type))" % __pyx_checksum)
 *     __pyx_result = WartsPingResponse.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_WartsPingResponse__set_state(<WartsPingResponse> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_10traceutils_7scamper_5warts_WartsPingResponse), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x78ac107 = (family, icmp_code, icmp_type, probe_ipid, reply_ipid, reply_proto, reply_size, reply_ttl, rtt, rx, seq, tx, type))" % __pyx_checksum)
 *     __pyx_result = WartsPingResponse.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_WartsPingResponse__set_state(<WartsPingResponse> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":9
 *     __pyx_result = WartsPingResponse.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_WartsPingResponse__set_state(<WartsPingResponse> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_WartsPingResponse__set_state(WartsPingResponse __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(2, 9, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_10traceutils_7scamper_5warts___pyx_unpickle_WartsPingResponse__set_state(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x78ac107 = (family, icmp_code, icmp_type, probe_ipid, reply_ipid, reply_proto, reply_size, reply_ttl, rtt, rx, seq, tx, type))" % __pyx_checksum)
 *     __pyx_result = WartsPingResponse.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_WartsPingResponse__set_state(<WartsPingResponse> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_WartsPingResponse__set_state(<WartsPingResponse> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_WartsPingResponse__set_state(WartsPingResponse __pyx_result, tuple __pyx_state):
 *     __pyx_result.family = __pyx_state[0]; __pyx_result.icmp_code = __pyx_state[1]; __pyx_result.icmp_type = __pyx_state[2]; __pyx_result.probe_ipid = __pyx_state[3]; __pyx_result.reply_ipid = __pyx_state[4]; __pyx_result.reply_proto = __pyx_state[5]; __pyx_result.reply_size = __pyx_state[6]; __pyx_result.reply_ttl = __pyx_state[7]; __pyx_result.rtt = __pyx_state[8]; __pyx_result.rx = __pyx_state[9]; __pyx_result.seq = __pyx_state[10]; __pyx_result.tx = __pyx_state[11]; __pyx_result.type = __pyx_state[12]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_WartsPingResponse(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("traceutils.scamper.warts.__pyx_unpickle_WartsPingResponse", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_WartsPingResponse__set_state(<WartsPingResponse> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_WartsPingResponse__set_state(WartsPingResponse __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.family = __pyx_state[0]; __pyx_result.icmp_code = __pyx_state[1]; __pyx_result.icmp_type = __pyx_state[2]; __pyx_result.probe_ipid = __pyx_state[3]; __pyx_result.reply_ipid = __pyx_state[4]; __pyx_result.reply_proto = __pyx_state[5]; __pyx_result.reply_size = __pyx_state[6]; __pyx_result.reply_ttl = __pyx_state[7]; __pyx_result.rtt = __pyx_state[8]; __pyx_result.rx = __pyx_state[9]; __pyx_result.seq = __pyx_state[10]; __pyx_result.tx = __pyx_state[11]; __pyx_result.type = __pyx_state[12]
 *     if len(__pyx_state) > 13 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_10traceutils_7scamper_5warts___pyx_unpickle_WartsPingResponse__set_state(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  double __pyx_t_3;
  enum __pyx_t_10traceutils_7scamper_3hop_ICMPType __pyx_t_4;
  int __pyx_t_5;
  Py_ssize_t __pyx_t_6;
  int __pyx_t_7;
  int __pyx_t_8;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  __Pyx_RefNannySetupContext("__pyx_unpickle_WartsPingResponse__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_WartsPingResponse__set_state(WartsPingResponse __pyx_result, tuple __pyx_state):
 *     __pyx_result.family = __pyx_state[0]; __pyx_result.icmp_code = __pyx_state[1]; __pyx_result.icmp_type = __pyx_state[2]; __pyx_result.probe_ipid = __pyx_state[3]; __pyx_result.reply_ipid = __pyx_state[4]; __pyx_result.reply_proto = __pyx_state[5]; __pyx_result.reply_size = __pyx_state[6]; __pyx_result.reply_ttl = __pyx_state[7]; __pyx_result.rtt = __pyx_state[8]; __pyx_result.rx = __pyx_state[9]; __pyx_result.seq = __pyx_state[10]; __pyx_result.tx = __pyx_state[11]; __pyx_result.type = __pyx_state[12]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 13 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[13])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->family = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->icmp_code = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->icmp_type = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->probe_ipid = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->reply_ipid = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 5, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyUnicode_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "unicode", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->reply_proto);
  __Pyx_DECREF(__pyx_v___pyx_result->reply_proto);
  __pyx_v___pyx_result->reply_proto = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 6, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->reply_size = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 7, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->reply_ttl = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 8, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_3 == (double)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->rtt = __pyx_t_3;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 9, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyDict_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->rx);
  __Pyx_DECREF(__pyx_v___pyx_result->rx);
  __pyx_v___pyx_result->rx = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 10, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->seq = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 11, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyDict_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->tx);
  __Pyx_DECREF(__pyx_v___pyx_result->tx);
  __pyx_v___pyx_result->tx = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 12, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = ((enum __pyx_t_10traceutils_7scamper_3hop_ICMPType)__Pyx_PyInt_As_enum____pyx_t_10traceutils_7scamper_3hop_ICMPType(__pyx_t_1)); if (unlikely(PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->type = __pyx_t_4;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_WartsPingResponse__set_state(WartsPingResponse __pyx_result, tuple __pyx_state):
 *     __pyx_result.family = __pyx_state[0]; __pyx_result.icmp_code = __pyx_state[1]; __pyx_result.icmp_type = __pyx_state[2]; __pyx_result.probe_ipid = __pyx_state[3]; __pyx_result.reply_ipid = __pyx_state[4]; __pyx_result.reply_proto = __pyx_state[5]; __pyx_result.reply_size = __pyx_state[6]; __pyx_result.reply_ttl = __pyx_state[7]; __pyx_result.rtt = __pyx_state[8]; __pyx_result.rx = __pyx_state[9]; __pyx_result.seq = __pyx_state[10]; __pyx_result.tx = __pyx_state[11]; __pyx_result.type = __pyx_state[12]
 *     if len(__pyx_state) > 13 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[13])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(2, 13, __pyx_L1_error)
  }
  __pyx_t_6 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_6 == ((Py_ssize_t)-1))) __PYX_ERR(2, 13, __pyx_L1_error)
  __pyx_t_7 = ((__pyx_t_6 > 13) != 0);
  if (__pyx_t_7) {
  } else {
    __pyx_t_5 = __pyx_t_7;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_7 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_7 == ((int)-1))) __PYX_ERR(2, 13, __pyx_L1_error)
  __pyx_t_8 = (__pyx_t_7 != 0);
  __pyx_t_5 = __pyx_t_8;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_5) {

    /* "(tree fragment)":14
 *     __pyx_result.family = __pyx_state[0]; __pyx_result.icmp_code = __pyx_state[1]; __pyx_result.icmp_type = __pyx_state[2]; __pyx_result.probe_ipid = __pyx_state[3]; __pyx_result.reply_ipid = __pyx_state[4]; __pyx_result.reply_proto = __pyx_state[5]; __pyx_result.reply_size = __pyx_state[6]; __pyx_result.reply_ttl = __pyx_state[7]; __pyx_result.rtt = __pyx_state[8]; __pyx_result.rx = __pyx_state[9]; __pyx_result.seq = __pyx_state[10]; __pyx_result.tx = __pyx_state[11]; __pyx_result.type = __pyx_state[12]
 *     if len(__pyx_state) > 13 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[13])             # <<<<<<<<<<<<<<
 */
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_9)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_t_9, __pyx_n_s_update); if (unlikely(!__pyx_t_10)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(2, 14, __pyx_L1_error)
    }
    __pyx_t_9 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 13, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_9)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_11 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_10))) {
      __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_10);
      if (likely(__pyx_t_11)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_10);
        __Pyx_INCREF(__pyx_t_11);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_10, function);
      }
    }
    __pyx_t_1 = (__pyx_t_11) ? __Pyx_PyObject_Call2Args(__pyx_t_10, __pyx_t_11, __pyx_t_9) : __Pyx_PyObject_CallOneArg(__pyx_t_10, __pyx_t_9);
    __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_WartsPingResponse__set_state(WartsPingResponse __pyx_result, tuple __pyx_state):
 *     __pyx_result.family = __pyx_state[0]; __pyx_result.icmp_code = __pyx_state[1]; __pyx_result.icmp_type = __pyx_state[2]; __pyx_result.probe_ipid = __pyx_state[3]; __pyx_result.reply_ipid = __pyx_state[4]; __pyx_result.reply_proto = __pyx_state[5]; __pyx_result.reply_size = __pyx_state[6]; __pyx_result.reply_ttl = __pyx_state[7]; __pyx_result.rtt = __pyx_state[8]; __pyx_result.rx = __pyx_state[9]; __pyx_result.seq = __pyx_state[10]; __pyx_result.tx = __pyx_state[11]; __pyx_result.type = __pyx_state[12]
 *     if len(__pyx_state) > 13 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[13])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_WartsPingResponse__set_state(<WartsPingResponse> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_WartsPingResponse__set_state(WartsPingResponse __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.family = __pyx_state[0]; __pyx_result.icmp_code = __pyx_state[1]; __pyx_result.icmp_type = __pyx_state[2]; __pyx_result.probe_ipid = __pyx_state[3]; __pyx_result.reply_ipid = __pyx_state[4]; __pyx_result.reply_proto = __pyx_state[5]; __pyx_result.reply_size = __pyx_state[6]; __pyx_result.reply_ttl = __pyx_state[7]; __pyx_result.rtt = __pyx_state[8]; __pyx_result.rx = __pyx_state[9]; __pyx_result.seq = __pyx_state[10]; __pyx_result.tx = __pyx_state[11]; __pyx_result.type = __pyx_state[12]
 *     if len(__pyx_state) > 13 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_AddTraceback("traceutils.scamper.warts.__pyx_unpickle_WartsPingResponse__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_WartsReader(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_9__pyx_unpickle_WartsReader(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_10traceutils_7scamper_5warts_8__pyx_unpickle_WartsReader[] = "__pyx_unpickle_WartsReader(__pyx_type, long __pyx_checksum, __pyx_state)";
static PyMethodDef __pyx_mdef_10traceutils_7scamper_5warts_9__pyx_unpickle_WartsReader = {"__pyx_unpickle_WartsReader", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_10traceutils_7scamper_5warts_9__pyx_unpickle_WartsReader, METH_VARARGS|METH_KEYWORDS, __pyx_doc_10traceutils_7scamper_5warts_8__pyx_unpickle_WartsReader};
static PyObject *__pyx_pw_10traceutils_7scamper_5warts_9__pyx_unpickle_WartsReader(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_WartsReader (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_WartsReader", 1, 3, 3, 1); __PYX_ERR(2, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_WartsReader", 1, 3, 3, 2); __PYX_ERR(2, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_WartsReader") < 0)) __PYX_ERR(2, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(2, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_WartsReader", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(2, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("traceutils.scamper.warts.__pyx_unpickle_WartsReader", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_10traceutils_7scamper_5warts_8__pyx_unpickle_WartsReader(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10traceutils_7scamper_5warts_8__pyx_unpickle_WartsReader(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  __Pyx_RefNannySetupContext("__pyx_unpickle_WartsReader", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x93358b7:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x93358b7 = (filename, p))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0x93358b7) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x93358b7:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x93358b7 = (filename, p))" % __pyx_checksum)
 *     __pyx_result = WartsReader.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum != 0x93358b7:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x93358b7 = (filename, p))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = WartsReader.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0x93, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(2, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x93358b7:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x93358b7 = (filename, p))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x93358b7 = (filename, p))" % __pyx_checksum)
 *     __pyx_result = WartsReader.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_WartsReader__set_state(<WartsReader> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_10traceutils_7scamper_5warts_WartsReader), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x93358b7 = (filename, p))" % __pyx_checksum)
 *     __pyx_result = WartsReader.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_WartsReader__set_state(<WartsReader> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":9
 *     __pyx_result = WartsReader.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_WartsReader__set_state(<WartsReader> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_WartsReader__set_state(WartsReader __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(2, 9, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_10traceutils_7scamper_5warts___pyx_unpickle_WartsReader__set_state(((struct __pyx_obj_10traceutils_7scamper_5warts_WartsReader *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x93358b7 = (filename, p))" % __pyx_checksum)
 *     __pyx_result = WartsReader.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_WartsReader__set_state(<WartsReader> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_WartsReader__set_state(<WartsReader> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_WartsReader__set_state(WartsReader __pyx_result, tuple __pyx_state):
 *     __pyx_result.filename = __pyx_state[0]; __pyx_result.p = __pyx_state[1]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_WartsReader(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("traceutils.scamper.warts.__pyx_unpickle_WartsReader", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_WartsReader__set_state(<WartsReader> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_WartsReader__set_state(WartsReader __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.filename = __pyx_state[0]; __pyx_result.p = __pyx_state[1]
 *     if len(__pyx_state) > 2 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_10traceutils_7scamper_5warts___pyx_unpickle_WartsReader__set_state(struct __pyx_obj_10traceutils_7scamper_5warts_WartsReader *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  __Pyx_RefNannySetupContext("__pyx_unpickle_WartsReader__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_WartsReader__set_state(WartsReader __pyx_result, tuple __pyx_state):
 *     __pyx_result.filename = __pyx_state[0]; __pyx_result.p = __pyx_state[1]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 2 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[2])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyUnicode_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "unicode", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.filename);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.filename);
  __pyx_v___pyx_result->__pyx_base.filename = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.p);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.p);
  __pyx_v___pyx_result->__pyx_base.p = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_WartsReader__set_state(WartsReader __pyx_result, tuple __pyx_state):
 *     __pyx_result.filename = __pyx_state[0]; __pyx_result.p = __pyx_state[1]
 *     if len(__pyx_state) > 2 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[2])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(2, 13, __pyx_L1_error)
  }
  __pyx_t_3 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_3 == ((Py_ssize_t)-1))) __PYX_ERR(2, 13, __pyx_L1_error)
  __pyx_t_4 = ((__pyx_t_3 > 2) != 0);
  if (__pyx_t_4) {
  } else {
    __pyx_t_2 = __pyx_t_4;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_4 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(2, 13, __pyx_L1_error)
  __pyx_t_5 = (__pyx_t_4 != 0);
  __pyx_t_2 = __pyx_t_5;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_2) {

    /* "(tree fragment)":14
 *     __pyx_result.filename = __pyx_state[0]; __pyx_result.p = __pyx_state[1]
 *     if len(__pyx_state) > 2 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[2])             # <<<<<<<<<<<<<<
 */
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_6)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_update); if (unlikely(!__pyx_t_7)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(2, 14, __pyx_L1_error)
    }
    __pyx_t_6 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_8 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
      __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_7);
      if (likely(__pyx_t_8)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
        __Pyx_INCREF(__pyx_t_8);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_7, function);
      }
    }
    __pyx_t_1 = (__pyx_t_8) ? __Pyx_PyObject_Call2Args(__pyx_t_7, __pyx_t_8, __pyx_t_6) : __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_t_6);
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_WartsReader__set_state(WartsReader __pyx_result, tuple __pyx_state):
 *     __pyx_result.filename = __pyx_state[0]; __pyx_result.p = __pyx_state[1]
 *     if len(__pyx_state) > 2 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[2])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_WartsReader__set_state(<WartsReader> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_WartsReader__set_state(WartsReader __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.filename = __pyx_state[0]; __pyx_result.p = __pyx_state[1]
 *     if len(__pyx_state) > 2 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("traceutils.scamper.warts.__pyx_unpickle_WartsReader__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
static struct __pyx_vtabstruct_10traceutils_7scamper_5warts_WartsHop __pyx_vtable_10traceutils_7scamper_5warts_WartsHop;

static PyObject *__pyx_tp_new_10traceutils_7scamper_5warts_WartsHop(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_10traceutils_7scamper_5warts_WartsHop *p;
  PyObject *o = __pyx_ptype_10traceutils_7scamper_3hop_Hop->tp_new(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_10traceutils_7scamper_5warts_WartsHop *)o);
  p->__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_10traceutils_7scamper_3hop_Hop*)__pyx_vtabptr_10traceutils_7scamper_5warts_WartsHop;
  p->icmpext = ((PyObject*)Py_None); Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_10traceutils_7scamper_5warts_WartsHop(PyObject *o) {
  struct __pyx_obj_10traceutils_7scamper_5warts_WartsHop *p = (struct __pyx_obj_10traceutils_7scamper_5warts_WartsHop *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->icmpext);
  #if CYTHON_USE_TYPE_SLOTS
  if (PyType_IS_GC(Py_TYPE(o)->tp_base))
  #endif
  PyObject_GC_Track(o);
  if (likely(__pyx_ptype_10traceutils_7scamper_3hop_Hop)) __pyx_ptype_10traceutils_7scamper_3hop_Hop->tp_dealloc(o); else __Pyx_call_next_tp_dealloc(o, __pyx_tp_dealloc_10traceutils_7scamper_5warts_WartsHop);
}

static int __pyx_tp_traverse_10traceutils_7scamper_5warts_WartsHop(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_10traceutils_7scamper_5warts_WartsHop *p = (struct __pyx_obj_10traceutils_7scamper_5warts_WartsHop *)o;
  e = ((likely(__pyx_ptype_10traceutils_7scamper_3hop_Hop)) ? ((__pyx_ptype_10traceutils_7scamper_3hop_Hop->tp_traverse) ? __pyx_ptype_10traceutils_7scamper_3hop_Hop->tp_traverse(o, v, a) : 0) : __Pyx_call_next_tp_traverse(o, v, a, __pyx_tp_traverse_10traceutils_7scamper_5warts_WartsHop)); if (e) return e;
  if (p->icmpext) {
    e = (*v)(p->icmpext, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_10traceutils_7scamper_5warts_WartsHop(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_10traceutils_7scamper_5warts_WartsHop *p = (struct __pyx_obj_10traceutils_7scamper_5warts_WartsHop *)o;
  if (likely(__pyx_ptype_10traceutils_7scamper_3hop_Hop)) { if (__pyx_ptype_10traceutils_7scamper_3hop_Hop->tp_clear) __pyx_ptype_10traceutils_7scamper_3hop_Hop->tp_clear(o); } else __Pyx_call_next_tp_clear(o, __pyx_tp_clear_10traceutils_7scamper_5warts_WartsHop);
  tmp = ((PyObject*)p->icmpext);
  p->icmpext = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_10traceutils_7scamper_5warts_8WartsHop_icmpext(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_10traceutils_7scamper_5warts_8WartsHop_7icmpext_1__get__(o);
}

static int __pyx_setprop_10traceutils_7scamper_5warts_8WartsHop_icmpext(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_10traceutils_7scamper_5warts_8WartsHop_7icmpext_3__set__(o, v);
  }
  else {
    return __pyx_pw_10traceutils_7scamper_5warts_8WartsHop_7icmpext_5__del__(o);
  }
}

static PyObject *__pyx_getprop_10traceutils_7scamper_5warts_8WartsHop_probe_id(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_10traceutils_7scamper_5warts_8WartsHop_8probe_id_1__get__(o);
}

static int __pyx_setprop_10traceutils_7scamper_5warts_8WartsHop_probe_id(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_10traceutils_7scamper_5warts_8WartsHop_8probe_id_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_10traceutils_7scamper_5warts_8WartsHop_probe_size(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_10traceutils_7scamper_5warts_8WartsHop_10probe_size_1__get__(o);
}

static int __pyx_setprop_10traceutils_7scamper_5warts_8WartsHop_probe_size(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_10traceutils_7scamper_5warts_8WartsHop_10probe_size_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_10traceutils_7scamper_5warts_8WartsHop_reply_ipid(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_10traceutils_7scamper_5warts_8WartsHop_10reply_ipid_1__get__(o);
}

static int __pyx_setprop_10traceutils_7scamper_5warts_8WartsHop_reply_ipid(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_10traceutils_7scamper_5warts_8WartsHop_10reply_ipid_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_10traceutils_7scamper_5warts_8WartsHop_icmp_q_ipl(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_10traceutils_7scamper_5warts_8WartsHop_10icmp_q_ipl_1__get__(o);
}

static int __pyx_setprop_10traceutils_7scamper_5warts_8WartsHop_icmp_q_ipl(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_10traceutils_7scamper_5warts_8WartsHop_10icmp_q_ipl_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_10traceutils_7scamper_5warts_8WartsHop_icmp_nhmtu(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_10traceutils_7scamper_5warts_8WartsHop_10icmp_nhmtu_1__get__(o);
}

static int __pyx_setprop_10traceutils_7scamper_5warts_8WartsHop_icmp_nhmtu(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_10traceutils_7scamper_5warts_8WartsHop_10icmp_nhmtu_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyMethodDef __pyx_methods_10traceutils_7scamper_5warts_WartsHop[] = {
  {"__reduce_cython__", (PyCFunction)__pyx_pw_10traceutils_7scamper_5warts_8WartsHop_3__reduce_cython__, METH_NOARGS, __pyx_doc_10traceutils_7scamper_5warts_8WartsHop_2__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_10traceutils_7scamper_5warts_8WartsHop_5__setstate_cython__, METH_O, __pyx_doc_10traceutils_7scamper_5warts_8WartsHop_4__setstate_cython__},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_10traceutils_7scamper_5warts_WartsHop[] = {
  {(char *)"icmpext", __pyx_getprop_10traceutils_7scamper_5warts_8WartsHop_icmpext, __pyx_setprop_10traceutils_7scamper_5warts_8WartsHop_icmpext, (char *)"icmpext: list", 0},
  {(char *)"probe_id", __pyx_getprop_10traceutils_7scamper_5warts_8WartsHop_probe_id, __pyx_setprop_10traceutils_7scamper_5warts_8WartsHop_probe_id, (char *)"probe_id: 'int'", 0},
  {(char *)"probe_size", __pyx_getprop_10traceutils_7scamper_5warts_8WartsHop_probe_size, __pyx_setprop_10traceutils_7scamper_5warts_8WartsHop_probe_size, (char *)"probe_size: 'unsigned short'", 0},
  {(char *)"reply_ipid", __pyx_getprop_10traceutils_7scamper_5warts_8WartsHop_reply_ipid, __pyx_setprop_10traceutils_7scamper_5warts_8WartsHop_reply_ipid, (char *)"reply_ipid: 'int'", 0},
  {(char *)"icmp_q_ipl", __pyx_getprop_10traceutils_7scamper_5warts_8WartsHop_icmp_q_ipl, __pyx_setprop_10traceutils_7scamper_5warts_8WartsHop_icmp_q_ipl, (char *)"icmp_q_ipl: 'int'", 0},
  {(char *)"icmp_nhmtu", __pyx_getprop_10traceutils_7scamper_5warts_8WartsHop_icmp_nhmtu, __pyx_setprop_10traceutils_7scamper_5warts_8WartsHop_icmp_nhmtu, (char *)"icmp_nhmtu: 'int'", 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_10traceutils_7scamper_5warts_WartsHop = {
  PyVarObject_HEAD_INIT(0, 0)
  "traceutils.scamper.warts.WartsHop", /*tp_name*/
  sizeof(struct __pyx_obj_10traceutils_7scamper_5warts_WartsHop), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_10traceutils_7scamper_5warts_WartsHop, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "WartsHop(unicode addr=u'', unsigned char probe_ttl=0, int probe_id=-1, unsigned short probe_size=0, double rtt=-1, unsigned char reply_ttl=0, int reply_tos=-1, unsigned short reply_size=0, int reply_ipid=-1, unsigned char icmp_type=0, unsigned char icmp_code=0, unsigned char icmp_q_ttl=1, int icmp_q_ipl=-1, int icmp_q_tos=-1, list icmpext=None, int icmp_nhmtu=-1, int family=0, **kwargs)", /*tp_doc*/
  __pyx_tp_traverse_10traceutils_7scamper_5warts_WartsHop, /*tp_traverse*/
  __pyx_tp_clear_10traceutils_7scamper_5warts_WartsHop, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_10traceutils_7scamper_5warts_WartsHop, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_10traceutils_7scamper_5warts_WartsHop, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_10traceutils_7scamper_5warts_8WartsHop_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_10traceutils_7scamper_5warts_WartsHop, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_10traceutils_7scamper_5warts_WartsTrace __pyx_vtable_10traceutils_7scamper_5warts_WartsTrace;

static PyObject *__pyx_tp_new_10traceutils_7scamper_5warts_WartsTrace(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *p;
  PyObject *o = __pyx_ptype_10traceutils_7scamper_3hop_Trace->tp_new(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *)o);
  p->__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_10traceutils_7scamper_3hop_Trace*)__pyx_vtabptr_10traceutils_7scamper_5warts_WartsTrace;
  p->type = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->version = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->method = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->stop_reason = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->start = ((PyObject*)Py_None); Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_10traceutils_7scamper_5warts_WartsTrace(PyObject *o) {
  struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *p = (struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->type);
  Py_CLEAR(p->version);
  Py_CLEAR(p->method);
  Py_CLEAR(p->stop_reason);
  Py_CLEAR(p->start);
  PyObject_GC_Track(o);
  if (likely(__pyx_ptype_10traceutils_7scamper_3hop_Trace)) __pyx_ptype_10traceutils_7scamper_3hop_Trace->tp_dealloc(o); else __Pyx_call_next_tp_dealloc(o, __pyx_tp_dealloc_10traceutils_7scamper_5warts_WartsTrace);
}

static int __pyx_tp_traverse_10traceutils_7scamper_5warts_WartsTrace(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *p = (struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *)o;
  e = ((likely(__pyx_ptype_10traceutils_7scamper_3hop_Trace)) ? ((__pyx_ptype_10traceutils_7scamper_3hop_Trace->tp_traverse) ? __pyx_ptype_10traceutils_7scamper_3hop_Trace->tp_traverse(o, v, a) : 0) : __Pyx_call_next_tp_traverse(o, v, a, __pyx_tp_traverse_10traceutils_7scamper_5warts_WartsTrace)); if (e) return e;
  if (p->start) {
    e = (*v)(p->start, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_10traceutils_7scamper_5warts_WartsTrace(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *p = (struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace *)o;
  if (likely(__pyx_ptype_10traceutils_7scamper_3hop_Trace)) { if (__pyx_ptype_10traceutils_7scamper_3hop_Trace->tp_clear) __pyx_ptype_10traceutils_7scamper_3hop_Trace->tp_clear(o); } else __Pyx_call_next_tp_clear(o, __pyx_tp_clear_10traceutils_7scamper_5warts_WartsTrace);
  tmp = ((PyObject*)p->start);
  p->start = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_10traceutils_7scamper_5warts_10WartsTrace_type(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_4type_1__get__(o);
}

static int __pyx_setprop_10traceutils_7scamper_5warts_10WartsTrace_type(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_4type_3__set__(o, v);
  }
  else {
    return __pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_4type_5__del__(o);
  }
}

static PyObject *__pyx_getprop_10traceutils_7scamper_5warts_10WartsTrace_version(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_7version_1__get__(o);
}

static int __pyx_setprop_10traceutils_7scamper_5warts_10WartsTrace_version(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_7version_3__set__(o, v);
  }
  else {
    return __pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_7version_5__del__(o);
  }
}

static PyObject *__pyx_getprop_10traceutils_7scamper_5warts_10WartsTrace_userid(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_6userid_1__get__(o);
}

static int __pyx_setprop_10traceutils_7scamper_5warts_10WartsTrace_userid(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_6userid_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_10traceutils_7scamper_5warts_10WartsTrace_method(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_6method_1__get__(o);
}

static int __pyx_setprop_10traceutils_7scamper_5warts_10WartsTrace_method(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_6method_3__set__(o, v);
  }
  else {
    return __pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_6method_5__del__(o);
  }
}

static PyObject *__pyx_getprop_10traceutils_7scamper_5warts_10WartsTrace_icmp_sum(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_8icmp_sum_1__get__(o);
}

static int __pyx_setprop_10traceutils_7scamper_5warts_10WartsTrace_icmp_sum(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_8icmp_sum_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_10traceutils_7scamper_5warts_10WartsTrace_stop_reason(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_11stop_reason_1__get__(o);
}

static int __pyx_setprop_10traceutils_7scamper_5warts_10WartsTrace_stop_reason(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_11stop_reason_3__set__(o, v);
  }
  else {
    return __pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_11stop_reason_5__del__(o);
  }
}

static PyObject *__pyx_getprop_10traceutils_7scamper_5warts_10WartsTrace_stop_data(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_9stop_data_1__get__(o);
}

static int __pyx_setprop_10traceutils_7scamper_5warts_10WartsTrace_stop_data(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_9stop_data_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_10traceutils_7scamper_5warts_10WartsTrace_start(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_5start_1__get__(o);
}

static int __pyx_setprop_10traceutils_7scamper_5warts_10WartsTrace_start(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_5start_3__set__(o, v);
  }
  else {
    return __pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_5start_5__del__(o);
  }
}

static PyObject *__pyx_getprop_10traceutils_7scamper_5warts_10WartsTrace_hop_count(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_9hop_count_1__get__(o);
}

static int __pyx_setprop_10traceutils_7scamper_5warts_10WartsTrace_hop_count(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_9hop_count_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_10traceutils_7scamper_5warts_10WartsTrace_attempts(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_8attempts_1__get__(o);
}

static int __pyx_setprop_10traceutils_7scamper_5warts_10WartsTrace_attempts(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_8attempts_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_10traceutils_7scamper_5warts_10WartsTrace_hoplimit(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_8hoplimit_1__get__(o);
}

static int __pyx_setprop_10traceutils_7scamper_5warts_10WartsTrace_hoplimit(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_8hoplimit_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_10traceutils_7scamper_5warts_10WartsTrace_firsthop(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_8firsthop_1__get__(o);
}

static int __pyx_setprop_10traceutils_7scamper_5warts_10WartsTrace_firsthop(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_8firsthop_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_10traceutils_7scamper_5warts_10WartsTrace_wait(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_4wait_1__get__(o);
}

static int __pyx_setprop_10traceutils_7scamper_5warts_10WartsTrace_wait(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_4wait_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_10traceutils_7scamper_5warts_10WartsTrace_wait_probe(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_10wait_probe_1__get__(o);
}

static int __pyx_setprop_10traceutils_7scamper_5warts_10WartsTrace_wait_probe(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_10wait_probe_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_10traceutils_7scamper_5warts_10WartsTrace_tos(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_3tos_1__get__(o);
}

static int __pyx_setprop_10traceutils_7scamper_5warts_10WartsTrace_tos(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_3tos_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_10traceutils_7scamper_5warts_10WartsTrace_probe_size(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_10probe_size_1__get__(o);
}

static int __pyx_setprop_10traceutils_7scamper_5warts_10WartsTrace_probe_size(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_10probe_size_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_10traceutils_7scamper_5warts_10WartsTrace_probe_count(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_11probe_count_1__get__(o);
}

static int __pyx_setprop_10traceutils_7scamper_5warts_10WartsTrace_probe_count(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_11probe_count_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyMethodDef __pyx_methods_10traceutils_7scamper_5warts_WartsTrace[] = {
  {"__reduce_cython__", (PyCFunction)__pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_3__reduce_cython__, METH_NOARGS, __pyx_doc_10traceutils_7scamper_5warts_10WartsTrace_2__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_5__setstate_cython__, METH_O, __pyx_doc_10traceutils_7scamper_5warts_10WartsTrace_4__setstate_cython__},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_10traceutils_7scamper_5warts_WartsTrace[] = {
  {(char *)"type", __pyx_getprop_10traceutils_7scamper_5warts_10WartsTrace_type, __pyx_setprop_10traceutils_7scamper_5warts_10WartsTrace_type, (char *)"type: unicode", 0},
  {(char *)"version", __pyx_getprop_10traceutils_7scamper_5warts_10WartsTrace_version, __pyx_setprop_10traceutils_7scamper_5warts_10WartsTrace_version, (char *)"version: unicode", 0},
  {(char *)"userid", __pyx_getprop_10traceutils_7scamper_5warts_10WartsTrace_userid, __pyx_setprop_10traceutils_7scamper_5warts_10WartsTrace_userid, (char *)"userid: 'int'", 0},
  {(char *)"method", __pyx_getprop_10traceutils_7scamper_5warts_10WartsTrace_method, __pyx_setprop_10traceutils_7scamper_5warts_10WartsTrace_method, (char *)"method: unicode", 0},
  {(char *)"icmp_sum", __pyx_getprop_10traceutils_7scamper_5warts_10WartsTrace_icmp_sum, __pyx_setprop_10traceutils_7scamper_5warts_10WartsTrace_icmp_sum, (char *)"icmp_sum: 'int'", 0},
  {(char *)"stop_reason", __pyx_getprop_10traceutils_7scamper_5warts_10WartsTrace_stop_reason, __pyx_setprop_10traceutils_7scamper_5warts_10WartsTrace_stop_reason, (char *)"stop_reason: unicode", 0},
  {(char *)"stop_data", __pyx_getprop_10traceutils_7scamper_5warts_10WartsTrace_stop_data, __pyx_setprop_10traceutils_7scamper_5warts_10WartsTrace_stop_data, (char *)"stop_data: 'int'", 0},
  {(char *)"start", __pyx_getprop_10traceutils_7scamper_5warts_10WartsTrace_start, __pyx_setprop_10traceutils_7scamper_5warts_10WartsTrace_start, (char *)"start: dict", 0},
  {(char *)"hop_count", __pyx_getprop_10traceutils_7scamper_5warts_10WartsTrace_hop_count, __pyx_setprop_10traceutils_7scamper_5warts_10WartsTrace_hop_count, (char *)"hop_count: 'int'", 0},
  {(char *)"attempts", __pyx_getprop_10traceutils_7scamper_5warts_10WartsTrace_attempts, __pyx_setprop_10traceutils_7scamper_5warts_10WartsTrace_attempts, (char *)"attempts: 'int'", 0},
  {(char *)"hoplimit", __pyx_getprop_10traceutils_7scamper_5warts_10WartsTrace_hoplimit, __pyx_setprop_10traceutils_7scamper_5warts_10WartsTrace_hoplimit, (char *)"hoplimit: 'unsigned char'", 0},
  {(char *)"firsthop", __pyx_getprop_10traceutils_7scamper_5warts_10WartsTrace_firsthop, __pyx_setprop_10traceutils_7scamper_5warts_10WartsTrace_firsthop, (char *)"firsthop: 'unsigned char'", 0},
  {(char *)"wait", __pyx_getprop_10traceutils_7scamper_5warts_10WartsTrace_wait, __pyx_setprop_10traceutils_7scamper_5warts_10WartsTrace_wait, (char *)"wait: 'double'", 0},
  {(char *)"wait_probe", __pyx_getprop_10traceutils_7scamper_5warts_10WartsTrace_wait_probe, __pyx_setprop_10traceutils_7scamper_5warts_10WartsTrace_wait_probe, (char *)"wait_probe: 'int'", 0},
  {(char *)"tos", __pyx_getprop_10traceutils_7scamper_5warts_10WartsTrace_tos, __pyx_setprop_10traceutils_7scamper_5warts_10WartsTrace_tos, (char *)"tos: 'int'", 0},
  {(char *)"probe_size", __pyx_getprop_10traceutils_7scamper_5warts_10WartsTrace_probe_size, __pyx_setprop_10traceutils_7scamper_5warts_10WartsTrace_probe_size, (char *)"probe_size: 'unsigned char'", 0},
  {(char *)"probe_count", __pyx_getprop_10traceutils_7scamper_5warts_10WartsTrace_probe_count, __pyx_setprop_10traceutils_7scamper_5warts_10WartsTrace_probe_count, (char *)"probe_count: 'unsigned char'", 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_10traceutils_7scamper_5warts_WartsTrace = {
  PyVarObject_HEAD_INIT(0, 0)
  "traceutils.scamper.warts.WartsTrace", /*tp_name*/
  sizeof(struct __pyx_obj_10traceutils_7scamper_5warts_WartsTrace), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_10traceutils_7scamper_5warts_WartsTrace, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "WartsTrace(unicode type=u'', unicode version=u'', int userid=-1, unicode method=u'', unicode src=u'', unicode dst=u'', int icmp_sum=-1, unicode stop_reason=u'', int stop_data=-1, dict start=None, int hop_count=-1, int attempts=-1, unsigned char hoplimit=0, unsigned char firsthop=1, double wait=-1, int wait_probe=-1, int tos=-1, unsigned short probe_size=0, unsigned char probe_count=0, list hops=None, unicode list_name=u'', int id=-1, unicode hostname=u'', long start_time=0)", /*tp_doc*/
  __pyx_tp_traverse_10traceutils_7scamper_5warts_WartsTrace, /*tp_traverse*/
  __pyx_tp_clear_10traceutils_7scamper_5warts_WartsTrace, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_10traceutils_7scamper_5warts_WartsTrace, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_10traceutils_7scamper_5warts_WartsTrace, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_10traceutils_7scamper_5warts_10WartsTrace_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_10traceutils_7scamper_5warts_WartsTrace, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static PyObject *__pyx_tp_new_10traceutils_7scamper_5warts_WartsPing(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *)o);
  p->type = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->version = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->method = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->src = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->dst = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->start = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->statistics = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->responses = ((PyObject*)Py_None); Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_10traceutils_7scamper_5warts_WartsPing(PyObject *o) {
  struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *p = (struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->type);
  Py_CLEAR(p->version);
  Py_CLEAR(p->method);
  Py_CLEAR(p->src);
  Py_CLEAR(p->dst);
  Py_CLEAR(p->start);
  Py_CLEAR(p->statistics);
  Py_CLEAR(p->responses);
  (*Py_TYPE(o)->tp_free)(o);
}

static int __pyx_tp_traverse_10traceutils_7scamper_5warts_WartsPing(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *p = (struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *)o;
  if (p->start) {
    e = (*v)(p->start, a); if (e) return e;
  }
  if (p->statistics) {
    e = (*v)(p->statistics, a); if (e) return e;
  }
  if (p->responses) {
    e = (*v)(p->responses, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_10traceutils_7scamper_5warts_WartsPing(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *p = (struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing *)o;
  tmp = ((PyObject*)p->start);
  p->start = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->statistics);
  p->statistics = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->responses);
  p->responses = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_10traceutils_7scamper_5warts_9WartsPing_type(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_10traceutils_7scamper_5warts_9WartsPing_4type_1__get__(o);
}

static int __pyx_setprop_10traceutils_7scamper_5warts_9WartsPing_type(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_10traceutils_7scamper_5warts_9WartsPing_4type_3__set__(o, v);
  }
  else {
    return __pyx_pw_10traceutils_7scamper_5warts_9WartsPing_4type_5__del__(o);
  }
}

static PyObject *__pyx_getprop_10traceutils_7scamper_5warts_9WartsPing_version(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_10traceutils_7scamper_5warts_9WartsPing_7version_1__get__(o);
}

static int __pyx_setprop_10traceutils_7scamper_5warts_9WartsPing_version(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_10traceutils_7scamper_5warts_9WartsPing_7version_3__set__(o, v);
  }
  else {
    return __pyx_pw_10traceutils_7scamper_5warts_9WartsPing_7version_5__del__(o);
  }
}

static PyObject *__pyx_getprop_10traceutils_7scamper_5warts_9WartsPing_method(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_10traceutils_7scamper_5warts_9WartsPing_6method_1__get__(o);
}

static int __pyx_setprop_10traceutils_7scamper_5warts_9WartsPing_method(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_10traceutils_7scamper_5warts_9WartsPing_6method_3__set__(o, v);
  }
  else {
    return __pyx_pw_10traceutils_7scamper_5warts_9WartsPing_6method_5__del__(o);
  }
}

static PyObject *__pyx_getprop_10traceutils_7scamper_5warts_9WartsPing_src(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_10traceutils_7scamper_5warts_9WartsPing_3src_1__get__(o);
}

static int __pyx_setprop_10traceutils_7scamper_5warts_9WartsPing_src(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_10traceutils_7scamper_5warts_9WartsPing_3src_3__set__(o, v);
  }
  else {
    return __pyx_pw_10traceutils_7scamper_5warts_9WartsPing_3src_5__del__(o);
  }
}

static PyObject *__pyx_getprop_10traceutils_7scamper_5warts_9WartsPing_dst(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_10traceutils_7scamper_5warts_9WartsPing_3dst_1__get__(o);
}

static int __pyx_setprop_10traceutils_7scamper_5warts_9WartsPing_dst(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_10traceutils_7scamper_5warts_9WartsPing_3dst_3__set__(o, v);
  }
  else {
    return __pyx_pw_10traceutils_7scamper_5warts_9WartsPing_3dst_5__del__(o);
  }
}

static PyObject *__pyx_getprop_10traceutils_7scamper_5warts_9WartsPing_start(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_10traceutils_7scamper_5warts_9WartsPing_5start_1__get__(o);
}

static int __pyx_setprop_10traceutils_7scamper_5warts_9WartsPing_start(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_10traceutils_7scamper_5warts_9WartsPing_5start_3__set__(o, v);
  }
  else {
    return __pyx_pw_10traceutils_7scamper_5warts_9WartsPing_5start_5__del__(o);
  }
}

static PyObject *__pyx_getprop_10traceutils_7scamper_5warts_9WartsPing_statistics(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_10traceutils_7scamper_5warts_9WartsPing_10statistics_1__get__(o);
}

static int __pyx_setprop_10traceutils_7scamper_5warts_9WartsPing_statistics(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_10traceutils_7scamper_5warts_9WartsPing_10statistics_3__set__(o, v);
  }
  else {
    return __pyx_pw_10traceutils_7scamper_5warts_9WartsPing_10statistics_5__del__(o);
  }
}

static PyObject *__pyx_getprop_10traceutils_7scamper_5warts_9WartsPing_ping_sent(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_10traceutils_7scamper_5warts_9WartsPing_9ping_sent_1__get__(o);
}

static int __pyx_setprop_10traceutils_7scamper_5warts_9WartsPing_ping_sent(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_10traceutils_7scamper_5warts_9WartsPing_9ping_sent_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_10traceutils_7scamper_5warts_9WartsPing_probe_size(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_10traceutils_7scamper_5warts_9WartsPing_10probe_size_1__get__(o);
}

static int __pyx_setprop_10traceutils_7scamper_5warts_9WartsPing_probe_size(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_10traceutils_7scamper_5warts_9WartsPing_10probe_size_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_10traceutils_7scamper_5warts_9WartsPing_userid(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_10traceutils_7scamper_5warts_9WartsPing_6userid_1__get__(o);
}

static int __pyx_setprop_10traceutils_7scamper_5warts_9WartsPing_userid(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_10traceutils_7scamper_5warts_9WartsPing_6userid_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_10traceutils_7scamper_5warts_9WartsPing_ttl(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_10traceutils_7scamper_5warts_9WartsPing_3ttl_1__get__(o);
}

static int __pyx_setprop_10traceutils_7scamper_5warts_9WartsPing_ttl(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_10traceutils_7scamper_5warts_9WartsPing_3ttl_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_10traceutils_7scamper_5warts_9WartsPing_wait(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_10traceutils_7scamper_5warts_9WartsPing_4wait_1__get__(o);
}

static int __pyx_setprop_10traceutils_7scamper_5warts_9WartsPing_wait(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_10traceutils_7scamper_5warts_9WartsPing_4wait_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_10traceutils_7scamper_5warts_9WartsPing_timeout(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_10traceutils_7scamper_5warts_9WartsPing_7timeout_1__get__(o);
}

static int __pyx_setprop_10traceutils_7scamper_5warts_9WartsPing_timeout(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_10traceutils_7scamper_5warts_9WartsPing_7timeout_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_10traceutils_7scamper_5warts_9WartsPing_responses(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_10traceutils_7scamper_5warts_9WartsPing_9responses_1__get__(o);
}

static int __pyx_setprop_10traceutils_7scamper_5warts_9WartsPing_responses(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_10traceutils_7scamper_5warts_9WartsPing_9responses_3__set__(o, v);
  }
  else {
    return __pyx_pw_10traceutils_7scamper_5warts_9WartsPing_9responses_5__del__(o);
  }
}

static PyObject *__pyx_getprop_10traceutils_7scamper_5warts_9WartsPing_family(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_10traceutils_7scamper_5warts_9WartsPing_6family_1__get__(o);
}

static int __pyx_setprop_10traceutils_7scamper_5warts_9WartsPing_family(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_10traceutils_7scamper_5warts_9WartsPing_6family_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyMethodDef __pyx_methods_10traceutils_7scamper_5warts_WartsPing[] = {
  {"__reduce_cython__", (PyCFunction)__pyx_pw_10traceutils_7scamper_5warts_9WartsPing_5__reduce_cython__, METH_NOARGS, __pyx_doc_10traceutils_7scamper_5warts_9WartsPing_4__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_10traceutils_7scamper_5warts_9WartsPing_7__setstate_cython__, METH_O, __pyx_doc_10traceutils_7scamper_5warts_9WartsPing_6__setstate_cython__},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_10traceutils_7scamper_5warts_WartsPing[] = {
  {(char *)"type", __pyx_getprop_10traceutils_7scamper_5warts_9WartsPing_type, __pyx_setprop_10traceutils_7scamper_5warts_9WartsPing_type, (char *)"type: unicode", 0},
  {(char *)"version", __pyx_getprop_10traceutils_7scamper_5warts_9WartsPing_version, __pyx_setprop_10traceutils_7scamper_5warts_9WartsPing_version, (char *)"version: unicode", 0},
  {(char *)"method", __pyx_getprop_10traceutils_7scamper_5warts_9WartsPing_method, __pyx_setprop_10traceutils_7scamper_5warts_9WartsPing_method, (char *)"method: unicode", 0},
  {(char *)"src", __pyx_getprop_10traceutils_7scamper_5warts_9WartsPing_src, __pyx_setprop_10traceutils_7scamper_5warts_9WartsPing_src, (char *)"src: unicode", 0},
  {(char *)"dst", __pyx_getprop_10traceutils_7scamper_5warts_9WartsPing_dst, __pyx_setprop_10traceutils_7scamper_5warts_9WartsPing_dst, (char *)"dst: unicode", 0},
  {(char *)"start", __pyx_getprop_10traceutils_7scamper_5warts_9WartsPing_start, __pyx_setprop_10traceutils_7scamper_5warts_9WartsPing_start, (char *)"start: dict", 0},
  {(char *)"statistics", __pyx_getprop_10traceutils_7scamper_5warts_9WartsPing_statistics, __pyx_setprop_10traceutils_7scamper_5warts_9WartsPing_statistics, (char *)"statistics: dict", 0},
  {(char *)"ping_sent", __pyx_getprop_10traceutils_7scamper_5warts_9WartsPing_ping_sent, __pyx_setprop_10traceutils_7scamper_5warts_9WartsPing_ping_sent, (char *)"ping_sent: 'int'", 0},
  {(char *)"probe_size", __pyx_getprop_10traceutils_7scamper_5warts_9WartsPing_probe_size, __pyx_setprop_10traceutils_7scamper_5warts_9WartsPing_probe_size, (char *)"probe_size: 'int'", 0},
  {(char *)"userid", __pyx_getprop_10traceutils_7scamper_5warts_9WartsPing_userid, __pyx_setprop_10traceutils_7scamper_5warts_9WartsPing_userid, (char *)"userid: 'int'", 0},
  {(char *)"ttl", __pyx_getprop_10traceutils_7scamper_5warts_9WartsPing_ttl, __pyx_setprop_10traceutils_7scamper_5warts_9WartsPing_ttl, (char *)"ttl: 'int'", 0},
  {(char *)"wait", __pyx_getprop_10traceutils_7scamper_5warts_9WartsPing_wait, __pyx_setprop_10traceutils_7scamper_5warts_9WartsPing_wait, (char *)"wait: 'double'", 0},
  {(char *)"timeout", __pyx_getprop_10traceutils_7scamper_5warts_9WartsPing_timeout, __pyx_setprop_10traceutils_7scamper_5warts_9WartsPing_timeout, (char *)"timeout: 'double'", 0},
  {(char *)"responses", __pyx_getprop_10traceutils_7scamper_5warts_9WartsPing_responses, __pyx_setprop_10traceutils_7scamper_5warts_9WartsPing_responses, (char *)"responses: list", 0},
  {(char *)"family", __pyx_getprop_10traceutils_7scamper_5warts_9WartsPing_family, __pyx_setprop_10traceutils_7scamper_5warts_9WartsPing_family, (char *)"family: 'int'", 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_10traceutils_7scamper_5warts_WartsPing = {
  PyVarObject_HEAD_INIT(0, 0)
  "traceutils.scamper.warts.WartsPing", /*tp_name*/
  sizeof(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPing), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_10traceutils_7scamper_5warts_WartsPing, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  __pyx_pw_10traceutils_7scamper_5warts_9WartsPing_3__repr__, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "WartsPing(unicode type=u'ping', unicode version=None, unicode method=None, unicode src=None, unicode dst=None, dict start=None, int ping_sent=-1, int probe_size=-1, int userid=-1, int ttl=-1, double wait=-1, double timeout=-1, list responses=None, dict statistics=None)", /*tp_doc*/
  __pyx_tp_traverse_10traceutils_7scamper_5warts_WartsPing, /*tp_traverse*/
  __pyx_tp_clear_10traceutils_7scamper_5warts_WartsPing, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_10traceutils_7scamper_5warts_WartsPing, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_10traceutils_7scamper_5warts_WartsPing, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_10traceutils_7scamper_5warts_9WartsPing_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_10traceutils_7scamper_5warts_WartsPing, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static PyObject *__pyx_tp_new_10traceutils_7scamper_5warts_WartsPingResponse(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *)o);
  p->reply_proto = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->tx = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->rx = ((PyObject*)Py_None); Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_10traceutils_7scamper_5warts_WartsPingResponse(PyObject *o) {
  struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *p = (struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->reply_proto);
  Py_CLEAR(p->tx);
  Py_CLEAR(p->rx);
  (*Py_TYPE(o)->tp_free)(o);
}

static int __pyx_tp_traverse_10traceutils_7scamper_5warts_WartsPingResponse(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *p = (struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *)o;
  if (p->tx) {
    e = (*v)(p->tx, a); if (e) return e;
  }
  if (p->rx) {
    e = (*v)(p->rx, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_10traceutils_7scamper_5warts_WartsPingResponse(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *p = (struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse *)o;
  tmp = ((PyObject*)p->tx);
  p->tx = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->rx);
  p->rx = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_10traceutils_7scamper_5warts_17WartsPingResponse_seq(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_10traceutils_7scamper_5warts_17WartsPingResponse_3seq_1__get__(o);
}

static int __pyx_setprop_10traceutils_7scamper_5warts_17WartsPingResponse_seq(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_10traceutils_7scamper_5warts_17WartsPingResponse_3seq_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_10traceutils_7scamper_5warts_17WartsPingResponse_reply_size(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_10traceutils_7scamper_5warts_17WartsPingResponse_10reply_size_1__get__(o);
}

static int __pyx_setprop_10traceutils_7scamper_5warts_17WartsPingResponse_reply_size(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_10traceutils_7scamper_5warts_17WartsPingResponse_10reply_size_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_10traceutils_7scamper_5warts_17WartsPingResponse_reply_ttl(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_10traceutils_7scamper_5warts_17WartsPingResponse_9reply_ttl_1__get__(o);
}

static int __pyx_setprop_10traceutils_7scamper_5warts_17WartsPingResponse_reply_ttl(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_10traceutils_7scamper_5warts_17WartsPingResponse_9reply_ttl_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_10traceutils_7scamper_5warts_17WartsPingResponse_probe_ipid(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_10traceutils_7scamper_5warts_17WartsPingResponse_10probe_ipid_1__get__(o);
}

static int __pyx_setprop_10traceutils_7scamper_5warts_17WartsPingResponse_probe_ipid(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_10traceutils_7scamper_5warts_17WartsPingResponse_10probe_ipid_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_10traceutils_7scamper_5warts_17WartsPingResponse_reply_ipid(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_10traceutils_7scamper_5warts_17WartsPingResponse_10reply_ipid_1__get__(o);
}

static int __pyx_setprop_10traceutils_7scamper_5warts_17WartsPingResponse_reply_ipid(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_10traceutils_7scamper_5warts_17WartsPingResponse_10reply_ipid_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_10traceutils_7scamper_5warts_17WartsPingResponse_icmp_type(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_10traceutils_7scamper_5warts_17WartsPingResponse_9icmp_type_1__get__(o);
}

static int __pyx_setprop_10traceutils_7scamper_5warts_17WartsPingResponse_icmp_type(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_10traceutils_7scamper_5warts_17WartsPingResponse_9icmp_type_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_10traceutils_7scamper_5warts_17WartsPingResponse_icmp_code(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_10traceutils_7scamper_5warts_17WartsPingResponse_9icmp_code_1__get__(o);
}

static int __pyx_setprop_10traceutils_7scamper_5warts_17WartsPingResponse_icmp_code(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_10traceutils_7scamper_5warts_17WartsPingResponse_9icmp_code_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_10traceutils_7scamper_5warts_17WartsPingResponse_reply_proto(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_10traceutils_7scamper_5warts_17WartsPingResponse_11reply_proto_1__get__(o);
}

static int __pyx_setprop_10traceutils_7scamper_5warts_17WartsPingResponse_reply_proto(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_10traceutils_7scamper_5warts_17WartsPingResponse_11reply_proto_3__set__(o, v);
  }
  else {
    return __pyx_pw_10traceutils_7scamper_5warts_17WartsPingResponse_11reply_proto_5__del__(o);
  }
}

static PyObject *__pyx_getprop_10traceutils_7scamper_5warts_17WartsPingResponse_tx(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_10traceutils_7scamper_5warts_17WartsPingResponse_2tx_1__get__(o);
}

static int __pyx_setprop_10traceutils_7scamper_5warts_17WartsPingResponse_tx(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_10traceutils_7scamper_5warts_17WartsPingResponse_2tx_3__set__(o, v);
  }
  else {
    return __pyx_pw_10traceutils_7scamper_5warts_17WartsPingResponse_2tx_5__del__(o);
  }
}

static PyObject *__pyx_getprop_10traceutils_7scamper_5warts_17WartsPingResponse_rx(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_10traceutils_7scamper_5warts_17WartsPingResponse_2rx_1__get__(o);
}

static int __pyx_setprop_10traceutils_7scamper_5warts_17WartsPingResponse_rx(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_10traceutils_7scamper_5warts_17WartsPingResponse_2rx_3__set__(o, v);
  }
  else {
    return __pyx_pw_10traceutils_7scamper_5warts_17WartsPingResponse_2rx_5__del__(o);
  }
}

static PyObject *__pyx_getprop_10traceutils_7scamper_5warts_17WartsPingResponse_rtt(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_10traceutils_7scamper_5warts_17WartsPingResponse_3rtt_1__get__(o);
}

static int __pyx_setprop_10traceutils_7scamper_5warts_17WartsPingResponse_rtt(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_10traceutils_7scamper_5warts_17WartsPingResponse_3rtt_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_10traceutils_7scamper_5warts_17WartsPingResponse_family(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_10traceutils_7scamper_5warts_17WartsPingResponse_6family_1__get__(o);
}

static int __pyx_setprop_10traceutils_7scamper_5warts_17WartsPingResponse_family(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_10traceutils_7scamper_5warts_17WartsPingResponse_6family_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_10traceutils_7scamper_5warts_17WartsPingResponse_type(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_10traceutils_7scamper_5warts_17WartsPingResponse_4type_1__get__(o);
}

static int __pyx_setprop_10traceutils_7scamper_5warts_17WartsPingResponse_type(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_10traceutils_7scamper_5warts_17WartsPingResponse_4type_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyMethodDef __pyx_methods_10traceutils_7scamper_5warts_WartsPingResponse[] = {
  {"__reduce_cython__", (PyCFunction)__pyx_pw_10traceutils_7scamper_5warts_17WartsPingResponse_5__reduce_cython__, METH_NOARGS, __pyx_doc_10traceutils_7scamper_5warts_17WartsPingResponse_4__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_10traceutils_7scamper_5warts_17WartsPingResponse_7__setstate_cython__, METH_O, __pyx_doc_10traceutils_7scamper_5warts_17WartsPingResponse_6__setstate_cython__},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_10traceutils_7scamper_5warts_WartsPingResponse[] = {
  {(char *)"seq", __pyx_getprop_10traceutils_7scamper_5warts_17WartsPingResponse_seq, __pyx_setprop_10traceutils_7scamper_5warts_17WartsPingResponse_seq, (char *)"seq: 'int'", 0},
  {(char *)"reply_size", __pyx_getprop_10traceutils_7scamper_5warts_17WartsPingResponse_reply_size, __pyx_setprop_10traceutils_7scamper_5warts_17WartsPingResponse_reply_size, (char *)"reply_size: 'int'", 0},
  {(char *)"reply_ttl", __pyx_getprop_10traceutils_7scamper_5warts_17WartsPingResponse_reply_ttl, __pyx_setprop_10traceutils_7scamper_5warts_17WartsPingResponse_reply_ttl, (char *)"reply_ttl: 'int'", 0},
  {(char *)"probe_ipid", __pyx_getprop_10traceutils_7scamper_5warts_17WartsPingResponse_probe_ipid, __pyx_setprop_10traceutils_7scamper_5warts_17WartsPingResponse_probe_ipid, (char *)"probe_ipid: 'int'", 0},
  {(char *)"reply_ipid", __pyx_getprop_10traceutils_7scamper_5warts_17WartsPingResponse_reply_ipid, __pyx_setprop_10traceutils_7scamper_5warts_17WartsPingResponse_reply_ipid, (char *)"reply_ipid: 'int'", 0},
  {(char *)"icmp_type", __pyx_getprop_10traceutils_7scamper_5warts_17WartsPingResponse_icmp_type, __pyx_setprop_10traceutils_7scamper_5warts_17WartsPingResponse_icmp_type, (char *)"icmp_type: 'int'", 0},
  {(char *)"icmp_code", __pyx_getprop_10traceutils_7scamper_5warts_17WartsPingResponse_icmp_code, __pyx_setprop_10traceutils_7scamper_5warts_17WartsPingResponse_icmp_code, (char *)"icmp_code: 'int'", 0},
  {(char *)"reply_proto", __pyx_getprop_10traceutils_7scamper_5warts_17WartsPingResponse_reply_proto, __pyx_setprop_10traceutils_7scamper_5warts_17WartsPingResponse_reply_proto, (char *)"reply_proto: unicode", 0},
  {(char *)"tx", __pyx_getprop_10traceutils_7scamper_5warts_17WartsPingResponse_tx, __pyx_setprop_10traceutils_7scamper_5warts_17WartsPingResponse_tx, (char *)"tx: dict", 0},
  {(char *)"rx", __pyx_getprop_10traceutils_7scamper_5warts_17WartsPingResponse_rx, __pyx_setprop_10traceutils_7scamper_5warts_17WartsPingResponse_rx, (char *)"rx: dict", 0},
  {(char *)"rtt", __pyx_getprop_10traceutils_7scamper_5warts_17WartsPingResponse_rtt, __pyx_setprop_10traceutils_7scamper_5warts_17WartsPingResponse_rtt, (char *)"rtt: 'double'", 0},
  {(char *)"family", __pyx_getprop_10traceutils_7scamper_5warts_17WartsPingResponse_family, __pyx_setprop_10traceutils_7scamper_5warts_17WartsPingResponse_family, (char *)"family: 'int'", 0},
  {(char *)"type", __pyx_getprop_10traceutils_7scamper_5warts_17WartsPingResponse_type, __pyx_setprop_10traceutils_7scamper_5warts_17WartsPingResponse_type, (char *)"type: 'ICMPType'", 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_10traceutils_7scamper_5warts_WartsPingResponse = {
  PyVarObject_HEAD_INIT(0, 0)
  "traceutils.scamper.warts.WartsPingResponse", /*tp_name*/
  sizeof(struct __pyx_obj_10traceutils_7scamper_5warts_WartsPingResponse), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_10traceutils_7scamper_5warts_WartsPingResponse, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  __pyx_pw_10traceutils_7scamper_5warts_17WartsPingResponse_3__repr__, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "WartsPingResponse(int seq=-1, int reply_size=-1, int reply_ttl=-1, unicode reply_proto=None, dict tx=None, dict rx=None, double rtt=-1, int probe_ipid=-1, int reply_ipid=-1, int icmp_type=-1, int icmp_code=-1, int family=0)", /*tp_doc*/
  __pyx_tp_traverse_10traceutils_7scamper_5warts_WartsPingResponse, /*tp_traverse*/
  __pyx_tp_clear_10traceutils_7scamper_5warts_WartsPingResponse, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_10traceutils_7scamper_5warts_WartsPingResponse, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_10traceutils_7scamper_5warts_WartsPingResponse, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_10traceutils_7scamper_5warts_17WartsPingResponse_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_10traceutils_7scamper_5warts_WartsPingResponse, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_10traceutils_7scamper_5warts_WartsReader __pyx_vtable_10traceutils_7scamper_5warts_WartsReader;

static PyObject *__pyx_tp_new_10traceutils_7scamper_5warts_WartsReader(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_10traceutils_7scamper_5warts_WartsReader *p;
  PyObject *o = __pyx_ptype_10traceutils_7scamper_3hop_Reader->tp_new(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_10traceutils_7scamper_5warts_WartsReader *)o);
  p->__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_10traceutils_7scamper_3hop_Reader*)__pyx_vtabptr_10traceutils_7scamper_5warts_WartsReader;
  return o;
}

static void __pyx_tp_dealloc_10traceutils_7scamper_5warts_WartsReader(PyObject *o) {
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  PyObject_GC_Track(o);
  if (likely(__pyx_ptype_10traceutils_7scamper_3hop_Reader)) __pyx_ptype_10traceutils_7scamper_3hop_Reader->tp_dealloc(o); else __Pyx_call_next_tp_dealloc(o, __pyx_tp_dealloc_10traceutils_7scamper_5warts_WartsReader);
}

static int __pyx_tp_traverse_10traceutils_7scamper_5warts_WartsReader(PyObject *o, visitproc v, void *a) {
  int e;
  e = ((likely(__pyx_ptype_10traceutils_7scamper_3hop_Reader)) ? ((__pyx_ptype_10traceutils_7scamper_3hop_Reader->tp_traverse) ? __pyx_ptype_10traceutils_7scamper_3hop_Reader->tp_traverse(o, v, a) : 0) : __Pyx_call_next_tp_traverse(o, v, a, __pyx_tp_traverse_10traceutils_7scamper_5warts_WartsReader)); if (e) return e;
  return 0;
}

static int __pyx_tp_clear_10traceutils_7scamper_5warts_WartsReader(PyObject *o) {
  if (likely(__pyx_ptype_10traceutils_7scamper_3hop_Reader)) { if (__pyx_ptype_10traceutils_7scamper_3hop_Reader->tp_clear) __pyx_ptype_10traceutils_7scamper_3hop_Reader->tp_clear(o); } else __Pyx_call_next_tp_clear(o, __pyx_tp_clear_10traceutils_7scamper_5warts_WartsReader);
  return 0;
}

static PyMethodDef __pyx_methods_10traceutils_7scamper_5warts_WartsReader[] = {
  {"open", (PyCFunction)__pyx_pw_10traceutils_7scamper_5warts_11WartsReader_6open, METH_NOARGS, __pyx_doc_10traceutils_7scamper_5warts_11WartsReader_5open},
  {"close", (PyCFunction)__pyx_pw_10traceutils_7scamper_5warts_11WartsReader_8close, METH_NOARGS, __pyx_doc_10traceutils_7scamper_5warts_11WartsReader_7close},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_10traceutils_7scamper_5warts_11WartsReader_10__reduce_cython__, METH_NOARGS, __pyx_doc_10traceutils_7scamper_5warts_11WartsReader_9__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_10traceutils_7scamper_5warts_11WartsReader_12__setstate_cython__, METH_O, __pyx_doc_10traceutils_7scamper_5warts_11WartsReader_11__setstate_cython__},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_10traceutils_7scamper_5warts_WartsReader = {
  PyVarObject_HEAD_INIT(0, 0)
  "traceutils.scamper.warts.WartsReader", /*tp_name*/
  sizeof(struct __pyx_obj_10traceutils_7scamper_5warts_WartsReader), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_10traceutils_7scamper_5warts_WartsReader, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "WartsReader(unicode filename)", /*tp_doc*/
  __pyx_tp_traverse_10traceutils_7scamper_5warts_WartsReader, /*tp_traverse*/
  __pyx_tp_clear_10traceutils_7scamper_5warts_WartsReader, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  __pyx_pw_10traceutils_7scamper_5warts_11WartsReader_3__iter__, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_10traceutils_7scamper_5warts_WartsReader, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_10traceutils_7scamper_5warts_11WartsReader_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_10traceutils_7scamper_5warts_WartsReader, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static struct __pyx_obj_10traceutils_7scamper_5warts___pyx_scope_struct____iter__ *__pyx_freelist_10traceutils_7scamper_5warts___pyx_scope_struct____iter__[8];
static int __pyx_freecount_10traceutils_7scamper_5warts___pyx_scope_struct____iter__ = 0;

static PyObject *__pyx_tp_new_10traceutils_7scamper_5warts___pyx_scope_struct____iter__(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  if (CYTHON_COMPILING_IN_CPYTHON && likely((__pyx_freecount_10traceutils_7scamper_5warts___pyx_scope_struct____iter__ > 0) & (t->tp_basicsize == sizeof(struct __pyx_obj_10traceutils_7scamper_5warts___pyx_scope_struct____iter__)))) {
    o = (PyObject*)__pyx_freelist_10traceutils_7scamper_5warts___pyx_scope_struct____iter__[--__pyx_freecount_10traceutils_7scamper_5warts___pyx_scope_struct____iter__];
    memset(o, 0, sizeof(struct __pyx_obj_10traceutils_7scamper_5warts___pyx_scope_struct____iter__));
    (void) PyObject_INIT(o, t);
    PyObject_GC_Track(o);
  } else {
    o = (*t->tp_alloc)(t, 0);
    if (unlikely(!o)) return 0;
  }
  return o;
}

static void __pyx_tp_dealloc_10traceutils_7scamper_5warts___pyx_scope_struct____iter__(PyObject *o) {
  struct __pyx_obj_10traceutils_7scamper_5warts___pyx_scope_struct____iter__ *p = (struct __pyx_obj_10traceutils_7scamper_5warts___pyx_scope_struct____iter__ *)o;
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->__pyx_v_j);
  Py_CLEAR(p->__pyx_v_line);
  Py_CLEAR(p->__pyx_v_rtype);
  Py_CLEAR(p->__pyx_v_self);
  Py_CLEAR(p->__pyx_t_0);
  if (CYTHON_COMPILING_IN_CPYTHON && ((__pyx_freecount_10traceutils_7scamper_5warts___pyx_scope_struct____iter__ < 8) & (Py_TYPE(o)->tp_basicsize == sizeof(struct __pyx_obj_10traceutils_7scamper_5warts___pyx_scope_struct____iter__)))) {
    __pyx_freelist_10traceutils_7scamper_5warts___pyx_scope_struct____iter__[__pyx_freecount_10traceutils_7scamper_5warts___pyx_scope_struct____iter__++] = ((struct __pyx_obj_10traceutils_7scamper_5warts___pyx_scope_struct____iter__ *)o);
  } else {
    (*Py_TYPE(o)->tp_free)(o);
  }
}

static int __pyx_tp_traverse_10traceutils_7scamper_5warts___pyx_scope_struct____iter__(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_10traceutils_7scamper_5warts___pyx_scope_struct____iter__ *p = (struct __pyx_obj_10traceutils_7scamper_5warts___pyx_scope_struct____iter__ *)o;
  if (p->__pyx_v_j) {
    e = (*v)(p->__pyx_v_j, a); if (e) return e;
  }
  if (p->__pyx_v_self) {
    e = (*v)(((PyObject *)p->__pyx_v_self), a); if (e) return e;
  }
  if (p->__pyx_t_0) {
    e = (*v)(p->__pyx_t_0, a); if (e) return e;
  }
  return 0;
}

static PyTypeObject __pyx_type_10traceutils_7scamper_5warts___pyx_scope_struct____iter__ = {
  PyVarObject_HEAD_INIT(0, 0)
  "traceutils.scamper.warts.__pyx_scope_struct____iter__", /*tp_name*/
  sizeof(struct __pyx_obj_10traceutils_7scamper_5warts___pyx_scope_struct____iter__), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_10traceutils_7scamper_5warts___pyx_scope_struct____iter__, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_10traceutils_7scamper_5warts___pyx_scope_struct____iter__, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  0, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_10traceutils_7scamper_5warts___pyx_scope_struct____iter__, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static PyMethodDef __pyx_methods[] = {
  {0, 0, 0, 0}
};

#if PY_MAJOR_VERSION >= 3
#if CYTHON_PEP489_MULTI_PHASE_INIT
static PyObject* __pyx_pymod_create(PyObject *spec, PyModuleDef *def); /*proto*/
static int __pyx_pymod_exec_warts(PyObject* module); /*proto*/
static PyModuleDef_Slot __pyx_moduledef_slots[] = {
  {Py_mod_create, (void*)__pyx_pymod_create},
  {Py_mod_exec, (void*)__pyx_pymod_exec_warts},
  {0, NULL}
};
#endif

static struct PyModuleDef __pyx_moduledef = {
    PyModuleDef_HEAD_INIT,
    "warts",
    0, /* m_doc */
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    0, /* m_size */
  #else
    -1, /* m_size */
  #endif
    __pyx_methods /* m_methods */,
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    __pyx_moduledef_slots, /* m_slots */
  #else
    NULL, /* m_reload */
  #endif
    NULL, /* m_traverse */
    NULL, /* m_clear */
    NULL /* m_free */
};
#endif
#ifndef CYTHON_SMALL_CODE
#if defined(__clang__)
    #define CYTHON_SMALL_CODE
#elif defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 3))
    #define CYTHON_SMALL_CODE __attribute__((cold))
#else
    #define CYTHON_SMALL_CODE
#endif
#endif

static __Pyx_StringTabEntry __pyx_string_tab[] = {
  {&__pyx_kp_u_, __pyx_k_, sizeof(__pyx_k_), 0, 1, 0, 0},
  {&__pyx_kp_s_Incompatible_checksums_s_vs_0x57, __pyx_k_Incompatible_checksums_s_vs_0x57, sizeof(__pyx_k_Incompatible_checksums_s_vs_0x57), 0, 0, 1, 0},
  {&__pyx_kp_s_Incompatible_checksums_s_vs_0x78, __pyx_k_Incompatible_checksums_s_vs_0x78, sizeof(__pyx_k_Incompatible_checksums_s_vs_0x78), 0, 0, 1, 0},
  {&__pyx_kp_s_Incompatible_checksums_s_vs_0x93, __pyx_k_Incompatible_checksums_s_vs_0x93, sizeof(__pyx_k_Incompatible_checksums_s_vs_0x93), 0, 0, 1, 0},
  {&__pyx_kp_s_Incompatible_checksums_s_vs_0xb9, __pyx_k_Incompatible_checksums_s_vs_0xb9, sizeof(__pyx_k_Incompatible_checksums_s_vs_0xb9), 0, 0, 1, 0},
  {&__pyx_kp_s_Incompatible_checksums_s_vs_0xff, __pyx_k_Incompatible_checksums_s_vs_0xff, sizeof(__pyx_k_Incompatible_checksums_s_vs_0xff), 0, 0, 1, 0},
  {&__pyx_n_s_PIPE, __pyx_k_PIPE, sizeof(__pyx_k_PIPE), 0, 0, 1, 1},
  {&__pyx_n_s_PickleError, __pyx_k_PickleError, sizeof(__pyx_k_PickleError), 0, 0, 1, 1},
  {&__pyx_n_s_Popen, __pyx_k_Popen, sizeof(__pyx_k_Popen), 0, 0, 1, 1},
  {&__pyx_kp_u_RTT_rtt, __pyx_k_RTT_rtt, sizeof(__pyx_k_RTT_rtt), 0, 1, 0, 0},
  {&__pyx_kp_u_Src_src_Dst_dst, __pyx_k_Src_src_Dst_dst, sizeof(__pyx_k_Src_src_Dst_dst), 0, 1, 0, 0},
  {&__pyx_n_s_WartsHop, __pyx_k_WartsHop, sizeof(__pyx_k_WartsHop), 0, 0, 1, 1},
  {&__pyx_n_s_WartsPing, __pyx_k_WartsPing, sizeof(__pyx_k_WartsPing), 0, 0, 1, 1},
  {&__pyx_n_s_WartsPingResponse, __pyx_k_WartsPingResponse, sizeof(__pyx_k_WartsPingResponse), 0, 0, 1, 1},
  {&__pyx_n_s_WartsReader, __pyx_k_WartsReader, sizeof(__pyx_k_WartsReader), 0, 0, 1, 1},
  {&__pyx_n_s_WartsReader___iter, __pyx_k_WartsReader___iter, sizeof(__pyx_k_WartsReader___iter), 0, 0, 1, 1},
  {&__pyx_n_s_WartsTrace, __pyx_k_WartsTrace, sizeof(__pyx_k_WartsTrace), 0, 0, 1, 1},
  {&__pyx_kp_u__2, __pyx_k__2, sizeof(__pyx_k__2), 0, 1, 0, 0},
  {&__pyx_kp_u__3, __pyx_k__3, sizeof(__pyx_k__3), 0, 1, 0, 0},
  {&__pyx_n_s_addr, __pyx_k_addr, sizeof(__pyx_k_addr), 0, 0, 1, 1},
  {&__pyx_n_s_args, __pyx_k_args, sizeof(__pyx_k_args), 0, 0, 1, 1},
  {&__pyx_n_s_attempts, __pyx_k_attempts, sizeof(__pyx_k_attempts), 0, 0, 1, 1},
  {&__pyx_kp_u_bz2, __pyx_k_bz2, sizeof(__pyx_k_bz2), 0, 1, 0, 0},
  {&__pyx_kp_u_bzip2, __pyx_k_bzip2, sizeof(__pyx_k_bzip2), 0, 1, 0, 0},
  {&__pyx_kp_u_bzip2_d_c_sc_warts2json, __pyx_k_bzip2_d_c_sc_warts2json, sizeof(__pyx_k_bzip2_d_c_sc_warts2json), 0, 1, 0, 0},
  {&__pyx_n_s_cline_in_traceback, __pyx_k_cline_in_traceback, sizeof(__pyx_k_cline_in_traceback), 0, 0, 1, 1},
  {&__pyx_n_s_close, __pyx_k_close, sizeof(__pyx_k_close), 0, 0, 1, 1},
  {&__pyx_n_s_dict, __pyx_k_dict, sizeof(__pyx_k_dict), 0, 0, 1, 1},
  {&__pyx_n_s_dst, __pyx_k_dst, sizeof(__pyx_k_dst), 0, 0, 1, 1},
  {&__pyx_n_s_family, __pyx_k_family, sizeof(__pyx_k_family), 0, 0, 1, 1},
  {&__pyx_n_s_filename, __pyx_k_filename, sizeof(__pyx_k_filename), 0, 0, 1, 1},
  {&__pyx_n_s_firsthop, __pyx_k_firsthop, sizeof(__pyx_k_firsthop), 0, 0, 1, 1},
  {&__pyx_n_s_format, __pyx_k_format, sizeof(__pyx_k_format), 0, 0, 1, 1},
  {&__pyx_n_u_from, __pyx_k_from, sizeof(__pyx_k_from), 0, 1, 0, 1},
  {&__pyx_n_s_getstate, __pyx_k_getstate, sizeof(__pyx_k_getstate), 0, 0, 1, 1},
  {&__pyx_kp_u_gz, __pyx_k_gz, sizeof(__pyx_k_gz), 0, 1, 0, 0},
  {&__pyx_kp_u_gzip_d_c_sc_warts2json, __pyx_k_gzip_d_c_sc_warts2json, sizeof(__pyx_k_gzip_d_c_sc_warts2json), 0, 1, 0, 0},
  {&__pyx_n_s_hop_count, __pyx_k_hop_count, sizeof(__pyx_k_hop_count), 0, 0, 1, 1},
  {&__pyx_n_s_hoplimit, __pyx_k_hoplimit, sizeof(__pyx_k_hoplimit), 0, 0, 1, 1},
  {&__pyx_n_s_hops, __pyx_k_hops, sizeof(__pyx_k_hops), 0, 0, 1, 1},
  {&__pyx_n_s_hostname, __pyx_k_hostname, sizeof(__pyx_k_hostname), 0, 0, 1, 1},
  {&__pyx_n_s_icmp_code, __pyx_k_icmp_code, sizeof(__pyx_k_icmp_code), 0, 0, 1, 1},
  {&__pyx_n_s_icmp_nhmtu, __pyx_k_icmp_nhmtu, sizeof(__pyx_k_icmp_nhmtu), 0, 0, 1, 1},
  {&__pyx_n_s_icmp_q_ipl, __pyx_k_icmp_q_ipl, sizeof(__pyx_k_icmp_q_ipl), 0, 0, 1, 1},
  {&__pyx_n_s_icmp_q_tos, __pyx_k_icmp_q_tos, sizeof(__pyx_k_icmp_q_tos), 0, 0, 1, 1},
  {&__pyx_n_s_icmp_q_ttl, __pyx_k_icmp_q_ttl, sizeof(__pyx_k_icmp_q_ttl), 0, 0, 1, 1},
  {&__pyx_n_s_icmp_sum, __pyx_k_icmp_sum, sizeof(__pyx_k_icmp_sum), 0, 0, 1, 1},
  {&__pyx_n_s_icmp_type, __pyx_k_icmp_type, sizeof(__pyx_k_icmp_type), 0, 0, 1, 1},
  {&__pyx_n_s_icmpext, __pyx_k_icmpext, sizeof(__pyx_k_icmpext), 0, 0, 1, 1},
  {&__pyx_n_s_id, __pyx_k_id, sizeof(__pyx_k_id), 0, 0, 1, 1},
  {&__pyx_n_s_import, __pyx_k_import, sizeof(__pyx_k_import), 0, 0, 1, 1},
  {&__pyx_n_s_items, __pyx_k_items, sizeof(__pyx_k_items), 0, 0, 1, 1},
  {&__pyx_n_s_iter, __pyx_k_iter, sizeof(__pyx_k_iter), 0, 0, 1, 1},
  {&__pyx_n_s_json, __pyx_k_json, sizeof(__pyx_k_json), 0, 0, 1, 1},
  {&__pyx_n_s_list_name, __pyx_k_list_name, sizeof(__pyx_k_list_name), 0, 0, 1, 1},
  {&__pyx_n_s_loads, __pyx_k_loads, sizeof(__pyx_k_loads), 0, 0, 1, 1},
  {&__pyx_n_s_main, __pyx_k_main, sizeof(__pyx_k_main), 0, 0, 1, 1},
  {&__pyx_n_s_method, __pyx_k_method, sizeof(__pyx_k_method), 0, 0, 1, 1},
  {&__pyx_n_u_mpls_labels, __pyx_k_mpls_labels, sizeof(__pyx_k_mpls_labels), 0, 1, 0, 1},
  {&__pyx_n_s_name, __pyx_k_name, sizeof(__pyx_k_name), 0, 0, 1, 1},
  {&__pyx_n_s_new, __pyx_k_new, sizeof(__pyx_k_new), 0, 0, 1, 1},
  {&__pyx_n_s_open, __pyx_k_open, sizeof(__pyx_k_open), 0, 0, 1, 1},
  {&__pyx_n_s_pickle, __pyx_k_pickle, sizeof(__pyx_k_pickle), 0, 0, 1, 1},
  {&__pyx_n_u_ping, __pyx_k_ping, sizeof(__pyx_k_ping), 0, 1, 0, 1},
  {&__pyx_n_s_ping_sent, __pyx_k_ping_sent, sizeof(__pyx_k_ping_sent), 0, 0, 1, 1},
  {&__pyx_n_s_pop, __pyx_k_pop, sizeof(__pyx_k_pop), 0, 0, 1, 1},
  {&__pyx_n_s_probe_count, __pyx_k_probe_count, sizeof(__pyx_k_probe_count), 0, 0, 1, 1},
  {&__pyx_n_s_probe_id, __pyx_k_probe_id, sizeof(__pyx_k_probe_id), 0, 0, 1, 1},
  {&__pyx_n_s_probe_ipid, __pyx_k_probe_ipid, sizeof(__pyx_k_probe_ipid), 0, 0, 1, 1},
  {&__pyx_n_s_probe_size, __pyx_k_probe_size, sizeof(__pyx_k_probe_size), 0, 0, 1, 1},
  {&__pyx_n_s_probe_ttl, __pyx_k_probe_ttl, sizeof(__pyx_k_probe_ttl), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_PickleError, __pyx_k_pyx_PickleError, sizeof(__pyx_k_pyx_PickleError), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_checksum, __pyx_k_pyx_checksum, sizeof(__pyx_k_pyx_checksum), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_result, __pyx_k_pyx_result, sizeof(__pyx_k_pyx_result), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_state, __pyx_k_pyx_state, sizeof(__pyx_k_pyx_state), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_type, __pyx_k_pyx_type, sizeof(__pyx_k_pyx_type), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_WartsHop, __pyx_k_pyx_unpickle_WartsHop, sizeof(__pyx_k_pyx_unpickle_WartsHop), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_WartsPing, __pyx_k_pyx_unpickle_WartsPing, sizeof(__pyx_k_pyx_unpickle_WartsPing), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_WartsPingResponse, __pyx_k_pyx_unpickle_WartsPingResponse, sizeof(__pyx_k_pyx_unpickle_WartsPingResponse), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_WartsReader, __pyx_k_pyx_unpickle_WartsReader, sizeof(__pyx_k_pyx_unpickle_WartsReader), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_WartsTrace, __pyx_k_pyx_unpickle_WartsTrace, sizeof(__pyx_k_pyx_unpickle_WartsTrace), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_vtable, __pyx_k_pyx_vtable, sizeof(__pyx_k_pyx_vtable), 0, 0, 1, 1},
  {&__pyx_n_s_reduce, __pyx_k_reduce, sizeof(__pyx_k_reduce), 0, 0, 1, 1},
  {&__pyx_n_s_reduce_cython, __pyx_k_reduce_cython, sizeof(__pyx_k_reduce_cython), 0, 0, 1, 1},
  {&__pyx_n_s_reduce_ex, __pyx_k_reduce_ex, sizeof(__pyx_k_reduce_ex), 0, 0, 1, 1},
  {&__pyx_n_s_reply_ipid, __pyx_k_reply_ipid, sizeof(__pyx_k_reply_ipid), 0, 0, 1, 1},
  {&__pyx_n_s_reply_proto, __pyx_k_reply_proto, sizeof(__pyx_k_reply_proto), 0, 0, 1, 1},
  {&__pyx_n_s_reply_size, __pyx_k_reply_size, sizeof(__pyx_k_reply_size), 0, 0, 1, 1},
  {&__pyx_n_s_reply_tos, __pyx_k_reply_tos, sizeof(__pyx_k_reply_tos), 0, 0, 1, 1},
  {&__pyx_n_s_reply_ttl, __pyx_k_reply_ttl, sizeof(__pyx_k_reply_ttl), 0, 0, 1, 1},
  {&__pyx_n_s_responses, __pyx_k_responses, sizeof(__pyx_k_responses), 0, 0, 1, 1},
  {&__pyx_n_s_rtt, __pyx_k_rtt, sizeof(__pyx_k_rtt), 0, 0, 1, 1},
  {&__pyx_n_s_rx, __pyx_k_rx, sizeof(__pyx_k_rx), 0, 0, 1, 1},
  {&__pyx_kp_u_sc_warts2json, __pyx_k_sc_warts2json, sizeof(__pyx_k_sc_warts2json), 0, 1, 0, 0},
  {&__pyx_n_s_send, __pyx_k_send, sizeof(__pyx_k_send), 0, 0, 1, 1},
  {&__pyx_n_s_seq, __pyx_k_seq, sizeof(__pyx_k_seq), 0, 0, 1, 1},
  {&__pyx_n_s_setstate, __pyx_k_setstate, sizeof(__pyx_k_setstate), 0, 0, 1, 1},
  {&__pyx_n_s_setstate_cython, __pyx_k_setstate_cython, sizeof(__pyx_k_setstate_cython), 0, 0, 1, 1},
  {&__pyx_n_s_shell, __pyx_k_shell, sizeof(__pyx_k_shell), 0, 0, 1, 1},
  {&__pyx_n_s_src, __pyx_k_src, sizeof(__pyx_k_src), 0, 0, 1, 1},
  {&__pyx_n_s_start, __pyx_k_start, sizeof(__pyx_k_start), 0, 0, 1, 1},
  {&__pyx_n_s_start_time, __pyx_k_start_time, sizeof(__pyx_k_start_time), 0, 0, 1, 1},
  {&__pyx_n_s_statistics, __pyx_k_statistics, sizeof(__pyx_k_statistics), 0, 0, 1, 1},
  {&__pyx_n_s_stdout, __pyx_k_stdout, sizeof(__pyx_k_stdout), 0, 0, 1, 1},
  {&__pyx_n_s_stop_data, __pyx_k_stop_data, sizeof(__pyx_k_stop_data), 0, 0, 1, 1},
  {&__pyx_n_s_stop_reason, __pyx_k_stop_reason, sizeof(__pyx_k_stop_reason), 0, 0, 1, 1},
  {&__pyx_kp_s_stringsource, __pyx_k_stringsource, sizeof(__pyx_k_stringsource), 0, 0, 1, 0},
  {&__pyx_n_s_subprocess, __pyx_k_subprocess, sizeof(__pyx_k_subprocess), 0, 0, 1, 1},
  {&__pyx_n_s_test, __pyx_k_test, sizeof(__pyx_k_test), 0, 0, 1, 1},
  {&__pyx_n_s_throw, __pyx_k_throw, sizeof(__pyx_k_throw), 0, 0, 1, 1},
  {&__pyx_n_s_timeout, __pyx_k_timeout, sizeof(__pyx_k_timeout), 0, 0, 1, 1},
  {&__pyx_n_s_tos, __pyx_k_tos, sizeof(__pyx_k_tos), 0, 0, 1, 1},
  {&__pyx_n_u_trace, __pyx_k_trace, sizeof(__pyx_k_trace), 0, 1, 0, 1},
  {&__pyx_n_s_traceutils_scamper_warts, __pyx_k_traceutils_scamper_warts, sizeof(__pyx_k_traceutils_scamper_warts), 0, 0, 1, 1},
  {&__pyx_n_s_ttl, __pyx_k_ttl, sizeof(__pyx_k_ttl), 0, 0, 1, 1},
  {&__pyx_n_s_tx, __pyx_k_tx, sizeof(__pyx_k_tx), 0, 0, 1, 1},
  {&__pyx_n_s_type, __pyx_k_type, sizeof(__pyx_k_type), 0, 0, 1, 1},
  {&__pyx_n_u_type, __pyx_k_type, sizeof(__pyx_k_type), 0, 1, 0, 1},
  {&__pyx_n_s_universal_newlines, __pyx_k_universal_newlines, sizeof(__pyx_k_universal_newlines), 0, 0, 1, 1},
  {&__pyx_n_s_update, __pyx_k_update, sizeof(__pyx_k_update), 0, 0, 1, 1},
  {&__pyx_n_s_userid, __pyx_k_userid, sizeof(__pyx_k_userid), 0, 0, 1, 1},
  {&__pyx_n_s_version, __pyx_k_version, sizeof(__pyx_k_version), 0, 0, 1, 1},
  {&__pyx_n_s_wait, __pyx_k_wait, sizeof(__pyx_k_wait), 0, 0, 1, 1},
  {&__pyx_n_s_wait_probe, __pyx_k_wait_probe, sizeof(__pyx_k_wait_probe), 0, 0, 1, 1},
  {0, 0, 0, 0, 0, 0, 0}
};
static CYTHON_SMALL_CODE int __Pyx_InitCachedBuiltins(void) {
  return 0;
}

static CYTHON_SMALL_CODE int __Pyx_InitCachedConstants(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_InitCachedConstants", 0);

  /* "(tree fragment)":1
 * def __pyx_unpickle_WartsTrace(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */
  __pyx_tuple__4 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__4)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__4);
  __Pyx_GIVEREF(__pyx_tuple__4);
  __pyx_codeobj__5 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__4, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_WartsTrace, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__5)) __PYX_ERR(2, 1, __pyx_L1_error)
  __pyx_tuple__6 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__6)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__6);
  __Pyx_GIVEREF(__pyx_tuple__6);
  __pyx_codeobj__7 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__6, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_WartsHop, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__7)) __PYX_ERR(2, 1, __pyx_L1_error)
  __pyx_tuple__8 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__8)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__8);
  __Pyx_GIVEREF(__pyx_tuple__8);
  __pyx_codeobj__9 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__8, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_WartsPing, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__9)) __PYX_ERR(2, 1, __pyx_L1_error)
  __pyx_tuple__10 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__10)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__10);
  __Pyx_GIVEREF(__pyx_tuple__10);
  __pyx_codeobj__11 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__10, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_WartsPingResponse, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__11)) __PYX_ERR(2, 1, __pyx_L1_error)
  __pyx_tuple__12 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__12)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__12);
  __Pyx_GIVEREF(__pyx_tuple__12);
  __pyx_codeobj__13 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__12, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_WartsReader, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__13)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static CYTHON_SMALL_CODE int __Pyx_InitGlobals(void) {
  __pyx_umethod_PyDict_Type_pop.type = (PyObject*)&PyDict_Type;
  __pyx_umethod_PyUnicode_Type_format.type = (PyObject*)&PyUnicode_Type;
  if (__Pyx_InitStrings(__pyx_string_tab) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  __pyx_int_91565645 = PyInt_FromLong(91565645L); if (unlikely(!__pyx_int_91565645)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_126533895 = PyInt_FromLong(126533895L); if (unlikely(!__pyx_int_126533895)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_154359991 = PyInt_FromLong(154359991L); if (unlikely(!__pyx_int_154359991)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_194082941 = PyInt_FromLong(194082941L); if (unlikely(!__pyx_int_194082941)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_268127653 = PyInt_FromLong(268127653L); if (unlikely(!__pyx_int_268127653)) __PYX_ERR(0, 1, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

static CYTHON_SMALL_CODE int __Pyx_modinit_global_init_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_export_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_export_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_init_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_import_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_import_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_import_code(void); /*proto*/

static int __Pyx_modinit_global_init_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_global_init_code", 0);
  /*--- Global init code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_variable_export_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_export_code", 0);
  /*--- Variable export code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_function_export_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_export_code", 0);
  /*--- Function export code ---*/
  if (__Pyx_ExportFunction("create_responses", (void (*)(void))__pyx_f_10traceutils_7scamper_5warts_create_responses, "PyObject *(PyObject *, int)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_type_init_code(void) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_init_code", 0);
  /*--- Type init code ---*/
  __pyx_t_1 = PyImport_ImportModule("traceutils.scamper.hop"); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_10traceutils_7scamper_3hop_Hop = __Pyx_ImportType(__pyx_t_1, "traceutils.scamper.hop", "Hop", sizeof(struct __pyx_obj_10traceutils_7scamper_3hop_Hop), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_10traceutils_7scamper_3hop_Hop) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_vtabptr_10traceutils_7scamper_3hop_Hop = (struct __pyx_vtabstruct_10traceutils_7scamper_3hop_Hop*)__Pyx_GetVtable(__pyx_ptype_10traceutils_7scamper_3hop_Hop->tp_dict); if (unlikely(!__pyx_vtabptr_10traceutils_7scamper_3hop_Hop)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_vtabptr_10traceutils_7scamper_5warts_WartsHop = &__pyx_vtable_10traceutils_7scamper_5warts_WartsHop;
  __pyx_vtable_10traceutils_7scamper_5warts_WartsHop.__pyx_base = *__pyx_vtabptr_10traceutils_7scamper_3hop_Hop;
  __pyx_type_10traceutils_7scamper_5warts_WartsHop.tp_base = __pyx_ptype_10traceutils_7scamper_3hop_Hop;
  if (PyType_Ready(&__pyx_type_10traceutils_7scamper_5warts_WartsHop) < 0) __PYX_ERR(0, 47, __pyx_L1_error)
  __pyx_type_10traceutils_7scamper_5warts_WartsHop.tp_print = 0;
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_10traceutils_7scamper_5warts_WartsHop.tp_dictoffset && __pyx_type_10traceutils_7scamper_5warts_WartsHop.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_10traceutils_7scamper_5warts_WartsHop.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_10traceutils_7scamper_5warts_WartsHop.tp_dict, __pyx_vtabptr_10traceutils_7scamper_5warts_WartsHop) < 0) __PYX_ERR(0, 47, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_WartsHop, (PyObject *)&__pyx_type_10traceutils_7scamper_5warts_WartsHop) < 0) __PYX_ERR(0, 47, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_10traceutils_7scamper_5warts_WartsHop) < 0) __PYX_ERR(0, 47, __pyx_L1_error)
  __pyx_ptype_10traceutils_7scamper_5warts_WartsHop = &__pyx_type_10traceutils_7scamper_5warts_WartsHop;
  __pyx_ptype_10traceutils_7scamper_3hop_Trace = __Pyx_ImportType(__pyx_t_1, "traceutils.scamper.hop", "Trace", sizeof(struct __pyx_obj_10traceutils_7scamper_3hop_Trace), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_10traceutils_7scamper_3hop_Trace) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_vtabptr_10traceutils_7scamper_3hop_Trace = (struct __pyx_vtabstruct_10traceutils_7scamper_3hop_Trace*)__Pyx_GetVtable(__pyx_ptype_10traceutils_7scamper_3hop_Trace->tp_dict); if (unlikely(!__pyx_vtabptr_10traceutils_7scamper_3hop_Trace)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_vtabptr_10traceutils_7scamper_5warts_WartsTrace = &__pyx_vtable_10traceutils_7scamper_5warts_WartsTrace;
  __pyx_vtable_10traceutils_7scamper_5warts_WartsTrace.__pyx_base = *__pyx_vtabptr_10traceutils_7scamper_3hop_Trace;
  __pyx_type_10traceutils_7scamper_5warts_WartsTrace.tp_base = __pyx_ptype_10traceutils_7scamper_3hop_Trace;
  if (PyType_Ready(&__pyx_type_10traceutils_7scamper_5warts_WartsTrace) < 0) __PYX_ERR(0, 14, __pyx_L1_error)
  __pyx_type_10traceutils_7scamper_5warts_WartsTrace.tp_print = 0;
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_10traceutils_7scamper_5warts_WartsTrace.tp_dictoffset && __pyx_type_10traceutils_7scamper_5warts_WartsTrace.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_10traceutils_7scamper_5warts_WartsTrace.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_10traceutils_7scamper_5warts_WartsTrace.tp_dict, __pyx_vtabptr_10traceutils_7scamper_5warts_WartsTrace) < 0) __PYX_ERR(0, 14, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_WartsTrace, (PyObject *)&__pyx_type_10traceutils_7scamper_5warts_WartsTrace) < 0) __PYX_ERR(0, 14, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_10traceutils_7scamper_5warts_WartsTrace) < 0) __PYX_ERR(0, 14, __pyx_L1_error)
  __pyx_ptype_10traceutils_7scamper_5warts_WartsTrace = &__pyx_type_10traceutils_7scamper_5warts_WartsTrace;
  if (PyType_Ready(&__pyx_type_10traceutils_7scamper_5warts_WartsPing) < 0) __PYX_ERR(0, 81, __pyx_L1_error)
  __pyx_type_10traceutils_7scamper_5warts_WartsPing.tp_print = 0;
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_10traceutils_7scamper_5warts_WartsPing.tp_dictoffset && __pyx_type_10traceutils_7scamper_5warts_WartsPing.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_10traceutils_7scamper_5warts_WartsPing.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_WartsPing, (PyObject *)&__pyx_type_10traceutils_7scamper_5warts_WartsPing) < 0) __PYX_ERR(0, 81, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_10traceutils_7scamper_5warts_WartsPing) < 0) __PYX_ERR(0, 81, __pyx_L1_error)
  __pyx_ptype_10traceutils_7scamper_5warts_WartsPing = &__pyx_type_10traceutils_7scamper_5warts_WartsPing;
  if (PyType_Ready(&__pyx_type_10traceutils_7scamper_5warts_WartsPingResponse) < 0) __PYX_ERR(0, 116, __pyx_L1_error)
  __pyx_type_10traceutils_7scamper_5warts_WartsPingResponse.tp_print = 0;
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_10traceutils_7scamper_5warts_WartsPingResponse.tp_dictoffset && __pyx_type_10traceutils_7scamper_5warts_WartsPingResponse.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_10traceutils_7scamper_5warts_WartsPingResponse.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_WartsPingResponse, (PyObject *)&__pyx_type_10traceutils_7scamper_5warts_WartsPingResponse) < 0) __PYX_ERR(0, 116, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_10traceutils_7scamper_5warts_WartsPingResponse) < 0) __PYX_ERR(0, 116, __pyx_L1_error)
  __pyx_ptype_10traceutils_7scamper_5warts_WartsPingResponse = &__pyx_type_10traceutils_7scamper_5warts_WartsPingResponse;
  __pyx_ptype_10traceutils_7scamper_3hop_Reader = __Pyx_ImportType(__pyx_t_1, "traceutils.scamper.hop", "Reader", sizeof(struct __pyx_obj_10traceutils_7scamper_3hop_Reader), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_10traceutils_7scamper_3hop_Reader) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_vtabptr_10traceutils_7scamper_3hop_Reader = (struct __pyx_vtabstruct_10traceutils_7scamper_3hop_Reader*)__Pyx_GetVtable(__pyx_ptype_10traceutils_7scamper_3hop_Reader->tp_dict); if (unlikely(!__pyx_vtabptr_10traceutils_7scamper_3hop_Reader)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_vtabptr_10traceutils_7scamper_5warts_WartsReader = &__pyx_vtable_10traceutils_7scamper_5warts_WartsReader;
  __pyx_vtable_10traceutils_7scamper_5warts_WartsReader.__pyx_base = *__pyx_vtabptr_10traceutils_7scamper_3hop_Reader;
  __pyx_vtable_10traceutils_7scamper_5warts_WartsReader.__pyx_base.open = (void (*)(struct __pyx_obj_10traceutils_7scamper_3hop_Reader *, int __pyx_skip_dispatch))__pyx_f_10traceutils_7scamper_5warts_11WartsReader_open;
  __pyx_vtable_10traceutils_7scamper_5warts_WartsReader.__pyx_base.close = (void (*)(struct __pyx_obj_10traceutils_7scamper_3hop_Reader *, int __pyx_skip_dispatch))__pyx_f_10traceutils_7scamper_5warts_11WartsReader_close;
  __pyx_type_10traceutils_7scamper_5warts_WartsReader.tp_base = __pyx_ptype_10traceutils_7scamper_3hop_Reader;
  if (PyType_Ready(&__pyx_type_10traceutils_7scamper_5warts_WartsReader) < 0) __PYX_ERR(0, 136, __pyx_L1_error)
  __pyx_type_10traceutils_7scamper_5warts_WartsReader.tp_print = 0;
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_10traceutils_7scamper_5warts_WartsReader.tp_dictoffset && __pyx_type_10traceutils_7scamper_5warts_WartsReader.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_10traceutils_7scamper_5warts_WartsReader.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_10traceutils_7scamper_5warts_WartsReader.tp_dict, __pyx_vtabptr_10traceutils_7scamper_5warts_WartsReader) < 0) __PYX_ERR(0, 136, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_WartsReader, (PyObject *)&__pyx_type_10traceutils_7scamper_5warts_WartsReader) < 0) __PYX_ERR(0, 136, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_10traceutils_7scamper_5warts_WartsReader) < 0) __PYX_ERR(0, 136, __pyx_L1_error)
  __pyx_ptype_10traceutils_7scamper_5warts_WartsReader = &__pyx_type_10traceutils_7scamper_5warts_WartsReader;
  if (PyType_Ready(&__pyx_type_10traceutils_7scamper_5warts___pyx_scope_struct____iter__) < 0) __PYX_ERR(0, 141, __pyx_L1_error)
  __pyx_type_10traceutils_7scamper_5warts___pyx_scope_struct____iter__.tp_print = 0;
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_10traceutils_7scamper_5warts___pyx_scope_struct____iter__.tp_dictoffset && __pyx_type_10traceutils_7scamper_5warts___pyx_scope_struct____iter__.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_10traceutils_7scamper_5warts___pyx_scope_struct____iter__.tp_getattro = __Pyx_PyObject_GenericGetAttrNoDict;
  }
  __pyx_ptype_10traceutils_7scamper_5warts___pyx_scope_struct____iter__ = &__pyx_type_10traceutils_7scamper_5warts___pyx_scope_struct____iter__;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_type_import_code(void) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_import_code", 0);
  /*--- Type import code ---*/
  __pyx_t_1 = PyImport_ImportModule("traceutils.radix.radix_prefix"); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_10traceutils_5radix_12radix_prefix_RadixPrefix = __Pyx_ImportType(__pyx_t_1, "traceutils.radix.radix_prefix", "RadixPrefix", sizeof(struct __pyx_obj_10traceutils_5radix_12radix_prefix_RadixPrefix), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_10traceutils_5radix_12radix_prefix_RadixPrefix) __PYX_ERR(3, 5, __pyx_L1_error)
  __pyx_vtabptr_10traceutils_5radix_12radix_prefix_RadixPrefix = (struct __pyx_vtabstruct_10traceutils_5radix_12radix_prefix_RadixPrefix*)__Pyx_GetVtable(__pyx_ptype_10traceutils_5radix_12radix_prefix_RadixPrefix->tp_dict); if (unlikely(!__pyx_vtabptr_10traceutils_5radix_12radix_prefix_RadixPrefix)) __PYX_ERR(3, 5, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("traceutils.radix.radix_node"); if (unlikely(!__pyx_t_1)) __PYX_ERR(4, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_10traceutils_5radix_10radix_node_RadixNode = __Pyx_ImportType(__pyx_t_1, "traceutils.radix.radix_node", "RadixNode", sizeof(struct __pyx_obj_10traceutils_5radix_10radix_node_RadixNode), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_10traceutils_5radix_10radix_node_RadixNode) __PYX_ERR(4, 3, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("traceutils.radix.radix_tree"); if (unlikely(!__pyx_t_1)) __PYX_ERR(5, 9, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_10traceutils_5radix_10radix_tree_RadixTree = __Pyx_ImportType(__pyx_t_1, "traceutils.radix.radix_tree", "RadixTree", sizeof(struct __pyx_obj_10traceutils_5radix_10radix_tree_RadixTree), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_10traceutils_5radix_10radix_tree_RadixTree) __PYX_ERR(5, 9, __pyx_L1_error)
  __pyx_vtabptr_10traceutils_5radix_10radix_tree_RadixTree = (struct __pyx_vtabstruct_10traceutils_5radix_10radix_tree_RadixTree*)__Pyx_GetVtable(__pyx_ptype_10traceutils_5radix_10radix_tree_RadixTree->tp_dict); if (unlikely(!__pyx_vtabptr_10traceutils_5radix_10radix_tree_RadixTree)) __PYX_ERR(5, 9, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("traceutils.radix.radix"); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_10traceutils_5radix_5radix_Radix = __Pyx_ImportType(__pyx_t_1, "traceutils.radix.radix", "Radix", sizeof(struct __pyx_obj_10traceutils_5radix_5radix_Radix), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_10traceutils_5radix_5radix_Radix) __PYX_ERR(6, 7, __pyx_L1_error)
  __pyx_vtabptr_10traceutils_5radix_5radix_Radix = (struct __pyx_vtabstruct_10traceutils_5radix_5radix_Radix*)__Pyx_GetVtable(__pyx_ptype_10traceutils_5radix_5radix_Radix->tp_dict); if (unlikely(!__pyx_vtabptr_10traceutils_5radix_5radix_Radix)) __PYX_ERR(6, 7, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("traceutils.radix.ip2as"); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 8, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_10traceutils_5radix_5ip2as_IP2AS = __Pyx_ImportType(__pyx_t_1, "traceutils.radix.ip2as", "IP2AS", sizeof(struct __pyx_obj_10traceutils_5radix_5ip2as_IP2AS), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_10traceutils_5radix_5ip2as_IP2AS) __PYX_ERR(7, 8, __pyx_L1_error)
  __pyx_vtabptr_10traceutils_5radix_5ip2as_IP2AS = (struct __pyx_vtabstruct_10traceutils_5radix_5ip2as_IP2AS*)__Pyx_GetVtable(__pyx_ptype_10traceutils_5radix_5ip2as_IP2AS->tp_dict); if (unlikely(!__pyx_vtabptr_10traceutils_5radix_5ip2as_IP2AS)) __PYX_ERR(7, 8, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_variable_import_code(void) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_import_code", 0);
  /*--- Variable import code ---*/
  __pyx_t_1 = PyImport_ImportModule("traceutils.radix.ip2as"); if (!__pyx_t_1) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "PRIVATE4", (void **)&__pyx_vp_10traceutils_5radix_5ip2as_PRIVATE4, "PyObject *") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "PRIVATE6", (void **)&__pyx_vp_10traceutils_5radix_5ip2as_PRIVATE6, "PyObject *") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "MULTICAST4", (void **)&__pyx_vp_10traceutils_5radix_5ip2as_MULTICAST4, "PyObject *") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "MULTICAST6", (void **)&__pyx_vp_10traceutils_5radix_5ip2as_MULTICAST6, "PyObject *") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_function_import_code(void) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_import_code", 0);
  /*--- Function import code ---*/
  __pyx_t_1 = PyImport_ImportModule("traceutils.scamper.hop"); if (!__pyx_t_1) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_1, "gettype", (void (**)(void))&__pyx_f_10traceutils_7scamper_3hop_gettype, "enum __pyx_t_10traceutils_7scamper_3hop_ICMPType (int, int, int __pyx_skip_dispatch)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_2 = PyImport_ImportModule("traceutils.utils.net"); if (!__pyx_t_2) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_2, "find_family", (void (**)(void))&__pyx_f_10traceutils_5utils_3net_find_family, "unsigned char (PyObject *, int __pyx_skip_dispatch)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_RefNannyFinishContext();
  return -1;
}


#if PY_MAJOR_VERSION < 3
#ifdef CYTHON_NO_PYINIT_EXPORT
#define __Pyx_PyMODINIT_FUNC void
#else
#define __Pyx_PyMODINIT_FUNC PyMODINIT_FUNC
#endif
#else
#ifdef CYTHON_NO_PYINIT_EXPORT
#define __Pyx_PyMODINIT_FUNC PyObject *
#else
#define __Pyx_PyMODINIT_FUNC PyMODINIT_FUNC
#endif
#endif


#if PY_MAJOR_VERSION < 3
__Pyx_PyMODINIT_FUNC initwarts(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC initwarts(void)
#else
__Pyx_PyMODINIT_FUNC PyInit_warts(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC PyInit_warts(void)
#if CYTHON_PEP489_MULTI_PHASE_INIT
{
  return PyModuleDef_Init(&__pyx_moduledef);
}
static CYTHON_SMALL_CODE int __Pyx_check_single_interpreter(void) {
    #if PY_VERSION_HEX >= 0x030700A1
    static PY_INT64_T main_interpreter_id = -1;
    PY_INT64_T current_id = PyInterpreterState_GetID(PyThreadState_Get()->interp);
    if (main_interpreter_id == -1) {
        main_interpreter_id = current_id;
        return (unlikely(current_id == -1)) ? -1 : 0;
    } else if (unlikely(main_interpreter_id != current_id))
    #else
    static PyInterpreterState *main_interpreter = NULL;
    PyInterpreterState *current_interpreter = PyThreadState_Get()->interp;
    if (!main_interpreter) {
        main_interpreter = current_interpreter;
    } else if (unlikely(main_interpreter != current_interpreter))
    #endif
    {
        PyErr_SetString(
            PyExc_ImportError,
            "Interpreter change detected - this module can only be loaded into one interpreter per process.");
        return -1;
    }
    return 0;
}
static CYTHON_SMALL_CODE int __Pyx_copy_spec_to_module(PyObject *spec, PyObject *moddict, const char* from_name, const char* to_name, int allow_none) {
    PyObject *value = PyObject_GetAttrString(spec, from_name);
    int result = 0;
    if (likely(value)) {
        if (allow_none || value != Py_None) {
            result = PyDict_SetItemString(moddict, to_name, value);
        }
        Py_DECREF(value);
    } else if (PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Clear();
    } else {
        result = -1;
    }
    return result;
}
static CYTHON_SMALL_CODE PyObject* __pyx_pymod_create(PyObject *spec, CYTHON_UNUSED PyModuleDef *def) {
    PyObject *module = NULL, *moddict, *modname;
    if (__Pyx_check_single_interpreter())
        return NULL;
    if (__pyx_m)
        return __Pyx_NewRef(__pyx_m);
    modname = PyObject_GetAttrString(spec, "name");
    if (unlikely(!modname)) goto bad;
    module = PyModule_NewObject(modname);
    Py_DECREF(modname);
    if (unlikely(!module)) goto bad;
    moddict = PyModule_GetDict(module);
    if (unlikely(!moddict)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "loader", "__loader__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "origin", "__file__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "parent", "__package__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "submodule_search_locations", "__path__", 0) < 0)) goto bad;
    return module;
bad:
    Py_XDECREF(module);
    return NULL;
}


static CYTHON_SMALL_CODE int __pyx_pymod_exec_warts(PyObject *__pyx_pyinit_module)
#endif
#endif
{
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannyDeclarations
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  if (__pyx_m) {
    if (__pyx_m == __pyx_pyinit_module) return 0;
    PyErr_SetString(PyExc_RuntimeError, "Module 'warts' has already been imported. Re-initialisation is not supported.");
    return -1;
  }
  #elif PY_MAJOR_VERSION >= 3
  if (__pyx_m) return __Pyx_NewRef(__pyx_m);
  #endif
  #if CYTHON_REFNANNY
__Pyx_RefNanny = __Pyx_RefNannyImportAPI("refnanny");
if (!__Pyx_RefNanny) {
  PyErr_Clear();
  __Pyx_RefNanny = __Pyx_RefNannyImportAPI("Cython.Runtime.refnanny");
  if (!__Pyx_RefNanny)
      Py_FatalError("failed to import 'refnanny' module");
}
#endif
  __Pyx_RefNannySetupContext("__Pyx_PyMODINIT_FUNC PyInit_warts(void)", 0);
  if (__Pyx_check_binary_version() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #ifdef __Pxy_PyFrame_Initialize_Offsets
  __Pxy_PyFrame_Initialize_Offsets();
  #endif
  __pyx_empty_tuple = PyTuple_New(0); if (unlikely(!__pyx_empty_tuple)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_bytes = PyBytes_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_bytes)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_unicode = PyUnicode_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_unicode)) __PYX_ERR(0, 1, __pyx_L1_error)
  #ifdef __Pyx_CyFunction_USED
  if (__pyx_CyFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_FusedFunction_USED
  if (__pyx_FusedFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Coroutine_USED
  if (__pyx_Coroutine_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Generator_USED
  if (__pyx_Generator_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_AsyncGen_USED
  if (__pyx_AsyncGen_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_StopAsyncIteration_USED
  if (__pyx_StopAsyncIteration_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  /*--- Library function declarations ---*/
  /*--- Threads initialization code ---*/
  #if defined(__PYX_FORCE_INIT_THREADS) && __PYX_FORCE_INIT_THREADS
  #ifdef WITH_THREAD /* Python build with threading support? */
  PyEval_InitThreads();
  #endif
  #endif
  /*--- Module creation code ---*/
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  __pyx_m = __pyx_pyinit_module;
  Py_INCREF(__pyx_m);
  #else
  #if PY_MAJOR_VERSION < 3
  __pyx_m = Py_InitModule4("warts", __pyx_methods, 0, 0, PYTHON_API_VERSION); Py_XINCREF(__pyx_m);
  #else
  __pyx_m = PyModule_Create(&__pyx_moduledef);
  #endif
  if (unlikely(!__pyx_m)) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  __pyx_d = PyModule_GetDict(__pyx_m); if (unlikely(!__pyx_d)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_d);
  __pyx_b = PyImport_AddModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_b)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_b);
  __pyx_cython_runtime = PyImport_AddModule((char *) "cython_runtime"); if (unlikely(!__pyx_cython_runtime)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_cython_runtime);
  if (PyObject_SetAttrString(__pyx_m, "__builtins__", __pyx_b) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  /*--- Initialize various global constants etc. ---*/
  if (__Pyx_InitGlobals() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #if PY_MAJOR_VERSION < 3 && (__PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT)
  if (__Pyx_init_sys_getdefaultencoding_params() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  if (__pyx_module_is_main_traceutils__scamper__warts) {
    if (PyObject_SetAttr(__pyx_m, __pyx_n_s_name, __pyx_n_s_main) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  }
  #if PY_MAJOR_VERSION >= 3
  {
    PyObject *modules = PyImport_GetModuleDict(); if (unlikely(!modules)) __PYX_ERR(0, 1, __pyx_L1_error)
    if (!PyDict_GetItemString(modules, "traceutils.scamper.warts")) {
      if (unlikely(PyDict_SetItemString(modules, "traceutils.scamper.warts", __pyx_m) < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
    }
  }
  #endif
  /*--- Builtin init code ---*/
  if (__Pyx_InitCachedBuiltins() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Constants init code ---*/
  if (__Pyx_InitCachedConstants() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Global type/function init code ---*/
  (void)__Pyx_modinit_global_init_code();
  (void)__Pyx_modinit_variable_export_code();
  if (unlikely(__Pyx_modinit_function_export_code() != 0)) goto __pyx_L1_error;
  if (unlikely(__Pyx_modinit_type_init_code() != 0)) goto __pyx_L1_error;
  if (unlikely(__Pyx_modinit_type_import_code() != 0)) goto __pyx_L1_error;
  if (unlikely(__Pyx_modinit_variable_import_code() != 0)) goto __pyx_L1_error;
  if (unlikely(__Pyx_modinit_function_import_code() != 0)) goto __pyx_L1_error;
  /*--- Execution code ---*/
  #if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
  if (__Pyx_patch_abc() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif

  /* "traceutils/scamper/warts.pyx":1
 * import json as json             # <<<<<<<<<<<<<<
 * from subprocess import Popen, PIPE
 * 
 */
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_json, 0, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_json, __pyx_t_1) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "traceutils/scamper/warts.pyx":2
 * import json as json
 * from subprocess import Popen, PIPE             # <<<<<<<<<<<<<<
 * 
 * from traceutils.scamper.hop cimport Hop, Trace, Reader, gettype
 */
  __pyx_t_1 = PyList_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_s_Popen);
  __Pyx_GIVEREF(__pyx_n_s_Popen);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_s_Popen);
  __Pyx_INCREF(__pyx_n_s_PIPE);
  __Pyx_GIVEREF(__pyx_n_s_PIPE);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_s_PIPE);
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_subprocess, __pyx_t_1, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_Popen); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_Popen, __pyx_t_1) < 0) __PYX_ERR(0, 2, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_PIPE); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_PIPE, __pyx_t_1) < 0) __PYX_ERR(0, 2, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_WartsTrace(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_10traceutils_7scamper_5warts_1__pyx_unpickle_WartsTrace, NULL, __pyx_n_s_traceutils_scamper_warts); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_WartsTrace, __pyx_t_2) < 0) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":11
 *         __pyx_unpickle_WartsTrace__set_state(<WartsTrace> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_WartsTrace__set_state(WartsTrace __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.attempts = __pyx_state[0]; __pyx_result.dst = __pyx_state[1]; __pyx_result.family = __pyx_state[2]; __pyx_result.firsthop = __pyx_state[3]; __pyx_result.hop_count = __pyx_state[4]; __pyx_result.hoplimit = __pyx_state[5]; __pyx_result.hops = __pyx_state[6]; __pyx_result.icmp_sum = __pyx_state[7]; __pyx_result.loop = __pyx_state[8]; __pyx_result.method = __pyx_state[9]; __pyx_result.probe_count = __pyx_state[10]; __pyx_result.probe_size = __pyx_state[11]; __pyx_result.src = __pyx_state[12]; __pyx_result.start = __pyx_state[13]; __pyx_result.stop_data = __pyx_state[14]; __pyx_result.stop_reason = __pyx_state[15]; __pyx_result.tos = __pyx_state[16]; __pyx_result.type = __pyx_state[17]; __pyx_result.userid = __pyx_state[18]; __pyx_result.version = __pyx_state[19]; __pyx_result.wait = __pyx_state[20]; __pyx_result.wait_probe = __pyx_state[21]
 *     if len(__pyx_state) > 22 and hasattr(__pyx_result, '__dict__'):
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_10traceutils_7scamper_5warts_3__pyx_unpickle_WartsHop, NULL, __pyx_n_s_traceutils_scamper_warts); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_WartsHop, __pyx_t_2) < 0) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_WartsPing(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_10traceutils_7scamper_5warts_5__pyx_unpickle_WartsPing, NULL, __pyx_n_s_traceutils_scamper_warts); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_WartsPing, __pyx_t_2) < 0) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":11
 *         __pyx_unpickle_WartsPing__set_state(<WartsPing> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_WartsPing__set_state(WartsPing __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.dst = __pyx_state[0]; __pyx_result.family = __pyx_state[1]; __pyx_result.method = __pyx_state[2]; __pyx_result.ping_sent = __pyx_state[3]; __pyx_result.probe_size = __pyx_state[4]; __pyx_result.responses = __pyx_state[5]; __pyx_result.src = __pyx_state[6]; __pyx_result.start = __pyx_state[7]; __pyx_result.statistics = __pyx_state[8]; __pyx_result.timeout = __pyx_state[9]; __pyx_result.ttl = __pyx_state[10]; __pyx_result.type = __pyx_state[11]; __pyx_result.userid = __pyx_state[12]; __pyx_result.version = __pyx_state[13]; __pyx_result.wait = __pyx_state[14]
 *     if len(__pyx_state) > 15 and hasattr(__pyx_result, '__dict__'):
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_10traceutils_7scamper_5warts_7__pyx_unpickle_WartsPingResponse, NULL, __pyx_n_s_traceutils_scamper_warts); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_WartsPingResponse, __pyx_t_2) < 0) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_WartsReader(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_10traceutils_7scamper_5warts_9__pyx_unpickle_WartsReader, NULL, __pyx_n_s_traceutils_scamper_warts); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_WartsReader, __pyx_t_2) < 0) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "traceutils/scamper/warts.pyx":1
 * import json as json             # <<<<<<<<<<<<<<
 * from subprocess import Popen, PIPE
 * 
 */
  __pyx_t_2 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_test, __pyx_t_2) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /*--- Wrapped vars code ---*/

  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  if (__pyx_m) {
    if (__pyx_d) {
      __Pyx_AddTraceback("init traceutils.scamper.warts", __pyx_clineno, __pyx_lineno, __pyx_filename);
    }
    Py_CLEAR(__pyx_m);
  } else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_ImportError, "init traceutils.scamper.warts");
  }
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  return (__pyx_m != NULL) ? 0 : -1;
  #elif PY_MAJOR_VERSION >= 3
  return __pyx_m;
  #else
  return;
  #endif
}

/* --- Runtime support code --- */
/* Refnanny */
#if CYTHON_REFNANNY
static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname) {
    PyObject *m = NULL, *p = NULL;
    void *r = NULL;
    m = PyImport_ImportModule(modname);
    if (!m) goto end;
    p = PyObject_GetAttrString(m, "RefNannyAPI");
    if (!p) goto end;
    r = PyLong_AsVoidPtr(p);
end:
    Py_XDECREF(p);
    Py_XDECREF(m);
    return (__Pyx_RefNannyAPIStruct *)r;
}
#endif

/* IterFinish */
static CYTHON_INLINE int __Pyx_IterFinish(void) {
#if CYTHON_FAST_THREAD_STATE
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject* exc_type = tstate->curexc_type;
    if (unlikely(exc_type)) {
        if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) {
            PyObject *exc_value, *exc_tb;
            exc_value = tstate->curexc_value;
            exc_tb = tstate->curexc_traceback;
            tstate->curexc_type = 0;
            tstate->curexc_value = 0;
            tstate->curexc_traceback = 0;
            Py_DECREF(exc_type);
            Py_XDECREF(exc_value);
            Py_XDECREF(exc_tb);
            return 0;
        } else {
            return -1;
        }
    }
    return 0;
#else
    if (unlikely(PyErr_Occurred())) {
        if (likely(PyErr_ExceptionMatches(PyExc_StopIteration))) {
            PyErr_Clear();
            return 0;
        } else {
            return -1;
        }
    }
    return 0;
#endif
}

/* PyFunctionFastCall */
#if CYTHON_FAST_PYCALL
static PyObject* __Pyx_PyFunction_FastCallNoKw(PyCodeObject *co, PyObject **args, Py_ssize_t na,
                                               PyObject *globals) {
    PyFrameObject *f;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject **fastlocals;
    Py_ssize_t i;
    PyObject *result;
    assert(globals != NULL);
    /* XXX Perhaps we should create a specialized
       PyFrame_New() that doesn't take locals, but does
       take builtins without sanity checking them.
       */
    assert(tstate != NULL);
    f = PyFrame_New(tstate, co, globals, NULL);
    if (f == NULL) {
        return NULL;
    }
    fastlocals = __Pyx_PyFrame_GetLocalsplus(f);
    for (i = 0; i < na; i++) {
        Py_INCREF(*args);
        fastlocals[i] = *args++;
    }
    result = PyEval_EvalFrameEx(f,0);
    ++tstate->recursion_depth;
    Py_DECREF(f);
    --tstate->recursion_depth;
    return result;
}
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, int nargs, PyObject *kwargs) {
    PyCodeObject *co = (PyCodeObject *)PyFunction_GET_CODE(func);
    PyObject *globals = PyFunction_GET_GLOBALS(func);
    PyObject *argdefs = PyFunction_GET_DEFAULTS(func);
    PyObject *closure;
#if PY_MAJOR_VERSION >= 3
    PyObject *kwdefs;
#endif
    PyObject *kwtuple, **k;
    PyObject **d;
    Py_ssize_t nd;
    Py_ssize_t nk;
    PyObject *result;
    assert(kwargs == NULL || PyDict_Check(kwargs));
    nk = kwargs ? PyDict_Size(kwargs) : 0;
    if (Py_EnterRecursiveCall((char*)" while calling a Python object")) {
        return NULL;
    }
    if (
#if PY_MAJOR_VERSION >= 3
            co->co_kwonlyargcount == 0 &&
#endif
            likely(kwargs == NULL || nk == 0) &&
            co->co_flags == (CO_OPTIMIZED | CO_NEWLOCALS | CO_NOFREE)) {
        if (argdefs == NULL && co->co_argcount == nargs) {
            result = __Pyx_PyFunction_FastCallNoKw(co, args, nargs, globals);
            goto done;
        }
        else if (nargs == 0 && argdefs != NULL
                 && co->co_argcount == Py_SIZE(argdefs)) {
            /* function called with no arguments, but all parameters have
               a default value: use default values as arguments .*/
            args = &PyTuple_GET_ITEM(argdefs, 0);
            result =__Pyx_PyFunction_FastCallNoKw(co, args, Py_SIZE(argdefs), globals);
            goto done;
        }
    }
    if (kwargs != NULL) {
        Py_ssize_t pos, i;
        kwtuple = PyTuple_New(2 * nk);
        if (kwtuple == NULL) {
            result = NULL;
            goto done;
        }
        k = &PyTuple_GET_ITEM(kwtuple, 0);
        pos = i = 0;
        while (PyDict_Next(kwargs, &pos, &k[i], &k[i+1])) {
            Py_INCREF(k[i]);
            Py_INCREF(k[i+1]);
            i += 2;
        }
        nk = i / 2;
    }
    else {
        kwtuple = NULL;
        k = NULL;
    }
    closure = PyFunction_GET_CLOSURE(func);
#if PY_MAJOR_VERSION >= 3
    kwdefs = PyFunction_GET_KW_DEFAULTS(func);
#endif
    if (argdefs != NULL) {
        d = &PyTuple_GET_ITEM(argdefs, 0);
        nd = Py_SIZE(argdefs);
    }
    else {
        d = NULL;
        nd = 0;
    }
#if PY_MAJOR_VERSION >= 3
    result = PyEval_EvalCodeEx((PyObject*)co, globals, (PyObject *)NULL,
                               args, nargs,
                               k, (int)nk,
                               d, (int)nd, kwdefs, closure);
#else
    result = PyEval_EvalCodeEx(co, globals, (PyObject *)NULL,
                               args, nargs,
                               k, (int)nk,
                               d, (int)nd, closure);
#endif
    Py_XDECREF(kwtuple);
done:
    Py_LeaveRecursiveCall();
    return result;
}
#endif
#endif

/* PyObjectCall */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *result;
    ternaryfunc call = func->ob_type->tp_call;
    if (unlikely(!call))
        return PyObject_Call(func, arg, kw);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = (*call)(func, arg, kw);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCallMethO */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg) {
    PyObject *self, *result;
    PyCFunction cfunc;
    cfunc = PyCFunction_GET_FUNCTION(func);
    self = PyCFunction_GET_SELF(func);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = cfunc(self, arg);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCallNoArg */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, NULL, 0);
    }
#endif
#ifdef __Pyx_CyFunction_USED
    if (likely(PyCFunction_Check(func) || __Pyx_CyFunction_Check(func)))
#else
    if (likely(PyCFunction_Check(func)))
#endif
    {
        if (likely(PyCFunction_GET_FLAGS(func) & METH_NOARGS)) {
            return __Pyx_PyObject_CallMethO(func, NULL);
        }
    }
    return __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL);
}
#endif

/* PyCFunctionFastCall */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject * __Pyx_PyCFunction_FastCall(PyObject *func_obj, PyObject **args, Py_ssize_t nargs) {
    PyCFunctionObject *func = (PyCFunctionObject*)func_obj;
    PyCFunction meth = PyCFunction_GET_FUNCTION(func);
    PyObject *self = PyCFunction_GET_SELF(func);
    int flags = PyCFunction_GET_FLAGS(func);
    assert(PyCFunction_Check(func));
    assert(METH_FASTCALL == (flags & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS | METH_STACKLESS)));
    assert(nargs >= 0);
    assert(nargs == 0 || args != NULL);
    /* _PyCFunction_FastCallDict() must not be called with an exception set,
       because it may clear it (directly or indirectly) and so the
       caller loses its exception */
    assert(!PyErr_Occurred());
    if ((PY_VERSION_HEX < 0x030700A0) || unlikely(flags & METH_KEYWORDS)) {
        return (*((__Pyx_PyCFunctionFastWithKeywords)(void*)meth)) (self, args, nargs, NULL);
    } else {
        return (*((__Pyx_PyCFunctionFast)(void*)meth)) (self, args, nargs);
    }
}
#endif

/* PyObjectCallOneArg */
#if CYTHON_COMPILING_IN_CPYTHON
static PyObject* __Pyx__PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_New(1);
    if (unlikely(!args)) return NULL;
    Py_INCREF(arg);
    PyTuple_SET_ITEM(args, 0, arg);
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, &arg, 1);
    }
#endif
    if (likely(PyCFunction_Check(func))) {
        if (likely(PyCFunction_GET_FLAGS(func) & METH_O)) {
            return __Pyx_PyObject_CallMethO(func, arg);
#if CYTHON_FAST_PYCCALL
        } else if (PyCFunction_GET_FLAGS(func) & METH_FASTCALL) {
            return __Pyx_PyCFunction_FastCall(func, &arg, 1);
#endif
        }
    }
    return __Pyx__PyObject_CallOneArg(func, arg);
}
#else
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_Pack(1, arg);
    if (unlikely(!args)) return NULL;
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
#endif

/* PyObjectGetAttrStr */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro))
        return tp->tp_getattro(obj, attr_name);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_getattr))
        return tp->tp_getattr(obj, PyString_AS_STRING(attr_name));
#endif
    return PyObject_GetAttr(obj, attr_name);
}
#endif

/* PyObjectGetMethod */
static int __Pyx_PyObject_GetMethod(PyObject *obj, PyObject *name, PyObject **method) {
    PyObject *attr;
#if CYTHON_UNPACK_METHODS && CYTHON_COMPILING_IN_CPYTHON && CYTHON_USE_PYTYPE_LOOKUP
    PyTypeObject *tp = Py_TYPE(obj);
    PyObject *descr;
    descrgetfunc f = NULL;
    PyObject **dictptr, *dict;
    int meth_found = 0;
    assert (*method == NULL);
    if (unlikely(tp->tp_getattro != PyObject_GenericGetAttr)) {
        attr = __Pyx_PyObject_GetAttrStr(obj, name);
        goto try_unpack;
    }
    if (unlikely(tp->tp_dict == NULL) && unlikely(PyType_Ready(tp) < 0)) {
        return 0;
    }
    descr = _PyType_Lookup(tp, name);
    if (likely(descr != NULL)) {
        Py_INCREF(descr);
#if PY_MAJOR_VERSION >= 3
        #ifdef __Pyx_CyFunction_USED
        if (likely(PyFunction_Check(descr) || (Py_TYPE(descr) == &PyMethodDescr_Type) || __Pyx_CyFunction_Check(descr)))
        #else
        if (likely(PyFunction_Check(descr) || (Py_TYPE(descr) == &PyMethodDescr_Type)))
        #endif
#else
        #ifdef __Pyx_CyFunction_USED
        if (likely(PyFunction_Check(descr) || __Pyx_CyFunction_Check(descr)))
        #else
        if (likely(PyFunction_Check(descr)))
        #endif
#endif
        {
            meth_found = 1;
        } else {
            f = Py_TYPE(descr)->tp_descr_get;
            if (f != NULL && PyDescr_IsData(descr)) {
                attr = f(descr, obj, (PyObject *)Py_TYPE(obj));
                Py_DECREF(descr);
                goto try_unpack;
            }
        }
    }
    dictptr = _PyObject_GetDictPtr(obj);
    if (dictptr != NULL && (dict = *dictptr) != NULL) {
        Py_INCREF(dict);
        attr = __Pyx_PyDict_GetItemStr(dict, name);
        if (attr != NULL) {
            Py_INCREF(attr);
            Py_DECREF(dict);
            Py_XDECREF(descr);
            goto try_unpack;
        }
        Py_DECREF(dict);
    }
    if (meth_found) {
        *method = descr;
        return 1;
    }
    if (f != NULL) {
        attr = f(descr, obj, (PyObject *)Py_TYPE(obj));
        Py_DECREF(descr);
        goto try_unpack;
    }
    if (descr != NULL) {
        *method = descr;
        return 0;
    }
    PyErr_Format(PyExc_AttributeError,
#if PY_MAJOR_VERSION >= 3
                 "'%.50s' object has no attribute '%U'",
                 tp->tp_name, name);
#else
                 "'%.50s' object has no attribute '%.400s'",
                 tp->tp_name, PyString_AS_STRING(name));
#endif
    return 0;
#else
    attr = __Pyx_PyObject_GetAttrStr(obj, name);
    goto try_unpack;
#endif
try_unpack:
#if CYTHON_UNPACK_METHODS
    if (likely(attr) && PyMethod_Check(attr) && likely(PyMethod_GET_SELF(attr) == obj)) {
        PyObject *function = PyMethod_GET_FUNCTION(attr);
        Py_INCREF(function);
        Py_DECREF(attr);
        *method = function;
        return 1;
    }
#endif
    *method = attr;
    return 0;
}

/* PyObjectCallMethod0 */
static PyObject* __Pyx_PyObject_CallMethod0(PyObject* obj, PyObject* method_name) {
    PyObject *method = NULL, *result = NULL;
    int is_method = __Pyx_PyObject_GetMethod(obj, method_name, &method);
    if (likely(is_method)) {
        result = __Pyx_PyObject_CallOneArg(method, obj);
        Py_DECREF(method);
        return result;
    }
    if (unlikely(!method)) goto bad;
    result = __Pyx_PyObject_CallNoArg(method);
    Py_DECREF(method);
bad:
    return result;
}

/* RaiseNeedMoreValuesToUnpack */
static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index) {
    PyErr_Format(PyExc_ValueError,
                 "need more than %" CYTHON_FORMAT_SSIZE_T "d value%.1s to unpack",
                 index, (index == 1) ? "" : "s");
}

/* RaiseTooManyValuesToUnpack */
static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected) {
    PyErr_Format(PyExc_ValueError,
                 "too many values to unpack (expected %" CYTHON_FORMAT_SSIZE_T "d)", expected);
}

/* UnpackItemEndCheck */
static int __Pyx_IternextUnpackEndCheck(PyObject *retval, Py_ssize_t expected) {
    if (unlikely(retval)) {
        Py_DECREF(retval);
        __Pyx_RaiseTooManyValuesError(expected);
        return -1;
    } else {
        return __Pyx_IterFinish();
    }
    return 0;
}

/* RaiseNoneIterError */
static CYTHON_INLINE void __Pyx_RaiseNoneNotIterableError(void) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
}

/* UnpackTupleError */
static void __Pyx_UnpackTupleError(PyObject *t, Py_ssize_t index) {
    if (t == Py_None) {
      __Pyx_RaiseNoneNotIterableError();
    } else if (PyTuple_GET_SIZE(t) < index) {
      __Pyx_RaiseNeedMoreValuesError(PyTuple_GET_SIZE(t));
    } else {
      __Pyx_RaiseTooManyValuesError(index);
    }
}

/* UnpackTuple2 */
static CYTHON_INLINE int __Pyx_unpack_tuple2_exact(
        PyObject* tuple, PyObject** pvalue1, PyObject** pvalue2, int decref_tuple) {
    PyObject *value1 = NULL, *value2 = NULL;
#if CYTHON_COMPILING_IN_PYPY
    value1 = PySequence_ITEM(tuple, 0);  if (unlikely(!value1)) goto bad;
    value2 = PySequence_ITEM(tuple, 1);  if (unlikely(!value2)) goto bad;
#else
    value1 = PyTuple_GET_ITEM(tuple, 0);  Py_INCREF(value1);
    value2 = PyTuple_GET_ITEM(tuple, 1);  Py_INCREF(value2);
#endif
    if (decref_tuple) {
        Py_DECREF(tuple);
    }
    *pvalue1 = value1;
    *pvalue2 = value2;
    return 0;
#if CYTHON_COMPILING_IN_PYPY
bad:
    Py_XDECREF(value1);
    Py_XDECREF(value2);
    if (decref_tuple) { Py_XDECREF(tuple); }
    return -1;
#endif
}
static int __Pyx_unpack_tuple2_generic(PyObject* tuple, PyObject** pvalue1, PyObject** pvalue2,
                                       int has_known_size, int decref_tuple) {
    Py_ssize_t index;
    PyObject *value1 = NULL, *value2 = NULL, *iter = NULL;
    iternextfunc iternext;
    iter = PyObject_GetIter(tuple);
    if (unlikely(!iter)) goto bad;
    if (decref_tuple) { Py_DECREF(tuple); tuple = NULL; }
    iternext = Py_TYPE(iter)->tp_iternext;
    value1 = iternext(iter); if (unlikely(!value1)) { index = 0; goto unpacking_failed; }
    value2 = iternext(iter); if (unlikely(!value2)) { index = 1; goto unpacking_failed; }
    if (!has_known_size && unlikely(__Pyx_IternextUnpackEndCheck(iternext(iter), 2))) goto bad;
    Py_DECREF(iter);
    *pvalue1 = value1;
    *pvalue2 = value2;
    return 0;
unpacking_failed:
    if (!has_known_size && __Pyx_IterFinish() == 0)
        __Pyx_RaiseNeedMoreValuesError(index);
bad:
    Py_XDECREF(iter);
    Py_XDECREF(value1);
    Py_XDECREF(value2);
    if (decref_tuple) { Py_XDECREF(tuple); }
    return -1;
}

/* dict_iter */
static CYTHON_INLINE PyObject* __Pyx_dict_iterator(PyObject* iterable, int is_dict, PyObject* method_name,
                                                   Py_ssize_t* p_orig_length, int* p_source_is_dict) {
    is_dict = is_dict || likely(PyDict_CheckExact(iterable));
    *p_source_is_dict = is_dict;
    if (is_dict) {
#if !CYTHON_COMPILING_IN_PYPY
        *p_orig_length = PyDict_Size(iterable);
        Py_INCREF(iterable);
        return iterable;
#elif PY_MAJOR_VERSION >= 3
        static PyObject *py_items = NULL, *py_keys = NULL, *py_values = NULL;
        PyObject **pp = NULL;
        if (method_name) {
            const char *name = PyUnicode_AsUTF8(method_name);
            if (strcmp(name, "iteritems") == 0) pp = &py_items;
            else if (strcmp(name, "iterkeys") == 0) pp = &py_keys;
            else if (strcmp(name, "itervalues") == 0) pp = &py_values;
            if (pp) {
                if (!*pp) {
                    *pp = PyUnicode_FromString(name + 4);
                    if (!*pp)
                        return NULL;
                }
                method_name = *pp;
            }
        }
#endif
    }
    *p_orig_length = 0;
    if (method_name) {
        PyObject* iter;
        iterable = __Pyx_PyObject_CallMethod0(iterable, method_name);
        if (!iterable)
            return NULL;
#if !CYTHON_COMPILING_IN_PYPY
        if (PyTuple_CheckExact(iterable) || PyList_CheckExact(iterable))
            return iterable;
#endif
        iter = PyObject_GetIter(iterable);
        Py_DECREF(iterable);
        return iter;
    }
    return PyObject_GetIter(iterable);
}
static CYTHON_INLINE int __Pyx_dict_iter_next(
        PyObject* iter_obj, CYTHON_NCP_UNUSED Py_ssize_t orig_length, CYTHON_NCP_UNUSED Py_ssize_t* ppos,
        PyObject** pkey, PyObject** pvalue, PyObject** pitem, int source_is_dict) {
    PyObject* next_item;
#if !CYTHON_COMPILING_IN_PYPY
    if (source_is_dict) {
        PyObject *key, *value;
        if (unlikely(orig_length != PyDict_Size(iter_obj))) {
            PyErr_SetString(PyExc_RuntimeError, "dictionary changed size during iteration");
            return -1;
        }
        if (unlikely(!PyDict_Next(iter_obj, ppos, &key, &value))) {
            return 0;
        }
        if (pitem) {
            PyObject* tuple = PyTuple_New(2);
            if (unlikely(!tuple)) {
                return -1;
            }
            Py_INCREF(key);
            Py_INCREF(value);
            PyTuple_SET_ITEM(tuple, 0, key);
            PyTuple_SET_ITEM(tuple, 1, value);
            *pitem = tuple;
        } else {
            if (pkey) {
                Py_INCREF(key);
                *pkey = key;
            }
            if (pvalue) {
                Py_INCREF(value);
                *pvalue = value;
            }
        }
        return 1;
    } else if (PyTuple_CheckExact(iter_obj)) {
        Py_ssize_t pos = *ppos;
        if (unlikely(pos >= PyTuple_GET_SIZE(iter_obj))) return 0;
        *ppos = pos + 1;
        next_item = PyTuple_GET_ITEM(iter_obj, pos);
        Py_INCREF(next_item);
    } else if (PyList_CheckExact(iter_obj)) {
        Py_ssize_t pos = *ppos;
        if (unlikely(pos >= PyList_GET_SIZE(iter_obj))) return 0;
        *ppos = pos + 1;
        next_item = PyList_GET_ITEM(iter_obj, pos);
        Py_INCREF(next_item);
    } else
#endif
    {
        next_item = PyIter_Next(iter_obj);
        if (unlikely(!next_item)) {
            return __Pyx_IterFinish();
        }
    }
    if (pitem) {
        *pitem = next_item;
    } else if (pkey && pvalue) {
        if (__Pyx_unpack_tuple2(next_item, pkey, pvalue, source_is_dict, source_is_dict, 1))
            return -1;
    } else if (pkey) {
        *pkey = next_item;
    } else {
        *pvalue = next_item;
    }
    return 1;
}

/* RaiseDoubleKeywords */
static void __Pyx_RaiseDoubleKeywordsError(
    const char* func_name,
    PyObject* kw_name)
{
    PyErr_Format(PyExc_TypeError,
        #if PY_MAJOR_VERSION >= 3
        "%s() got multiple values for keyword argument '%U'", func_name, kw_name);
        #else
        "%s() got multiple values for keyword argument '%s'", func_name,
        PyString_AsString(kw_name));
        #endif
}

/* MergeKeywords */
static int __Pyx_MergeKeywords(PyObject *kwdict, PyObject *source_mapping) {
    PyObject *iter, *key = NULL, *value = NULL;
    int source_is_dict, result;
    Py_ssize_t orig_length, ppos = 0;
    iter = __Pyx_dict_iterator(source_mapping, 0, __pyx_n_s_items, &orig_length, &source_is_dict);
    if (unlikely(!iter)) {
        PyObject *args;
        if (!PyErr_ExceptionMatches(PyExc_AttributeError)) goto bad;
        PyErr_Clear();
        args = PyTuple_Pack(1, source_mapping);
        if (likely(args)) {
            PyObject *fallback = PyObject_Call((PyObject*)&PyDict_Type, args, NULL);
            Py_DECREF(args);
            if (likely(fallback)) {
                iter = __Pyx_dict_iterator(fallback, 1, __pyx_n_s_items, &orig_length, &source_is_dict);
                Py_DECREF(fallback);
            }
        }
        if (unlikely(!iter)) goto bad;
    }
    while (1) {
        result = __Pyx_dict_iter_next(iter, orig_length, &ppos, &key, &value, NULL, source_is_dict);
        if (unlikely(result < 0)) goto bad;
        if (!result) break;
        if (unlikely(PyDict_Contains(kwdict, key))) {
            __Pyx_RaiseDoubleKeywordsError("function", key);
            result = -1;
        } else {
            result = PyDict_SetItem(kwdict, key, value);
        }
        Py_DECREF(key);
        Py_DECREF(value);
        if (unlikely(result < 0)) goto bad;
    }
    Py_XDECREF(iter);
    return 0;
bad:
    Py_XDECREF(iter);
    return -1;
}

/* ParseKeywords */
static int __Pyx_ParseOptionalKeywords(
    PyObject *kwds,
    PyObject **argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    const char* function_name)
{
    PyObject *key = 0, *value = 0;
    Py_ssize_t pos = 0;
    PyObject*** name;
    PyObject*** first_kw_arg = argnames + num_pos_args;
    while (PyDict_Next(kwds, &pos, &key, &value)) {
        name = first_kw_arg;
        while (*name && (**name != key)) name++;
        if (*name) {
            values[name-argnames] = value;
            continue;
        }
        name = first_kw_arg;
        #if PY_MAJOR_VERSION < 3
        if (likely(PyString_CheckExact(key)) || likely(PyString_Check(key))) {
            while (*name) {
                if ((CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**name) == PyString_GET_SIZE(key))
                        && _PyString_Eq(**name, key)) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    if ((**argname == key) || (
                            (CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**argname) == PyString_GET_SIZE(key))
                             && _PyString_Eq(**argname, key))) {
                        goto arg_passed_twice;
                    }
                    argname++;
                }
            }
        } else
        #endif
        if (likely(PyUnicode_Check(key))) {
            while (*name) {
                int cmp = (**name == key) ? 0 :
                #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                    (PyUnicode_GET_SIZE(**name) != PyUnicode_GET_SIZE(key)) ? 1 :
                #endif
                    PyUnicode_Compare(**name, key);
                if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                if (cmp == 0) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    int cmp = (**argname == key) ? 0 :
                    #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                        (PyUnicode_GET_SIZE(**argname) != PyUnicode_GET_SIZE(key)) ? 1 :
                    #endif
                        PyUnicode_Compare(**argname, key);
                    if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                    if (cmp == 0) goto arg_passed_twice;
                    argname++;
                }
            }
        } else
            goto invalid_keyword_type;
        if (kwds2) {
            if (unlikely(PyDict_SetItem(kwds2, key, value))) goto bad;
        } else {
            goto invalid_keyword;
        }
    }
    return 0;
arg_passed_twice:
    __Pyx_RaiseDoubleKeywordsError(function_name, key);
    goto bad;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    goto bad;
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
    #if PY_MAJOR_VERSION < 3
        "%.200s() got an unexpected keyword argument '%.200s'",
        function_name, PyString_AsString(key));
    #else
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
bad:
    return -1;
}

/* RaiseArgTupleInvalid */
static void __Pyx_RaiseArgtupleInvalid(
    const char* func_name,
    int exact,
    Py_ssize_t num_min,
    Py_ssize_t num_max,
    Py_ssize_t num_found)
{
    Py_ssize_t num_expected;
    const char *more_or_less;
    if (num_found < num_min) {
        num_expected = num_min;
        more_or_less = "at least";
    } else {
        num_expected = num_max;
        more_or_less = "at most";
    }
    if (exact) {
        more_or_less = "exactly";
    }
    PyErr_Format(PyExc_TypeError,
                 "%.200s() takes %.8s %" CYTHON_FORMAT_SSIZE_T "d positional argument%.1s (%" CYTHON_FORMAT_SSIZE_T "d given)",
                 func_name, more_or_less, num_expected,
                 (num_expected == 1) ? "" : "s", num_found);
}

/* ArgTypeTest */
static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact)
{
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    else if (exact) {
        #if PY_MAJOR_VERSION == 2
        if ((type == &PyBaseString_Type) && likely(__Pyx_PyBaseString_CheckExact(obj))) return 1;
        #endif
    }
    else {
        if (likely(__Pyx_TypeCheck(obj, type))) return 1;
    }
    PyErr_Format(PyExc_TypeError,
        "Argument '%.200s' has incorrect type (expected %.200s, got %.200s)",
        name, type->tp_name, Py_TYPE(obj)->tp_name);
    return 0;
}

/* PyErrExceptionMatches */
#if CYTHON_FAST_THREAD_STATE
static int __Pyx_PyErr_ExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    n = PyTuple_GET_SIZE(tuple);
#if PY_MAJOR_VERSION >= 3
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
#endif
    for (i=0; i<n; i++) {
        if (__Pyx_PyErr_GivenExceptionMatches(exc_type, PyTuple_GET_ITEM(tuple, i))) return 1;
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err) {
    PyObject *exc_type = tstate->curexc_type;
    if (exc_type == err) return 1;
    if (unlikely(!exc_type)) return 0;
    if (unlikely(PyTuple_Check(err)))
        return __Pyx_PyErr_ExceptionMatchesTuple(exc_type, err);
    return __Pyx_PyErr_GivenExceptionMatches(exc_type, err);
}
#endif

/* PyErrFetchRestore */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    tmp_type = tstate->curexc_type;
    tmp_value = tstate->curexc_value;
    tmp_tb = tstate->curexc_traceback;
    tstate->curexc_type = type;
    tstate->curexc_value = value;
    tstate->curexc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    *type = tstate->curexc_type;
    *value = tstate->curexc_value;
    *tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
}
#endif

/* GetAttr */
static CYTHON_INLINE PyObject *__Pyx_GetAttr(PyObject *o, PyObject *n) {
#if CYTHON_USE_TYPE_SLOTS
#if PY_MAJOR_VERSION >= 3
    if (likely(PyUnicode_Check(n)))
#else
    if (likely(PyString_Check(n)))
#endif
        return __Pyx_PyObject_GetAttrStr(o, n);
#endif
    return PyObject_GetAttr(o, n);
}

/* GetAttr3 */
static PyObject *__Pyx_GetAttr3Default(PyObject *d) {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    if (unlikely(!__Pyx_PyErr_ExceptionMatches(PyExc_AttributeError)))
        return NULL;
    __Pyx_PyErr_Clear();
    Py_INCREF(d);
    return d;
}
static CYTHON_INLINE PyObject *__Pyx_GetAttr3(PyObject *o, PyObject *n, PyObject *d) {
    PyObject *r = __Pyx_GetAttr(o, n);
    return (likely(r)) ? r : __Pyx_GetAttr3Default(d);
}

/* GetBuiltinName */
static PyObject *__Pyx_GetBuiltinName(PyObject *name) {
    PyObject* result = __Pyx_PyObject_GetAttrStr(__pyx_b, name);
    if (unlikely(!result)) {
        PyErr_Format(PyExc_NameError,
#if PY_MAJOR_VERSION >= 3
            "name '%U' is not defined", name);
#else
            "name '%.200s' is not defined", PyString_AS_STRING(name));
#endif
    }
    return result;
}

/* PyDictVersioning */
#if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    return likely(dict) ? __PYX_GET_DICT_VERSION(dict) : 0;
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj) {
    PyObject **dictptr = NULL;
    Py_ssize_t offset = Py_TYPE(obj)->tp_dictoffset;
    if (offset) {
#if CYTHON_COMPILING_IN_CPYTHON
        dictptr = (likely(offset > 0)) ? (PyObject **) ((char *)obj + offset) : _PyObject_GetDictPtr(obj);
#else
        dictptr = _PyObject_GetDictPtr(obj);
#endif
    }
    return (dictptr && *dictptr) ? __PYX_GET_DICT_VERSION(*dictptr) : 0;
}
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    if (unlikely(!dict) || unlikely(tp_dict_version != __PYX_GET_DICT_VERSION(dict)))
        return 0;
    return obj_dict_version == __Pyx_get_object_dict_version(obj);
}
#endif

/* GetModuleGlobalName */
#if CYTHON_USE_DICT_VERSIONS
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value)
#else
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name)
#endif
{
    PyObject *result;
#if !CYTHON_AVOID_BORROWED_REFS
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030500A1
    result = _PyDict_GetItem_KnownHash(__pyx_d, name, ((PyASCIIObject *) name)->hash);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    } else if (unlikely(PyErr_Occurred())) {
        return NULL;
    }
#else
    result = PyDict_GetItem(__pyx_d, name);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    }
#endif
#else
    result = PyObject_GetItem(__pyx_d, name);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    }
    PyErr_Clear();
#endif
    return __Pyx_GetBuiltinName(name);
}

/* GetItemInt */
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j) {
    PyObject *r;
    if (!j) return NULL;
    r = PyObject_GetItem(o, j);
    Py_DECREF(j);
    return r;
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyList_GET_SIZE(o);
    }
    if ((!boundscheck) || likely(__Pyx_is_valid_index(wrapped_i, PyList_GET_SIZE(o)))) {
        PyObject *r = PyList_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyTuple_GET_SIZE(o);
    }
    if ((!boundscheck) || likely(__Pyx_is_valid_index(wrapped_i, PyTuple_GET_SIZE(o)))) {
        PyObject *r = PyTuple_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i, int is_list,
                                                     CYTHON_NCP_UNUSED int wraparound,
                                                     CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS && CYTHON_USE_TYPE_SLOTS
    if (is_list || PyList_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyList_GET_SIZE(o);
        if ((!boundscheck) || (likely(__Pyx_is_valid_index(n, PyList_GET_SIZE(o))))) {
            PyObject *r = PyList_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    }
    else if (PyTuple_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyTuple_GET_SIZE(o);
        if ((!boundscheck) || likely(__Pyx_is_valid_index(n, PyTuple_GET_SIZE(o)))) {
            PyObject *r = PyTuple_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    } else {
        PySequenceMethods *m = Py_TYPE(o)->tp_as_sequence;
        if (likely(m && m->sq_item)) {
            if (wraparound && unlikely(i < 0) && likely(m->sq_length)) {
                Py_ssize_t l = m->sq_length(o);
                if (likely(l >= 0)) {
                    i += l;
                } else {
                    if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                        return NULL;
                    PyErr_Clear();
                }
            }
            return m->sq_item(o, i);
        }
    }
#else
    if (is_list || PySequence_Check(o)) {
        return PySequence_GetItem(o, i);
    }
#endif
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
}

/* PyObjectCall2Args */
static CYTHON_UNUSED PyObject* __Pyx_PyObject_Call2Args(PyObject* function, PyObject* arg1, PyObject* arg2) {
    PyObject *args, *result = NULL;
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(function)) {
        PyObject *args[2] = {arg1, arg2};
        return __Pyx_PyFunction_FastCall(function, args, 2);
    }
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(function)) {
        PyObject *args[2] = {arg1, arg2};
        return __Pyx_PyCFunction_FastCall(function, args, 2);
    }
    #endif
    args = PyTuple_New(2);
    if (unlikely(!args)) goto done;
    Py_INCREF(arg1);
    PyTuple_SET_ITEM(args, 0, arg1);
    Py_INCREF(arg2);
    PyTuple_SET_ITEM(args, 1, arg2);
    Py_INCREF(function);
    result = __Pyx_PyObject_Call(function, args, NULL);
    Py_DECREF(args);
    Py_DECREF(function);
done:
    return result;
}

/* UnpackUnboundCMethod */
static int __Pyx_TryUnpackUnboundCMethod(__Pyx_CachedCFunction* target) {
    PyObject *method;
    method = __Pyx_PyObject_GetAttrStr(target->type, *target->method_name);
    if (unlikely(!method))
        return -1;
    target->method = method;
#if CYTHON_COMPILING_IN_CPYTHON
    #if PY_MAJOR_VERSION >= 3
    if (likely(__Pyx_TypeCheck(method, &PyMethodDescr_Type)))
    #endif
    {
        PyMethodDescrObject *descr = (PyMethodDescrObject*) method;
        target->func = descr->d_method->ml_meth;
        target->flag = descr->d_method->ml_flags & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_STACKLESS);
    }
#endif
    return 0;
}

/* CallUnboundCMethod2 */
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030600B1
static CYTHON_INLINE PyObject *__Pyx_CallUnboundCMethod2(__Pyx_CachedCFunction *cfunc, PyObject *self, PyObject *arg1, PyObject *arg2) {
    if (likely(cfunc->func)) {
        PyObject *args[2] = {arg1, arg2};
        if (cfunc->flag == METH_FASTCALL) {
            #if PY_VERSION_HEX >= 0x030700A0
            return (*(__Pyx_PyCFunctionFast)(void*)(PyCFunction)cfunc->func)(self, args, 2);
            #else
            return (*(__Pyx_PyCFunctionFastWithKeywords)(void*)(PyCFunction)cfunc->func)(self, args, 2, NULL);
            #endif
        }
        #if PY_VERSION_HEX >= 0x030700A0
        if (cfunc->flag == (METH_FASTCALL | METH_KEYWORDS))
            return (*(__Pyx_PyCFunctionFastWithKeywords)(void*)(PyCFunction)cfunc->func)(self, args, 2, NULL);
        #endif
    }
    return __Pyx__CallUnboundCMethod2(cfunc, self, arg1, arg2);
}
#endif
static PyObject* __Pyx__CallUnboundCMethod2(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg1, PyObject* arg2){
    PyObject *args, *result = NULL;
    if (unlikely(!cfunc->func && !cfunc->method) && unlikely(__Pyx_TryUnpackUnboundCMethod(cfunc) < 0)) return NULL;
#if CYTHON_COMPILING_IN_CPYTHON
    if (cfunc->func && (cfunc->flag & METH_VARARGS)) {
        args = PyTuple_New(2);
        if (unlikely(!args)) goto bad;
        Py_INCREF(arg1);
        PyTuple_SET_ITEM(args, 0, arg1);
        Py_INCREF(arg2);
        PyTuple_SET_ITEM(args, 1, arg2);
        if (cfunc->flag & METH_KEYWORDS)
            result = (*(PyCFunctionWithKeywords)(void*)(PyCFunction)cfunc->func)(self, args, NULL);
        else
            result = (*cfunc->func)(self, args);
    } else {
        args = PyTuple_New(3);
        if (unlikely(!args)) goto bad;
        Py_INCREF(self);
        PyTuple_SET_ITEM(args, 0, self);
        Py_INCREF(arg1);
        PyTuple_SET_ITEM(args, 1, arg1);
        Py_INCREF(arg2);
        PyTuple_SET_ITEM(args, 2, arg2);
        result = __Pyx_PyObject_Call(cfunc->method, args, NULL);
    }
#else
    args = PyTuple_Pack(3, self, arg1, arg2);
    if (unlikely(!args)) goto bad;
    result = __Pyx_PyObject_Call(cfunc->method, args, NULL);
#endif
bad:
    Py_XDECREF(args);
    return result;
}

/* CallUnboundCMethod1 */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_CallUnboundCMethod1(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg) {
    if (likely(cfunc->func)) {
        int flag = cfunc->flag;
        if (flag == METH_O) {
            return (*(cfunc->func))(self, arg);
        } else if (PY_VERSION_HEX >= 0x030600B1 && flag == METH_FASTCALL) {
            if (PY_VERSION_HEX >= 0x030700A0) {
                return (*(__Pyx_PyCFunctionFast)(void*)(PyCFunction)cfunc->func)(self, &arg, 1);
            } else {
                return (*(__Pyx_PyCFunctionFastWithKeywords)(void*)(PyCFunction)cfunc->func)(self, &arg, 1, NULL);
            }
        } else if (PY_VERSION_HEX >= 0x030700A0 && flag == (METH_FASTCALL | METH_KEYWORDS)) {
            return (*(__Pyx_PyCFunctionFastWithKeywords)(void*)(PyCFunction)cfunc->func)(self, &arg, 1, NULL);
        }
    }
    return __Pyx__CallUnboundCMethod1(cfunc, self, arg);
}
#endif
static PyObject* __Pyx__CallUnboundCMethod1(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg){
    PyObject *args, *result = NULL;
    if (unlikely(!cfunc->func && !cfunc->method) && unlikely(__Pyx_TryUnpackUnboundCMethod(cfunc) < 0)) return NULL;
#if CYTHON_COMPILING_IN_CPYTHON
    if (cfunc->func && (cfunc->flag & METH_VARARGS)) {
        args = PyTuple_New(1);
        if (unlikely(!args)) goto bad;
        Py_INCREF(arg);
        PyTuple_SET_ITEM(args, 0, arg);
        if (cfunc->flag & METH_KEYWORDS)
            result = (*(PyCFunctionWithKeywords)(void*)(PyCFunction)cfunc->func)(self, args, NULL);
        else
            result = (*cfunc->func)(self, args);
    } else {
        args = PyTuple_New(2);
        if (unlikely(!args)) goto bad;
        Py_INCREF(self);
        PyTuple_SET_ITEM(args, 0, self);
        Py_INCREF(arg);
        PyTuple_SET_ITEM(args, 1, arg);
        result = __Pyx_PyObject_Call(cfunc->method, args, NULL);
    }
#else
    args = PyTuple_Pack(2, self, arg);
    if (unlikely(!args)) goto bad;
    result = __Pyx_PyObject_Call(cfunc->method, args, NULL);
#endif
bad:
    Py_XDECREF(args);
    return result;
}

/* py_dict_pop */
static CYTHON_INLINE PyObject *__Pyx_PyDict_Pop(PyObject *d, PyObject *key, PyObject *default_value) {
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX > 0x030600B3
    if ((1)) {
        return _PyDict_Pop(d, key, default_value);
    } else
#endif
    if (default_value) {
        return __Pyx_CallUnboundCMethod2(&__pyx_umethod_PyDict_Type_pop, d, key, default_value);
    } else {
        return __Pyx_CallUnboundCMethod1(&__pyx_umethod_PyDict_Type_pop, d, key);
    }
}

/* DictGetItem */
#if PY_MAJOR_VERSION >= 3 && !CYTHON_COMPILING_IN_PYPY
static PyObject *__Pyx_PyDict_GetItem(PyObject *d, PyObject* key) {
    PyObject *value;
    value = PyDict_GetItemWithError(d, key);
    if (unlikely(!value)) {
        if (!PyErr_Occurred()) {
            if (unlikely(PyTuple_Check(key))) {
                PyObject* args = PyTuple_Pack(1, key);
                if (likely(args)) {
                    PyErr_SetObject(PyExc_KeyError, args);
                    Py_DECREF(args);
                }
            } else {
                PyErr_SetObject(PyExc_KeyError, key);
            }
        }
        return NULL;
    }
    Py_INCREF(value);
    return value;
}
#endif

/* BytesEquals */
static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY
    return PyObject_RichCompareBool(s1, s2, equals);
#else
    if (s1 == s2) {
        return (equals == Py_EQ);
    } else if (PyBytes_CheckExact(s1) & PyBytes_CheckExact(s2)) {
        const char *ps1, *ps2;
        Py_ssize_t length = PyBytes_GET_SIZE(s1);
        if (length != PyBytes_GET_SIZE(s2))
            return (equals == Py_NE);
        ps1 = PyBytes_AS_STRING(s1);
        ps2 = PyBytes_AS_STRING(s2);
        if (ps1[0] != ps2[0]) {
            return (equals == Py_NE);
        } else if (length == 1) {
            return (equals == Py_EQ);
        } else {
            int result;
#if CYTHON_USE_UNICODE_INTERNALS
            Py_hash_t hash1, hash2;
            hash1 = ((PyBytesObject*)s1)->ob_shash;
            hash2 = ((PyBytesObject*)s2)->ob_shash;
            if (hash1 != hash2 && hash1 != -1 && hash2 != -1) {
                return (equals == Py_NE);
            }
#endif
            result = memcmp(ps1, ps2, (size_t)length);
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & PyBytes_CheckExact(s2)) {
        return (equals == Py_NE);
    } else if ((s2 == Py_None) & PyBytes_CheckExact(s1)) {
        return (equals == Py_NE);
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
#endif
}

/* UnicodeEquals */
static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY
    return PyObject_RichCompareBool(s1, s2, equals);
#else
#if PY_MAJOR_VERSION < 3
    PyObject* owned_ref = NULL;
#endif
    int s1_is_unicode, s2_is_unicode;
    if (s1 == s2) {
        goto return_eq;
    }
    s1_is_unicode = PyUnicode_CheckExact(s1);
    s2_is_unicode = PyUnicode_CheckExact(s2);
#if PY_MAJOR_VERSION < 3
    if ((s1_is_unicode & (!s2_is_unicode)) && PyString_CheckExact(s2)) {
        owned_ref = PyUnicode_FromObject(s2);
        if (unlikely(!owned_ref))
            return -1;
        s2 = owned_ref;
        s2_is_unicode = 1;
    } else if ((s2_is_unicode & (!s1_is_unicode)) && PyString_CheckExact(s1)) {
        owned_ref = PyUnicode_FromObject(s1);
        if (unlikely(!owned_ref))
            return -1;
        s1 = owned_ref;
        s1_is_unicode = 1;
    } else if (((!s2_is_unicode) & (!s1_is_unicode))) {
        return __Pyx_PyBytes_Equals(s1, s2, equals);
    }
#endif
    if (s1_is_unicode & s2_is_unicode) {
        Py_ssize_t length;
        int kind;
        void *data1, *data2;
        if (unlikely(__Pyx_PyUnicode_READY(s1) < 0) || unlikely(__Pyx_PyUnicode_READY(s2) < 0))
            return -1;
        length = __Pyx_PyUnicode_GET_LENGTH(s1);
        if (length != __Pyx_PyUnicode_GET_LENGTH(s2)) {
            goto return_ne;
        }
#if CYTHON_USE_UNICODE_INTERNALS
        {
            Py_hash_t hash1, hash2;
        #if CYTHON_PEP393_ENABLED
            hash1 = ((PyASCIIObject*)s1)->hash;
            hash2 = ((PyASCIIObject*)s2)->hash;
        #else
            hash1 = ((PyUnicodeObject*)s1)->hash;
            hash2 = ((PyUnicodeObject*)s2)->hash;
        #endif
            if (hash1 != hash2 && hash1 != -1 && hash2 != -1) {
                goto return_ne;
            }
        }
#endif
        kind = __Pyx_PyUnicode_KIND(s1);
        if (kind != __Pyx_PyUnicode_KIND(s2)) {
            goto return_ne;
        }
        data1 = __Pyx_PyUnicode_DATA(s1);
        data2 = __Pyx_PyUnicode_DATA(s2);
        if (__Pyx_PyUnicode_READ(kind, data1, 0) != __Pyx_PyUnicode_READ(kind, data2, 0)) {
            goto return_ne;
        } else if (length == 1) {
            goto return_eq;
        } else {
            int result = memcmp(data1, data2, (size_t)(length * kind));
            #if PY_MAJOR_VERSION < 3
            Py_XDECREF(owned_ref);
            #endif
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & s2_is_unicode) {
        goto return_ne;
    } else if ((s2 == Py_None) & s1_is_unicode) {
        goto return_ne;
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        #if PY_MAJOR_VERSION < 3
        Py_XDECREF(owned_ref);
        #endif
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
return_eq:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(owned_ref);
    #endif
    return (equals == Py_EQ);
return_ne:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(owned_ref);
    #endif
    return (equals == Py_NE);
#endif
}

/* unicode_tailmatch */
static int __Pyx_PyUnicode_TailmatchTuple(PyObject* s, PyObject* substrings,
                                          Py_ssize_t start, Py_ssize_t end, int direction) {
    Py_ssize_t i, count = PyTuple_GET_SIZE(substrings);
    for (i = 0; i < count; i++) {
        Py_ssize_t result;
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        result = PyUnicode_Tailmatch(s, PyTuple_GET_ITEM(substrings, i),
                                     start, end, direction);
#else
        PyObject* sub = PySequence_ITEM(substrings, i);
        if (unlikely(!sub)) return -1;
        result = PyUnicode_Tailmatch(s, sub, start, end, direction);
        Py_DECREF(sub);
#endif
        if (result) {
            return (int) result;
        }
    }
    return 0;
}
static int __Pyx_PyUnicode_Tailmatch(PyObject* s, PyObject* substr,
                                     Py_ssize_t start, Py_ssize_t end, int direction) {
    if (unlikely(PyTuple_Check(substr))) {
        return __Pyx_PyUnicode_TailmatchTuple(s, substr, start, end, direction);
    }
    return (int) PyUnicode_Tailmatch(s, substr, start, end, direction);
}

/* Import */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level) {
    PyObject *empty_list = 0;
    PyObject *module = 0;
    PyObject *global_dict = 0;
    PyObject *empty_dict = 0;
    PyObject *list;
    #if PY_MAJOR_VERSION < 3
    PyObject *py_import;
    py_import = __Pyx_PyObject_GetAttrStr(__pyx_b, __pyx_n_s_import);
    if (!py_import)
        goto bad;
    #endif
    if (from_list)
        list = from_list;
    else {
        empty_list = PyList_New(0);
        if (!empty_list)
            goto bad;
        list = empty_list;
    }
    global_dict = PyModule_GetDict(__pyx_m);
    if (!global_dict)
        goto bad;
    empty_dict = PyDict_New();
    if (!empty_dict)
        goto bad;
    {
        #if PY_MAJOR_VERSION >= 3
        if (level == -1) {
            if (strchr(__Pyx_MODULE_NAME, '.')) {
                module = PyImport_ImportModuleLevelObject(
                    name, global_dict, empty_dict, list, 1);
                if (!module) {
                    if (!PyErr_ExceptionMatches(PyExc_ImportError))
                        goto bad;
                    PyErr_Clear();
                }
            }
            level = 0;
        }
        #endif
        if (!module) {
            #if PY_MAJOR_VERSION < 3
            PyObject *py_level = PyInt_FromLong(level);
            if (!py_level)
                goto bad;
            module = PyObject_CallFunctionObjArgs(py_import,
                name, global_dict, empty_dict, list, py_level, (PyObject *)NULL);
            Py_DECREF(py_level);
            #else
            module = PyImport_ImportModuleLevelObject(
                name, global_dict, empty_dict, list, level);
            #endif
        }
    }
bad:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(py_import);
    #endif
    Py_XDECREF(empty_list);
    Py_XDECREF(empty_dict);
    return module;
}

/* ImportFrom */
static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name) {
    PyObject* value = __Pyx_PyObject_GetAttrStr(module, name);
    if (unlikely(!value) && PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Format(PyExc_ImportError,
        #if PY_MAJOR_VERSION < 3
            "cannot import name %.230s", PyString_AS_STRING(name));
        #else
            "cannot import name %S", name);
        #endif
    }
    return value;
}

/* RaiseException */
#if PY_MAJOR_VERSION < 3
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb,
                        CYTHON_UNUSED PyObject *cause) {
    __Pyx_PyThreadState_declare
    Py_XINCREF(type);
    if (!value || value == Py_None)
        value = NULL;
    else
        Py_INCREF(value);
    if (!tb || tb == Py_None)
        tb = NULL;
    else {
        Py_INCREF(tb);
        if (!PyTraceBack_Check(tb)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: arg 3 must be a traceback or None");
            goto raise_error;
        }
    }
    if (PyType_Check(type)) {
#if CYTHON_COMPILING_IN_PYPY
        if (!value) {
            Py_INCREF(Py_None);
            value = Py_None;
        }
#endif
        PyErr_NormalizeException(&type, &value, &tb);
    } else {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto raise_error;
        }
        value = type;
        type = (PyObject*) Py_TYPE(type);
        Py_INCREF(type);
        if (!PyType_IsSubtype((PyTypeObject *)type, (PyTypeObject *)PyExc_BaseException)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: exception class must be a subclass of BaseException");
            goto raise_error;
        }
    }
    __Pyx_PyThreadState_assign
    __Pyx_ErrRestore(type, value, tb);
    return;
raise_error:
    Py_XDECREF(value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
    return;
}
#else
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause) {
    PyObject* owned_instance = NULL;
    if (tb == Py_None) {
        tb = 0;
    } else if (tb && !PyTraceBack_Check(tb)) {
        PyErr_SetString(PyExc_TypeError,
            "raise: arg 3 must be a traceback or None");
        goto bad;
    }
    if (value == Py_None)
        value = 0;
    if (PyExceptionInstance_Check(type)) {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto bad;
        }
        value = type;
        type = (PyObject*) Py_TYPE(value);
    } else if (PyExceptionClass_Check(type)) {
        PyObject *instance_class = NULL;
        if (value && PyExceptionInstance_Check(value)) {
            instance_class = (PyObject*) Py_TYPE(value);
            if (instance_class != type) {
                int is_subclass = PyObject_IsSubclass(instance_class, type);
                if (!is_subclass) {
                    instance_class = NULL;
                } else if (unlikely(is_subclass == -1)) {
                    goto bad;
                } else {
                    type = instance_class;
                }
            }
        }
        if (!instance_class) {
            PyObject *args;
            if (!value)
                args = PyTuple_New(0);
            else if (PyTuple_Check(value)) {
                Py_INCREF(value);
                args = value;
            } else
                args = PyTuple_Pack(1, value);
            if (!args)
                goto bad;
            owned_instance = PyObject_Call(type, args, NULL);
            Py_DECREF(args);
            if (!owned_instance)
                goto bad;
            value = owned_instance;
            if (!PyExceptionInstance_Check(value)) {
                PyErr_Format(PyExc_TypeError,
                             "calling %R should have returned an instance of "
                             "BaseException, not %R",
                             type, Py_TYPE(value));
                goto bad;
            }
        }
    } else {
        PyErr_SetString(PyExc_TypeError,
            "raise: exception class must be a subclass of BaseException");
        goto bad;
    }
    if (cause) {
        PyObject *fixed_cause;
        if (cause == Py_None) {
            fixed_cause = NULL;
        } else if (PyExceptionClass_Check(cause)) {
            fixed_cause = PyObject_CallObject(cause, NULL);
            if (fixed_cause == NULL)
                goto bad;
        } else if (PyExceptionInstance_Check(cause)) {
            fixed_cause = cause;
            Py_INCREF(fixed_cause);
        } else {
            PyErr_SetString(PyExc_TypeError,
                            "exception causes must derive from "
                            "BaseException");
            goto bad;
        }
        PyException_SetCause(value, fixed_cause);
    }
    PyErr_SetObject(type, value);
    if (tb) {
#if CYTHON_COMPILING_IN_PYPY
        PyObject *tmp_type, *tmp_value, *tmp_tb;
        PyErr_Fetch(&tmp_type, &tmp_value, &tmp_tb);
        Py_INCREF(tb);
        PyErr_Restore(tmp_type, tmp_value, tb);
        Py_XDECREF(tmp_tb);
#else
        PyThreadState *tstate = __Pyx_PyThreadState_Current;
        PyObject* tmp_tb = tstate->curexc_traceback;
        if (tb != tmp_tb) {
            Py_INCREF(tb);
            tstate->curexc_traceback = tb;
            Py_XDECREF(tmp_tb);
        }
#endif
    }
bad:
    Py_XDECREF(owned_instance);
    return;
}
#endif

/* HasAttr */
static CYTHON_INLINE int __Pyx_HasAttr(PyObject *o, PyObject *n) {
    PyObject *r;
    if (unlikely(!__Pyx_PyBaseString_Check(n))) {
        PyErr_SetString(PyExc_TypeError,
                        "hasattr(): attribute name must be string");
        return -1;
    }
    r = __Pyx_GetAttr(o, n);
    if (unlikely(!r)) {
        PyErr_Clear();
        return 0;
    } else {
        Py_DECREF(r);
        return 1;
    }
}

/* CallNextTpDealloc */
static void __Pyx_call_next_tp_dealloc(PyObject* obj, destructor current_tp_dealloc) {
    PyTypeObject* type = Py_TYPE(obj);
    while (type && type->tp_dealloc != current_tp_dealloc)
        type = type->tp_base;
    while (type && type->tp_dealloc == current_tp_dealloc)
        type = type->tp_base;
    if (type)
        type->tp_dealloc(obj);
}

/* CallNextTpTraverse */
static int __Pyx_call_next_tp_traverse(PyObject* obj, visitproc v, void *a, traverseproc current_tp_traverse) {
    PyTypeObject* type = Py_TYPE(obj);
    while (type && type->tp_traverse != current_tp_traverse)
        type = type->tp_base;
    while (type && type->tp_traverse == current_tp_traverse)
        type = type->tp_base;
    if (type && type->tp_traverse)
        return type->tp_traverse(obj, v, a);
    return 0;
}

/* CallNextTpClear */
static void __Pyx_call_next_tp_clear(PyObject* obj, inquiry current_tp_clear) {
    PyTypeObject* type = Py_TYPE(obj);
    while (type && type->tp_clear != current_tp_clear)
        type = type->tp_base;
    while (type && type->tp_clear == current_tp_clear)
        type = type->tp_base;
    if (type && type->tp_clear)
        type->tp_clear(obj);
}

/* TypeImport */
#ifndef __PYX_HAVE_RT_ImportType
#define __PYX_HAVE_RT_ImportType
static PyTypeObject *__Pyx_ImportType(PyObject *module, const char *module_name, const char *class_name,
    size_t size, enum __Pyx_ImportType_CheckSize check_size)
{
    PyObject *result = 0;
    char warning[200];
    Py_ssize_t basicsize;
#ifdef Py_LIMITED_API
    PyObject *py_basicsize;
#endif
    result = PyObject_GetAttrString(module, class_name);
    if (!result)
        goto bad;
    if (!PyType_Check(result)) {
        PyErr_Format(PyExc_TypeError,
            "%.200s.%.200s is not a type object",
            module_name, class_name);
        goto bad;
    }
#ifndef Py_LIMITED_API
    basicsize = ((PyTypeObject *)result)->tp_basicsize;
#else
    py_basicsize = PyObject_GetAttrString(result, "__basicsize__");
    if (!py_basicsize)
        goto bad;
    basicsize = PyLong_AsSsize_t(py_basicsize);
    Py_DECREF(py_basicsize);
    py_basicsize = 0;
    if (basicsize == (Py_ssize_t)-1 && PyErr_Occurred())
        goto bad;
#endif
    if ((size_t)basicsize < size) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize);
        goto bad;
    }
    if (check_size == __Pyx_ImportType_CheckSize_Error && (size_t)basicsize != size) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize);
        goto bad;
    }
    else if (check_size == __Pyx_ImportType_CheckSize_Warn && (size_t)basicsize > size) {
        PyOS_snprintf(warning, sizeof(warning),
            "%s.%s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize);
        if (PyErr_WarnEx(NULL, warning, 0) < 0) goto bad;
    }
    return (PyTypeObject *)result;
bad:
    Py_XDECREF(result);
    return NULL;
}
#endif

/* GetVTable */
static void* __Pyx_GetVtable(PyObject *dict) {
    void* ptr;
    PyObject *ob = PyObject_GetItem(dict, __pyx_n_s_pyx_vtable);
    if (!ob)
        goto bad;
#if PY_VERSION_HEX >= 0x02070000
    ptr = PyCapsule_GetPointer(ob, 0);
#else
    ptr = PyCObject_AsVoidPtr(ob);
#endif
    if (!ptr && !PyErr_Occurred())
        PyErr_SetString(PyExc_RuntimeError, "invalid vtable found for imported type");
    Py_DECREF(ob);
    return ptr;
bad:
    Py_XDECREF(ob);
    return NULL;
}

/* PyObject_GenericGetAttrNoDict */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject *__Pyx_RaiseGenericGetAttributeError(PyTypeObject *tp, PyObject *attr_name) {
    PyErr_Format(PyExc_AttributeError,
#if PY_MAJOR_VERSION >= 3
                 "'%.50s' object has no attribute '%U'",
                 tp->tp_name, attr_name);
#else
                 "'%.50s' object has no attribute '%.400s'",
                 tp->tp_name, PyString_AS_STRING(attr_name));
#endif
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_GenericGetAttrNoDict(PyObject* obj, PyObject* attr_name) {
    PyObject *descr;
    PyTypeObject *tp = Py_TYPE(obj);
    if (unlikely(!PyString_Check(attr_name))) {
        return PyObject_GenericGetAttr(obj, attr_name);
    }
    assert(!tp->tp_dictoffset);
    descr = _PyType_Lookup(tp, attr_name);
    if (unlikely(!descr)) {
        return __Pyx_RaiseGenericGetAttributeError(tp, attr_name);
    }
    Py_INCREF(descr);
    #if PY_MAJOR_VERSION < 3
    if (likely(PyType_HasFeature(Py_TYPE(descr), Py_TPFLAGS_HAVE_CLASS)))
    #endif
    {
        descrgetfunc f = Py_TYPE(descr)->tp_descr_get;
        if (unlikely(f)) {
            PyObject *res = f(descr, obj, (PyObject *)tp);
            Py_DECREF(descr);
            return res;
        }
    }
    return descr;
}
#endif

/* PyObject_GenericGetAttr */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject* __Pyx_PyObject_GenericGetAttr(PyObject* obj, PyObject* attr_name) {
    if (unlikely(Py_TYPE(obj)->tp_dictoffset)) {
        return PyObject_GenericGetAttr(obj, attr_name);
    }
    return __Pyx_PyObject_GenericGetAttrNoDict(obj, attr_name);
}
#endif

/* SetVTable */
static int __Pyx_SetVtable(PyObject *dict, void *vtable) {
#if PY_VERSION_HEX >= 0x02070000
    PyObject *ob = PyCapsule_New(vtable, 0, 0);
#else
    PyObject *ob = PyCObject_FromVoidPtr(vtable, 0);
#endif
    if (!ob)
        goto bad;
    if (PyDict_SetItem(dict, __pyx_n_s_pyx_vtable, ob) < 0)
        goto bad;
    Py_DECREF(ob);
    return 0;
bad:
    Py_XDECREF(ob);
    return -1;
}

/* SetupReduce */
static int __Pyx_setup_reduce_is_named(PyObject* meth, PyObject* name) {
  int ret;
  PyObject *name_attr;
  name_attr = __Pyx_PyObject_GetAttrStr(meth, __pyx_n_s_name);
  if (likely(name_attr)) {
      ret = PyObject_RichCompareBool(name_attr, name, Py_EQ);
  } else {
      ret = -1;
  }
  if (unlikely(ret < 0)) {
      PyErr_Clear();
      ret = 0;
  }
  Py_XDECREF(name_attr);
  return ret;
}
static int __Pyx_setup_reduce(PyObject* type_obj) {
    int ret = 0;
    PyObject *object_reduce = NULL;
    PyObject *object_reduce_ex = NULL;
    PyObject *reduce = NULL;
    PyObject *reduce_ex = NULL;
    PyObject *reduce_cython = NULL;
    PyObject *setstate = NULL;
    PyObject *setstate_cython = NULL;
#if CYTHON_USE_PYTYPE_LOOKUP
    if (_PyType_Lookup((PyTypeObject*)type_obj, __pyx_n_s_getstate)) goto GOOD;
#else
    if (PyObject_HasAttr(type_obj, __pyx_n_s_getstate)) goto GOOD;
#endif
#if CYTHON_USE_PYTYPE_LOOKUP
    object_reduce_ex = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_reduce_ex); if (!object_reduce_ex) goto BAD;
#else
    object_reduce_ex = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_n_s_reduce_ex); if (!object_reduce_ex) goto BAD;
#endif
    reduce_ex = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce_ex); if (unlikely(!reduce_ex)) goto BAD;
    if (reduce_ex == object_reduce_ex) {
#if CYTHON_USE_PYTYPE_LOOKUP
        object_reduce = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_reduce); if (!object_reduce) goto BAD;
#else
        object_reduce = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_n_s_reduce); if (!object_reduce) goto BAD;
#endif
        reduce = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce); if (unlikely(!reduce)) goto BAD;
        if (reduce == object_reduce || __Pyx_setup_reduce_is_named(reduce, __pyx_n_s_reduce_cython)) {
            reduce_cython = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce_cython); if (unlikely(!reduce_cython)) goto BAD;
            ret = PyDict_SetItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_reduce, reduce_cython); if (unlikely(ret < 0)) goto BAD;
            ret = PyDict_DelItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_reduce_cython); if (unlikely(ret < 0)) goto BAD;
            setstate = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_setstate);
            if (!setstate) PyErr_Clear();
            if (!setstate || __Pyx_setup_reduce_is_named(setstate, __pyx_n_s_setstate_cython)) {
                setstate_cython = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_setstate_cython); if (unlikely(!setstate_cython)) goto BAD;
                ret = PyDict_SetItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_setstate, setstate_cython); if (unlikely(ret < 0)) goto BAD;
                ret = PyDict_DelItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_setstate_cython); if (unlikely(ret < 0)) goto BAD;
            }
            PyType_Modified((PyTypeObject*)type_obj);
        }
    }
    goto GOOD;
BAD:
    if (!PyErr_Occurred())
        PyErr_Format(PyExc_RuntimeError, "Unable to initialize pickling for %s", ((PyTypeObject*)type_obj)->tp_name);
    ret = -1;
GOOD:
#if !CYTHON_USE_PYTYPE_LOOKUP
    Py_XDECREF(object_reduce);
    Py_XDECREF(object_reduce_ex);
#endif
    Py_XDECREF(reduce);
    Py_XDECREF(reduce_ex);
    Py_XDECREF(reduce_cython);
    Py_XDECREF(setstate);
    Py_XDECREF(setstate_cython);
    return ret;
}

/* CLineInTraceback */
#ifndef CYTHON_CLINE_IN_TRACEBACK
static int __Pyx_CLineForTraceback(PyThreadState *tstate, int c_line) {
    PyObject *use_cline;
    PyObject *ptype, *pvalue, *ptraceback;
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject **cython_runtime_dict;
#endif
    if (unlikely(!__pyx_cython_runtime)) {
        return c_line;
    }
    __Pyx_ErrFetchInState(tstate, &ptype, &pvalue, &ptraceback);
#if CYTHON_COMPILING_IN_CPYTHON
    cython_runtime_dict = _PyObject_GetDictPtr(__pyx_cython_runtime);
    if (likely(cython_runtime_dict)) {
        __PYX_PY_DICT_LOOKUP_IF_MODIFIED(
            use_cline, *cython_runtime_dict,
            __Pyx_PyDict_GetItemStr(*cython_runtime_dict, __pyx_n_s_cline_in_traceback))
    } else
#endif
    {
      PyObject *use_cline_obj = __Pyx_PyObject_GetAttrStr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback);
      if (use_cline_obj) {
        use_cline = PyObject_Not(use_cline_obj) ? Py_False : Py_True;
        Py_DECREF(use_cline_obj);
      } else {
        PyErr_Clear();
        use_cline = NULL;
      }
    }
    if (!use_cline) {
        c_line = 0;
        PyObject_SetAttr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback, Py_False);
    }
    else if (use_cline == Py_False || (use_cline != Py_True && PyObject_Not(use_cline) != 0)) {
        c_line = 0;
    }
    __Pyx_ErrRestoreInState(tstate, ptype, pvalue, ptraceback);
    return c_line;
}
#endif

/* CodeObjectCache */
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line) {
    int start = 0, mid = 0, end = count - 1;
    if (end >= 0 && code_line > entries[end].code_line) {
        return count;
    }
    while (start < end) {
        mid = start + (end - start) / 2;
        if (code_line < entries[mid].code_line) {
            end = mid;
        } else if (code_line > entries[mid].code_line) {
             start = mid + 1;
        } else {
            return mid;
        }
    }
    if (code_line <= entries[mid].code_line) {
        return mid;
    } else {
        return mid + 1;
    }
}
static PyCodeObject *__pyx_find_code_object(int code_line) {
    PyCodeObject* code_object;
    int pos;
    if (unlikely(!code_line) || unlikely(!__pyx_code_cache.entries)) {
        return NULL;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if (unlikely(pos >= __pyx_code_cache.count) || unlikely(__pyx_code_cache.entries[pos].code_line != code_line)) {
        return NULL;
    }
    code_object = __pyx_code_cache.entries[pos].code_object;
    Py_INCREF(code_object);
    return code_object;
}
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object) {
    int pos, i;
    __Pyx_CodeObjectCacheEntry* entries = __pyx_code_cache.entries;
    if (unlikely(!code_line)) {
        return;
    }
    if (unlikely(!entries)) {
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Malloc(64*sizeof(__Pyx_CodeObjectCacheEntry));
        if (likely(entries)) {
            __pyx_code_cache.entries = entries;
            __pyx_code_cache.max_count = 64;
            __pyx_code_cache.count = 1;
            entries[0].code_line = code_line;
            entries[0].code_object = code_object;
            Py_INCREF(code_object);
        }
        return;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if ((pos < __pyx_code_cache.count) && unlikely(__pyx_code_cache.entries[pos].code_line == code_line)) {
        PyCodeObject* tmp = entries[pos].code_object;
        entries[pos].code_object = code_object;
        Py_DECREF(tmp);
        return;
    }
    if (__pyx_code_cache.count == __pyx_code_cache.max_count) {
        int new_max = __pyx_code_cache.max_count + 64;
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Realloc(
            __pyx_code_cache.entries, (size_t)new_max*sizeof(__Pyx_CodeObjectCacheEntry));
        if (unlikely(!entries)) {
            return;
        }
        __pyx_code_cache.entries = entries;
        __pyx_code_cache.max_count = new_max;
    }
    for (i=__pyx_code_cache.count; i>pos; i--) {
        entries[i] = entries[i-1];
    }
    entries[pos].code_line = code_line;
    entries[pos].code_object = code_object;
    __pyx_code_cache.count++;
    Py_INCREF(code_object);
}

/* AddTraceback */
#include "compile.h"
#include "frameobject.h"
#include "traceback.h"
static PyCodeObject* __Pyx_CreateCodeObjectForTraceback(
            const char *funcname, int c_line,
            int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyObject *py_srcfile = 0;
    PyObject *py_funcname = 0;
    #if PY_MAJOR_VERSION < 3
    py_srcfile = PyString_FromString(filename);
    #else
    py_srcfile = PyUnicode_FromString(filename);
    #endif
    if (!py_srcfile) goto bad;
    if (c_line) {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #else
        py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #endif
    }
    else {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromString(funcname);
        #else
        py_funcname = PyUnicode_FromString(funcname);
        #endif
    }
    if (!py_funcname) goto bad;
    py_code = __Pyx_PyCode_New(
        0,
        0,
        0,
        0,
        0,
        __pyx_empty_bytes, /*PyObject *code,*/
        __pyx_empty_tuple, /*PyObject *consts,*/
        __pyx_empty_tuple, /*PyObject *names,*/
        __pyx_empty_tuple, /*PyObject *varnames,*/
        __pyx_empty_tuple, /*PyObject *freevars,*/
        __pyx_empty_tuple, /*PyObject *cellvars,*/
        py_srcfile,   /*PyObject *filename,*/
        py_funcname,  /*PyObject *name,*/
        py_line,
        __pyx_empty_bytes  /*PyObject *lnotab*/
    );
    Py_DECREF(py_srcfile);
    Py_DECREF(py_funcname);
    return py_code;
bad:
    Py_XDECREF(py_srcfile);
    Py_XDECREF(py_funcname);
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyFrameObject *py_frame = 0;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    if (c_line) {
        c_line = __Pyx_CLineForTraceback(tstate, c_line);
    }
    py_code = __pyx_find_code_object(c_line ? -c_line : py_line);
    if (!py_code) {
        py_code = __Pyx_CreateCodeObjectForTraceback(
            funcname, c_line, py_line, filename);
        if (!py_code) goto bad;
        __pyx_insert_code_object(c_line ? -c_line : py_line, py_code);
    }
    py_frame = PyFrame_New(
        tstate,            /*PyThreadState *tstate,*/
        py_code,           /*PyCodeObject *code,*/
        __pyx_d,    /*PyObject *globals,*/
        0                  /*PyObject *locals*/
    );
    if (!py_frame) goto bad;
    __Pyx_PyFrame_SetLineNumber(py_frame, py_line);
    PyTraceBack_Here(py_frame);
bad:
    Py_XDECREF(py_code);
    Py_XDECREF(py_frame);
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value) {
    const int neg_one = (int) ((int) 0 - (int) 1), const_zero = (int) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(int) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(int) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(int) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(int),
                                     little, !is_unsigned);
    }
}

/* CIntFromPyVerify */
#define __PYX_VERIFY_RETURN_INT(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 0)
#define __PYX_VERIFY_RETURN_INT_EXC(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 1)
#define __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, exc)\
    {\
        func_type value = func_value;\
        if (sizeof(target_type) < sizeof(func_type)) {\
            if (unlikely(value != (func_type) (target_type) value)) {\
                func_type zero = 0;\
                if (exc && unlikely(value == (func_type)-1 && PyErr_Occurred()))\
                    return (target_type) -1;\
                if (is_unsigned && unlikely(value < zero))\
                    goto raise_neg_overflow;\
                else\
                    goto raise_overflow;\
            }\
        }\
        return (target_type) value;\
    }

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_unsigned_char(unsigned char value) {
    const unsigned char neg_one = (unsigned char) ((unsigned char) 0 - (unsigned char) 1), const_zero = (unsigned char) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(unsigned char) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(unsigned char) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(unsigned char) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(unsigned char) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(unsigned char) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(unsigned char),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_unsigned_short(unsigned short value) {
    const unsigned short neg_one = (unsigned short) ((unsigned short) 0 - (unsigned short) 1), const_zero = (unsigned short) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(unsigned short) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(unsigned short) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(unsigned short) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(unsigned short) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(unsigned short) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(unsigned short),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_enum____pyx_t_10traceutils_7scamper_3hop_ICMPType(enum __pyx_t_10traceutils_7scamper_3hop_ICMPType value) {
    const enum __pyx_t_10traceutils_7scamper_3hop_ICMPType neg_one = (enum __pyx_t_10traceutils_7scamper_3hop_ICMPType) ((enum __pyx_t_10traceutils_7scamper_3hop_ICMPType) 0 - (enum __pyx_t_10traceutils_7scamper_3hop_ICMPType) 1), const_zero = (enum __pyx_t_10traceutils_7scamper_3hop_ICMPType) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(enum __pyx_t_10traceutils_7scamper_3hop_ICMPType) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(enum __pyx_t_10traceutils_7scamper_3hop_ICMPType) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(enum __pyx_t_10traceutils_7scamper_3hop_ICMPType) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(enum __pyx_t_10traceutils_7scamper_3hop_ICMPType) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(enum __pyx_t_10traceutils_7scamper_3hop_ICMPType) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(enum __pyx_t_10traceutils_7scamper_3hop_ICMPType),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value) {
    const long neg_one = (long) ((long) 0 - (long) 1), const_zero = (long) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(long) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(long) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(long),
                                     little, !is_unsigned);
    }
}

/* CIntFromPy */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *x) {
    const int neg_one = (int) ((int) 0 - (int) 1), const_zero = (int) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(int) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(int, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (int) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case  1: __PYX_VERIFY_RETURN_INT(int, digit, digits[0])
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 2 * PyLong_SHIFT) {
                            return (int) (((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 3 * PyLong_SHIFT) {
                            return (int) (((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 4 * PyLong_SHIFT) {
                            return (int) (((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (int) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(int) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case -1: __PYX_VERIFY_RETURN_INT(int, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(int,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(int) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) ((((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) ((((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) ((((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(int) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            int val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (int) -1;
        }
    } else {
        int val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (int) -1;
        val = __Pyx_PyInt_As_int(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int");
    return (int) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int");
    return (int) -1;
}

/* CIntFromPy */
static CYTHON_INLINE unsigned char __Pyx_PyInt_As_unsigned_char(PyObject *x) {
    const unsigned char neg_one = (unsigned char) ((unsigned char) 0 - (unsigned char) 1), const_zero = (unsigned char) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(unsigned char) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(unsigned char, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (unsigned char) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (unsigned char) 0;
                case  1: __PYX_VERIFY_RETURN_INT(unsigned char, digit, digits[0])
                case 2:
                    if (8 * sizeof(unsigned char) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned char, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned char) >= 2 * PyLong_SHIFT) {
                            return (unsigned char) (((((unsigned char)digits[1]) << PyLong_SHIFT) | (unsigned char)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(unsigned char) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned char, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned char) >= 3 * PyLong_SHIFT) {
                            return (unsigned char) (((((((unsigned char)digits[2]) << PyLong_SHIFT) | (unsigned char)digits[1]) << PyLong_SHIFT) | (unsigned char)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(unsigned char) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned char, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned char) >= 4 * PyLong_SHIFT) {
                            return (unsigned char) (((((((((unsigned char)digits[3]) << PyLong_SHIFT) | (unsigned char)digits[2]) << PyLong_SHIFT) | (unsigned char)digits[1]) << PyLong_SHIFT) | (unsigned char)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (unsigned char) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(unsigned char) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(unsigned char, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(unsigned char) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(unsigned char, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (unsigned char) 0;
                case -1: __PYX_VERIFY_RETURN_INT(unsigned char, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(unsigned char,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(unsigned char) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned char, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned char) - 1 > 2 * PyLong_SHIFT) {
                            return (unsigned char) (((unsigned char)-1)*(((((unsigned char)digits[1]) << PyLong_SHIFT) | (unsigned char)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(unsigned char) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned char, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned char) - 1 > 2 * PyLong_SHIFT) {
                            return (unsigned char) ((((((unsigned char)digits[1]) << PyLong_SHIFT) | (unsigned char)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(unsigned char) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned char, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned char) - 1 > 3 * PyLong_SHIFT) {
                            return (unsigned char) (((unsigned char)-1)*(((((((unsigned char)digits[2]) << PyLong_SHIFT) | (unsigned char)digits[1]) << PyLong_SHIFT) | (unsigned char)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(unsigned char) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned char, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned char) - 1 > 3 * PyLong_SHIFT) {
                            return (unsigned char) ((((((((unsigned char)digits[2]) << PyLong_SHIFT) | (unsigned char)digits[1]) << PyLong_SHIFT) | (unsigned char)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(unsigned char) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned char, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned char) - 1 > 4 * PyLong_SHIFT) {
                            return (unsigned char) (((unsigned char)-1)*(((((((((unsigned char)digits[3]) << PyLong_SHIFT) | (unsigned char)digits[2]) << PyLong_SHIFT) | (unsigned char)digits[1]) << PyLong_SHIFT) | (unsigned char)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(unsigned char) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned char, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned char) - 1 > 4 * PyLong_SHIFT) {
                            return (unsigned char) ((((((((((unsigned char)digits[3]) << PyLong_SHIFT) | (unsigned char)digits[2]) << PyLong_SHIFT) | (unsigned char)digits[1]) << PyLong_SHIFT) | (unsigned char)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(unsigned char) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(unsigned char, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(unsigned char) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(unsigned char, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            unsigned char val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (unsigned char) -1;
        }
    } else {
        unsigned char val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (unsigned char) -1;
        val = __Pyx_PyInt_As_unsigned_char(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to unsigned char");
    return (unsigned char) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to unsigned char");
    return (unsigned char) -1;
}

/* CIntFromPy */
static CYTHON_INLINE unsigned short __Pyx_PyInt_As_unsigned_short(PyObject *x) {
    const unsigned short neg_one = (unsigned short) ((unsigned short) 0 - (unsigned short) 1), const_zero = (unsigned short) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(unsigned short) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(unsigned short, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (unsigned short) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (unsigned short) 0;
                case  1: __PYX_VERIFY_RETURN_INT(unsigned short, digit, digits[0])
                case 2:
                    if (8 * sizeof(unsigned short) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned short, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned short) >= 2 * PyLong_SHIFT) {
                            return (unsigned short) (((((unsigned short)digits[1]) << PyLong_SHIFT) | (unsigned short)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(unsigned short) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned short, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned short) >= 3 * PyLong_SHIFT) {
                            return (unsigned short) (((((((unsigned short)digits[2]) << PyLong_SHIFT) | (unsigned short)digits[1]) << PyLong_SHIFT) | (unsigned short)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(unsigned short) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned short, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned short) >= 4 * PyLong_SHIFT) {
                            return (unsigned short) (((((((((unsigned short)digits[3]) << PyLong_SHIFT) | (unsigned short)digits[2]) << PyLong_SHIFT) | (unsigned short)digits[1]) << PyLong_SHIFT) | (unsigned short)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (unsigned short) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(unsigned short) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(unsigned short, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(unsigned short) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(unsigned short, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (unsigned short) 0;
                case -1: __PYX_VERIFY_RETURN_INT(unsigned short, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(unsigned short,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(unsigned short) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned short, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned short) - 1 > 2 * PyLong_SHIFT) {
                            return (unsigned short) (((unsigned short)-1)*(((((unsigned short)digits[1]) << PyLong_SHIFT) | (unsigned short)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(unsigned short) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned short, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned short) - 1 > 2 * PyLong_SHIFT) {
                            return (unsigned short) ((((((unsigned short)digits[1]) << PyLong_SHIFT) | (unsigned short)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(unsigned short) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned short, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned short) - 1 > 3 * PyLong_SHIFT) {
                            return (unsigned short) (((unsigned short)-1)*(((((((unsigned short)digits[2]) << PyLong_SHIFT) | (unsigned short)digits[1]) << PyLong_SHIFT) | (unsigned short)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(unsigned short) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned short, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned short) - 1 > 3 * PyLong_SHIFT) {
                            return (unsigned short) ((((((((unsigned short)digits[2]) << PyLong_SHIFT) | (unsigned short)digits[1]) << PyLong_SHIFT) | (unsigned short)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(unsigned short) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned short, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned short) - 1 > 4 * PyLong_SHIFT) {
                            return (unsigned short) (((unsigned short)-1)*(((((((((unsigned short)digits[3]) << PyLong_SHIFT) | (unsigned short)digits[2]) << PyLong_SHIFT) | (unsigned short)digits[1]) << PyLong_SHIFT) | (unsigned short)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(unsigned short) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned short, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned short) - 1 > 4 * PyLong_SHIFT) {
                            return (unsigned short) ((((((((((unsigned short)digits[3]) << PyLong_SHIFT) | (unsigned short)digits[2]) << PyLong_SHIFT) | (unsigned short)digits[1]) << PyLong_SHIFT) | (unsigned short)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(unsigned short) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(unsigned short, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(unsigned short) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(unsigned short, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            unsigned short val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (unsigned short) -1;
        }
    } else {
        unsigned short val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (unsigned short) -1;
        val = __Pyx_PyInt_As_unsigned_short(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to unsigned short");
    return (unsigned short) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to unsigned short");
    return (unsigned short) -1;
}

/* CIntFromPy */
static CYTHON_INLINE enum __pyx_t_10traceutils_7scamper_3hop_ICMPType __Pyx_PyInt_As_enum____pyx_t_10traceutils_7scamper_3hop_ICMPType(PyObject *x) {
    const enum __pyx_t_10traceutils_7scamper_3hop_ICMPType neg_one = (enum __pyx_t_10traceutils_7scamper_3hop_ICMPType) ((enum __pyx_t_10traceutils_7scamper_3hop_ICMPType) 0 - (enum __pyx_t_10traceutils_7scamper_3hop_ICMPType) 1), const_zero = (enum __pyx_t_10traceutils_7scamper_3hop_ICMPType) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(enum __pyx_t_10traceutils_7scamper_3hop_ICMPType) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(enum __pyx_t_10traceutils_7scamper_3hop_ICMPType, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (enum __pyx_t_10traceutils_7scamper_3hop_ICMPType) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (enum __pyx_t_10traceutils_7scamper_3hop_ICMPType) 0;
                case  1: __PYX_VERIFY_RETURN_INT(enum __pyx_t_10traceutils_7scamper_3hop_ICMPType, digit, digits[0])
                case 2:
                    if (8 * sizeof(enum __pyx_t_10traceutils_7scamper_3hop_ICMPType) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(enum __pyx_t_10traceutils_7scamper_3hop_ICMPType, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(enum __pyx_t_10traceutils_7scamper_3hop_ICMPType) >= 2 * PyLong_SHIFT) {
                            return (enum __pyx_t_10traceutils_7scamper_3hop_ICMPType) (((((enum __pyx_t_10traceutils_7scamper_3hop_ICMPType)digits[1]) << PyLong_SHIFT) | (enum __pyx_t_10traceutils_7scamper_3hop_ICMPType)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(enum __pyx_t_10traceutils_7scamper_3hop_ICMPType) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(enum __pyx_t_10traceutils_7scamper_3hop_ICMPType, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(enum __pyx_t_10traceutils_7scamper_3hop_ICMPType) >= 3 * PyLong_SHIFT) {
                            return (enum __pyx_t_10traceutils_7scamper_3hop_ICMPType) (((((((enum __pyx_t_10traceutils_7scamper_3hop_ICMPType)digits[2]) << PyLong_SHIFT) | (enum __pyx_t_10traceutils_7scamper_3hop_ICMPType)digits[1]) << PyLong_SHIFT) | (enum __pyx_t_10traceutils_7scamper_3hop_ICMPType)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(enum __pyx_t_10traceutils_7scamper_3hop_ICMPType) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(enum __pyx_t_10traceutils_7scamper_3hop_ICMPType, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(enum __pyx_t_10traceutils_7scamper_3hop_ICMPType) >= 4 * PyLong_SHIFT) {
                            return (enum __pyx_t_10traceutils_7scamper_3hop_ICMPType) (((((((((enum __pyx_t_10traceutils_7scamper_3hop_ICMPType)digits[3]) << PyLong_SHIFT) | (enum __pyx_t_10traceutils_7scamper_3hop_ICMPType)digits[2]) << PyLong_SHIFT) | (enum __pyx_t_10traceutils_7scamper_3hop_ICMPType)digits[1]) << PyLong_SHIFT) | (enum __pyx_t_10traceutils_7scamper_3hop_ICMPType)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (enum __pyx_t_10traceutils_7scamper_3hop_ICMPType) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(enum __pyx_t_10traceutils_7scamper_3hop_ICMPType) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(enum __pyx_t_10traceutils_7scamper_3hop_ICMPType, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(enum __pyx_t_10traceutils_7scamper_3hop_ICMPType) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(enum __pyx_t_10traceutils_7scamper_3hop_ICMPType, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (enum __pyx_t_10traceutils_7scamper_3hop_ICMPType) 0;
                case -1: __PYX_VERIFY_RETURN_INT(enum __pyx_t_10traceutils_7scamper_3hop_ICMPType, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(enum __pyx_t_10traceutils_7scamper_3hop_ICMPType,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(enum __pyx_t_10traceutils_7scamper_3hop_ICMPType) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(enum __pyx_t_10traceutils_7scamper_3hop_ICMPType, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(enum __pyx_t_10traceutils_7scamper_3hop_ICMPType) - 1 > 2 * PyLong_SHIFT) {
                            return (enum __pyx_t_10traceutils_7scamper_3hop_ICMPType) (((enum __pyx_t_10traceutils_7scamper_3hop_ICMPType)-1)*(((((enum __pyx_t_10traceutils_7scamper_3hop_ICMPType)digits[1]) << PyLong_SHIFT) | (enum __pyx_t_10traceutils_7scamper_3hop_ICMPType)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(enum __pyx_t_10traceutils_7scamper_3hop_ICMPType) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(enum __pyx_t_10traceutils_7scamper_3hop_ICMPType, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(enum __pyx_t_10traceutils_7scamper_3hop_ICMPType) - 1 > 2 * PyLong_SHIFT) {
                            return (enum __pyx_t_10traceutils_7scamper_3hop_ICMPType) ((((((enum __pyx_t_10traceutils_7scamper_3hop_ICMPType)digits[1]) << PyLong_SHIFT) | (enum __pyx_t_10traceutils_7scamper_3hop_ICMPType)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(enum __pyx_t_10traceutils_7scamper_3hop_ICMPType) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(enum __pyx_t_10traceutils_7scamper_3hop_ICMPType, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(enum __pyx_t_10traceutils_7scamper_3hop_ICMPType) - 1 > 3 * PyLong_SHIFT) {
                            return (enum __pyx_t_10traceutils_7scamper_3hop_ICMPType) (((enum __pyx_t_10traceutils_7scamper_3hop_ICMPType)-1)*(((((((enum __pyx_t_10traceutils_7scamper_3hop_ICMPType)digits[2]) << PyLong_SHIFT) | (enum __pyx_t_10traceutils_7scamper_3hop_ICMPType)digits[1]) << PyLong_SHIFT) | (enum __pyx_t_10traceutils_7scamper_3hop_ICMPType)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(enum __pyx_t_10traceutils_7scamper_3hop_ICMPType) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(enum __pyx_t_10traceutils_7scamper_3hop_ICMPType, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(enum __pyx_t_10traceutils_7scamper_3hop_ICMPType) - 1 > 3 * PyLong_SHIFT) {
                            return (enum __pyx_t_10traceutils_7scamper_3hop_ICMPType) ((((((((enum __pyx_t_10traceutils_7scamper_3hop_ICMPType)digits[2]) << PyLong_SHIFT) | (enum __pyx_t_10traceutils_7scamper_3hop_ICMPType)digits[1]) << PyLong_SHIFT) | (enum __pyx_t_10traceutils_7scamper_3hop_ICMPType)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(enum __pyx_t_10traceutils_7scamper_3hop_ICMPType) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(enum __pyx_t_10traceutils_7scamper_3hop_ICMPType, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(enum __pyx_t_10traceutils_7scamper_3hop_ICMPType) - 1 > 4 * PyLong_SHIFT) {
                            return (enum __pyx_t_10traceutils_7scamper_3hop_ICMPType) (((enum __pyx_t_10traceutils_7scamper_3hop_ICMPType)-1)*(((((((((enum __pyx_t_10traceutils_7scamper_3hop_ICMPType)digits[3]) << PyLong_SHIFT) | (enum __pyx_t_10traceutils_7scamper_3hop_ICMPType)digits[2]) << PyLong_SHIFT) | (enum __pyx_t_10traceutils_7scamper_3hop_ICMPType)digits[1]) << PyLong_SHIFT) | (enum __pyx_t_10traceutils_7scamper_3hop_ICMPType)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(enum __pyx_t_10traceutils_7scamper_3hop_ICMPType) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(enum __pyx_t_10traceutils_7scamper_3hop_ICMPType, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(enum __pyx_t_10traceutils_7scamper_3hop_ICMPType) - 1 > 4 * PyLong_SHIFT) {
                            return (enum __pyx_t_10traceutils_7scamper_3hop_ICMPType) ((((((((((enum __pyx_t_10traceutils_7scamper_3hop_ICMPType)digits[3]) << PyLong_SHIFT) | (enum __pyx_t_10traceutils_7scamper_3hop_ICMPType)digits[2]) << PyLong_SHIFT) | (enum __pyx_t_10traceutils_7scamper_3hop_ICMPType)digits[1]) << PyLong_SHIFT) | (enum __pyx_t_10traceutils_7scamper_3hop_ICMPType)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(enum __pyx_t_10traceutils_7scamper_3hop_ICMPType) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(enum __pyx_t_10traceutils_7scamper_3hop_ICMPType, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(enum __pyx_t_10traceutils_7scamper_3hop_ICMPType) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(enum __pyx_t_10traceutils_7scamper_3hop_ICMPType, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            enum __pyx_t_10traceutils_7scamper_3hop_ICMPType val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (enum __pyx_t_10traceutils_7scamper_3hop_ICMPType) -1;
        }
    } else {
        enum __pyx_t_10traceutils_7scamper_3hop_ICMPType val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (enum __pyx_t_10traceutils_7scamper_3hop_ICMPType) -1;
        val = __Pyx_PyInt_As_enum____pyx_t_10traceutils_7scamper_3hop_ICMPType(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to enum __pyx_t_10traceutils_7scamper_3hop_ICMPType");
    return (enum __pyx_t_10traceutils_7scamper_3hop_ICMPType) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to enum __pyx_t_10traceutils_7scamper_3hop_ICMPType");
    return (enum __pyx_t_10traceutils_7scamper_3hop_ICMPType) -1;
}

/* CIntFromPy */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *x) {
    const long neg_one = (long) ((long) 0 - (long) 1), const_zero = (long) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(long) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(long, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (long) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case  1: __PYX_VERIFY_RETURN_INT(long, digit, digits[0])
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 2 * PyLong_SHIFT) {
                            return (long) (((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 3 * PyLong_SHIFT) {
                            return (long) (((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 4 * PyLong_SHIFT) {
                            return (long) (((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (long) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(long) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case -1: __PYX_VERIFY_RETURN_INT(long, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(long,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(long) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) ((((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) ((((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) ((((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(long) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            long val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (long) -1;
        }
    } else {
        long val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (long) -1;
        val = __Pyx_PyInt_As_long(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to long");
    return (long) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to long");
    return (long) -1;
}

/* FastTypeChecks */
#if CYTHON_COMPILING_IN_CPYTHON
static int __Pyx_InBases(PyTypeObject *a, PyTypeObject *b) {
    while (a) {
        a = a->tp_base;
        if (a == b)
            return 1;
    }
    return b == &PyBaseObject_Type;
}
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b) {
    PyObject *mro;
    if (a == b) return 1;
    mro = a->tp_mro;
    if (likely(mro)) {
        Py_ssize_t i, n;
        n = PyTuple_GET_SIZE(mro);
        for (i = 0; i < n; i++) {
            if (PyTuple_GET_ITEM(mro, i) == (PyObject *)b)
                return 1;
        }
        return 0;
    }
    return __Pyx_InBases(a, b);
}
#if PY_MAJOR_VERSION == 2
static int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject* exc_type2) {
    PyObject *exception, *value, *tb;
    int res;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&exception, &value, &tb);
    res = exc_type1 ? PyObject_IsSubclass(err, exc_type1) : 0;
    if (unlikely(res == -1)) {
        PyErr_WriteUnraisable(err);
        res = 0;
    }
    if (!res) {
        res = PyObject_IsSubclass(err, exc_type2);
        if (unlikely(res == -1)) {
            PyErr_WriteUnraisable(err);
            res = 0;
        }
    }
    __Pyx_ErrRestore(exception, value, tb);
    return res;
}
#else
static CYTHON_INLINE int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject *exc_type2) {
    int res = exc_type1 ? __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type1) : 0;
    if (!res) {
        res = __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type2);
    }
    return res;
}
#endif
static int __Pyx_PyErr_GivenExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    assert(PyExceptionClass_Check(exc_type));
    n = PyTuple_GET_SIZE(tuple);
#if PY_MAJOR_VERSION >= 3
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
#endif
    for (i=0; i<n; i++) {
        PyObject *t = PyTuple_GET_ITEM(tuple, i);
        #if PY_MAJOR_VERSION < 3
        if (likely(exc_type == t)) return 1;
        #endif
        if (likely(PyExceptionClass_Check(t))) {
            if (__Pyx_inner_PyErr_GivenExceptionMatches2(exc_type, NULL, t)) return 1;
        } else {
        }
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject* exc_type) {
    if (likely(err == exc_type)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        if (likely(PyExceptionClass_Check(exc_type))) {
            return __Pyx_inner_PyErr_GivenExceptionMatches2(err, NULL, exc_type);
        } else if (likely(PyTuple_Check(exc_type))) {
            return __Pyx_PyErr_GivenExceptionMatchesTuple(err, exc_type);
        } else {
        }
    }
    return PyErr_GivenExceptionMatches(err, exc_type);
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *exc_type1, PyObject *exc_type2) {
    assert(PyExceptionClass_Check(exc_type1));
    assert(PyExceptionClass_Check(exc_type2));
    if (likely(err == exc_type1 || err == exc_type2)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        return __Pyx_inner_PyErr_GivenExceptionMatches2(err, exc_type1, exc_type2);
    }
    return (PyErr_GivenExceptionMatches(err, exc_type1) || PyErr_GivenExceptionMatches(err, exc_type2));
}
#endif

/* FetchCommonType */
static PyTypeObject* __Pyx_FetchCommonType(PyTypeObject* type) {
    PyObject* fake_module;
    PyTypeObject* cached_type = NULL;
    fake_module = PyImport_AddModule((char*) "_cython_" CYTHON_ABI);
    if (!fake_module) return NULL;
    Py_INCREF(fake_module);
    cached_type = (PyTypeObject*) PyObject_GetAttrString(fake_module, type->tp_name);
    if (cached_type) {
        if (!PyType_Check((PyObject*)cached_type)) {
            PyErr_Format(PyExc_TypeError,
                "Shared Cython type %.200s is not a type object",
                type->tp_name);
            goto bad;
        }
        if (cached_type->tp_basicsize != type->tp_basicsize) {
            PyErr_Format(PyExc_TypeError,
                "Shared Cython type %.200s has the wrong size, try recompiling",
                type->tp_name);
            goto bad;
        }
    } else {
        if (!PyErr_ExceptionMatches(PyExc_AttributeError)) goto bad;
        PyErr_Clear();
        if (PyType_Ready(type) < 0) goto bad;
        if (PyObject_SetAttrString(fake_module, type->tp_name, (PyObject*) type) < 0)
            goto bad;
        Py_INCREF(type);
        cached_type = type;
    }
done:
    Py_DECREF(fake_module);
    return cached_type;
bad:
    Py_XDECREF(cached_type);
    cached_type = NULL;
    goto done;
}

/* GetTopmostException */
#if CYTHON_USE_EXC_INFO_STACK
static _PyErr_StackItem *
__Pyx_PyErr_GetTopmostException(PyThreadState *tstate)
{
    _PyErr_StackItem *exc_info = tstate->exc_info;
    while ((exc_info->exc_type == NULL || exc_info->exc_type == Py_None) &&
           exc_info->previous_item != NULL)
    {
        exc_info = exc_info->previous_item;
    }
    return exc_info;
}
#endif

/* SaveResetException */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    #if CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = __Pyx_PyErr_GetTopmostException(tstate);
    *type = exc_info->exc_type;
    *value = exc_info->exc_value;
    *tb = exc_info->exc_traceback;
    #else
    *type = tstate->exc_type;
    *value = tstate->exc_value;
    *tb = tstate->exc_traceback;
    #endif
    Py_XINCREF(*type);
    Py_XINCREF(*value);
    Py_XINCREF(*tb);
}
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    #if CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = tstate->exc_info;
    tmp_type = exc_info->exc_type;
    tmp_value = exc_info->exc_value;
    tmp_tb = exc_info->exc_traceback;
    exc_info->exc_type = type;
    exc_info->exc_value = value;
    exc_info->exc_traceback = tb;
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = type;
    tstate->exc_value = value;
    tstate->exc_traceback = tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
#endif

/* SwapException */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx__ExceptionSwap(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    #if CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = tstate->exc_info;
    tmp_type = exc_info->exc_type;
    tmp_value = exc_info->exc_value;
    tmp_tb = exc_info->exc_traceback;
    exc_info->exc_type = *type;
    exc_info->exc_value = *value;
    exc_info->exc_traceback = *tb;
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = *type;
    tstate->exc_value = *value;
    tstate->exc_traceback = *tb;
    #endif
    *type = tmp_type;
    *value = tmp_value;
    *tb = tmp_tb;
}
#else
static CYTHON_INLINE void __Pyx_ExceptionSwap(PyObject **type, PyObject **value, PyObject **tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    PyErr_GetExcInfo(&tmp_type, &tmp_value, &tmp_tb);
    PyErr_SetExcInfo(*type, *value, *tb);
    *type = tmp_type;
    *value = tmp_value;
    *tb = tmp_tb;
}
#endif

/* PyObjectCallMethod1 */
static PyObject* __Pyx__PyObject_CallMethod1(PyObject* method, PyObject* arg) {
    PyObject *result = __Pyx_PyObject_CallOneArg(method, arg);
    Py_DECREF(method);
    return result;
}
static PyObject* __Pyx_PyObject_CallMethod1(PyObject* obj, PyObject* method_name, PyObject* arg) {
    PyObject *method = NULL, *result;
    int is_method = __Pyx_PyObject_GetMethod(obj, method_name, &method);
    if (likely(is_method)) {
        result = __Pyx_PyObject_Call2Args(method, obj, arg);
        Py_DECREF(method);
        return result;
    }
    if (unlikely(!method)) return NULL;
    return __Pyx__PyObject_CallMethod1(method, arg);
}

/* CoroutineBase */
#include <structmember.h>
#include <frameobject.h>
#define __Pyx_Coroutine_Undelegate(gen) Py_CLEAR((gen)->yieldfrom)
static int __Pyx_PyGen__FetchStopIterationValue(CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject **pvalue) {
    PyObject *et, *ev, *tb;
    PyObject *value = NULL;
    __Pyx_ErrFetch(&et, &ev, &tb);
    if (!et) {
        Py_XDECREF(tb);
        Py_XDECREF(ev);
        Py_INCREF(Py_None);
        *pvalue = Py_None;
        return 0;
    }
    if (likely(et == PyExc_StopIteration)) {
        if (!ev) {
            Py_INCREF(Py_None);
            value = Py_None;
        }
#if PY_VERSION_HEX >= 0x030300A0
        else if (Py_TYPE(ev) == (PyTypeObject*)PyExc_StopIteration) {
            value = ((PyStopIterationObject *)ev)->value;
            Py_INCREF(value);
            Py_DECREF(ev);
        }
#endif
        else if (unlikely(PyTuple_Check(ev))) {
            if (PyTuple_GET_SIZE(ev) >= 1) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
                value = PyTuple_GET_ITEM(ev, 0);
                Py_INCREF(value);
#else
                value = PySequence_ITEM(ev, 0);
#endif
            } else {
                Py_INCREF(Py_None);
                value = Py_None;
            }
            Py_DECREF(ev);
        }
        else if (!__Pyx_TypeCheck(ev, (PyTypeObject*)PyExc_StopIteration)) {
            value = ev;
        }
        if (likely(value)) {
            Py_XDECREF(tb);
            Py_DECREF(et);
            *pvalue = value;
            return 0;
        }
    } else if (!__Pyx_PyErr_GivenExceptionMatches(et, PyExc_StopIteration)) {
        __Pyx_ErrRestore(et, ev, tb);
        return -1;
    }
    PyErr_NormalizeException(&et, &ev, &tb);
    if (unlikely(!PyObject_TypeCheck(ev, (PyTypeObject*)PyExc_StopIteration))) {
        __Pyx_ErrRestore(et, ev, tb);
        return -1;
    }
    Py_XDECREF(tb);
    Py_DECREF(et);
#if PY_VERSION_HEX >= 0x030300A0
    value = ((PyStopIterationObject *)ev)->value;
    Py_INCREF(value);
    Py_DECREF(ev);
#else
    {
        PyObject* args = __Pyx_PyObject_GetAttrStr(ev, __pyx_n_s_args);
        Py_DECREF(ev);
        if (likely(args)) {
            value = PySequence_GetItem(args, 0);
            Py_DECREF(args);
        }
        if (unlikely(!value)) {
            __Pyx_ErrRestore(NULL, NULL, NULL);
            Py_INCREF(Py_None);
            value = Py_None;
        }
    }
#endif
    *pvalue = value;
    return 0;
}
static CYTHON_INLINE
void __Pyx_Coroutine_ExceptionClear(__Pyx_ExcInfoStruct *exc_state) {
    PyObject *t, *v, *tb;
    t = exc_state->exc_type;
    v = exc_state->exc_value;
    tb = exc_state->exc_traceback;
    exc_state->exc_type = NULL;
    exc_state->exc_value = NULL;
    exc_state->exc_traceback = NULL;
    Py_XDECREF(t);
    Py_XDECREF(v);
    Py_XDECREF(tb);
}
#define __Pyx_Coroutine_AlreadyRunningError(gen)  (__Pyx__Coroutine_AlreadyRunningError(gen), (PyObject*)NULL)
static void __Pyx__Coroutine_AlreadyRunningError(CYTHON_UNUSED __pyx_CoroutineObject *gen) {
    const char *msg;
    if ((0)) {
    #ifdef __Pyx_Coroutine_USED
    } else if (__Pyx_Coroutine_Check((PyObject*)gen)) {
        msg = "coroutine already executing";
    #endif
    #ifdef __Pyx_AsyncGen_USED
    } else if (__Pyx_AsyncGen_CheckExact((PyObject*)gen)) {
        msg = "async generator already executing";
    #endif
    } else {
        msg = "generator already executing";
    }
    PyErr_SetString(PyExc_ValueError, msg);
}
#define __Pyx_Coroutine_NotStartedError(gen)  (__Pyx__Coroutine_NotStartedError(gen), (PyObject*)NULL)
static void __Pyx__Coroutine_NotStartedError(CYTHON_UNUSED PyObject *gen) {
    const char *msg;
    if ((0)) {
    #ifdef __Pyx_Coroutine_USED
    } else if (__Pyx_Coroutine_Check(gen)) {
        msg = "can't send non-None value to a just-started coroutine";
    #endif
    #ifdef __Pyx_AsyncGen_USED
    } else if (__Pyx_AsyncGen_CheckExact(gen)) {
        msg = "can't send non-None value to a just-started async generator";
    #endif
    } else {
        msg = "can't send non-None value to a just-started generator";
    }
    PyErr_SetString(PyExc_TypeError, msg);
}
#define __Pyx_Coroutine_AlreadyTerminatedError(gen, value, closing)  (__Pyx__Coroutine_AlreadyTerminatedError(gen, value, closing), (PyObject*)NULL)
static void __Pyx__Coroutine_AlreadyTerminatedError(CYTHON_UNUSED PyObject *gen, PyObject *value, CYTHON_UNUSED int closing) {
    #ifdef __Pyx_Coroutine_USED
    if (!closing && __Pyx_Coroutine_Check(gen)) {
        PyErr_SetString(PyExc_RuntimeError, "cannot reuse already awaited coroutine");
    } else
    #endif
    if (value) {
        #ifdef __Pyx_AsyncGen_USED
        if (__Pyx_AsyncGen_CheckExact(gen))
            PyErr_SetNone(__Pyx_PyExc_StopAsyncIteration);
        else
        #endif
        PyErr_SetNone(PyExc_StopIteration);
    }
}
static
PyObject *__Pyx_Coroutine_SendEx(__pyx_CoroutineObject *self, PyObject *value, int closing) {
    __Pyx_PyThreadState_declare
    PyThreadState *tstate;
    __Pyx_ExcInfoStruct *exc_state;
    PyObject *retval;
    assert(!self->is_running);
    if (unlikely(self->resume_label == 0)) {
        if (unlikely(value && value != Py_None)) {
            return __Pyx_Coroutine_NotStartedError((PyObject*)self);
        }
    }
    if (unlikely(self->resume_label == -1)) {
        return __Pyx_Coroutine_AlreadyTerminatedError((PyObject*)self, value, closing);
    }
#if CYTHON_FAST_THREAD_STATE
    __Pyx_PyThreadState_assign
    tstate = __pyx_tstate;
#else
    tstate = __Pyx_PyThreadState_Current;
#endif
    exc_state = &self->gi_exc_state;
    if (exc_state->exc_type) {
        #if CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_PYSTON
        #else
        if (exc_state->exc_traceback) {
            PyTracebackObject *tb = (PyTracebackObject *) exc_state->exc_traceback;
            PyFrameObject *f = tb->tb_frame;
            Py_XINCREF(tstate->frame);
            assert(f->f_back == NULL);
            f->f_back = tstate->frame;
        }
        #endif
    }
#if CYTHON_USE_EXC_INFO_STACK
    exc_state->previous_item = tstate->exc_info;
    tstate->exc_info = exc_state;
#else
    if (exc_state->exc_type) {
        __Pyx_ExceptionSwap(&exc_state->exc_type, &exc_state->exc_value, &exc_state->exc_traceback);
    } else {
        __Pyx_Coroutine_ExceptionClear(exc_state);
        __Pyx_ExceptionSave(&exc_state->exc_type, &exc_state->exc_value, &exc_state->exc_traceback);
    }
#endif
    self->is_running = 1;
    retval = self->body((PyObject *) self, tstate, value);
    self->is_running = 0;
#if CYTHON_USE_EXC_INFO_STACK
    exc_state = &self->gi_exc_state;
    tstate->exc_info = exc_state->previous_item;
    exc_state->previous_item = NULL;
    __Pyx_Coroutine_ResetFrameBackpointer(exc_state);
#endif
    return retval;
}
static CYTHON_INLINE void __Pyx_Coroutine_ResetFrameBackpointer(__Pyx_ExcInfoStruct *exc_state) {
    PyObject *exc_tb = exc_state->exc_traceback;
    if (likely(exc_tb)) {
#if CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_PYSTON
#else
        PyTracebackObject *tb = (PyTracebackObject *) exc_tb;
        PyFrameObject *f = tb->tb_frame;
        Py_CLEAR(f->f_back);
#endif
    }
}
static CYTHON_INLINE
PyObject *__Pyx_Coroutine_MethodReturn(CYTHON_UNUSED PyObject* gen, PyObject *retval) {
    if (unlikely(!retval)) {
        __Pyx_PyThreadState_declare
        __Pyx_PyThreadState_assign
        if (!__Pyx_PyErr_Occurred()) {
            PyObject *exc = PyExc_StopIteration;
            #ifdef __Pyx_AsyncGen_USED
            if (__Pyx_AsyncGen_CheckExact(gen))
                exc = __Pyx_PyExc_StopAsyncIteration;
            #endif
            __Pyx_PyErr_SetNone(exc);
        }
    }
    return retval;
}
static CYTHON_INLINE
PyObject *__Pyx_Coroutine_FinishDelegation(__pyx_CoroutineObject *gen) {
    PyObject *ret;
    PyObject *val = NULL;
    __Pyx_Coroutine_Undelegate(gen);
    __Pyx_PyGen__FetchStopIterationValue(__Pyx_PyThreadState_Current, &val);
    ret = __Pyx_Coroutine_SendEx(gen, val, 0);
    Py_XDECREF(val);
    return ret;
}
static PyObject *__Pyx_Coroutine_Send(PyObject *self, PyObject *value) {
    PyObject *retval;
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject*) self;
    PyObject *yf = gen->yieldfrom;
    if (unlikely(gen->is_running))
        return __Pyx_Coroutine_AlreadyRunningError(gen);
    if (yf) {
        PyObject *ret;
        gen->is_running = 1;
        #ifdef __Pyx_Generator_USED
        if (__Pyx_Generator_CheckExact(yf)) {
            ret = __Pyx_Coroutine_Send(yf, value);
        } else
        #endif
        #ifdef __Pyx_Coroutine_USED
        if (__Pyx_Coroutine_Check(yf)) {
            ret = __Pyx_Coroutine_Send(yf, value);
        } else
        #endif
        #ifdef __Pyx_AsyncGen_USED
        if (__pyx_PyAsyncGenASend_CheckExact(yf)) {
            ret = __Pyx_async_gen_asend_send(yf, value);
        } else
        #endif
        #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x03030000 && (defined(__linux__) || PY_VERSION_HEX >= 0x030600B3)
        if (PyGen_CheckExact(yf)) {
            ret = _PyGen_Send((PyGenObject*)yf, value == Py_None ? NULL : value);
        } else
        #endif
        #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x03050000 && defined(PyCoro_CheckExact) && (defined(__linux__) || PY_VERSION_HEX >= 0x030600B3)
        if (PyCoro_CheckExact(yf)) {
            ret = _PyGen_Send((PyGenObject*)yf, value == Py_None ? NULL : value);
        } else
        #endif
        {
            if (value == Py_None)
                ret = Py_TYPE(yf)->tp_iternext(yf);
            else
                ret = __Pyx_PyObject_CallMethod1(yf, __pyx_n_s_send, value);
        }
        gen->is_running = 0;
        if (likely(ret)) {
            return ret;
        }
        retval = __Pyx_Coroutine_FinishDelegation(gen);
    } else {
        retval = __Pyx_Coroutine_SendEx(gen, value, 0);
    }
    return __Pyx_Coroutine_MethodReturn(self, retval);
}
static int __Pyx_Coroutine_CloseIter(__pyx_CoroutineObject *gen, PyObject *yf) {
    PyObject *retval = NULL;
    int err = 0;
    #ifdef __Pyx_Generator_USED
    if (__Pyx_Generator_CheckExact(yf)) {
        retval = __Pyx_Coroutine_Close(yf);
        if (!retval)
            return -1;
    } else
    #endif
    #ifdef __Pyx_Coroutine_USED
    if (__Pyx_Coroutine_Check(yf)) {
        retval = __Pyx_Coroutine_Close(yf);
        if (!retval)
            return -1;
    } else
    if (__Pyx_CoroutineAwait_CheckExact(yf)) {
        retval = __Pyx_CoroutineAwait_Close((__pyx_CoroutineAwaitObject*)yf, NULL);
        if (!retval)
            return -1;
    } else
    #endif
    #ifdef __Pyx_AsyncGen_USED
    if (__pyx_PyAsyncGenASend_CheckExact(yf)) {
        retval = __Pyx_async_gen_asend_close(yf, NULL);
    } else
    if (__pyx_PyAsyncGenAThrow_CheckExact(yf)) {
        retval = __Pyx_async_gen_athrow_close(yf, NULL);
    } else
    #endif
    {
        PyObject *meth;
        gen->is_running = 1;
        meth = __Pyx_PyObject_GetAttrStr(yf, __pyx_n_s_close);
        if (unlikely(!meth)) {
            if (!PyErr_ExceptionMatches(PyExc_AttributeError)) {
                PyErr_WriteUnraisable(yf);
            }
            PyErr_Clear();
        } else {
            retval = PyObject_CallFunction(meth, NULL);
            Py_DECREF(meth);
            if (!retval)
                err = -1;
        }
        gen->is_running = 0;
    }
    Py_XDECREF(retval);
    return err;
}
static PyObject *__Pyx_Generator_Next(PyObject *self) {
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject*) self;
    PyObject *yf = gen->yieldfrom;
    if (unlikely(gen->is_running))
        return __Pyx_Coroutine_AlreadyRunningError(gen);
    if (yf) {
        PyObject *ret;
        gen->is_running = 1;
        #ifdef __Pyx_Generator_USED
        if (__Pyx_Generator_CheckExact(yf)) {
            ret = __Pyx_Generator_Next(yf);
        } else
        #endif
        #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x03030000 && (defined(__linux__) || PY_VERSION_HEX >= 0x030600B3)
        if (PyGen_CheckExact(yf)) {
            ret = _PyGen_Send((PyGenObject*)yf, NULL);
        } else
        #endif
        #ifdef __Pyx_Coroutine_USED
        if (__Pyx_Coroutine_Check(yf)) {
            ret = __Pyx_Coroutine_Send(yf, Py_None);
        } else
        #endif
            ret = Py_TYPE(yf)->tp_iternext(yf);
        gen->is_running = 0;
        if (likely(ret)) {
            return ret;
        }
        return __Pyx_Coroutine_FinishDelegation(gen);
    }
    return __Pyx_Coroutine_SendEx(gen, Py_None, 0);
}
static PyObject *__Pyx_Coroutine_Close_Method(PyObject *self, CYTHON_UNUSED PyObject *arg) {
    return __Pyx_Coroutine_Close(self);
}
static PyObject *__Pyx_Coroutine_Close(PyObject *self) {
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    PyObject *retval, *raised_exception;
    PyObject *yf = gen->yieldfrom;
    int err = 0;
    if (unlikely(gen->is_running))
        return __Pyx_Coroutine_AlreadyRunningError(gen);
    if (yf) {
        Py_INCREF(yf);
        err = __Pyx_Coroutine_CloseIter(gen, yf);
        __Pyx_Coroutine_Undelegate(gen);
        Py_DECREF(yf);
    }
    if (err == 0)
        PyErr_SetNone(PyExc_GeneratorExit);
    retval = __Pyx_Coroutine_SendEx(gen, NULL, 1);
    if (unlikely(retval)) {
        const char *msg;
        Py_DECREF(retval);
        if ((0)) {
        #ifdef __Pyx_Coroutine_USED
        } else if (__Pyx_Coroutine_Check(self)) {
            msg = "coroutine ignored GeneratorExit";
        #endif
        #ifdef __Pyx_AsyncGen_USED
        } else if (__Pyx_AsyncGen_CheckExact(self)) {
#if PY_VERSION_HEX < 0x03060000
            msg = "async generator ignored GeneratorExit - might require Python 3.6+ finalisation (PEP 525)";
#else
            msg = "async generator ignored GeneratorExit";
#endif
        #endif
        } else {
            msg = "generator ignored GeneratorExit";
        }
        PyErr_SetString(PyExc_RuntimeError, msg);
        return NULL;
    }
    raised_exception = PyErr_Occurred();
    if (likely(!raised_exception || __Pyx_PyErr_GivenExceptionMatches2(raised_exception, PyExc_GeneratorExit, PyExc_StopIteration))) {
        if (raised_exception) PyErr_Clear();
        Py_INCREF(Py_None);
        return Py_None;
    }
    return NULL;
}
static PyObject *__Pyx__Coroutine_Throw(PyObject *self, PyObject *typ, PyObject *val, PyObject *tb,
                                        PyObject *args, int close_on_genexit) {
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    PyObject *yf = gen->yieldfrom;
    if (unlikely(gen->is_running))
        return __Pyx_Coroutine_AlreadyRunningError(gen);
    if (yf) {
        PyObject *ret;
        Py_INCREF(yf);
        if (__Pyx_PyErr_GivenExceptionMatches(typ, PyExc_GeneratorExit) && close_on_genexit) {
            int err = __Pyx_Coroutine_CloseIter(gen, yf);
            Py_DECREF(yf);
            __Pyx_Coroutine_Undelegate(gen);
            if (err < 0)
                return __Pyx_Coroutine_MethodReturn(self, __Pyx_Coroutine_SendEx(gen, NULL, 0));
            goto throw_here;
        }
        gen->is_running = 1;
        if (0
        #ifdef __Pyx_Generator_USED
            || __Pyx_Generator_CheckExact(yf)
        #endif
        #ifdef __Pyx_Coroutine_USED
            || __Pyx_Coroutine_Check(yf)
        #endif
            ) {
            ret = __Pyx__Coroutine_Throw(yf, typ, val, tb, args, close_on_genexit);
        #ifdef __Pyx_Coroutine_USED
        } else if (__Pyx_CoroutineAwait_CheckExact(yf)) {
            ret = __Pyx__Coroutine_Throw(((__pyx_CoroutineAwaitObject*)yf)->coroutine, typ, val, tb, args, close_on_genexit);
        #endif
        } else {
            PyObject *meth = __Pyx_PyObject_GetAttrStr(yf, __pyx_n_s_throw);
            if (unlikely(!meth)) {
                Py_DECREF(yf);
                if (!PyErr_ExceptionMatches(PyExc_AttributeError)) {
                    gen->is_running = 0;
                    return NULL;
                }
                PyErr_Clear();
                __Pyx_Coroutine_Undelegate(gen);
                gen->is_running = 0;
                goto throw_here;
            }
            if (likely(args)) {
                ret = PyObject_CallObject(meth, args);
            } else {
                ret = PyObject_CallFunctionObjArgs(meth, typ, val, tb, NULL);
            }
            Py_DECREF(meth);
        }
        gen->is_running = 0;
        Py_DECREF(yf);
        if (!ret) {
            ret = __Pyx_Coroutine_FinishDelegation(gen);
        }
        return __Pyx_Coroutine_MethodReturn(self, ret);
    }
throw_here:
    __Pyx_Raise(typ, val, tb, NULL);
    return __Pyx_Coroutine_MethodReturn(self, __Pyx_Coroutine_SendEx(gen, NULL, 0));
}
static PyObject *__Pyx_Coroutine_Throw(PyObject *self, PyObject *args) {
    PyObject *typ;
    PyObject *val = NULL;
    PyObject *tb = NULL;
    if (!PyArg_UnpackTuple(args, (char *)"throw", 1, 3, &typ, &val, &tb))
        return NULL;
    return __Pyx__Coroutine_Throw(self, typ, val, tb, args, 1);
}
static CYTHON_INLINE int __Pyx_Coroutine_traverse_excstate(__Pyx_ExcInfoStruct *exc_state, visitproc visit, void *arg) {
    Py_VISIT(exc_state->exc_type);
    Py_VISIT(exc_state->exc_value);
    Py_VISIT(exc_state->exc_traceback);
    return 0;
}
static int __Pyx_Coroutine_traverse(__pyx_CoroutineObject *gen, visitproc visit, void *arg) {
    Py_VISIT(gen->closure);
    Py_VISIT(gen->classobj);
    Py_VISIT(gen->yieldfrom);
    return __Pyx_Coroutine_traverse_excstate(&gen->gi_exc_state, visit, arg);
}
static int __Pyx_Coroutine_clear(PyObject *self) {
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    Py_CLEAR(gen->closure);
    Py_CLEAR(gen->classobj);
    Py_CLEAR(gen->yieldfrom);
    __Pyx_Coroutine_ExceptionClear(&gen->gi_exc_state);
#ifdef __Pyx_AsyncGen_USED
    if (__Pyx_AsyncGen_CheckExact(self)) {
        Py_CLEAR(((__pyx_PyAsyncGenObject*)gen)->ag_finalizer);
    }
#endif
    Py_CLEAR(gen->gi_code);
    Py_CLEAR(gen->gi_name);
    Py_CLEAR(gen->gi_qualname);
    Py_CLEAR(gen->gi_modulename);
    return 0;
}
static void __Pyx_Coroutine_dealloc(PyObject *self) {
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    PyObject_GC_UnTrack(gen);
    if (gen->gi_weakreflist != NULL)
        PyObject_ClearWeakRefs(self);
    if (gen->resume_label >= 0) {
        PyObject_GC_Track(self);
#if PY_VERSION_HEX >= 0x030400a1 && CYTHON_USE_TP_FINALIZE
        if (PyObject_CallFinalizerFromDealloc(self))
#else
        Py_TYPE(gen)->tp_del(self);
        if (self->ob_refcnt > 0)
#endif
        {
            return;
        }
        PyObject_GC_UnTrack(self);
    }
#ifdef __Pyx_AsyncGen_USED
    if (__Pyx_AsyncGen_CheckExact(self)) {
        /* We have to handle this case for asynchronous generators
           right here, because this code has to be between UNTRACK
           and GC_Del. */
        Py_CLEAR(((__pyx_PyAsyncGenObject*)self)->ag_finalizer);
    }
#endif
    __Pyx_Coroutine_clear(self);
    PyObject_GC_Del(gen);
}
static void __Pyx_Coroutine_del(PyObject *self) {
    PyObject *error_type, *error_value, *error_traceback;
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    __Pyx_PyThreadState_declare
    if (gen->resume_label < 0) {
        return;
    }
#if !CYTHON_USE_TP_FINALIZE
    assert(self->ob_refcnt == 0);
    self->ob_refcnt = 1;
#endif
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&error_type, &error_value, &error_traceback);
#ifdef __Pyx_AsyncGen_USED
    if (__Pyx_AsyncGen_CheckExact(self)) {
        __pyx_PyAsyncGenObject *agen = (__pyx_PyAsyncGenObject*)self;
        PyObject *finalizer = agen->ag_finalizer;
        if (finalizer && !agen->ag_closed) {
            PyObject *res = __Pyx_PyObject_CallOneArg(finalizer, self);
            if (unlikely(!res)) {
                PyErr_WriteUnraisable(self);
            } else {
                Py_DECREF(res);
            }
            __Pyx_ErrRestore(error_type, error_value, error_traceback);
            return;
        }
    }
#endif
    if (unlikely(gen->resume_label == 0 && !error_value)) {
#ifdef __Pyx_Coroutine_USED
#ifdef __Pyx_Generator_USED
    if (!__Pyx_Generator_CheckExact(self))
#endif
        {
        PyObject_GC_UnTrack(self);
#if PY_MAJOR_VERSION >= 3  || defined(PyErr_WarnFormat)
        if (unlikely(PyErr_WarnFormat(PyExc_RuntimeWarning, 1, "coroutine '%.50S' was never awaited", gen->gi_qualname) < 0))
            PyErr_WriteUnraisable(self);
#else
        {PyObject *msg;
        char *cmsg;
        #if CYTHON_COMPILING_IN_PYPY
        msg = NULL;
        cmsg = (char*) "coroutine was never awaited";
        #else
        char *cname;
        PyObject *qualname;
        qualname = gen->gi_qualname;
        cname = PyString_AS_STRING(qualname);
        msg = PyString_FromFormat("coroutine '%.50s' was never awaited", cname);
        if (unlikely(!msg)) {
            PyErr_Clear();
            cmsg = (char*) "coroutine was never awaited";
        } else {
            cmsg = PyString_AS_STRING(msg);
        }
        #endif
        if (unlikely(PyErr_WarnEx(PyExc_RuntimeWarning, cmsg, 1) < 0))
            PyErr_WriteUnraisable(self);
        Py_XDECREF(msg);}
#endif
        PyObject_GC_Track(self);
        }
#endif
    } else {
        PyObject *res = __Pyx_Coroutine_Close(self);
        if (unlikely(!res)) {
            if (PyErr_Occurred())
                PyErr_WriteUnraisable(self);
        } else {
            Py_DECREF(res);
        }
    }
    __Pyx_ErrRestore(error_type, error_value, error_traceback);
#if !CYTHON_USE_TP_FINALIZE
    assert(self->ob_refcnt > 0);
    if (--self->ob_refcnt == 0) {
        return;
    }
    {
        Py_ssize_t refcnt = self->ob_refcnt;
        _Py_NewReference(self);
        self->ob_refcnt = refcnt;
    }
#if CYTHON_COMPILING_IN_CPYTHON
    assert(PyType_IS_GC(self->ob_type) &&
           _Py_AS_GC(self)->gc.gc_refs != _PyGC_REFS_UNTRACKED);
    _Py_DEC_REFTOTAL;
#endif
#ifdef COUNT_ALLOCS
    --Py_TYPE(self)->tp_frees;
    --Py_TYPE(self)->tp_allocs;
#endif
#endif
}
static PyObject *
__Pyx_Coroutine_get_name(__pyx_CoroutineObject *self, CYTHON_UNUSED void *context)
{
    PyObject *name = self->gi_name;
    if (unlikely(!name)) name = Py_None;
    Py_INCREF(name);
    return name;
}
static int
__Pyx_Coroutine_set_name(__pyx_CoroutineObject *self, PyObject *value, CYTHON_UNUSED void *context)
{
    PyObject *tmp;
#if PY_MAJOR_VERSION >= 3
    if (unlikely(value == NULL || !PyUnicode_Check(value)))
#else
    if (unlikely(value == NULL || !PyString_Check(value)))
#endif
    {
        PyErr_SetString(PyExc_TypeError,
                        "__name__ must be set to a string object");
        return -1;
    }
    tmp = self->gi_name;
    Py_INCREF(value);
    self->gi_name = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_Coroutine_get_qualname(__pyx_CoroutineObject *self, CYTHON_UNUSED void *context)
{
    PyObject *name = self->gi_qualname;
    if (unlikely(!name)) name = Py_None;
    Py_INCREF(name);
    return name;
}
static int
__Pyx_Coroutine_set_qualname(__pyx_CoroutineObject *self, PyObject *value, CYTHON_UNUSED void *context)
{
    PyObject *tmp;
#if PY_MAJOR_VERSION >= 3
    if (unlikely(value == NULL || !PyUnicode_Check(value)))
#else
    if (unlikely(value == NULL || !PyString_Check(value)))
#endif
    {
        PyErr_SetString(PyExc_TypeError,
                        "__qualname__ must be set to a string object");
        return -1;
    }
    tmp = self->gi_qualname;
    Py_INCREF(value);
    self->gi_qualname = value;
    Py_XDECREF(tmp);
    return 0;
}
static __pyx_CoroutineObject *__Pyx__Coroutine_New(
            PyTypeObject* type, __pyx_coroutine_body_t body, PyObject *code, PyObject *closure,
            PyObject *name, PyObject *qualname, PyObject *module_name) {
    __pyx_CoroutineObject *gen = PyObject_GC_New(__pyx_CoroutineObject, type);
    if (unlikely(!gen))
        return NULL;
    return __Pyx__Coroutine_NewInit(gen, body, code, closure, name, qualname, module_name);
}
static __pyx_CoroutineObject *__Pyx__Coroutine_NewInit(
            __pyx_CoroutineObject *gen, __pyx_coroutine_body_t body, PyObject *code, PyObject *closure,
            PyObject *name, PyObject *qualname, PyObject *module_name) {
    gen->body = body;
    gen->closure = closure;
    Py_XINCREF(closure);
    gen->is_running = 0;
    gen->resume_label = 0;
    gen->classobj = NULL;
    gen->yieldfrom = NULL;
    gen->gi_exc_state.exc_type = NULL;
    gen->gi_exc_state.exc_value = NULL;
    gen->gi_exc_state.exc_traceback = NULL;
#if CYTHON_USE_EXC_INFO_STACK
    gen->gi_exc_state.previous_item = NULL;
#endif
    gen->gi_weakreflist = NULL;
    Py_XINCREF(qualname);
    gen->gi_qualname = qualname;
    Py_XINCREF(name);
    gen->gi_name = name;
    Py_XINCREF(module_name);
    gen->gi_modulename = module_name;
    Py_XINCREF(code);
    gen->gi_code = code;
    PyObject_GC_Track(gen);
    return gen;
}

/* PatchModuleWithCoroutine */
static PyObject* __Pyx_Coroutine_patch_module(PyObject* module, const char* py_code) {
#if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
    int result;
    PyObject *globals, *result_obj;
    globals = PyDict_New();  if (unlikely(!globals)) goto ignore;
    result = PyDict_SetItemString(globals, "_cython_coroutine_type",
    #ifdef __Pyx_Coroutine_USED
        (PyObject*)__pyx_CoroutineType);
    #else
        Py_None);
    #endif
    if (unlikely(result < 0)) goto ignore;
    result = PyDict_SetItemString(globals, "_cython_generator_type",
    #ifdef __Pyx_Generator_USED
        (PyObject*)__pyx_GeneratorType);
    #else
        Py_None);
    #endif
    if (unlikely(result < 0)) goto ignore;
    if (unlikely(PyDict_SetItemString(globals, "_module", module) < 0)) goto ignore;
    if (unlikely(PyDict_SetItemString(globals, "__builtins__", __pyx_b) < 0)) goto ignore;
    result_obj = PyRun_String(py_code, Py_file_input, globals, globals);
    if (unlikely(!result_obj)) goto ignore;
    Py_DECREF(result_obj);
    Py_DECREF(globals);
    return module;
ignore:
    Py_XDECREF(globals);
    PyErr_WriteUnraisable(module);
    if (unlikely(PyErr_WarnEx(PyExc_RuntimeWarning, "Cython module failed to patch module with custom type", 1) < 0)) {
        Py_DECREF(module);
        module = NULL;
    }
#else
    py_code++;
#endif
    return module;
}

/* PatchGeneratorABC */
#ifndef CYTHON_REGISTER_ABCS
#define CYTHON_REGISTER_ABCS 1
#endif
#if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
static PyObject* __Pyx_patch_abc_module(PyObject *module);
static PyObject* __Pyx_patch_abc_module(PyObject *module) {
    module = __Pyx_Coroutine_patch_module(
        module, ""
"if _cython_generator_type is not None:\n"
"    try: Generator = _module.Generator\n"
"    except AttributeError: pass\n"
"    else: Generator.register(_cython_generator_type)\n"
"if _cython_coroutine_type is not None:\n"
"    try: Coroutine = _module.Coroutine\n"
"    except AttributeError: pass\n"
"    else: Coroutine.register(_cython_coroutine_type)\n"
    );
    return module;
}
#endif
static int __Pyx_patch_abc(void) {
#if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
    static int abc_patched = 0;
    if (CYTHON_REGISTER_ABCS && !abc_patched) {
        PyObject *module;
        module = PyImport_ImportModule((PY_MAJOR_VERSION >= 3) ? "collections.abc" : "collections");
        if (!module) {
            PyErr_WriteUnraisable(NULL);
            if (unlikely(PyErr_WarnEx(PyExc_RuntimeWarning,
                    ((PY_MAJOR_VERSION >= 3) ?
                        "Cython module failed to register with collections.abc module" :
                        "Cython module failed to register with collections module"), 1) < 0)) {
                return -1;
            }
        } else {
            module = __Pyx_patch_abc_module(module);
            abc_patched = 1;
            if (unlikely(!module))
                return -1;
            Py_DECREF(module);
        }
        module = PyImport_ImportModule("backports_abc");
        if (module) {
            module = __Pyx_patch_abc_module(module);
            Py_XDECREF(module);
        }
        if (!module) {
            PyErr_Clear();
        }
    }
#else
    if ((0)) __Pyx_Coroutine_patch_module(NULL, NULL);
#endif
    return 0;
}

/* Generator */
static PyMethodDef __pyx_Generator_methods[] = {
    {"send", (PyCFunction) __Pyx_Coroutine_Send, METH_O,
     (char*) PyDoc_STR("send(arg) -> send 'arg' into generator,\nreturn next yielded value or raise StopIteration.")},
    {"throw", (PyCFunction) __Pyx_Coroutine_Throw, METH_VARARGS,
     (char*) PyDoc_STR("throw(typ[,val[,tb]]) -> raise exception in generator,\nreturn next yielded value or raise StopIteration.")},
    {"close", (PyCFunction) __Pyx_Coroutine_Close_Method, METH_NOARGS,
     (char*) PyDoc_STR("close() -> raise GeneratorExit inside generator.")},
    {0, 0, 0, 0}
};
static PyMemberDef __pyx_Generator_memberlist[] = {
    {(char *) "gi_running", T_BOOL, offsetof(__pyx_CoroutineObject, is_running), READONLY, NULL},
    {(char*) "gi_yieldfrom", T_OBJECT, offsetof(__pyx_CoroutineObject, yieldfrom), READONLY,
     (char*) PyDoc_STR("object being iterated by 'yield from', or None")},
    {(char*) "gi_code", T_OBJECT, offsetof(__pyx_CoroutineObject, gi_code), READONLY, NULL},
    {0, 0, 0, 0, 0}
};
static PyGetSetDef __pyx_Generator_getsets[] = {
    {(char *) "__name__", (getter)__Pyx_Coroutine_get_name, (setter)__Pyx_Coroutine_set_name,
     (char*) PyDoc_STR("name of the generator"), 0},
    {(char *) "__qualname__", (getter)__Pyx_Coroutine_get_qualname, (setter)__Pyx_Coroutine_set_qualname,
     (char*) PyDoc_STR("qualified name of the generator"), 0},
    {0, 0, 0, 0, 0}
};
static PyTypeObject __pyx_GeneratorType_type = {
    PyVarObject_HEAD_INIT(0, 0)
    "generator",
    sizeof(__pyx_CoroutineObject),
    0,
    (destructor) __Pyx_Coroutine_dealloc,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC | Py_TPFLAGS_HAVE_FINALIZE,
    0,
    (traverseproc) __Pyx_Coroutine_traverse,
    0,
    0,
    offsetof(__pyx_CoroutineObject, gi_weakreflist),
    0,
    (iternextfunc) __Pyx_Generator_Next,
    __pyx_Generator_methods,
    __pyx_Generator_memberlist,
    __pyx_Generator_getsets,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
#if CYTHON_USE_TP_FINALIZE
    0,
#else
    __Pyx_Coroutine_del,
#endif
    0,
#if CYTHON_USE_TP_FINALIZE
    __Pyx_Coroutine_del,
#elif PY_VERSION_HEX >= 0x030400a1
    0,
#endif
};
static int __pyx_Generator_init(void) {
    __pyx_GeneratorType_type.tp_getattro = __Pyx_PyObject_GenericGetAttrNoDict;
    __pyx_GeneratorType_type.tp_iter = PyObject_SelfIter;
    __pyx_GeneratorType = __Pyx_FetchCommonType(&__pyx_GeneratorType_type);
    if (unlikely(!__pyx_GeneratorType)) {
        return -1;
    }
    return 0;
}

/* CheckBinaryVersion */
static int __Pyx_check_binary_version(void) {
    char ctversion[4], rtversion[4];
    PyOS_snprintf(ctversion, 4, "%d.%d", PY_MAJOR_VERSION, PY_MINOR_VERSION);
    PyOS_snprintf(rtversion, 4, "%s", Py_GetVersion());
    if (ctversion[0] != rtversion[0] || ctversion[2] != rtversion[2]) {
        char message[200];
        PyOS_snprintf(message, sizeof(message),
                      "compiletime version %s of module '%.100s' "
                      "does not match runtime version %s",
                      ctversion, __Pyx_MODULE_NAME, rtversion);
        return PyErr_WarnEx(NULL, message, 1);
    }
    return 0;
}

/* FunctionExport */
static int __Pyx_ExportFunction(const char *name, void (*f)(void), const char *sig) {
    PyObject *d = 0;
    PyObject *cobj = 0;
    union {
        void (*fp)(void);
        void *p;
    } tmp;
    d = PyObject_GetAttrString(__pyx_m, (char *)"__pyx_capi__");
    if (!d) {
        PyErr_Clear();
        d = PyDict_New();
        if (!d)
            goto bad;
        Py_INCREF(d);
        if (PyModule_AddObject(__pyx_m, (char *)"__pyx_capi__", d) < 0)
            goto bad;
    }
    tmp.fp = f;
#if PY_VERSION_HEX >= 0x02070000
    cobj = PyCapsule_New(tmp.p, sig, 0);
#else
    cobj = PyCObject_FromVoidPtrAndDesc(tmp.p, (void *)sig, 0);
#endif
    if (!cobj)
        goto bad;
    if (PyDict_SetItemString(d, name, cobj) < 0)
        goto bad;
    Py_DECREF(cobj);
    Py_DECREF(d);
    return 0;
bad:
    Py_XDECREF(cobj);
    Py_XDECREF(d);
    return -1;
}

/* VoidPtrImport */
#ifndef __PYX_HAVE_RT_ImportVoidPtr
#define __PYX_HAVE_RT_ImportVoidPtr
static int __Pyx_ImportVoidPtr(PyObject *module, const char *name, void **p, const char *sig) {
    PyObject *d = 0;
    PyObject *cobj = 0;
    d = PyObject_GetAttrString(module, (char *)"__pyx_capi__");
    if (!d)
        goto bad;
    cobj = PyDict_GetItemString(d, name);
    if (!cobj) {
        PyErr_Format(PyExc_ImportError,
            "%.200s does not export expected C variable %.200s",
                PyModule_GetName(module), name);
        goto bad;
    }
#if PY_VERSION_HEX >= 0x02070000
    if (!PyCapsule_IsValid(cobj, sig)) {
        PyErr_Format(PyExc_TypeError,
            "C variable %.200s.%.200s has wrong signature (expected %.500s, got %.500s)",
             PyModule_GetName(module), name, sig, PyCapsule_GetName(cobj));
        goto bad;
    }
    *p = PyCapsule_GetPointer(cobj, sig);
#else
    {const char *desc, *s1, *s2;
    desc = (const char *)PyCObject_GetDesc(cobj);
    if (!desc)
        goto bad;
    s1 = desc; s2 = sig;
    while (*s1 != '\0' && *s1 == *s2) { s1++; s2++; }
    if (*s1 != *s2) {
        PyErr_Format(PyExc_TypeError,
            "C variable %.200s.%.200s has wrong signature (expected %.500s, got %.500s)",
             PyModule_GetName(module), name, sig, desc);
        goto bad;
    }
    *p = PyCObject_AsVoidPtr(cobj);}
#endif
    if (!(*p))
        goto bad;
    Py_DECREF(d);
    return 0;
bad:
    Py_XDECREF(d);
    return -1;
}
#endif

/* FunctionImport */
#ifndef __PYX_HAVE_RT_ImportFunction
#define __PYX_HAVE_RT_ImportFunction
static int __Pyx_ImportFunction(PyObject *module, const char *funcname, void (**f)(void), const char *sig) {
    PyObject *d = 0;
    PyObject *cobj = 0;
    union {
        void (*fp)(void);
        void *p;
    } tmp;
    d = PyObject_GetAttrString(module, (char *)"__pyx_capi__");
    if (!d)
        goto bad;
    cobj = PyDict_GetItemString(d, funcname);
    if (!cobj) {
        PyErr_Format(PyExc_ImportError,
            "%.200s does not export expected C function %.200s",
                PyModule_GetName(module), funcname);
        goto bad;
    }
#if PY_VERSION_HEX >= 0x02070000
    if (!PyCapsule_IsValid(cobj, sig)) {
        PyErr_Format(PyExc_TypeError,
            "C function %.200s.%.200s has wrong signature (expected %.500s, got %.500s)",
             PyModule_GetName(module), funcname, sig, PyCapsule_GetName(cobj));
        goto bad;
    }
    tmp.p = PyCapsule_GetPointer(cobj, sig);
#else
    {const char *desc, *s1, *s2;
    desc = (const char *)PyCObject_GetDesc(cobj);
    if (!desc)
        goto bad;
    s1 = desc; s2 = sig;
    while (*s1 != '\0' && *s1 == *s2) { s1++; s2++; }
    if (*s1 != *s2) {
        PyErr_Format(PyExc_TypeError,
            "C function %.200s.%.200s has wrong signature (expected %.500s, got %.500s)",
             PyModule_GetName(module), funcname, sig, desc);
        goto bad;
    }
    tmp.p = PyCObject_AsVoidPtr(cobj);}
#endif
    *f = tmp.fp;
    if (!(*f))
        goto bad;
    Py_DECREF(d);
    return 0;
bad:
    Py_XDECREF(d);
    return -1;
}
#endif

/* InitStrings */
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t) {
    while (t->p) {
        #if PY_MAJOR_VERSION < 3
        if (t->is_unicode) {
            *t->p = PyUnicode_DecodeUTF8(t->s, t->n - 1, NULL);
        } else if (t->intern) {
            *t->p = PyString_InternFromString(t->s);
        } else {
            *t->p = PyString_FromStringAndSize(t->s, t->n - 1);
        }
        #else
        if (t->is_unicode | t->is_str) {
            if (t->intern) {
                *t->p = PyUnicode_InternFromString(t->s);
            } else if (t->encoding) {
                *t->p = PyUnicode_Decode(t->s, t->n - 1, t->encoding, NULL);
            } else {
                *t->p = PyUnicode_FromStringAndSize(t->s, t->n - 1);
            }
        } else {
            *t->p = PyBytes_FromStringAndSize(t->s, t->n - 1);
        }
        #endif
        if (!*t->p)
            return -1;
        if (PyObject_Hash(*t->p) == -1)
            return -1;
        ++t;
    }
    return 0;
}

static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char* c_str) {
    return __Pyx_PyUnicode_FromStringAndSize(c_str, (Py_ssize_t)strlen(c_str));
}
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject* o) {
    Py_ssize_t ignore;
    return __Pyx_PyObject_AsStringAndSize(o, &ignore);
}
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
#if !CYTHON_PEP393_ENABLED
static const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    char* defenc_c;
    PyObject* defenc = _PyUnicode_AsDefaultEncodedString(o, NULL);
    if (!defenc) return NULL;
    defenc_c = PyBytes_AS_STRING(defenc);
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    {
        char* end = defenc_c + PyBytes_GET_SIZE(defenc);
        char* c;
        for (c = defenc_c; c < end; c++) {
            if ((unsigned char) (*c) >= 128) {
                PyUnicode_AsASCIIString(o);
                return NULL;
            }
        }
    }
#endif
    *length = PyBytes_GET_SIZE(defenc);
    return defenc_c;
}
#else
static CYTHON_INLINE const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    if (unlikely(__Pyx_PyUnicode_READY(o) == -1)) return NULL;
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    if (likely(PyUnicode_IS_ASCII(o))) {
        *length = PyUnicode_GET_LENGTH(o);
        return PyUnicode_AsUTF8(o);
    } else {
        PyUnicode_AsASCIIString(o);
        return NULL;
    }
#else
    return PyUnicode_AsUTF8AndSize(o, length);
#endif
}
#endif
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
    if (
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
            __Pyx_sys_getdefaultencoding_not_ascii &&
#endif
            PyUnicode_Check(o)) {
        return __Pyx_PyUnicode_AsStringAndSize(o, length);
    } else
#endif
#if (!CYTHON_COMPILING_IN_PYPY) || (defined(PyByteArray_AS_STRING) && defined(PyByteArray_GET_SIZE))
    if (PyByteArray_Check(o)) {
        *length = PyByteArray_GET_SIZE(o);
        return PyByteArray_AS_STRING(o);
    } else
#endif
    {
        char* result;
        int r = PyBytes_AsStringAndSize(o, &result, length);
        if (unlikely(r < 0)) {
            return NULL;
        } else {
            return result;
        }
    }
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject* x) {
   int is_true = x == Py_True;
   if (is_true | (x == Py_False) | (x == Py_None)) return is_true;
   else return PyObject_IsTrue(x);
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject* x) {
    int retval;
    if (unlikely(!x)) return -1;
    retval = __Pyx_PyObject_IsTrue(x);
    Py_DECREF(x);
    return retval;
}
static PyObject* __Pyx_PyNumber_IntOrLongWrongResultType(PyObject* result, const char* type_name) {
#if PY_MAJOR_VERSION >= 3
    if (PyLong_Check(result)) {
        if (PyErr_WarnFormat(PyExc_DeprecationWarning, 1,
                "__int__ returned non-int (type %.200s).  "
                "The ability to return an instance of a strict subclass of int "
                "is deprecated, and may be removed in a future version of Python.",
                Py_TYPE(result)->tp_name)) {
            Py_DECREF(result);
            return NULL;
        }
        return result;
    }
#endif
    PyErr_Format(PyExc_TypeError,
                 "__%.4s__ returned non-%.4s (type %.200s)",
                 type_name, type_name, Py_TYPE(result)->tp_name);
    Py_DECREF(result);
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x) {
#if CYTHON_USE_TYPE_SLOTS
  PyNumberMethods *m;
#endif
  const char *name = NULL;
  PyObject *res = NULL;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_Check(x) || PyLong_Check(x)))
#else
  if (likely(PyLong_Check(x)))
#endif
    return __Pyx_NewRef(x);
#if CYTHON_USE_TYPE_SLOTS
  m = Py_TYPE(x)->tp_as_number;
  #if PY_MAJOR_VERSION < 3
  if (m && m->nb_int) {
    name = "int";
    res = m->nb_int(x);
  }
  else if (m && m->nb_long) {
    name = "long";
    res = m->nb_long(x);
  }
  #else
  if (likely(m && m->nb_int)) {
    name = "int";
    res = m->nb_int(x);
  }
  #endif
#else
  if (!PyBytes_CheckExact(x) && !PyUnicode_CheckExact(x)) {
    res = PyNumber_Int(x);
  }
#endif
  if (likely(res)) {
#if PY_MAJOR_VERSION < 3
    if (unlikely(!PyInt_Check(res) && !PyLong_Check(res))) {
#else
    if (unlikely(!PyLong_CheckExact(res))) {
#endif
        return __Pyx_PyNumber_IntOrLongWrongResultType(res, name);
    }
  }
  else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_TypeError,
                    "an integer is required");
  }
  return res;
}
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject* b) {
  Py_ssize_t ival;
  PyObject *x;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_CheckExact(b))) {
    if (sizeof(Py_ssize_t) >= sizeof(long))
        return PyInt_AS_LONG(b);
    else
        return PyInt_AsSsize_t(b);
  }
#endif
  if (likely(PyLong_CheckExact(b))) {
    #if CYTHON_USE_PYLONG_INTERNALS
    const digit* digits = ((PyLongObject*)b)->ob_digit;
    const Py_ssize_t size = Py_SIZE(b);
    if (likely(__Pyx_sst_abs(size) <= 1)) {
        ival = likely(size) ? digits[0] : 0;
        if (size == -1) ival = -ival;
        return ival;
    } else {
      switch (size) {
         case 2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
      }
    }
    #endif
    return PyLong_AsSsize_t(b);
  }
  x = PyNumber_Index(b);
  if (!x) return -1;
  ival = PyInt_AsSsize_t(x);
  Py_DECREF(x);
  return ival;
}
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b) {
  return b ? __Pyx_NewRef(Py_True) : __Pyx_NewRef(Py_False);
}
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t ival) {
    return PyInt_FromSize_t(ival);
}


#endif /* Py_PYTHON_H */
